#!/bin/bash                                                                     

usage()
{
    cat <<EOF

    Usage:
    MarkGoodRun.sh [options]

    Prints the metadata of the given file and the metadata of all ancestors. Optionally, given a run, subrun and detector, all related data files will have their metadata changed

    Options:
    --dev              Use the development SAM db
    --file <file>      Sets the file whose metadata is to be modified. Supercedes run based selection.
    --run <run>        Sets the run number to be marked
    --subrun <subrun>  Sets the subrun to be marked
    --det  <det>       Sets the detector to be marked. Can be fardet, neardet, or ndos.
    --setGood          Sets good run metadata to true
    --setBad           Sets good run metadata to false
    --help             Prints this message
EOF
}

MakeFileList()
{
    rawList="$1"
    rawArr=($(echo ${rawList}))
    
    descendantsList=""

    for i in ${rawArr[@]}
      do
      descendantsList+="$(samweb -e nova file-lineage descendants $i) "
    done
 
    fileList="$rawList $descendantsList"
}

DEVOPT=""
FILE=""
ISGOOD=""
RUN=""
DET=""
SUBRUN=""

while [ $# -gt 0 ]
do
    case "x$1" in
    x--help)        usage; shift; exit 0;    continue;;
    x--dev)         DEVOPT="--dev"; shift;   continue;;
    x--file)        FILE="$2";      shift 2; continue;;
    x--setGood)     ISGOOD="true";  shift;   continue;;
    x--setBad)      ISGOOD="false"; shift;   continue;;
    x--det)         DET="$2";       shift 2; continue;;
    x--run)         RUN="$2";       shift 2; continue;;
    x--subrun)      SUBRUN="$2";    shift 2; continue;;
    *)              echo "Unknown argument!"; shift; usage; exit 1; continue;;
    esac
    break
done

#make sure ISGOOD is non-null

if [ "x${ISGOOD}" = "x" ]
    then
    echo "No good run flag set!"
    usage
    exit 1
fi

#fill rawList with raw files to be marked

rawList=""

#if FILE is non-null make sure FILE contains no /s.

if [ "x${FILE}" != "x" ]
    then
    #look for /s in the filename (SAM does not use the full path)
    if [ "$FILE" != "${FILE///}" ]
	then
	echo "SAM only accepts basenames, not full paths!"
	usage
	exit 1
    fi
    rawList="$FILE"
elif [ "x${RUN}" != "x" -a "x${SUBRUN}" != "x" -a "x${DET}" != "x"  ]
    then
    rawList=$(samweb -e nova list-files "Online.Detector = ${DET} and data_tier = raw and online.runnumber = ${RUN} and online.subrun = ${SUBRUN}")
else
    echo "Incorrect switches!"
    usage
    exit 1    
fi

#makes $fileList

MakeFileList "$rawList"

fileArr=($(echo ${fileList})) 

jsonfile=`mktemp -p .`

cat > $jsonfile <<EOF
{"DQ.IsGoodRun": "$ISGOOD"}
EOF

for i in ${fileArr[@]}
  do 
  echo
  echo "Modifying file: $i"
  samweb -v -e nova modify-metadata $i $jsonfile
done

rm $jsonfile

