##############
# Setup the Job specific parameters
DATASET=S14-02-05CryFD-Draining-Faster
SITES="FZU_NOVA,FNAL_nova,FNAL_nova_opportunistic,Harvard,UCSD,SMU,OSC,Wisconsin,MWT2" #Removed Nebraska from site list while they have Oasis issues
NUMFILES=1
NUMJOBS=1280
JOBSCRIPT="${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh"
JOBNAME="SAMProductionJobFDCry"
OUTPUT_DIR="/nova/prod/fts_dropbox"
EXECUTABLE="nova"
TIMEOUT=1296000 # 36 Hours
ROLE=Production # Which role to use, options are blank, Production
PROJECTNAME="$1"
echo "Project name is: $PROJECTNAME"

#############
# Set the Experiment Name
export EXPERIMENT=nova
#############
# Set the NOvA Base Release to use
nova_release="S14-02-05"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:$PRODUCTS

#############
# Setup Jobsub and IFDH (art)
setup jobsub_tools v1_2p
setup ifdh_art v1_3_1 -z /nusoft/app/externals -q e4:debug:nu

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

#############
# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"
if [[ "x$PROJECTNAME" == "x" ]]; then
  DATE="`date +%Y%m%d_%H%M%S_%N`"
  PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
  ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  if [ $? -eq 0 ]; then
      export SAM_PROJECT_NAME="${PROJECTNAME}"
  fi
else
  export SAM_PROJECT_NAME="${PROJECTNAME}"
fi
echo "$SAM_PROJECT_NAME" >>projectnames.log


##########################################
# Submit Onsite AND Offsite
#

##########################################
# The Magic Jobsub command
#--rename2 "s/daq/reco"
jobsub \
    -N ${NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_STATION \
    -e SAM_PROJECT_NAME \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -l "+JobType = \"MC\"" \
    -g \
    ${JOBSCRIPT} \
            --export EXTERNALS='$PRODUCTS' \
            --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
            --getconfig \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft_cvmfs.sh:-b:maxopt:-r:'"$nova_release" \
	    --limit ${NUMFILES} \
            --addoutput '*.daq.root' \
	    --rename unique \
            --self-destruct-timer $TIMEOUT \
	    --dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=importedSimulated \
	        --sam-application-family=nova \
	        --sam-data-tier=out1:artdaq \
	        --sam-application-version=$nova_release
