##########################################
# Basic Configuration
#
# If you're using this script to start
# multiple projects, these are the parameters
# that should be shared between all of them
##########################################

##############
# These will have to change based on the type of job you're running
OUTPUT_DIR="/nova/ana/users/eflumerf" # Where are the output files going to go?
TIMEOUT=12960000 # If the job runs longer than this (in seconds), it will try to kill itself to get log files back
ROLE=Production # Which role to use, options are blank, Production
NUMINPUTS=1088 # What is the target number of files in the input dataset?
NUMJOBS=5 # How many jobs are we going to start? -1 for as many are in the dataset
RECOVERY_DATASET=1 # Is this a recovery run? 1: Yes, 0: No. Recovery runs will start NUMJOBS jobs, even if there aren't NUMINPUTS files in the dataset
DATA_TIER="out1:artdaq" # Data tier: one of out1:artdaq, reco, pidpart, ...
FILETYPE="importedSimulated" # File Type: importedSimulated for MC, importedDetector for Data
FCL_FILE= # Job FCL file to use, blank for getconfig (MC Generation) case
DATASET=S14-03-40-CryFD-FullSet # SAM Dataset to use. ******EDIT ME!!!!******
OVERLAY_DATASET= # Dataset for overlay files, blank if not doing ND Rock Overlays
JOBNAME=SAMOffsitePing # How do you want your jobs to show up in SAM? $USER-$JOBNAME-$DATETIME
PROJECTNAME=$1 # If you want to add jobs to an existing project, send the project name as a command line option
NOVA_RELEASE="S14-05-12" # Which release of the Offline to use?
DEDICATED_SITES="FNAL_nova FNAL_nova_opportunistic" # Half of jobs will be submitted to DEDICATED_SITES, and the other
ALL_SITES="${DEDICATED_SITES} FZU Harvard HARVARD_nova MWT2 Nebraska Omaha OSC SMU SMU_nova UCSD Wisconsin"      # to ALL_SITES. Site list: OSC,Harvard,FZU,MWT2,UCSD,Wisconsin
HIGHPRIO= # Should we run on FermiGrid with High Priority? 1: yes, blank: no.

#############
# Loop Control Parameters
JobStarted=0
#Job2Started=0 Example of Multiple-Project Submission!

##########################################
# Submission Wait Loop
# 
# This script will wait for all of anticipated files to be declared
# then it will submit jobs
##########################################
if [ "x$FUNCTIONS_DEFINED" == "x" ] && [ "x$DEFINING_FUNCTIONS" == "x" ]; then
   echo "$(tput setaf 6)Sourcing myself to learn about my functions..."
   export DEFINING_FUNCTIONS=1
   pushd `dirname $0` > /dev/null
   SCRIPTPATH=`pwd`
   popd > /dev/null

   source "$SCRIPTPATH/$0"
fi
if [ "x$FUNCTIONS_DEFINED" != "x" ]; then
  while [ $JobStarted -eq 0 ]; do
  # Multi-project submission should use this:
  #while [ [ $JobStarted -eq 0 ] && [ $Job2Started -eq 0 ] ]; do
    
    # if [ $JobStarted -eq 0 ]; then
    #   JOBNAME=<First Job's Name>
    #   DATASET=<First Job's Dataset>
    #   submit_prioritized
    #   JobStarted=$(( 1 - $? ))
    # fi

    # if [ $Job2Started -eq 0 ]; then
    #   JOBNAME=<Second Job's Name>
    #   DATASET=<Second Job's Dataset>
    #   submit_prioritized
    #   Job2Started=$(( 1 - $? ))
    # fi

    if [ $JobStarted -eq 0 ]; then
      submit_prioritized
      JobStarted=$(( 1 - $? ))
    fi
    # Be nice about samweb calls, space things out a little bit
    if [ $JobStarted -eq 0 ]; then
      sleep 300
    fi
  done
fi

##########################################
# Function Declarations
#
# For Job-specific parameters, 
#   skip this section
##########################################
function do_submit()
{
echo "$(tput setaf 1)Running Jobsub..."


# Determine whether to use --config or --getconfig
CONFIG="--config ${FCL_FILE}"
if [ "x${FCL_FILE}" == "x" ]; then
    CONFIG="--getconfig"
fi
HIGHPRIO_CMD="-G 'nova_high_prio'"
if [ "x${highprio}" == "x" ]; then
    HIGHPRIO_CMD=""
fi
ROLEARG="--role Production"
if [ "x${ROLE}" == "x" ]; then
    ROLEARG=""
fi
# Submit Onsite AND Offsite
#
# The Magic Jobsub command
#jobsub
#jobsub_submit.py --group nova --debug \
#    --jobsub-server $SERVER \
#    ${ROLEARG} 
jobsub \
    -N ${JOBSUB_NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_STATION \
    -e SAM_PROJECT_NAME \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -e NOVADBTIMEOUT \
    -e NOVADBUSER  \
    -e NOVADBPWDFILE \
    -l "+JobType = \"MC\"" \
    -g ${HIGHPRIO_CMD} \
    ${JOBSCRIPT} \
            --multifile \
            --export EXTERNALS='$PRODUCTS' \
            --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
            ${CONFIG} \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft.sh:-b:maxopt:-r:'"$nova_release" \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novagrid.sh:-r:'"${NOVA_RELEASE}:-b:maxopt" \
	    --source "/cvmfs/oasis.opensciencegrid.org/fermilab/products/common/etc/setups.sh;setup:sam_web_client:v1_5:-z:/cvmfs/oasis.opensciencegrid.org/fermilab/products/common/db" \
            --limit ${NUMFILES} \
            --self-destruct-timer $TIMEOUT \
            --addoutput '*.daq.root' \
            --rename unique \
	    --hash \
            --dest ${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=${FILETYPE} \
	        --sam-application-family=nova \
	        --sam-data-tier=${DATA_TIER} \
	        --sam-application-version=$nova_release

# For overlay processing
  #          --export SECONDARY_DATASET="$OVERLAY_DATASET" \
  #          --export NUMBER_OF_SECONDARY="40" \
  #          --export OVERLAY_FCL='prodNDRockMixMerge.fcl' \
  #          --inputfile "$SRT_PUBLIC_CONTEXT/Metadata/samUtils/overlay_files.sh" \
  #          --postscript 'chmod:ug+x:./overlay_files.sh' \
  #          --postscript './overlay_files.sh' \

}

function make_project() {
# Set up SAM-specific parameters and create a project if necessary
echo "$(tput setaf 5)Setting up SAM; I'll create a project if I need to"

# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"

# If there's no project name defined, create a default name
if [[ "x$PROJECTNAME" == "x" ]]; then
  DATE="`date +%Y%m%d_%H%M%S_%N`"
  PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
  ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  if [ $? -eq 0 ]; then
      export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
     echo "Error starting project!" >&2
     exit 1
  fi
else
  # See if this is an already-running project...
  ifdh findProject "${PROJECTNAME}"
  if [ $? -ne 0 ]; then
    # It's not; start it
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  fi
  if [ $? -eq 0 ]; then
    export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
    echo "Error starting project!" >&2
    exit 1
  fi
fi

# For keeping track...
echo "$(tput sgr0)$SAM_PROJECT_NAME" >>projectnames.log

}

function safe_submit {
  if [ $JOBSUB_NUMJOBS -le 1000 ]; then
    do_submit
  else
    local total=$JOBSUB_NUMJOBS
    local submitted=0
    echo "$(tput setaf 2)Submitting $(( $total / 1000 + 1 )) batches of 1000 jobs each"
    while [ $submitted -lt $total ]; do
       JOBSUB_NUMJOBS=1000
       do_submit
       submitted=$(( $submitted + 1000 ))
    done
    JOBSUB_NUMJOBS=$total
  fi
}

function submit_prioritized {
  setup_environment
  # Submit half the jobs to the NOvA Dedicated sites. The rest can go anywhere
  local ok_to_submit=0
  local definition_files=`samweb -e nova count-definition-files $DATASET`
  if [ $definition_files -ge ${NUMINPUTS} ] && [ ${RECOVERY_DATASET} -eq 0 ]; then
    ok_to_submit=1
  fi

  if [ $definition_files -gt 0 ] && [ ${RECOVERY_DATASET} -eq 1 ]; then
    ok_to_submit=1
  elif [ ${RECOVERY_DATASET} -eq 1 ]; then
    return 0
  fi

   if [ ${ok_to_submit} -eq 1 ]; then
    if [ $NUMJOBS -lt 0 ]; then
      NUMJOBS=$definition_files
    fi
    make_project
    JOBSUB_NUMJOBS=$NUMJOBS

    for SITES in $ALL_SITES; do
        safe_submit
    done
    return 0
  fi

  return 1
}

function setup_environment {
echo "$(tput setaf 6)Setting up Environment..." 
#############
# Job wrapper parameters
#EXECUTABLE="nova"
EXECUTABLE="nova"
NUMFILES=1
JOBSCRIPT="${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh"

#############
# Set the Experiment Name
export EXPERIMENT=nova

#############
# Set the NOvA Base Release to use
nova_release=$NOVA_RELEASE

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:/grid/fermiapp/products/common/db:$PRODUCTS

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

#############
# Configure X509
kx509
getcert -s
export X509_CERT_DIR=/etc/grid-security/certificates
export X509_USER_CERT=/tmp/x509up_u${UID}
export X509_USER_KEY=/tmp/x509up_u${UID}
export SERVER=https://fifebatch.fnal.gov:8443

#############
# Setup Jobsub and IFDH (art)
setup sam_web_client
setup jobsub_tools v1_2q
setup jobsub_client v0_3_1
setup ifdh_art v1_4_2 -z /nusoft/app/externals -q e5:debug:nu
#if [ "`which $JOBSCRIPT`" != $JOBSCRIPT ]; then
#    JOBSCRIPT="`which $JOBSCRIPT`"
#fi

}

##########################################
# END of Function Declarations
##########################################
export FUNCTIONS_DEFINED=1

echo "$(tput sgr0)"
trap 'echo "$(tput sgr0)"' EXIT
echo DONE
