##############
# Setup the Job specific parameters
DATASET=$1
JOBNAME='prodrecohists_'$DATASET
NUMFILES=$2
NUMJOBS=-1 # if <= 0, it submits all of the jobs in the project
JOBFCL=$3
OUTPUT_DIR='/nova/prod/data/validation/FA14-10-03/'$4

echo "OUTPUT_DIR is: $OUTPUT_DIR"
echo "JOBFCL is $JOBFCL"

DATA_TIER="reco"
ROLE="Production"

#############
# Setup the SAM project here
DATE="`date +%Y%m%d_%H%M`"
PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
if [ $? -eq 0 ]; then
    export SAM_PROJECT_NAME="${PROJECTNAME}"
fi


if [ ${NUMJOBS} -le 0 ]; then
  export NUMPROJECTFILES=`samweb -e nova count-definition-files ${DATASET}`
  if [ ${NUMFILES} -eq 1 ]; then
    export NUMJOBS=${NUMPROJECTFILES}
  else
    export NUMJOBS=`echo "(${NUMPROJECTFILES}/ ${NUMFILES}) + 1" | bc`
  fi
  

fi

echo "NUMJOBS IS $NUMJOBS"

echo "DATASET" $DATASET

ln -s $SCRIPT /nova/app/condor-exec/$USER/${PROJECTNAME}.sh
JOBSCRIPT=/nova/app/condor-exec/$USER/${PROJECTNAME}.sh
echo "Project name is: $SAM_PROJECT_NAME"
echo "Station Monitor: http://samweb.fnal.gov:8480/station_monitor/nova/stations/nova/projects/$SAM_PROJECT_NAME"
echo "$SAM_PROJECT_NAME" >> $0.projectname.log


##########################################
# The Magic Jobsub command
jobsub \
    -N ${NUMJOBS} \
    --OS=SL6 \
    --opportunistic \
    --nowrapfile \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -g \
    -G nova_high_prio \
    -x /scratch/$USER/grid/$USER.nova.Production.proxy \
    ${JOBSCRIPT} \
      --multifile \
      --config $JOBFCL \
      --source '/grid/fermiapp/nova/novaart/novasvn/setup/setup_nova.sh:-r:'"${NOVA_RELEASE}:-b:${BUILD}" \
      --limit ${NUMFILES} \
      --export DEST=${OUTPUT_DIR} \
      -X ${EXECUTABLE} \
        --copyOut \
        --histTier recohist \
