#!/bin/bash

############
# Command Line Arguments project name and director where the log files are.
PROJECT=$1
SCAN_DIR=$2

#############
# Set the Experiment Name
export EXPERIMENT=nova
#############
# Set the NOvA Base Release to use
nova_release="S14-02-05"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
export PRODUCTS=/nusoft/app/externals:$PRODUCTS
setup ifdh_art v1_3_1 -z /nusoft/app/externals -q e4:debug:nu

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"

PROJECT_URI="`ifdh findProject $PROJECT $SAM_STATION`"

wget "$PROJECT_URI/summary?format=json"
mv summary?format=json $PROJECT.log
#ifdh endProject $PROJECT_URI

function scrapeoutfile
{
  grep -m 1 -H "Art has completed" $1 >>samlog.log
  grep -m 1 -H "Running:" $1 >>samlog.log
  grep -m 1 -H -A 1 "TrigReport.*Event  Summary" $logfile|tail -1 >>samlog.log
  grep -m 1 -H -A 1 "TimeReport.*Event  Summary" $logfile|tail -1 >>samlog.log
  grep -E -H "TimeReport *[0-9.]+ *[0-9.]+ *[0-9.]+ *[0-9.]+ *[0-9a-z]+ *$" $1 >>samlog.log

}
function scrapeerrfile
{
  grep -m 1 -H "hostname=" $1 >>samlog.log
  grep -m 1 -H "description=" $1 >>samlog.log
  grep -m 1 -H -B 1 "Aborted" $logfile |head -1 >>samlog.log
  grep -H -E '[0-9]+ -lt [0-9]+' $logfile|tail -1 >>samlog.log
}

# Find the last time marker
for logfile in $SCAN_DIR/*${PROJECT}*.out; do
  scrapeoutfile $logfile
done

# Find the last time marker
for logfile in $SCAN_DIR/*${PROJECT}*.err; do
  scrapeerrfile $logfile
done



