#!/usr/bin/perl
#
# Prepare a summary of a scan file from the event display.
#
# usage:
# % numi_scan_eventlist.pl scan1.txt [ scan2.txt scan3.txt ...] > summary.txt
#
# The output summary.txt file is in a format expected by the
# RunEventFilter module in the Utilities package
#
# The fields expected in the files scan1.txt, scan2.txt, etc. are:
#
# Event identifiers: 0:run/1:subrun/2:event 
# Event category: 3:golden/4:interesting/5:small/6:not candidate/7:other 
# Failure mode: 8:missing dcm/9:feb flash/10:missing hits/
#   11:highly vertical/12:reco failure/13:other
#
#-----------------------------------------------------------------------
#
# Fri Nov 15 08:25:57 CST 2013
# messier@indiana.edu
#=======================================================================
$run_lo    = 999999999;
$subrun_lo = 999999999;
$run_hi    = 0;
$subrun_hi = 0;
$icomment = 0;
foreach $file(@ARGV) {
    open (FILE, $file) or die "ERROR: Cannot open file $ARGV[0]\n";
    $lines = 0;
    for ($lines=0; <FILE>; ++$lines) {
	chomp;
	#
	# Skip first line of file which describes the fields
	#
	if ($lines>0) {
	    @fields = split(' ',$_);
	    $run    = @fields[0];
	    $subrun = @fields[1];
	    $event  = @fields[2];

	    if ($run<=$run_lo) {
		$run_lo = $run;
		if ($subrun<=$subrun_lo) {
		    $subrun_lo = $subrun;
		}
	    }
	    if ($run>=$run_hi) {
		$run_hi = $run;
		if ($subrun>=$subrun_hi) {
		    $subrun_hi = $subrun;
		}
	    }
	
	    $f = 3;
	    $golden        = @fields[$f++];
	    $possible      = @fields[$f++];
	    $not_candidate = @fields[$f++];

	    $fail_nplane       = @fields[$f++];
	    $fail_hw_dcm       = @fields[$f++];
	    $fail_hw_feb       = @fields[$f++];
	    $fail_missing_hits = @fields[$f++];
	    $fail_bad_track    = @fields[$f++];
	    $fail_other        = @fields[$f++];
	    
	    #
	    # Comment may have spaces and be spread across many
	    # fields.
            #
	    $have_comment = 0;
	    $comment = "";
	    for $j ($f .. $#fields) {
		$comment .= $fields[$j];
		$comment .= ' ';
		$have_comment = 1;
	    }
	    if ($have_comment==1) {
		@comments[$icomment++] = $comment;
	    }
	    
	    $n_golden   += $golden;
	    $n_possible += $possible;
	
	    $n_nplane    += $fail_nplane;
	    $n_hw_dcm    += $fail_hw_dcm;
	    $n_hw_feb    += $fail_hw_feb;
	    $n_fail_hits += $fail_missing_hits;
	    $n_bad_track += $fail_bad_track;
	    $n_other     += $fail_other;

	    ++$n;
	    if ($golden==1 || $possible==1) {
		print "$run $subrun $event\n";
	    }
	}
    }
    close(FILE);
}
print "#\n";
print "# Comments:\n";
for ($i=0; $i<$icomment; ++$i) {
  print "# - @comments[$i]\n";
}
print "#\n";
print "# Scan for runs $run_lo:$subrun_lo to $run_hi:$subrun_hi\n";
print "# $n events total\n";
print "#\n";
print "# $n_golden Golden events\n";
print "# $n_possible Possible events\n";
print "#\n";
print "# $n_hw_dcm DCM-associated failures\n";
print "# $n_hw_feb FEB-associated failures\n";
print "# $n_nplane Low-nplane events\n";
print "# $n_fail_hits Missing hit related failures\n";
print "# $n_bad_track Tracking failures\n";
########################################################################
