#!/bin/bash
# Much of the project managment code here is copied from art_sam_wrap.sh

# Check for expected environment variables
# To rnsure uniqueness of hadron secondaries vs. cosmic primaries generate 
# the secondaries first then use a previously declared secondary project.
: ${SECONDARY_PROJECT:?"Need to set SECONDARY_PROJECT non-empty"}
: ${OVERLAY_FCL:?"Need the OVERLAY_FCL file non-empty"}
: ${conf:?"Need the name of the fcl configuration file (conf) from generation stage"}

if [[ !("$conf" = *.fcl && ("$conf" = fardet* || "$conf" = neardet*)) ]]; then
    echo "No original config file found exiting (error code 5)"
    exit 5
fi

# Input file: under this scheme these are the secondary files (for example hadrons)
input_file=`find ./ -name '*.root'`

out_file=`echo ${input_file##*} | sed s/g4.root/overlay.daq.root/`

# Set nova.special correctly
if [ "$SPECIAL_NAME" != "" ];then
    if [ "$SPECIAL_NAME" = "geantonly" ]; then
	SPECIAL_NAME="none"
    else
	SPECIAL_NAME=`echo $SPECIAL_NAME | sed s/-geantonly//`
    fi
else
    # If SPECIAL_NAME is not set something when wrong.
    # Set it to to something innocuous.
    SPECIAL_NAME="none"
fi

# Collect information about the fcl files used for doing the overlay
FCL_PATH=`dirname $OVERLAY_FCL`
FCL_FILE=`basename $OVERLAY_FCL`
if [ "$FCL_PATH" = "." ] ; then
    FCL_PATH=$SRT_PUBLIC_CONTEXT/job
fi


# Setup the ifdh base uri
if [ "x$IFDH_BASE_URI" = "x" ]
then
    export IFDH_BASE_URI=http://samweb.fnal.gov:8480/sam/$EXPERIMENT/api
fi

# Get some condor enviroment stuff
if [ -n "$CLUSTER" ]
then
   description="$CLUSTER.$PROCESS"
else
   description=""
fi

projurl=`ifdh findProject $SECONDARY_PROJECT ${SAM_STATION:-$EXPERIMENT}`

consumer_id=''
count=0
while [ "$consumer_id" = "" ]
do
    sleep 5
    consumer_id=`IFDH_DEBUG= ifdh establishProcess "$projurl" "$appname" "$ART_VERSION" "$hostname" "$GRID_USER" "art" "$description" "$limit"`
    count=$((count + 1))
    if [ $count -gt 10 ]
    then
        echo "Unable to establish consumer id!"
        echo "Unable to establish consumer id!" >&2 
        exit 
    fi
done

echo project url: $projurl
echo consumer id: $consumer_id

# I don't know if this is needed but it's in art_sam_wrap.sh so it's also here...
export IFDH_GRIDFTP_EXTRA="-p 0 -dp"

#args="$args \"--sam-web-uri=$projurl\" \"--sam-process-id=$consumer_id\""

# Tell sam to set the next file for this consumer_id
uri=`IFDH_DEBUG= ifdh getNextFile $projurl $consumer_id | tail -1`

# Retrieve the next input file from the project.
fname=`IFDH_DEBUG= ifdh fetchInput "$uri" | tail -1 `
ifdh updateFileStatus $projurl $consumer_id $fname transferred


# Construct the overlay fcl file from the base file stored in svn
ls_cmd="ls $FCL_PATH/$FCL_FILE"
echo "doing $ls_cmd"
echo `eval $ls_cmd`
base_conf=`basename $conf`
hist_file=`echo $base_conf | sed s/root/hist.root/`
fcl_cmd="cat $FCL_PATH/$FCL_FILE | sed s/24580/$RANDOM/ | sed s/config.fcl/$base_conf/ | sed s/_SPECIAL_NAME_/$SPECIAL_NAME/ | sed s/hist.root/$hist_file/ > ./$FCL_FILE"
echo "$fcl_cmd"
eval "$fcl_cmd"
echo 'filters.singlemixer.readMode: "sequential"'  >> ./$FCL_FILE
echo "filters.singlemixer.fileNames: [\"$fname\"]" >> ./$FCL_FILE

# This is metadata crap for the nova command
metadata_args="--sam-file-type=importedSimulated --sam-application-family=nova --sam-data-tier=artdaq --sam-application-version=$SRT_BASE_RELEASE --sam-web-uri=$projurl --sam-process-id=$consumer_id"

# The actual nova command.
nova_cmd="nova -c ./$FCL_FILE -s $input_file -o $out_file $metadata_args"
echo "Running nova command using command:"
echo "$nova_cmd"

# Run the nova command then mark the secondary files appropriately 
if eval $nova_cmd; then
    ifdh updateFileStatus $projurl  $consumer_id $fname consumed
else
    res=$?
    ifdh updateFileStatus $projurl  $consumer_id $fname skipped
fi
ifdh endProcess "$projurl" "$consumer_id"

# Delete the retrieved secondary file.  This way it won't inadvertantly be copied back from --addoutput
rm $fname
