#!/usr/bin/env python
import sys
import getopt
import shlex, subprocess
import re, urllib2, csv
from datetime import date, datetime, timedelta
import psycopg2
import psycopg2.extras

def read_list(file):
    f = open(file, 'r')
    runs=[]
    subruns=[]
    while (1):
        line = f.readline()
        if (line==''):
            break
        r = line.split("\t")
        runs.append(int(r[0]))
        subruns.append( (int(r[0]),int(r[1])) )
    runs.sort()
    return (runs,subruns)

#.......................................................................

def run_minmax(runs):
    run_min = runs[0]
    run_max = runs[0]
    for r in runs:
        if (r<run_min):
            run_min = r
        if (r>run_max):
            run_max = r
    return (run_min, run_max)

#.......................................................................

def read_file_list(file):
    print 'Read file list:',file
    f = open(file,'r')
    runs=[]
    subruns=[]
    while (1):
        r = f.readline()
        if (r==''):
            break;
        #
        # Fragile! Assume file names are fixed length formats
        #
        run    = int(r[64:72])
        subrun = int(r[74:76])        
        runs.append(run)
        subruns.append( (run,subrun) )
    runs = set(runs)
    runs = list(runs)
    runs.sort()

    return (runs,subruns)

#.......................................................................

def runs_db(rmin, rmax):
    try:
        conn = psycopg2.connect(\
        "dbname=nova_prod "\
        "host=ifdbprod.fnal.gov "\
        "user=nova_reader "\
        "password=nova2pass port=5433"\
        )
    except:
        print "I am unable to connect to the runs database"

    db_cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    sql = "select run,nsubruns,tstart,tstop from fardet.runs where partition=1 and tstop is not null and run>=%d and run<=%d order by tstart"%(rmin,rmax)

    db_cursor.execute(sql)
    runs = db_cursor.fetchall();

    return runs

#.......................................................................

def in_unix(input):
    """ Convert a datetime to UNIX time

    Args:
    input: A datetime object

    Returns:
    A floating points number of seconds since Jan. 1, 1970 (UNIX standard)
    
    """
    start = datetime(year=1970,month=1,day=1,hour=0,minute=0,second=0)
    diff = input - start
    return diff.total_seconds()

#.......................................................................

def spills_between(t1, t2):
    """
    Return spill information (unix time and POT) for a datetime range

    Args:
    t1 (datetime): Start time
    t2 (datetime): End time

    Returns:
    List of (time,pot) tuples
    """

    spills = [];    

    tf1 = in_unix(t1)
    tf2 = in_unix(t2)
    webaddress = 'http://ifb-data.fnal.gov:8099'\
                 '/ifbeam/data/data?'\
                 'v=E:TRTGTD&e=e,a9&t0={}&t1={}&&f=csv'.\
                 format(tf1,tf2)

    dd = None
    #
    # Make three tries and then bail...
    #
    try:
        dd = urllib2.urlopen(webaddress,None,600)
    except:
        try:
            dd = urllib2.urlopen(webaddress,None,600)
        except:
            try:
                dd = urllib2.urlopen(webaddress,None,600)
            except:
                return spills

    if (dd==None):
        return spills
    
    csvdata = csv.reader(dd)

    for row in csvdata:
        if (row.count('E:TRTGTD')>0):
            if (row[4]!='null'):
                spills.append( (float(row[2])/1000., float(row[4])) )

    return spills

#.......................................................................

def count_subruns(subruns, run):
    count = 0
    for (r,s) in subruns:
        if (run==r):
            count = count + 1
    return count

#.......................................................................

def usage():
    print ""
    print "pot-for-runs takes a list of files or subruns and prints a table of:"
    print "1. run number"
    print "2. total number of subruns recorded for the run"
    print "3. number of subruns appearing in the list"
    print "4. run start time"
    print "5. run end time"
    print "6. total POT for the run"
    print "7. estimated POT for subruns in list, scaled by ratio of col3/col2 above."
    print ""
    print "Usage:"
    print ""
    print "% pot-for-runs -h -r [subrun list] -f [file list]"
    print ""
    print "Where:"
    print ""
    print "-h prints this help"
    print "-s skips runs that are not in list. Otherwise steps sequentially by run number."
    print "-r file.txt points to a text file of [run subrun] pairs separated by a space."
    print "-f file.txt points to a text file of file names in standard format"
    print ""
    print "You cannot give both -r and -f together."
    print ""
    
#.......................................................................

def main():
    file = None
    fmt  = None
    skip = True
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "hsr:f:",
                                   ["help","sequential","runs=","files="])
    except getopt.GetoptError as err:
        print str(err)
        usage()
        sys.exit(2)
    for o, a in opts:
        if (o in ("-h","--help")):
            usage()
            sys.exit(0)
        if (o in ("-s","--sequential")):
            skip = False
        if (o in ("-r","--runs")):
            file = a
            fmt  = 1
        if (o in ("-f","--files")):
            file = a
            fmt  = 2
    if (file==None):
        usage()
        sys.exit(1)
    if (fmt==1):
        (runs, subruns) = read_list(file)
    if (fmt==2):
        (runs, subruns) = read_file_list(file)

    (rmin, rmax) = run_minmax(runs)
    runs = runs_db(rmin, rmax)

    fmt = "%d/%m/%y %H:%M:%S"

    pot_total = 0
    pot_total_scaled = 0
    for r in runs:
        run  = r['run']
        t1   = r['tstart']
        t2   = r['tstop']
        nsub = r['nsubruns']
        
        spills = spills_between(t1, t2)
        pot_run = 0
        for (t,q) in spills:
            pot_run += q
        pot_total += pot_run

        count = count_subruns(subruns, run)
        if (skip & (count==0) ):
            continue

        pot_run_scaled = pot_run*float(count)/float(nsub)
        pot_total_scaled += pot_run_scaled
        
        print run,'\t',nsub,'\t',count,'\t',t1.strftime(fmt),'\t',t2.strftime(fmt),'\t',pot_run,'\t',pot_run_scaled

    print "TOTAL POT ", pot_total
    print "TOTAL SCALED POT ", pot_total_scaled

    return
    
#.......................................................................

if __name__=="__main__":
    main()

########################################################################
