#! /usr/bin/env python 

# Python std stuff
import sys, datetime
import os
# all of root 
from ROOT import *
# My utilities
import novaStyle, canMan, HTMLTools as html 
# Hist set for early data plots
from collections import OrderedDict
import psycopg2
from WatchdogFx import *


gEnv.SetValue("TFile.Recover", 0)


webDir = "/nusoft/app/web/htdoc/nova/datacheck/watchdogs/"




fullNames = OrderedDict()
fullNames["fardet"] = "Far Detector"
fullNames["neardet"] = "Near Detector"

now = datetime.datetime.now()
dateStr = str(now.year) + "-"+ str(now.month) + "-" + str(now.day) + "_" + str(now.hour) + "-" + str(now.minute)
outputDir = webDir + dateStr + "/"
print "Output results in : ", outputDir   
if not os.path.isdir(outputDir):
  os.mkdir(outputDir)

url = "http://nusoft.fnal.gov/nova/datacheck/watchdogs/" + dateStr + "/watchdog.html"

recoveryUrl = "http://cdcvs.fnal.gov/redmine/projects/datacheck/wiki/Recovering_Missing_Files_from_FTS"


outputHtml = open(outputDir + "watchdog.html", 'w')
outputHtml.write(html.preventCaching())

outputHtml.write(html.bootStrap())

files = os.listdir(outputDir)
outputHtml.write(html.title("Watchdog page for " + dateStr))
jumboText = html.heading1("NOvA Offline Data Watchdog Summary")
jumboText += html.paragraph("Generated on " + now.strftime("%Y/%m/%d at %H:%M") + ".  ")
jumboText +=  html.button("Permalink", url)
jumboText +=  html.button("Latest page", "http://nusoft.fnal.gov/nova/datacheck/watchdogs/latest/watchdog.html")

outputHtml.write(html.jumbo(jumboText))

outputHtml.write(html.paragraph("If you notice missing files, follow the " + html.link("recovery instructions", recoveryUrl, newTab=True)+"."))

outputHtml.write(html.paragraph("Note: negative number of missing files indicate that file transfer got ahead of either Nearline or database replication."))



outputHtml.write(html.paragraph("For older runs, see the " + html.link("All-time Processing History", "/nova/datacheck/processinghistory/history.html", newTab=True)+"."))

for detector, onMonDir in onMonDirs.items():
  (maxRunNumDB, minRunNumDB) = getMaxMinRunNumDB(detector) 
  onMonRunsHist =  getCompletedRunsHist(onMonDir, "OnMon Files", maxRunNumDB, minRunNumDB, lineStyle=7, lineColor=onMonColor, additionalDir=onMonTriggerDirs[detector], skipStr=".SMALL")
  dbRunsHist = getDBRunsHist(minRunNumDB, maxRunNumDB, detector, lineStyle=1, lineColor=dbColor)
  print 'export PGPASSWORD=' + pw + '; psql -h ifdbprod.fnal.gov -d nova_prod -p '+dbPort[detector] + ' -U nova_reader  -H -c "select run, nsubruns, nevents, partition, tstart, shifter, begin_comment, end_comment  from fardet.runs where run > '+str(minRunNumDB) +' and run < '+str(maxRunNumDB) +' order by run asc;" > ' + outputDir + 'dbDump_'+detector+'.html'
  #os.system('export PGPASSWORD=' + pw + '; psql -h ifdbprod.fnal.gov -d nova_prod -p '+dbPort[detector] + ' -U nova_reader  -H -c "select run, nsubruns, nevents, partition, tstart, shifter, begin_comment, end_comment  from fardet.runs where run > '+str(minRunNumDB) +' and run < '+str(maxRunNumDB) +' order by run asc;" > ' + outputDir + 'dbDump_'+detector+'.html')
  outputHtml.write(html.heading2(fullNames[detector] + " Processing History - Last Seven Days"))
  #outputHtml.write(html.paragraph(html.link("Database table dump", "dbDump_"+detector+".html") + " for these runs."))


  for key, path in paths.items():
    if not detector in key:
      continue
    plotName = path.name + ".png"
    (can,hists) = path.getPlotCanvas(maxRunNumDB, minRunNumDB, dbRunsHist, onMonRunsHist)
    can.SaveAs(os.path.join(outputDir, plotName))
    onMonDiffHist = (onMonRunsHist - hists[0])
    dbDiffHist = dbRunsHist - hists[0]
    onMonDiff = onMonDiffHist.Integral(1, onMonDiffHist.GetNbinsX() - 1) 
    dbDiff = dbDiffHist.Integral(1, dbDiffHist.GetNbinsX() - 1) 
    text = ""
    if dbDiff > 0 or onMonDiff > 0:
      text+= html.alertDanger(html.bold("Warning! ")+" Files appear to be missing, please follow the " + html.alertLink("recovery instructions", recoveryUrl, newTab=True)+".")
    text +=  html.img(plotName,"")
    text += html.paragraph("Missing raw files according to database: " + str(dbDiff))
    text += html.paragraph("Missing raw files according to OnMon files: " + str(onMonDiff))

    outputHtml.write(html.panel(path.title, text))

    



outputHtml.close()
redirectUrl = dateStr + "/watchdog.html"

apacheFile = open(webDir + ".htaccess", 'w')
apacheFile.write("RewriteEngine on \n")
apacheFile.write("RewriteRule   latest.html  /nova/datacheck/watchdogs/" + redirectUrl + " [R] \n")
apacheFile.write("RewriteRule   latest/(.*)  /nova/datacheck/watchdogs/"+ dateStr +"/$1 [PT] \n")
apacheFile.close()


