///
/// \file   dEdxTable.h
/// \brief  Tables of mean energy loss in NOvA materials
/// \author messier@indiana.edu
///
#ifndef BPFIT_DEDXTABLES_H
#define BPFIT_DEDXTABLES_H

namespace bpfit {
  ///
  /// \brief Tables of mean energy loss in NOvA materials
  ///
  class dEdxTable {
  public:
    ///
    /// \brief Build dEdx table for named material
    /// \param mat - Material name
    ///
    /// \warning If the material is unknown will quietly return 0's always.
    ///
    dEdxTable(const char* mat);
    
    ///
    /// Return dE/dx in units of GeV/cm
    ///
    /// \param T - kinetic energy of particle (GeV)
    /// \param m - mass of particle (GeV)
    /// \return dEdx in units of GeV/cm
    ///
    double operator()(double T, double m) const;
    
  private:
    void InitScintillator();
    void InitPVC();
    void InitSteel();
    void InitGlue();
    void InitVac();

  private:
    double TtoBetaGamma(double t,  double m) const;
    double BetaGammaToT(double bg, double m) const;
    
    double Interp(double t,
		  double m,
		  const double T[],
		  const double dedx[],
		  unsigned int imax) const;
  private:
    unsigned int  fSz;   ///< Table size
    const double* fT;    ///< Muon kinetic energy
    const double* fdEdx; ///< Muon energy loss
    double        fRho;  ///< Material density
  };
}
#endif
////////////////////////////////////////////////////////////////////////
