///////////////////////////////////////////////////////////////////////////
/// \brief   A DCMSummary contains summary information to calculate dcm timing delays
/// \author  edniner@indiana.edu
/// \date    Feb. 2013
///////////////////////////////////////////////////////////////////////////
#ifndef DCMSummary_H
#define DCMSummary_H

#include <vector>
#include <stdint.h>

/// Calibration data products
namespace caldp
{
  ///maximum number of dcms possible on a detector
  static const int kDCMNumMax = 168;
  

  class DCMSummary
  {
  public:
    DCMSummary();
    
    virtual ~DCMSummary();
    
    /// return start time offset is valid for
    uint32_t    StartTime () const {return sTime; };
    /// return end time offset is valid for
    uint32_t    EndTime () const {return eTime; };
    std::vector<int> DCMUse(std::vector<int> in) const {return dcmUse;};
    std::vector<std::vector<int> > DCMOffsetCount() const {return dcmOffsetCount;};
    std::vector<std::vector<float> > DCMOffsetSum() const {return dcmOffsetSum;};
    std::vector<std::vector<float> > DCMOffsetSumSquare() const {return dcmOffsetSumSquare;};

    int DCMOffsetCountIndx(int i, int j) const {return dcmOffsetCount[i][j];};
    float DCMOffsetSumIndx(int i, int j)const  {return dcmOffsetSum[i][j];};
    float DCMOffsetSumSquareIndx(int i, int j) const  {return dcmOffsetSumSquare[i][j];};
    int DCMUseIndx(int i) const {return dcmUse[i];};

    void  SetStartTime (uint32_t aStart) {sTime = aStart; };
    /// Set end time offset is valid for
    void SetEndTime (uint32_t aEnd) {eTime = aEnd; };
    void SetDCMUse(std::vector<int> in) {dcmUse = in;};
    void SetDCMOffsetCount(std::vector<std::vector<int> > in) {dcmOffsetCount = in;};
    void SetDCMOffsetSum(std::vector<std::vector<float> > in) {dcmOffsetSum = in;};
    void SetDCMOffsetSumSquare(std::vector<std::vector<float> > in) {dcmOffsetSumSquare = in;};
    void SetDCMOffsetCountIndx(int i, int j, int val) {dcmOffsetCount[i][j] = val;};
    void SetDCMOffsetSumIndx(int i, int j, float val) {dcmOffsetSum[i][j] = val;};
    void SetDCMOffsetSumSquareIndx(int i, int j, float val) {dcmOffsetSumSquare[i][j] = val;};
    void SetDCMUseIndx(int i, int val) {dcmUse[i] = val;};



    
  protected:    
    uint32_t sTime; ///< start time offset is valid for
    uint32_t eTime; ///< end time offset is valid for
    ///vector of number of uses for a dcm pair
    std::vector<std::vector<int> > dcmOffsetCount;
    ///vector running sum of relative dcm offsets
    std::vector<std::vector<float> > dcmOffsetSum;
    ///vector running sum squared of dcm offsets
    std::vector<std::vector<float> > dcmOffsetSumSquare;
    ///vector of times a dcm is used
    std::vector<int > dcmUse;

  };
  
} // end namespace

#endif // DCMSummary_H
