////////////////////////////////////////////////////////////////////////
// \file    Chunk.h
// \version $Id: Chunk.h,v 1.13 2012-10-30 23:33:08 bckhouse Exp $
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef DTCHUNK_H
#define DTCHUNK_H

#include "RecoBase/Cluster.h"

#include "DiscreteTracker/Segment.h"

namespace dt
{
  /// Sequence of contiguous hits and dead cells all on the same plane
  class Chunk
  {
  public:
    Chunk(){}
    Chunk(int plane, int cellInLo, int cellInHi, int cellOutLo, int cellOutHi, bool up,
          art::PtrVector<rb::CellHit> hits);
    int Plane() const {return fPlane;}
    int EntryHitCell()      const {return fUp ? fEntryCellHi : fEntryCellLo;}
    int ExitHitCell()       const {return fUp ? fExitCellLo  : fExitCellHi; }
    int EntryExtremeCell()  const {return fUp ? fEntryCellLo : fEntryCellHi;}
    int ExitExtremeCell()   const {return fUp ? fExitCellHi  : fExitCellLo; }
    int TopHitCell()        const {return fUp ? fExitCellLo  : fEntryCellLo;}
    int BottomHitCell()     const {return fUp ? fEntryCellHi : fExitCellHi; }
    int TopExtremeCell()    const {return fUp ? fExitCellHi  : fEntryCellHi;}
    int BottomExtremeCell() const {return fUp ? fEntryCellLo : fExitCellLo; }
    bool Up() const;
    void SetUp(bool up);
    bool Matches(const Chunk& ch) const;
    bool HasHits() const {return !fHits.empty();}
    bool AllDead() const {return  fHits.empty();}
    art::PtrVector<rb::CellHit> AllHits() const {return fHits;}
    art::PtrVector<rb::CellHit> HitsOnLine(double grad, double icept) const;
    // TODO: variants taking a Direction and returning only one?
    std::pair<Segment, Segment> GetSegs() const;

    bool IsWorthShowering() const;
    bool IsShowerChunk() const {return fShowerChunk;}
    void SetShowerChunk(){fSegsDirty = true; fShowerChunk = true;}

    bool operator<(const Chunk& rhs) const {return fPlane < rhs.fPlane;}
  protected:
    /// \param v0 lowest point not in a neighbouring cell
    /// \param v1 lowest point in this cell
    /// \param v2 highest point in this cell
    /// \param v3 highest point not in a neighbouring cell
    void CellPoints(const geo::PlaneGeo* plane, int cell,
                    double& v0, double& v1, double& v2, double& v3,
                    double& z0, double& z1) const;
    /// \param v0 cell centre
    /// \param v1 cell half-width
    void CellMeasurements(geo::View_t view, const geo::CellGeo* cell,
                          double& v0, double& dv,
                          double& z0, double& dz) const;
    int fPlane;
    int fEntryCellLo, fEntryCellHi, fExitCellLo, fExitCellHi;
    bool fUp;
    bool fShowerChunk; ///< "Shower chunk" can enter and exit anywhere

    mutable bool fSegsDirty;
    mutable std::pair<Segment, Segment> fSegs;

    art::PtrVector<rb::CellHit> fHits;
  };

} // namespace

#endif
