#ifndef EARMS_MINIMIZER_H
#define EARMS_MINIMIZER_H

namespace earms 
{
  class ElasticArms;
  ///
  ///
  ///
  class Minimizer
  {
  public:
    Minimizer(ElasticArms* arms);

    void   SeedVertex(double x, double y, double z);
    void   SeedArm(unsigned int, double th, double ph);
    double Fit();
    void   SetT0(double T0);
    void   SetT1(double T1);
    void   SetAnnealFrac(double f);
    void   SetAnnealMaxItr(unsigned int i);

  public:
    /// Maximum number of prongs
    static const unsigned int fMmax = 20;
  public:
    ElasticArms* fArms;         ///< The elastic arms to fit
    unsigned int fNdim;         ///< Number of variables in fit
    unsigned int fM;            ///< Number of prongs
    double       fVtxStep;      ///< Initial vertex step size (cm)
    double       fAngleStep;    ///< Initial angle step size (rad);
    unsigned int fAnnealMax;    ///< Maximum number of annealing steps
    double       fT0;           ///< Initial temperature
    double       fT1;           ///< Final temperature
    double       fAnnealFrac;   ///< Fraction to reduce temperature by
    double       fP[2*fMmax+3]; ///< The fit parameters
  };
}
#endif
////////////////////////////////////////////////////////////////////////
