///
/// \file    MCTrueView.h
/// \brief   A window that gives full list of MC particles, allowing user to
///          turn hits on and off in other views.
/// \author  jpaley@anl.gov
/// \version $Id: MCTrueView.h,v 1.2 2010-11-10 22:38:34 p-novaart Exp $
///
#ifndef EVD_MCTRUEVIEW_H
#define EVD_MCTRUEVIEW_H
#include "TGFrame.h"

#include "EventDisplayBase/ObjListCanvas.h"

#include <map>
#ifndef __CINT__
#include "Simulation/Particle.h"
#endif

class TGTransientFrame;
class TGCompositeFrame;
class TGLayoutHints;
class TGCheckButton;
class TGTextButton;
class TGVScrollBar;
class TGLabel;
class TParticle;
namespace sim { class Particle; }

namespace evd {
  class SimulationDrawer;
  class RawDataDrawer;
  class RecoBaseDrawer;

  /// Window containing list of MC particles
  class MCTrueView : public evdb::ObjListCanvas {
    RQ_OBJECT("evd::MCTrueView")

  public:
    MCTrueView(TGMainFrame* mf);
    ~MCTrueView();
    
    const char* Description() const { return "MC Particle List Viewer"; }
    const char* PrintTag()    const { return "mctrue-view";         }
    void Draw(const char* opt="u");
    void Update();
    void UpdateAndDraw();
    void LoadEvent();

    enum defs_ {
      kNPart    = 10, // number of particles to list at in the window
      kPartName = 0,  
      kMother   = 1,
      kTrackId  = 2,
      kMomentum = 3,
      kVtxZ     = 4,
      kNHit     = 5,
      kTime     = 6,
      kEDep     = 7,
      kNPanel   = 8 // number of panels
    };

  public:
    void SortParticle();
    void SortMother();
    void SortTrackId();
    void SortMomentum();
    void SortVtxZ();
    void SortNHit(); 
    void SortTime(); 
    void SortEDep(); 
    void HiLite();

    void HandleMouseWheel(Event_t *event);

    static bool CompParticle(const sim::Particle& a, const sim::Particle& b);
    static bool CompMother(const sim::Particle& a, const sim::Particle& b);
    static bool CompTrackId(const sim::Particle& a, const sim::Particle& b);
    static bool CompMomentum(const sim::Particle& a, const sim::Particle& b);
    static bool CompVtxZ(const sim::Particle& a, const sim::Particle& b);
    static bool CompNHit(const sim::Particle& a, const sim::Particle& b);
    static bool CompTime(const sim::Particle& a, const sim::Particle& b);
    static bool CompEDep(const sim::Particle& a, const sim::Particle& b);

  private:    
    int fFirstParticle;
    int fCurrEvent;
    const sim::Particle* fCheckMap[kNPart];

    std::vector<int> fPartList;

    TGTransientFrame* fMain;
    TGCompositeFrame* fBigFrame;
    TGCompositeFrame* fPartFrame[kNPanel];
  
    TGLayoutHints fL1,fL2,fL3,fL4,fL5;

    TGCheckButton* fDrawCheck[kNPart];
    TGLabel* fPartLabel[kNPanel][kNPart];
    TGTextButton* fColumnLabel[kNPanel];

    /*
    TGCompositeFrame* fButtonFrame;
    TGTextButton*     fDrawAllButton;
    TGTextButton*     fClearAllButton;
    TGTextButton*     fApplyButton;
    */

    TGVScrollBar*     fScrollBar;
    TGCompositeFrame* fShowFrame;
    TGCheckButton*    fShowPositive;
    TGCheckButton*    fShowNegative;
    TGCheckButton*    fShowNeutral;

    SimulationDrawer* fSim;
    RawDataDrawer*    fRaw;
    RecoBaseDrawer*   fReco;
  };
}

#endif
////////////////////////////////////////////////////////////////////////
