///
/// \file    PlotView.h
/// \brief   A view showing collections of plots related to this event
/// \author  messier@indiana.edu
/// \version $Id: PlotView.h,v 1.2 2012-01-15 07:47:51 messier Exp $
///
#ifndef EVD_PLOTVIEW_H
#define EVD_PLOTVIEW_H
#include "EventDisplayBase/Canvas.h"
#include "RQ_OBJECT.h"
#include <vector>

namespace evd 
{
  /// Data for a single plot appearing in the view
  class Plot {
  public:
    std::string              fModule;
    std::string              fObject;
    std::vector<std::string> fOptions;
  };
  
  /// Data for a single plotting pad
  class PlotPad {
  public:
    std::vector<Plot> fPlot;
  };

  /// View showing collections of plots related to this event
  class PlotView : public evdb::Canvas {
  public:
    RQ_OBJECT("evd::PlotView");
  public:
    PlotView(TGMainFrame* mf);
    ~PlotView();
    
    const char* Description() const { return "Event Plots"; }
    const char* PrintTag()    const { return "plots";       }
    
    void Parse(std::vector<PlotPad>& pads);
    void Draw(const char* opt="");

  private:
  };
}
#endif
////////////////////////////////////////////////////////////////////////
