/* 
 * File:   BufferNodeSimulator.h
 * Author: denis
 *
 * This class simulates the Buffer Node in NOvA DAQ
 *
 * Created on February 14, 2012, 2:50 PM
 */

#ifndef _BUFFERNODESIMULATOR_H
#define	_BUFFERNODESIMULATOR_H
#include "DAQDataFormats/RawDataBlock.h"
#include <vector>

// forward declarations
namespace daqdataformats{
	class RawMilliSlice;
	class RawTrigger;
}


namespace mcdatadaq {

    class BufferNodeSimulator {
        public:
            BufferNodeSimulator(uint32_t buffer_node_id); ///< Constructor
            ~BufferNodeSimulator();                       ///< Destructor

            /// Given all MilliSlices for all DCMs and a Trigger from GlobalTrigger, produce DataBlock
            daqdataformats::RawDataBlock* getDataBlock(const std::vector<daqdataformats::RawMilliSlice*>&,
                                                       daqdataformats::RawTrigger*);

            /// This Buffer Node ID
            uint32_t getBufferNodeID() const;

            /// Advance to the next event - clearing the DataBlock
            bool nextEvent();

        private:

            /// Add a millislice to the dataBlock
            bool addMilliSlice(daqdataformats::RawMilliSlice*);

        private:
            uint32_t                     _bufferNodeID; ///< Buffer Node ID

            daqdataformats::RawDataBlock _dataBlock;    ///< Data Block
    };
}
#endif	/* _BUFFERNODESIMULATOR_H */

