/* 
 * File:   Configuration.h
 * Author: denis
 * Configuration of the MockDataDAQ job
 * Created on February 1, 2012, 8:41 AM
 */

#ifndef _CONFIGURATION_H
#define	_CONFIGURATION_H
#include "DAQDataFormats/RawDAQData.h"

// forward declarations
namespace fhicl{
	class ParameterSet;
}


namespace mcdatadaq {

    static const uint32_t                  DEFAULT_NUMBER_MICROSLICES_IN_MILLISLICE = 1000;
    static const uint32_t                  DEFAULT_DCM_MICROSLICE_DT                = 320; /* length of microslice in clock ticks = 5 us/ 15.625 ns */
    static const uint32_t                  DEFAULT_NUMBER_OF_BUFFERNODES            = 200;
    static const daqdataformats::version_t DEFAULT_NANOSLICE_VERSION                = 0;


    class Configuration {
        public:
            static bool setInstance(fhicl::ParameterSet const& pset); //!< Method to set the class instance with parameters from fhicl
            static Configuration* getInstance(); //!< Method to obtain the class instance

            ~Configuration();
            
            /// initialize detector
            bool init();
            /// Print Configuration
            bool print() const;


            int         getDetector() const;       ///< Get Detector ID
            bool        isDetectorUnknown() const;                  ///< Check Detector if it's unknown ID

            uint32_t    getMicrosliceDT() const;                    ///< MicroSlice DT
            uint32_t    getNumberOfMicroslicesInMilliSlice() const; ///< NumberOfMicroslicesInMilliSlice
            uint32_t    getNumberOfBufferNodes() const;             ///< Number of buffer nodes
            uint32_t    getEventTimeDifference() const;             ///< Event time difference in NOvA clock ticks (15.626 ns)
            bool        isProducingJustMicroSlices() const;         ///< Do we produce just microslices?
            bool        isProducingRunObject() const;               ///< Do we produce Run object?
            bool        isProducingMilliSlices() const{ return fIsProducingMilliSlices;}
            uint32_t    getDAQSimulatorVersion() const;             ///< Get Global DAQSimulator version
            bool        writeDAQSimulatorVersionIntoFile(FILE*) const; ///< Write Global DAQSimulator version into the input file


            daqdataformats::version_t getNanoSliceVersion() const; ///< Get NanoSlice version
            daqdataformats::version_t getMicroBlockVersion() const;///< Get MicroBlock version


            /// Get full file name with a given additional name to specify what kind of file it is,
            /// like "MicroSlices", "Events" or "Run"
            std::string getFullFileName(const std::string addition_to_file_name) const;

            /// Utility method of writing DAQ data into a file
            bool  writeToFile(FILE*, const daqdataformats::RawDAQData&) const;

            /// Open file for binary write access
            FILE* openFile(const std::string file_name) const;

        private:
            Configuration();                                ///< default constructor
            Configuration(fhicl::ParameterSet const& pset); ///< Constructor using fcl parameters

            std::string getOutputDir() const;               ///< GetOutput Directory
            std::string getFileName() const;                ///< Get File name

        private:
            static Configuration*        finstanceID;                      ///!< Instance pointer for singleton class
            
            int                          fDetector;                        ///!< ipnd, near, far


            uint32_t                     fNumberOfMicroslicesInMilliSlice; ///< Number of Microslices in the MilliSlice
            uint32_t                     fMicrosliceDT;                    ///< MicroSlice length in NOvA clock ticks (15.626 ns)

            uint32_t                     fNumberOfBufferNodes;             ///< Number of buffer nodes

            std::string                  fOutputDir;                       ///< Directory for the output files
            std::string                  fFileName;                        ///< Output file names

            uint32_t                     fEventTimeDifference;             ///< Event time difference in NOvA clock ticks (15.626 ns)

            bool                         fIsProducingJustMicroSlices;      ///!< Are we producing just microslices?
            bool                         fIsProducingRunObject;            ///!< Are we producing Run Object?
            bool                         fIsProducingMilliSlices;          ///!< Are me going to produce MilliSlices?

            daqdataformats::version_t    fNanoSliceVersion;
            daqdataformats::version_t    fMicroBlockVersion;


    };// end of class Configuration
}// end of namespace mcdatadaq
#endif	/* _CONFIGURATION_H */

