/* 
 * File:   NanoSliceOrder.h
 * Author: denis
 * 
 * This class calculates the relative weight for NanoSlice to produce nanoslice order
 * given the position of the pixel (diblock, dcm, dcmLink, pixel) and its time.
 * The less weight, the earlier the order is assigned for the nanoslice.
 *
 * Created on February 13, 2012, 4:04 PM
 */

#ifndef _NANOSLICEORDER_H
#define	_NANOSLICEORDER_H
#include "DAQChannelMap/DAQChannelMapConstants.h"
#include <stdint.h>

// using unsigned long long rather than uint64_t to ensure consistent behavior on OSX and LINUX
typedef uint32_t           shift_t;
typedef unsigned long long mask_t;
typedef uint32_t           feb_timestamp;

namespace mcdatadaq {


    class NanoSliceOrder {
        public:
            static NanoSliceOrder* getInstance(); //!< Method to obtain the class instance
            ~NanoSliceOrder();

            /// Calculate the Weight for the given nanoslice position and time
            /// Weight is calculated from the appropriate masks
            mask_t calculateOrderingWeight(const daqchannelmap::diblock_t,
					   const daqchannelmap::dcm_id_t,
					   const daqchannelmap::feb_t,
					   const daqchannelmap::pixel_t,
					   const feb_timestamp) const;
            /// Printing shifts and masks
            bool print() const;

            /// Print information about the weight
            bool print(const mask_t weight) const;

        private:

            NanoSliceOrder(); ///< Constructor

            /// Calculate shift from a given mask
            /// Used in the constructor only
            shift_t calculateShiftFromMask(const mask_t mask) const;

            /// Calculate the weight using mask and shift for a given number
            mask_t getNumberWithShiftsAndMask(const uint32_t number,
					      const shift_t  shift,
					      const mask_t   mask) const;
            
        private:

            static NanoSliceOrder* finstanceID; //!< Instance pointer for singleton class

            
            /// Internal masks which define the nanoslice order
            static const mask_t mask_diblock = 0x0FF0000000000000ULL;
            static const mask_t mask_dcm     = 0x000F000000000000ULL;
            static const mask_t mask_feb     = 0x0000FF0000000000ULL;
            static const mask_t mask_time    = 0x000000FFFFFFFF00ULL;
            static const mask_t mask_pixel   = 0x00000000000000FFULL;

            /// Corresponding shifts
            shift_t             shift_diblock;
            shift_t             shift_dcm;
            shift_t             shift_feb;
            shift_t             shift_time;
            shift_t             shift_pixel;
    };// end of NanoSliceOrder class
}// end of mcdatadaq namespace

#endif	/* _NANOSLICEORDER_H */

