#ifndef OSC_OSCCALCULATORSTERILE_H
#define OSC_OSCCALCULATORSTERILE_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// \file OscCalculatorSterile.h                                         //
//                                                                      //
// Adapt the PMNS_Sterile calculator to standard interface              //
// <aurisaam@ucmail.uc.edu>						//
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include "IOscCalculator.h"
#include "PMNS_Sterile.h"

namespace osc
{
  /// \brief Adapt the PMNS_Sterile calculator to standard interface
  ///
  /// Adapt the \ref PMNS_Sterile calculator (3+N with matter effects) to standard interface
  class OscCalculatorSterile: public IOscCalculatorAdjustable
  {
  public:
    OscCalculatorSterile();
    virtual ~OscCalculatorSterile();

    void SetNFlavors(int nflavors);

    virtual IOscCalculatorAdjustable* Copy() const;
    
    // if flavAfter == 0, give the active fraction
    virtual double P(int flavBefore, int flavAfter, double E);

    virtual void SetL     (double L     ){fDirty = true; fL      = L;}
    virtual void SetRho   (double rho   ){fDirty = true; fRho    = rho;}

    void SetAngle(int i, int j, double th);
    void SetDelta(int i, int j, double delta);
    void SetDm(int i, double dm);

    //Getters
    int    GetNFlavors()          const { return fPMNS_Sterile->GetNFlavors(); }
    double GetL()                 const { return fL; }
    double GetRho()               const { return fRho; }
    double GetDm(int i)           const { return fPMNS_Sterile->GetDm(i); }
    double GetAngle(int i, int j) const { return fPMNS_Sterile->GetAngle(i, j); }
    double GetDelta(int i, int j) const { return fPMNS_Sterile->GetDelta(i, j); }

    virtual TMD5* GetParamsHash() const override
    {
      return IOscCalculatorAdjustable::GetParamsHashDefault("PMNSSterile");
    }
  protected:
    PMNS_Sterile* fPMNS_Sterile;

    virtual void SetDmsq21(double dmsq21);
    virtual void SetDmsq32(double dmsq32);
    virtual void SetTh12  (double th12  );
    virtual void SetTh13  (double th13  );
    virtual void SetTh23  (double th23  );
    virtual void SetdCP   (double dCP   );

    int    fNFlavors;
    double fRho; 
    bool   fDirty;
    double fPrevE;
    int    fPrevAnti;
    int    fPrevFlavBefore;
  };

  const OscCalculatorSterile* DowncastToSterile(const IOscCalculator* calc);
  OscCalculatorSterile* DowncastToSterile(IOscCalculator* calc);

} // namespace

#endif
