///////////////////////////////////////////////////////////////////////////
/// \brief  Simple service to provide a RunHistory configured to the right run
/// \author Christopher Backhouse - bckhouse@caltech.edu
//////////////////////////////////////////////////////////////////////////

#ifndef RUNHISTORYSERVICE_H
#define RUNHISTORYSERVICE_H

#include "RunHistory/RunHistory.h"

//Framework includes
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "art/Framework/Principal/Run.h"

namespace nova
{
namespace dbi
{

  /// \brief Simple service to provide a RunHistory configured to the right run
  class RunHistoryService : public RunHistory
  {
  public:
    // Get a RunHistoryService instance here
    RunHistoryService(const fhicl::ParameterSet& pset,
		      art::ActivityRegistry& reg);
    virtual ~RunHistoryService();

    void reconfigure(const fhicl::ParameterSet& pset);
    
    void preBeginRun(const art::Run& run);

  protected:
    int fFakeRunNumber;
    bool fTimeQueries;
    bool fUseMasking;
    bool fAnalysisDiBlockMasking;
    bool fCustomMasking;
    std::vector<int> fCustomMask;
    std::string fCalHitLabel;
    bool fAbort;
    bool fAbortIfMissingRunInDB;

  private:
    // Hide these functions. Irrelevant since we call them automatically.
    void SetRunNumber(int run);
    void SetDetector(const std::string& det);
    void SetDetector(novadaq::cnv::DetId det);
  };

}
}

DECLARE_ART_SERVICE(nova::dbi::RunHistoryService, LEGACY)

#endif
////////////////////////////////////////////////////////////////////////
