////////////////////////////////////////////////////////////////////////
// \file    ShowerPID.h
// \brief   A Shower class that records pid information, inherited from ShowerLID 
//          
// \version 
// \author  Jianming Bian (bianjm@physics.umn.edu)
////////////////////////////////////////////////////////////////////////
#ifndef SHOWERPID_H
#define SHOWERPID_H

#include "ShowerLID/ShowerLID.h"

#include <string>
#include <map>

namespace slid 
{

  class ShowerPID : public  slid::ShowerLID 
  {
  public:
    
    inline ShowerPID();
    ~ShowerPID(){}
#ifndef __GCCXML__
    ShowerPID( const slid::ShowerLID & showerlid);


    /// Setters
    void SetValEPi0      (float in) { fPIDEPi0   = in; }
    void SetValEPi0EL    (float in) { fPIDEPi0EL = in; }

    void SetDedx0           (float in) { fDedx0  = in; }
    void SetDedx1           (float in) { fDedx1  = in; }
    void SetDedx2           (float in) { fDedx2  = in; }
    void SetDedx3           (float in) { fDedx3  = in; }
    void SetDedx4           (float in) { fDedx4  = in; }
    void SetDedx5           (float in) { fDedx5  = in; }

    void SetELLL         (float in) { fELLL    = in; }
    void SetELLT         (float in) { fELLT    = in; }
    void SetGLLL        (float in) { fGLLL    = in; }
    void SetGLLT        (float in) { fGLLT    = in; }
    void SetMuLLL       (float in) { fMuLLL   = in; }
    void SetMuLLT       (float in) { fMuLLT   = in; }
    void SetPi0LLL      (float in) { fPi0LLL  = in; }
    void SetPi0LLT      (float in) { fPi0LLT  = in; }
    void SetPLLL        (float in) { fPLLL    = in; }
    void SetPLLT        (float in) { fPLLT    = in; }
    void SetNLLL        (float in) { fNLLL    = in; }
    void SetNLLT        (float in) { fNLLT    = in; }
    void SetPiLLL       (float in) { fPiLLL   = in; }
    void SetPiLLT       (float in) { fPiLLT   = in; }



    // Getters                                                                                                                                  
    float ValueEPi0     () const { return fPIDEPi0;   }
    float ValueEPi0EL   () const { return fPIDEPi0EL; }
    float Dedx0         () const  { return fDedx0;          }
    float Dedx1         () const  { return fDedx1;          }
    float Dedx2         () const  { return fDedx2;          }
    float Dedx3         () const  { return fDedx3;          }
    float Dedx4         () const  { return fDedx4;          }
    float Dedx5         () const  { return fDedx5;          }

    float ELLL        () const { return fELLL;    }
    float ELLT        () const { return fELLT;    }
    float GLLL        () const { return fGLLL;    }
    float GLLT        () const { return fGLLT;    }
    float MuLLL       () const { return fMuLLL;   }
    float MuLLT       () const { return fMuLLT;   }
    float Pi0LLL      () const { return fPi0LLL;  }
    float Pi0LLT      () const { return fPi0LLT;  }
    float PLLL        () const { return fPLLL;    }
    float PLLT        () const { return fPLLT;    }
    float NLLL        () const { return fNLLL;    }
    float NLLT        () const { return fNLLT;    }
    float PiLLL       () const { return fPiLLL;   }
    float PiLLT       () const { return fPiLLT;   }
#endif // __GCCXML__

  protected:
    /// Algorithm name


  public:
    /// \brief training variables
    float fPIDEPi0;
    float fPIDEPi0EL;
    float fDedx0;
    float fDedx1;
    float fDedx2;
    float fDedx3;
    float fDedx4;
    float fDedx5;

    float fELLL;
    float fELLT;
    float fGLLL;
    float fGLLT;
    float fMuLLL;
    float fMuLLT;
    float fPi0LLL;
    float fPi0LLT;
    float fPLLL;
    float fPLLT;
    float fNLLL;
    float fNLLT;
    float fPiLLL;
    float fPiLLT;

  };

  inline ShowerPID::ShowerPID()
    :
    fPIDEPi0(-5.0), fPIDEPi0EL(-5.0),
    fDedx0(-5.0), fDedx1(-5.0), fDedx2(-5.0), fDedx3(-5.0), fDedx4(-5.0), fDedx5(-5.0)
    {
  }

}
#endif
