////////////////////////////////////////////////////////////////////////
// \file    SRBPFEnergy.h
// \brief   This is a class for the BreakPointFitter energy estimator
//          output in the standard record.  
// \author  $Author: mbaird42 $
// \date    
////////////////////////////////////////////////////////////////////////
#ifndef SRBPFENERGY_H
#define SRBPFENERGY_H

namespace caf
{
  /// This is a container for the bpf energy estimator output in the standard record.  
  class SRBPFEnergy
    {
    public:
      /// Default constructor. 
      SRBPFEnergy();
      
      /// Default destuctor.
      virtual ~SRBPFEnergy();

      /// variables...
      float E1;        ///< Total event energy [GeV] (using an unoscillated training sample)
      float Eres1;     ///< Estimated event energy resolution [GeV] (using an unoscillated training sample)
      float E2;        ///< Total event energy [GeV] (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 0.95)
      float Eres2;     ///< Estimated event energy resolution [GeV] (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 0.95)
      float E3;        ///< Total event energy [GeV] (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 1.0)
      float Eres3;     ///< Estimated event energy resolution [GeV] (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 1.0)
      float EventID;   ///< BPF muon PID value used to pick out the most muon like track
      float PMuon;     ///< BPF reconstructed momentum for the prong with the best muon PID value
      float DirZMuon;  ///< BPF reconstructed track direction for the prong with the best muon PID value
      float N3DProngs; ///< Number of fuzzyK 3D prongs.
      float EFuzzyK3D; ///< Summed energy from hits on fuzzyK 3D prongs (not including the prong with the best muonPID)
      float ERemain;   ///< Summed remaining energy in the slice (not on fuzzyK 3D prongs)
      float SumPE;     ///< Summed PE for all hits not on the muon track

      /// Function to set the default values.
      virtual void setDefault();
      
    };
  
} // end namespace

#endif // SRBPFENERGY_H
//////////////////////////////////////////////////////////////////////////////
