////////////////////////////////////////////////////////////////////////
/// \brief   This class adds additional information for a Cosmic object
/// \author  Gavin S. Davies - gsdavies@iastate.edu
/// \date    February 2013
////////////////////////////////////////////////////////////////////////
#ifndef SRCOSMIC_H
#define SRCOSMIC_H
#include "StandardRecord/SRTruth.h"

#include "TVector3.h"

namespace caf
{
  
  
  /// A class to add Cosmic characteristics to TruthBranch on top of the inherited 
  /// SRTruth member variables.
  class SRCosmic
    {
    public:
      ///  Default constructor for SRCosmic. 
      SRCosmic();
      
      /// Default destuctor. 
      ~SRCosmic() {  };

      short          pdg;       ///< pdg code
      float          E;         ///< True energy [GeV]
      float          visE;      ///< Sum of FLS hits that made CellHits from this neutrino [GeV]
      float          visEinslc; ///< Sum of FLS hits that made CellHits  from this neutrino in this subevent [GeV]
      float          eff;       ///< Slicer efficiency for this truth interaction.
      float          pur;       ///< Slicer purity for this truth interaction
      unsigned int   nhitslc;   ///< Number of hits recorded in this slice by this truth interaction
      unsigned int   nhittot;   ///< Total number of hits recorded for this truth interaction


      float          time;          ///< interaction time.
      TLorentzVector p;             ///< True momentum [GeV]
      TLorentzVector penter;        ///< True momentum when entering the detector [GeV]
      TVector3       vtx;           ///< Vertex postion in detector coordinates [cm]

      std::vector<SRTrueMichelE> michel;  ///< Vector of true Michel electrons 
      
      float          azimuth; ///< Azimuth angle (w.r.t y-axis)
      float          zenith;  ///< Zenith angle (w.r.t y-axis)
      TVector3       enter;   ///< Cosmic entrance point in detector coordinates. [cm]  When the primary doesn't enter the detector, coordinates are -5e9
      TVector3       exit;   ///< Cosmic exit point in detector coordinates. [cm]   When the primary doesn't enter the detector, coordinates are -5e9

      TVector3       stop;    ///< Cosmic end point in detector coordinates, regardles of whether it is inside or outside the detector.  [cm]
    };
  
} // end namespace

#endif // SRCOSMIC_H
//////////////////////////////////////////////////////////////////////////////
