////////////////////////////////////////////////////////////////////////
// \file    SRElastic.h
// \brief   An  SRElastic represents a potential interaction point.
// \version $Id: SRElastic.h,v 1.4 2012-11-13 22:06:10 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-11-13 22:06:10 $
////////////////////////////////////////////////////////////////////////
#ifndef SRELASTIC_H
#define SRELASTIC_H

#include <vector>	

#include "TVector3.h"
#include "StandardRecord/SRProng.h"
#include "StandardRecord/SRVertex.h"

namespace caf
{
  /// An  SRElastic represents a potential interaction point.
  /// It inherits from SRVertex, a simple 3-vector and time.
  /// SRElastic adds vectors of prong objects.
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  
  class SRElastic
  {
  public:
    ///  Default constructor for SRElastic. 
    SRElastic();
    
    /// Default destuctor.
    virtual ~SRElastic();

    float      time;    ///< Time      [ns]
    TVector3   vtx;     ///< Vertex position in detector coordinates. [cm]

    /// Vector of 3D prong objects
    std::vector<SRProng>   png3d;     

    /// Vector of 2D prong objects
    std::vector<SRProng> png2d;

    
    size_t npng3d;
    size_t npng2d;
    size_t ntot;
    
    void fillSizes();
    virtual void setDefault();
  };
  
} // end namespace

#endif // SRELASTIC_H
//////////////////////////////////////////////////////////////////////////////
