////////////////////////////////////////////////////////////////////////
// \file    SRShower.h
// \brief   An SRShower is a high level track object.  The SRJMShower inherits from it.
//          It knows its direction and length, but does not own its cell hits.
// \version $Id: SRJMShower.h,v 1.3 2012-11-13 22:06:10 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-13 22:06:10 $
////////////////////////////////////////////////////////////////////////
#ifndef SRJMSHOWER_H
#define SRJMSHOWER_H
#include "StandardRecord/SRShower.h"

#include "TVector3.h"

namespace caf
{
  /// An SRShower is a simple descriptor for a shower.  The SRJMShower inherits 
  /// from it to add more fields.  This class does not contain individual  
  /// cell hits, but does know it's energy and direction.  This class is meant to be used in 
  /// Common Analysis Files as part of the StandardRecord.
  class SRJMShower
    {
    public:
      ///  Default constructor for SRShower. 
      SRJMShower();

      ~SRJMShower();

      unsigned short nhit;         ///< number of hits
      unsigned short nhitx;        ///< number of hits in x-view
      unsigned short nhity;        ///< number of hits in y-view
      unsigned short nplane;       ///< number of planes spanned
      unsigned short maxplanecont; ///< maximum number of contiguous planes in prong
      unsigned short maxplanegap;  ///< maximum number of gapped planes in prong
      float          calE;         ///< energy based on summed calibrated deposited charge [GeV]
      TVector3       start;        ///< Shower start point in detector coordinates. [cm]
      TVector3       dir;          ///< Shower direction at start point [unit vector recommended]
      float          len;          ///< track length [cm]
      View_t         view;         ///< Prong view caf::kX = 0, caf::kY = 1 or caf::kXorY = 2

      float          width;   ///< Shower width [cm]
      unsigned short nplanex; ///< number of planes spanned in x view
      unsigned short nplaney; ///< number of planes spanned in y view
      float          gap;     ///< gap from shower start to ElasticArms vertex [cm]
      TVector3       stop;    ///< shower stop point

      float          shwE;    ///< reconstructed shower energy [GeV]
      float          vtxE;    ///< calorometric energy of  [GeV]
    };
  
} // end namespace

#endif // SRJMSHOWER_H
//////////////////////////////////////////////////////////////////////////////
