////////////////////////////////////////////////////////////////////////
// \file    SRRemid.h
// \brief   This class contains the reco muon PID (ReMId) output.  
// \version $Id: SRRemid.h,v 1.4 2012-11-19 22:43:57 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 22:43:57 $
////////////////////////////////////////////////////////////////////////
#ifndef SRREMID_H
#define SRREMID_H

#include "StandardRecord/SRMuId.h"

#include <cstddef>
#include <vector>


namespace caf
{
  /// This class contains the reco muon PID (ReMId) output.  
  class SRRemid
    {
    public:
      ///  Default constructor. 
      SRRemid();
      
      /// Default destuctor.
      virtual ~SRRemid();

      int   pdg;        ///< PDG code of identified track, pion or muon 
      float pid;        ///< PID value output by kNN 
      float scatllh;    ///< Log-likelihood value from scattering angle
      float dedxllh;    ///< Log-likelihood value from dE/dx
      float len;        ///< Track length of identified track 
      float measfrac;   ///< Fraction of planes used to measure dE/dx
      int   cont;       ///< True if track is contained as defined by ReMId

      std::vector<SRMuId> muid;        ///< Vector of SRMuid objects, one for each track, sorted by track length, same as kalman
      size_t              nmuid;       ///< Number of  SRMuid in vector
      size_t              bestidx;     ///< Index of best track in remid muid object (which is good 3D tracks only)

      void fillSizes();
      virtual void setDefault();
      
    };
  
} // end namespace

#endif // SRREMID_H
//////////////////////////////////////////////////////////////////////////////
