////////////////////////////////////////////////////////////////////////
// \file    SRSlice.h
// \brief   An  SRSlice contains overarching information for a slice.
// \version $Id: SRSlice.h,v 1.5 2012-12-07 21:06:22 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-12-07 21:06:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRSLICE_H
#define SRSLICE_H
#include"TVector3.h"

namespace caf
{
  /// An  SRSlice contains overarching information for a slice.
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRSlice
    {
    public:
      ///  Default constructor for SRSlice.
      SRSlice();
      
      /// Default destuctor.
      virtual ~SRSlice();

      
      unsigned int nhit;            ///< number of hits
      unsigned int ncalhit;        ///< number of hits with calibration
      unsigned int nmiphit;         ///< number of minimum ionizing hits
      unsigned int ncontplanes;     ///< number of continuous planes
      unsigned int firstplane;      ///< first plane
      unsigned int lastplane;       ///< last plane
      unsigned int firstcell;       ///< first cell
      unsigned int lastcell;        ///< last cell
      unsigned int ncellsfromedge;  ///< minumum number of cells to edge of detector
      unsigned int nnonnoise;       ///< number of non noise slices in event from which this slice came
                                    ///< or number of slices in CAF coming from the same event.  

      
      float       calE;             ///< sum of uncalibrated ADC  
      float       starttime;        ///< start time [ns]
      float       endtime;          ///< end time [ns]  
      float       meantime;         ///< mean time, weighted by charge [ns]
      TVector3    boxmin;           ///< Minumum coordinates box containing all the hits [cm]
      TVector3    boxmax;           ///< Maximum coordinates box containing all the hits [cm]
      TVector3    meanpos;          ///< Mean position of hits in slice, weighted by charge [cm]

      void setDefault();

    };  
} // end namespace

#endif // SRSLICE_H
//////////////////////////////////////////////////////////////////////////////
