////////////////////////////////////////////////////////////////////////
/// \brief   An  SRTruth represents a true neutrino.
/// \author  Dominick Rocco
/// \date    
////////////////////////////////////////////////////////////////////////
#ifndef SRTRUTH_H
#define SRTRUTH_H

#include <vector>

#include "TVector3.h"
#include "TLorentzVector.h"

#include "StandardRecord/SRTrueMichelE.h"
namespace caf
{
  
  /// The SRTruth is boilerplate  
  /// showers and tracks that make up an interaction.  This class is 
  /// meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRTruth 
    {
    public:
      ///  Default constructor for SRTruth. 
      SRTruth();

      /// Default destuctor.
      ~SRTruth() {  };
      
      short          pdg;           ///< pdg code
      float          E;             ///< True energy [GeV]
      float          visE;          ///< Sum of FLS hits that made CellHits from this neutrino [GeV]
      float          visEinslc;     ///< Sum of FLS hits that made CellHits  from this neutrino in this subevent [GeV]
      float          eff;           ///< Slicer efficiency for this truth interaction. 
      float          pur;           ///< Slicer purity for this truth interaction 
      unsigned int   nhitslc;       ///< Number of hits recorded in this slice by this truth interaction 
      unsigned int   nhittot;       ///< Total number of hits recorded for this truth interaction 
      
      
      float          time;          ///< interaction time.
      TLorentzVector p;             ///< True momentum [GeV]
      TVector3       vtx;           ///< Vertex postion in detector coordinates [cm]
  
      std::vector<SRTrueMichelE> michel;  ///< Vector of true Michel electrons 
      
    };
  
} // end namespace

#endif // SRTRUTH_H
//////////////////////////////////////////////////////////////////////////////
