////////////////////////////////////////////////////////////////////////
// Class:       AirShower
// Module Type: analyzer
// File:        AirShower_module.cc
//
// Generated at Thu Jan  9 13:51:59 2014 by Martin Frank using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////

#include <iostream>

#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/HoughResult.h"

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

#include <TH1.h>

#include <boost/format.hpp>

namespace air {
  class AirShower;
}

class air::AirShower : public art::EDAnalyzer {
public:
  explicit AirShower(fhicl::ParameterSet const & p);
  virtual ~AirShower();

  void beginJob();
  void analyze(art::Event const & e) override;

private:
  std::string hit_label_, slice_label_;
  
  std::map<std::string, TH1*> h_;
};



air::AirShower::AirShower(fhicl::ParameterSet const & p) : 
  EDAnalyzer(p),
  hit_label_(p.get<std::string>("hit_label")),
  slice_label_(p.get<std::string>("slice_label"))
{
}



air::AirShower::~AirShower()
{
}



void air::AirShower::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;
  
  h_["adc"] = tfs->make<TH1I>("adc", "ADC Distribution", 4096, -0.5, 4095.5);
  h_["nhits"] = 
    tfs->make<TH1I>("nhits", "Number of Hits per Event", 101, -0.5, 100.5);

  h_["nhits_high"] =
    tfs->make<TH1I>("nhits_high", 
		    "Number of Hits per Event Above 100 ADC", 
		    101, -0.5, 100.5);

  h_["nhits_per_slice"] =
    tfs->make<TH1I>("nhits_per_slice", 
		    "Number of Hits per Non-Noise Slice",
		    1001, -0.5, 1000.5);
}



void air::AirShower::analyze(art::Event const & e)
{
  art::Handle<std::vector<rb::CellHit> > hits;
  e.getByLabel(hit_label_, hits);

  art::Handle<std::vector<rb::Cluster> > slices;
  e.getByLabel(slice_label_, slices);

  art::Handle<std::vector<rb::HoughResult> > hough_results;
  e.getByLabel("hought", hough_results);

  // Let's play around with the raw Hough information a bit:
  for (auto const& hough_result : *hough_results)
  {
    std::cout << "MF: Hough Result" << std::endl;
    for (auto const & peak : hough_result.fPeak)
      std::cout << "MF: \tRho   = " << peak.fRho
		<< "\nMF: \tTheta = " << peak.fTheta
		<< std::endl;
  }
  

  unsigned n_hits(0), n_hits_high(0);
  for (auto const& hit : *hits)
  {
    if (hit.ADC(0) > 0)
      n_hits++;

    if (hit.ADC(0) > 100)
      n_hits_high++;

    h_["adc"]->Fill(hit.ADC(0));

    if (n_hits > 0)
      h_["nhits"]->Fill(n_hits);

    if (n_hits_high > 0)
      h_["nhits_high"]->Fill(n_hits_high);
  }

  boost::format header("%15s %15s %15s %15s");
  boost::format form("%15.0f %15.0f %15.2f %15.2f");
  std::cout << header % "Slice Number" % "N Hits" % "Total ADC" % "Z Extent"
	    << std::endl;

  unsigned n_slice = 0;
  for (auto const& slice : *slices)
  {
    if (!slice.IsNoise())
    {
      h_["nhits_per_slice"]->Fill(slice.NCell());
      std::cout << form % 
	++n_slice % slice.NCell() % slice.TotalADC() % slice.ExtentZ()
		<< std::endl;
    }
  }
  
  std::cout << std::endl;
}



DEFINE_ART_MODULE(air::AirShower)
