////////////////////////////////////////////////////////////////////////
// Class:       BreakPointAna
// Module Type: analyzer
// File:        BreakPointAna_module.cc
//
// Generated at Wed Sep 24 07:35:37 2014 by Hayes Merritt using artmod
// from cetpkgsupport v1_07_00.
////////////////////////////////////////////////////////////////////////

#include <cmath>
#include <iostream>
#include <algorithm>

// Histogram includes
#include "TH1.h"
#include "TH2.h"
#include "TTree.h"
#include "TVector.h"
#include "TProfile.h"
#include "TMath.h"

// ART includes
#include "Calibrator/Calibrator.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/FindManyP.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Persistency/Common/Ptr.h"
#include "art/Persistency/Common/PtrVector.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "RecoBase/Vertex.h"
#include "RecoBase/FitSum.h"
#include "RecoBase/RecoHit.h"
#include "Utilities/AssociationUtil.h"
#include "Utilities/func/MathUtil.h"

#include "MCCheater/BackTracker.h"
#include "SimulationBase/MCNeutrino.h"
#include "SimulationBase/MCParticle.h"
#include "SimulationBase/MCTruth.h"

#include "SummaryData/EventQuality.h"
#include "SummaryData/POTSum.h"
#include "SummaryData/SpillData.h"

namespace bpfit {
  class BreakPointAna;
}

class bpfit::BreakPointAna : public art::EDAnalyzer {
public:
  explicit BreakPointAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  BreakPointAna(BreakPointAna const &) = delete;
  BreakPointAna(BreakPointAna &&) = delete;
  BreakPointAna & operator = (BreakPointAna const &) = delete;
  BreakPointAna & operator = (BreakPointAna &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;
  void beginJob();
  void endJob();
  virtual void endSubRun(const art::SubRun &sr);
  void testHadScale(art::Ptr<rb::Cluster>, art::Ptr<rb::Track>);

private:

  // get BackTracker handle
  art::ServiceHandle<cheat::BackTracker> bt;
  // Grab calibrator 
  art::ServiceHandle< calib::Calibrator > hCal;

  // Declare member data here.
  std::string fSlicerLabel;      ///< Where to find clusters
  std::string fVertexLabel;      ///< Where to find vertices
  std::string fProngLabel;       ///< Where to find prongs
  std::string fProngInstance;    ///< Which type of prong
  std::string fTrackLabel;       ///< Where to find tracks
  std::string fFitSumLabel;      ///< Where to find FitSum objects
  std::string fMCTruthLabel;     ///< Where to find MCTruth information
  std::string fGeneratorLabel;   ///< Where to find spill information in MC
  std::string fSpillLabel;       ///< Where to find spill information in data
  bool        fDoTruth;          ///< Access truth information
  bool        fIsMC;             ///< Is MC or data
  bool        fDoHadScale;       ///< Grab HadEnergyScale data.
  double      fTotalPOT;         ///< Total POT for one subrun
 
  ////////////////////
  //// Histograms ////
  ////////////////////

  // TH1F *fMuonMomentum;
  // TH1F *fDelMuonMomentum;
  // TH1F *fMuonTheta;
  // TH1F *fMuonEnergy;
  // TH1F *fPionMomentum;
  // TH1F *fPionTheta;
  // TH1F *fProtonMomentum;
  // TH1F *fProtonTheta;
  // TH1F *fChi2MuonHit;
  // TH1F *fDegFreeMuonHit;
  // TH1F *fChi2PionHit;
  // TH1F *fDegFreePionHit;
  // TH1F *fChi2ProtonHit;
  // TH1F *fDegFreeProtonHit;
  // TH1F *fChi2MuonAngle;
  // TH1F *fDegFreeMuonAngle;
  // TH1F *fChi2PionAngle;
  // TH1F *fDegFreePionAngle;
  // TH1F *fChi2ProtonAngle;
  // TH1F *fDegFreeProtonAngle;
  // TH1F *fDelX2AngleMuPi;
  // TH1F *fDelX2HitMuPi;
  // TH1F *fDelX2AngleMuPr;
  // TH1F *fDelX2HitMuPr;
  // TH1F *fDelX2MuPi;
  // TH1F *fDelX2MuPr;
  TH2F *fQEHadNRGScale;
  TH2F *fNonQEHadNRGScale;
  TH1D *fSRPOT;
  TH1F *fTrkLngthGammaMu;
  TH1F *fTrkLngthGammaPr;
  TH1F *fTrkLngthMuMu;
  TH1F *fTrkLngthPrPr;
  TH1F *fChi2GammaMu;
  TH1F *fChi2GammaPr;
  TH1F *fChi2MuMu;
  TH1F *fChi2PrPr;
};


bpfit::BreakPointAna::BreakPointAna(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p),
  fSlicerLabel   (p.get< std::string >("SlicerLabel"   )),
  fVertexLabel   (p.get< std::string >("VertexLabel"   )),
  fProngLabel    (p.get< std::string >("ProngLabel"    )),
  fProngInstance (p.get< std::string >("ProngInstance" )),
  fTrackLabel    (p.get< std::string >("TrackLabel"    )),
  fFitSumLabel   (p.get< std::string >("FitSumLabel"   )),
  fMCTruthLabel  (p.get< std::string >("MCTruthLabel"  )),
  fGeneratorLabel(p.get< std::string >("GeneratorLabel")),
  fSpillLabel    (p.get< std::string >("SpillLabel"    )),
  fDoTruth       (p.get< bool        >("DoTruth"       )),
  fIsMC          (p.get< bool        >("IsMC"          )),
  fDoHadScale    (p.get< bool        >("DoHadScale"    ))
  // More initializers here.
{}

void bpfit::BreakPointAna::beginJob()
{
  art::ServiceHandle< art::TFileService > tfs;

  fTotalPOT = 0.0;

  // Create histograms
  // fMuonMomentum        = tfs->make<TH1F>("fMuonMomentum","Muon Momentum",100,0,5);
  // fDelMuonMomentum     = tfs->make<TH1F>("fDelMuonMomentum","(p-p_{true})/p_{true}",100,-1,1);
  // fMuonTheta           = tfs->make<TH1F>("fMuonTheta","Muon Theta",180,0,180);
  // fMuonEnergy          = tfs->make<TH1F>("fMuonEnergy","Muon Energy",50,0,5);
  // fPionMomentum        = tfs->make<TH1F>("fPionMomentum","Pion Momentum",100,0,5);
  // fPionTheta           = tfs->make<TH1F>("fPionTheta","Pion Theta",180,0,180);
  // fProtonMomentum      = tfs->make<TH1F>("fProtonMomentum","Proton Momentum",100,0,5);
  // fProtonTheta         = tfs->make<TH1F>("fProtonTheta","Proton Theta",180,0,180);
  // fChi2MuonHit         = tfs->make<TH1F>("fChi2MuonHit","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreeMuonHit      = tfs->make<TH1F>("fDegFreeMuonHit","Degrees of freedom",1000,0,1000);
  // fChi2PionHit         = tfs->make<TH1F>("fChi2PionHit","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreePionHit      = tfs->make<TH1F>("fDegFreePionHit","Degrees of freedom",1000,0,1000);
  // fChi2ProtonHit       = tfs->make<TH1F>("fChi2ProtonHit","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreeProtonHit    = tfs->make<TH1F>("fDegFreeProtonHit","Degrees of freedom",1000,0,1000);
  // fChi2MuonAngle       = tfs->make<TH1F>("fChi2MuonAngle","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreeMuonAngle    = tfs->make<TH1F>("fDegFreeMuonAngle","Degrees of freedom",1000,0,1000);
  // fChi2PionAngle       = tfs->make<TH1F>("fChi2PionAngle","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreePionAngle    = tfs->make<TH1F>("fDegFreePionAngle","Degrees of freedom",1000,0,1000);
  // fChi2ProtonAngle     = tfs->make<TH1F>("fChi2ProtonAngle","Chi Squared Per Degree of Freedom",1000,0,15);
  // fDegFreeProtonAngle  = tfs->make<TH1F>("fDegFreeProtonAngle","Degrees of freedom",1000,0,1000);
  // fDelX2HitMuPi        = tfs->make<TH1F>("fDelX2HitMuPi","Delta Chi Squared Of Hits Per Degree of Freedom",1000,-10,10);
  // fDelX2AngleMuPi      = tfs->make<TH1F>("fDelX2AngleMuPi","Delta Chi Squared Of Angle Per Degree of Freedom",1000,-10,10);
  // fDelX2HitMuPr        = tfs->make<TH1F>("fDelX2HitMuProton","Delta Chi Squared Of Hits Per Degree of Freedom",1000,-10,10);
  // fDelX2AngleMuPr      = tfs->make<TH1F>("fDelX2AngleMuProton","Delta Chi Squared Of Angle Per Degree of Freedom",1000,-500,500);
  // fDelX2MuPi           = tfs->make<TH1F>("fDelX2MuPi","Delta Chi Squared Of Per Degree of Freedom",1000,-10,10);
  // fDelX2MuPr           = tfs->make<TH1F>("fDelX2MuProton","Delta Chi Squared Of Per Degree of Freedom",1000,-10,10);
  fQEHadNRGScale       = tfs->make<TH2F>("fQEHadNRGScale","Number of Hadronic Hits Vs. (E_{#nu} - p_{#mu}",1000,0,1000,100,0,10);
  fNonQEHadNRGScale    = tfs->make<TH2F>("fNonQEHadNRGScale","Number of Hadronic Hits Vs. (E_{#nu} - p_{#mu}",1000,0,1000,100,0,10);
  fSRPOT               = tfs->make<TH1D>("fSRPOT","Total POT For One SubRun",5,0,5);
  fTrkLngthGammaMu     = tfs->make<TH1F>("fTrkLngthGammaMu","Photon Track Length ID'd As Muon",1000,0,6000);
  fTrkLngthGammaPr     = tfs->make<TH1F>("fTrkLngthGammaPr","Photon Track Length ID'd As Proton",1000,0,6000);
  fTrkLngthMuMu        = tfs->make<TH1F>("fTrkLngthMuMu","Muon Track Length ID'd As Muon",1000,0,6000);
  fTrkLngthPrPr        = tfs->make<TH1F>("fTrkLngthPrPr","Proton Track Length ID'd As Proton",1000,0,6000);
  fChi2GammaMu         = tfs->make<TH1F>("fChi2GammaMu","#chi^2_{#alpha} per DoF Truth Muon Fit As #gamma",1500,0,15);
  fChi2GammaPr         = tfs->make<TH1F>("fChi2GammaPr","#chi^2_{#alpha} per DoF Truth Proton Fit As #gamma",1500,0,15);
  fChi2MuMu            = tfs->make<TH1F>("fChi2MuMu","#chi^2_{#alpha} per DoF Truth Muon Fit As Muon",1500,0,15);
  fChi2PrPr            = tfs->make<TH1F>("fChi2PrPr","#chi^2_{#alpha} per DoF Truth Proton Fit As Proton",1500,0,15);
}

void bpfit::BreakPointAna::endSubRun(const art::SubRun& sr)
{
  art::Handle<sumdata::POTSum> pot;
  
  if(fIsMC) sr.getByLabel("generator",    pot);
  else      sr.getByLabel("ifdbspillinfo",pot);
  
  if(!pot.failedToGet()){
    if(!fIsMC) fTotalPOT += (pot->totgoodpot*1.0e12);
    else       fTotalPOT += pot->totgoodpot;
  }  
}

void bpfit::BreakPointAna::endJob()
{
  std::cout<<" \n\n\n Total POT:  "<<fTotalPOT<<std::endl;
  // Fill histogram containing POT for file/subrun
  fSRPOT->Fill(3,fTotalPOT); 
}

void bpfit::BreakPointAna::analyze(art::Event const & e)
{
  ///////////////////////////////////////////////////////////////////////
  /// For reference purposes InteractionType 1001(1002) is CCQE(NCQE).///
  ///////////////////////////////////////////////////////////////////////
  
  // get slices
  art::Handle< std::vector< rb::Cluster > > hslice;
  e.getByLabel(fSlicerLabel, hslice);

  // get associations between slices and tracks
  art::FindManyP< rb::Track > fSlTrk(hslice, e, fTrackLabel);

  // get associations between slices and vertices.
  art::FindManyP<rb::Vertex> fSlVrtx(hslice, e, fVertexLabel);

  // get associations between slices and 3D prongs
  art::FindManyP<rb::Prong>
    fSlPrng(hslice, e, art::InputTag(fProngLabel,fProngInstance));

  // Fill number of slices histogram
  //fNumSlices->Fill(int(hslice->size()));

  // Get slices
  art::PtrVector< rb::Cluster > slices;
  for(unsigned int i=0;i<hslice->size();i++){
    slices.push_back(art::Ptr<rb::Cluster>(hslice,i));
  }

  // Get truth neutrino stuff
  std::vector< std::vector< cheat::NeutrinoEffPur > > NuEffPur;
  if(fIsMC) NuEffPur = bt->SlicesToMCTruthsTable(slices);

  // Loop over number of slices 
  for(unsigned int islice=0; islice<slices.size();islice++){
    // get vertices
    std::vector < art::Ptr< rb::Vertex > > vtx;
    if(fSlVrtx.isValid()) vtx  = fSlVrtx.at(islice);
    bool isCntdVtx = false;
    art::Ptr<rb::Vertex> vtx_0;
    // by design have a max of 1 vertex per slice
    if(vtx.size()==1){
      vtx_0 = vtx.at(0);
      isCntdVtx = fabs(vtx_0->GetX())<150 
	&& fabs(vtx_0->GetY())<150 
	&& vtx_0->GetZ()>50 && vtx_0->GetZ()<900;
    }
    // get prongs
    std::vector < art::Ptr< rb::Prong > > prongs;
    if(fSlPrng.isValid()) prongs = fSlPrng.at(islice);
    // get associations between fuzzyk 3d prongs and tracks
    art::FindManyP< rb::Track > fPrngTrk(prongs, e, fTrackLabel);   

    if(!isCntdVtx) continue;
   
    /// The goal here is to be able to compare the X^2 of each
    /// particle type iteration of each track. This is why we
    /// loop over the tracks associated to each prong.
    for(unsigned int iprong=0; iprong<prongs.size();iprong++){
      // Grab the current prong to get truth information
      // art::Ptr< rb::Prong > prongi = prongs.at(iprong);
      // get tracks associated with prongs
      std::vector < art::Ptr< rb::Track > > pr_bp_trks;
      if(fPrngTrk.isValid()){pr_bp_trks = fPrngTrk.at(iprong);}
      else std::cout<<" Prong Track isn't valid!!!!"<<std::endl;
      art::FindManyP< rb::FitSum > fKin(pr_bp_trks, e, fFitSumLabel);

      // Number of tracks per prong
      //fNumTracks->Fill(pr_bp_trks.size());
      unsigned int index = -1;
      //unsigned int lngst_trk_index = -1;
      
      for(unsigned int itrk=0; itrk<pr_bp_trks.size();itrk++){
	art::Ptr< rb::Track > tracki = pr_bp_trks.at(itrk);
	std::vector<art::Ptr<rb::FitSum>> ikinsum;
	if(fKin.isValid()) ikinsum = fKin.at(itrk);

	// kinsum object is always size 1 by design so we don't need a loop
	// and instead just call the first and only element in the collection.
	art::Ptr< rb::FitSum > ifit = ikinsum.at(0);

	if(fIsMC){
	  // Sort neutrinos associated with each slice and sort them according
	  // to purity.
	  std::vector< cheat::NeutrinoEffPur > iNuEffPur = NuEffPur[islice];
	  //int purest_index = 0;
	  for(int ipur=0; ipur<int(iNuEffPur.size())-1;ipur++){
	    if(iNuEffPur[ipur].purity<iNuEffPur[ipur+1].purity){
	      cheat::NeutrinoEffPur temp = iNuEffPur[ipur];
	      iNuEffPur[ipur] = iNuEffPur[ipur+1];
	      iNuEffPur[ipur+1] = temp;
	    }
	  }
    	  // Get most pure neutrino
	  simb::MCNeutrino nu     = iNuEffPur[0].neutrinoInt->GetNeutrino();
	  simb::MCParticle nupart = nu.Nu();
	  const std::vector< const sim::Particle *> iparticles = 
	    bt->HitsToParticle((*tracki).AllCells());
	  	  
	  ///-------------------------------------------------------------------
	  /// Compute the hadronic energy scaling factor.  The idea is to plot 
	  /// number of hadronic hits (N hits in slice - N hits on muon track) 
	  /// versus hadronic energy (neutrino energy (truth) - muon momentum).
	  ///-------------------------------------------------------------------
	  
	  if(fDoHadScale){
	    ///
	    /// Begin gathering information used to calculate the hadronic energy scale factor.
	    ///
	    if(fabs(iparticles.at(0)->PdgCode())==13 && ifit->PDG()==13){
	      float NHadHits   = 0.;
	      float HadNRG     = 0.;
	      float NSliceHits = 0.;
	      float NMuonHits  = 0.;
	      
	      NSliceHits = slices[islice]->NCell();
	      NMuonHits  = tracki->NCell();
	      NHadHits   = NSliceHits - NMuonHits;
	      HadNRG     = nupart.E() - ifit->FourMom().P();
	      
	      if(nu.InteractionType()==1001) fQEHadNRGScale->Fill(NHadHits,HadNRG);
	      if(nu.InteractionType()!=1002) fNonQEHadNRGScale->Fill(NHadHits,HadNRG);
	    }
	  }	    
	} // Closing brace for IsMC
	
	// I want to compare a track's kinematics with other track's kinematics
	for(unsigned int jtrk=0; jtrk<pr_bp_trks.size();jtrk++){
	  art::Ptr< rb::Track > trackj = pr_bp_trks.at(jtrk);
	  // Don't want to compare the track to itself...
	  if(jtrk==itrk) continue;
	  // Get the kinematic information for each track
	  std::vector<art::Ptr<rb::FitSum>> jkinsum;
	  if(fKin.isValid()) jkinsum = fKin.at(jtrk);
	  // Grab kinematics of compared track
	  art::Ptr< rb::FitSum > jfit = jkinsum.at(0);
	  if(fIsMC){
	    // MC Truth info via backtracker...
	    const std::vector< const sim::Particle *> jparticles = 
	      bt->HitsToParticle((*trackj).AllCells());
	  }

	  if(itrk<1){
	    // Determine which particle assumption is best fit
	    //float long_trk    = tracki->TotalLength();
	    //float cur_trk_len = trackj->TotalLength();
	    float min_X2      = (ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4);
	      //+(ifit->Chi2(0)+ifit->Chi2(1))/(ifit->Ndof(0)+ifit->Ndof(1)-4);
	    float cur_X2      = (jfit->Chi2(2)+jfit->Chi2(3))/(jfit->Ndof(2)+jfit->Ndof(3)-4);
	      //+(jfit->Chi2(0)+jfit->Chi2(1))/(jfit->Ndof(0)+jfit->Ndof(1)-4);
	    if(cur_X2 < min_X2){
	      min_X2 = cur_X2;
	      index = jtrk;
	    }
	    else{
	      index = itrk;
	    }
	    // if(cur_trk_len>long_trk){
	    //   long_trk = cur_trk_len;
	    //   lngst_trk_index = jtrk;
	    // }
	    // else{
	    //   lngst_trk_index = itrk;
	    // }
	  }
	  
	} // end loop over jtrk
	
	if(fIsMC){
	  if(itrk==index){
	    //fChosenID->Fill(ifit->PDG());
	    const std::vector< const sim::Particle *> trupart = 
	      bt->HitsToParticle((*tracki).AllCells());
	    if(ifit->PDG()==13){
	      if(fabs(trupart.at(0)->PdgCode())==13){
		fTrkLngthMuMu->Fill(tracki->TotalLength());
		fChi2MuMu->Fill((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-1));
	      }
	      if(fabs(trupart.at(0)->PdgCode())==22){
		fTrkLngthGammaMu->Fill(tracki->TotalLength());
		fChi2GammaMu->Fill((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-1));
	      }
	    }
	    if(ifit->PDG()==2212){
	      if(fabs(trupart.at(0)->PdgCode())==2212){
		fTrkLngthPrPr->Fill(tracki->TotalLength());
		fChi2PrPr->Fill((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-1));
	      }
	      if(fabs(trupart.at(0)->PdgCode())==22){
		fTrkLngthGammaPr->Fill(tracki->TotalLength());
		fChi2GammaPr->Fill((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-1));
	      }
	    }
	  }
	}
	
      } // end loop over itrk
    } // end loop over 3d prongs
  } // end loop over slices
  
} // end analyze module
  
/*
void testHadScale(art::Ptr<rb::Cluster> hadslice, art::Ptr<rb::Track> hadtrack){
  // Fill reconstructed neutrino energy without truth information
  float NHadHits   = 0.;
  float NSliceHits = 0.;
  float NMuonHits  = 0.;
  
  NSliceHits = hadslice->NCell();
  NMuonHits  = hadtrack->NCell();
  NHadHits   = NSliceHits - NMuonHits;
  fNumHadHits->Fill(NHadHits);
  
  ///
  /// Testing hadronic energy scale I have derived...
  ///
  double had_scale = 0.0175115;
  double had_base  = 0.526434;
  double nu_energy = (NHadHits*had_scale + had_base) + ifit->FourMom().P();
  //fRecoNuE->Fill(nu_energy);
  }*/

DEFINE_ART_MODULE(bpfit::BreakPointAna)


