////////////////////////////////////////////////////////////////////////
// Class:       BreakPointProtonAna
// Module Type: analyzer
// File:        BreakPointProtonAna_module.cc
//
// Generated at Wed Sep 24 07:35:37 2014 by Hayes Merritt using artmod
// from cetpkgsupport v1_07_00.
////////////////////////////////////////////////////////////////////////

#include <cmath>
#include <iostream>
#include <algorithm>

// Histogram includes
#include "TH1.h"
#include "TH2.h"
#include "TTree.h"
#include "TVector.h"
#include "TProfile.h"
#include "TMath.h"

// ART includes
#include "Calibrator/Calibrator.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/FindManyP.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Persistency/Common/Ptr.h"
#include "art/Persistency/Common/PtrVector.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "RecoBase/Vertex.h"
#include "RecoBase/FitSum.h"
#include "RecoBase/RecoHit.h"
#include "Utilities/AssociationUtil.h"
#include "Utilities/func/MathUtil.h"

#include "MCCheater/BackTracker.h"
#include "SimulationBase/MCNeutrino.h"
#include "SimulationBase/MCParticle.h"
#include "SimulationBase/MCTruth.h"

#include "SummaryData/EventQuality.h"
#include "SummaryData/POTSum.h"
#include "SummaryData/SpillData.h"

namespace bpfit {
  class BreakPointProtonAna;
}

class bpfit::BreakPointProtonAna : public art::EDAnalyzer {
public:
  explicit BreakPointProtonAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  BreakPointProtonAna(BreakPointProtonAna const &) = delete;
  BreakPointProtonAna(BreakPointProtonAna &&) = delete;
  BreakPointProtonAna & operator = (BreakPointProtonAna const &) = delete;
  BreakPointProtonAna & operator = (BreakPointProtonAna &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;
  void beginJob();
  void endJob();
  virtual void endSubRun(const art::SubRun &sr);

private:

  // get BackTracker handle
  art::ServiceHandle<cheat::BackTracker> bt;
  // Grab calibrator 
  art::ServiceHandle< calib::Calibrator > hCal;

  // Declare member data here.
  std::string fSlicerLabel;      ///< Where to find clusters
  std::string fVertexLabel;      ///< Where to find vertices
  std::string fProngLabel;       ///< Where to find prongs
  std::string fProngInstance;    ///< Which type of prong
  std::string fTrackLabel;       ///< Where to find tracks
  std::string fFitSumLabel;      ///< Where to find FitSum objects
  std::string fMCTruthLabel;     ///< Where to find MCTruth information
  std::string fGeneratorLabel;   ///< Where to find spill information in MC
  std::string fSpillLabel;       ///< Where to find spill information in data
  bool        fIsMC;             ///< Is MC or data
  double      fTotalPOT;         ///< Total POT for one subrun

  TTree *fOutTree;
 
  ////////////////////
  //// Histograms ////
  ////////////////////
  TH1D *fSRPOT;
  TH1F *fTrueNuE;
  TH1F *fRecoNuE;
  TH1F *fRecoNuP;
  TH1F *fPrDotXPr;
  TH1F *fPrDotXPrSlcE;
  TH1F *fPrjxnDiff;
  TH1F *fPrMomFracBias;
  TH1F *fMuonPurity;
  TH1F *fProtonPurity;
  TH1F *fProtonMomentum;
  TH1F *fNuMomentum;
  TH1F *fMuonMomentum;
  TH1F *fRecoNuETrue;
  TH1F *fRecoNuPTrue;
  TH1F *fPrDotXPrTrue;
  TH1F *fPrMomFracBiasTrue;
  TH1F *fProtonMomentumTrue;
  TH1F *fMuonMomentumTrue;
  TH1F *fNuMomentumTrue;
  TH1F *fRecoNuETrueQE;
  TH1F *fRecoNuPTrueQE;
  TH1F *fPrDotXPrTrueQE;
  TH1F *fPrDotXPrTrueQESlcE;
  TH1F *fPrMomFracBiasTrueQE;
  TH1F *fProtonMomentumTrueQE;
  TH1F *fMuonMomentumTrueQE;
  TH1F *fNuMomentumTrueQE;
  TH1F *fShowerWidth;
  TH2D *fPrjxnVsRecoNuE;
  TH1F *fPmuFracDiff;
  TH1F *fPprFracDiff;
  TH1F *fPmuFracDiffTrue;
  TH1F *fPprFracDiffTrue;
  TH1F *fPmuFracDiffTrueQE;
  TH1F *fPprFracDiffTrueQE;
  TH1F *fPhotonAsMuonMotherType;
  TH1F *fPhotonAsProtonMotherType;
  TH1F *fMuonTrkLngth;
  TH1F *fProtonTrkLngth;
  TH1F *fMuonTrkLngthGood;
  TH1F *fProtonTrkLngthGood;
  TH1F *fEvtQEPurity1;
  TH1F *fEvtQEPurity2;
  TH1F *fEvtQEPurity3;
  TH1F *fEvtQEPurity4;
  TH1F *fEvtQEPurity5;
  TH1F *fEvtQEPurity6;
  TH1F *fEvtQEPurity7;
  TH1F *fEvtQEPurity8;
  TH1F *fEvtQEPurity9;
  TH1F *fEvtQEPurity10;
  TH1F *fEvtQEPurity1slcE;
  TH1F *fEvtQEPurity2slcE;
  TH1F *fEvtQEPurity3slcE;
  TH1F *fEvtQEPurity4slcE;
  TH1F *fEvtQEPurity5slcE;
  TH1F *fEvtQEPurity6slcE;
  TH1F *fEvtQEPurity7slcE;
  TH1F *fEvtQEPurity8slcE;
  TH1F *fEvtQEPurity9slcE;
  TH1F *fEvtQEPurity10slcE;
  TH1F *fMuonPurityGood;
  TH1F *fProtonPurityGood;
  TH1F *fMuonPurityGoodSlcE;
  TH1F *fProtonPurityGoodSlcE;
  TH1F *fPrDelMomFracDiff;
  TH1F *fMuDelMomFracDiff;
  TH1F *fPrCosTheta;
  TH1F *fMuCosTheta;
  TH2F *fMuVPrCosTheta;
  unsigned int Run    = -1;
  unsigned int SubRun = -1;
  unsigned int Event  = -1;
  unsigned int Slice  = -1;
};


bpfit::BreakPointProtonAna::BreakPointProtonAna(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p),
  fSlicerLabel   (p.get< std::string >("SlicerLabel"   )),
  fVertexLabel   (p.get< std::string >("VertexLabel"   )),
  fProngLabel    (p.get< std::string >("ProngLabel"    )),
  fProngInstance (p.get< std::string >("ProngInstance" )),
  fTrackLabel    (p.get< std::string >("TrackLabel"    )),
  fFitSumLabel   (p.get< std::string >("FitSumLabel"   )),
  fMCTruthLabel  (p.get< std::string >("MCTruthLabel"  )),
  fGeneratorLabel(p.get< std::string >("GeneratorLabel")),
  fSpillLabel    (p.get< std::string >("SpillLabel"    )),
  fIsMC          (p.get< bool        >("IsMC"          ))
  // More initializers here.
{}

void bpfit::BreakPointProtonAna::beginJob()
{
  art::ServiceHandle< art::TFileService > tfs;
  
  fTotalPOT = 0.0;

  fSRPOT                     = tfs->make<TH1D>("fSRPOT","Total POT For One SubRun",5,0,5);
  fTrueNuE                   = tfs->make<TH1F>("fTrueNuE","True Neutrino Energy",120,0,6);
  fRecoNuE                   = tfs->make<TH1F>("fRecoNuE","Reconstructed Neutrino Energy",120,0,6);
  fRecoNuP                   = tfs->make<TH1F>("fRecoNuP","Reconstructed Neutrino Momentum",120,0,6);
  fPrDotXPr                  = tfs->make<TH1F>("fPrDotXPr","Expected Proton Momentum Dot Product With Reco Proton Momentum",100,-1,1);
  fPrDotXPrSlcE              = tfs->make<TH1F>("fPrDotXPrSlcE","Expected Proton Momentum Dot Product With Reco Proton Momentum Using Slice Energy",100,-1,1);
  fPrMomFracBias             = tfs->make<TH1F>("fMomDiff","(Expected Proton Momentum - Reco Proton Momentum)/Expected",100,-1,1);
  fMuonMomentum              = tfs->make<TH1F>("fMuonMomentum","Reconstructed Muon Momentum",120,0,6);  
  fProtonMomentum            = tfs->make<TH1F>("fProtonMomentum","Reconstructed Proton Momentum",120,0,6);
  fMuonPurity                = tfs->make<TH1F>("fMuonPurity","Selected Muon Purity By Truth ID",6000,-3000,3000);
  fProtonPurity              = tfs->make<TH1F>("fProtonPurity","Selected Proton Purity By Truth ID",6000,-3000,3000);
  fNuMomentum                = tfs->make<TH1F>("fNuMomentum","True Neutrino Momentum",120,0,6);
  fRecoNuETrue               = tfs->make<TH1F>("fRecoNuETrue","Reconstructed Neutrino Energy",120,0,6);
  fRecoNuPTrue               = tfs->make<TH1F>("fRecoNuPTrue","Reconstructed Neutrino Momentum",120,0,6);
  fPrDotXPrTrue              = tfs->make<TH1F>("fPrDotXPrTrue","Expected Proton Momentum Dot Product With Reco Proton Momentum",100,-1,1);
  fPrMomFracBiasTrue         = tfs->make<TH1F>("fMomDiffTrue","(Expected Proton Momentum - Reco Proton Momentum)/Expected",100,-1,1);
  fMuonMomentumTrue          = tfs->make<TH1F>("fMuonMomentumTrue","Reconstructed Muon Momentum",120,0,6);  
  fProtonMomentumTrue        = tfs->make<TH1F>("fProtonMomentumTrue","Reconstructed Proton Momentum",120,0,6);
  fNuMomentumTrue            = tfs->make<TH1F>("fNuMomentumTrue","True Neutrino Momentum",120,0,6);
  fRecoNuETrueQE             = tfs->make<TH1F>("fRecoNuETrueQE","Reconstructed Neutrino Energy",120,0,6);
  fRecoNuPTrueQE             = tfs->make<TH1F>("fRecoNuPTrueQE","Reconstructed Neutrino Momentum",120,0,6);
  fPrDotXPrTrueQE            = tfs->make<TH1F>("fPrDotXPrTrueQE","Expected Proton Momentum Dot Product With Reco Proton Momentum",100,-1,1);
  fPrDotXPrTrueQESlcE        = tfs->make<TH1F>("fPrDotXPrTrueQESlcE","Expected Proton Momentum Dot Product With Reco Proton Momentum Using Slice Energy",100,-1,1);
  fPrMomFracBiasTrueQE       = tfs->make<TH1F>("fMomDiffTrueQE","(Expected Proton Momentum - Reco Proton Momentum)/Expected",100,-1,1);
  fMuonMomentumTrueQE        = tfs->make<TH1F>("fMuonMomentumTrueQE","Reconstructed Muon Momentum",120,0,6);  
  fProtonMomentumTrueQE      = tfs->make<TH1F>("fProtonMomentumTrueQE","Reconstructed Proton Momentum",120,0,6);
  fNuMomentumTrueQE          = tfs->make<TH1F>("fNuMomentumTrueQE","True Neutrino Momentum",120,0,6);
  fPrjxnVsRecoNuE            = tfs->make<TH2D>("fPrjxnVsRecoNuE","E^{reco}_{#nu} Vs Projection",120,0,6,100,-1,1);
  fPmuFracDiff               = tfs->make<TH1F>("fPmuFracDiff","Fractional Momentum Difference",400,-2,2);
  fPprFracDiff               = tfs->make<TH1F>("fPprFracDiff","Fractional Momentum Difference",400,-2,2);  
  fPmuFracDiffTrue           = tfs->make<TH1F>("fPmuFracDiffTrue","Fractional Momentum Difference",400,-2,2);
  fPprFracDiffTrue           = tfs->make<TH1F>("fPprFracDiffTrue","Fractional Momentum Difference",400,-2,2);
  fPmuFracDiffTrueQE         = tfs->make<TH1F>("fPmuFracDiffTrueQE","Fractional Momentum Difference",400,-2,2);
  fPprFracDiffTrueQE         = tfs->make<TH1F>("fPprFracDiffTrueQE","Fractional Momentum Difference",400,-2,2);
  fPhotonAsMuonMotherType    = tfs->make<TH1F>("fPhotonAsMuonMotherType","Photon Mother Type",6000,-3000,3000);
  fPhotonAsProtonMotherType  = tfs->make<TH1F>("fPhotonAsProtonMotherType","Photon Mother Type",6000,-3000,3000);
  fMuonTrkLngth              = tfs->make<TH1F>("fMuonTrkLngth","Muon Track Length",1000,0,6000);
  fProtonTrkLngth            = tfs->make<TH1F>("fProtonTrkLngth","Proton Track Length",1000,0,6000);
  fMuonTrkLngthGood          = tfs->make<TH1F>("fMuonTrkLngthGood","Muon Track Length",1000,0,6000);
  fProtonTrkLngthGood        = tfs->make<TH1F>("fProtonTrkLngthGood","Proton Track Length",1000,0,6000);
  fEvtQEPurity1              = tfs->make<TH1F>("fEvtQEPurity1","Event Interaction Type",10000,0,10000);
  fEvtQEPurity2              = tfs->make<TH1F>("fEvtQEPurity2","Event Interaction Type",10000,0,10000);
  fEvtQEPurity3              = tfs->make<TH1F>("fEvtQEPurity3","Event Interaction Type",10000,0,10000);
  fEvtQEPurity4              = tfs->make<TH1F>("fEvtQEPurity4","Event Interaction Type",10000,0,10000);
  fEvtQEPurity5              = tfs->make<TH1F>("fEvtQEPurity5","Event Interaction Type",10000,0,10000);
  fEvtQEPurity6              = tfs->make<TH1F>("fEvtQEPurity6","Event Interaction Type",10000,0,10000);
  fEvtQEPurity7              = tfs->make<TH1F>("fEvtQEPurity7","Event Interaction Type",10000,0,10000);
  fEvtQEPurity8              = tfs->make<TH1F>("fEvtQEPurity8","Event Interaction Type",10000,0,10000);
  fEvtQEPurity9              = tfs->make<TH1F>("fEvtQEPurity9","Event Interaction Type",10000,0,10000);
  fEvtQEPurity10             = tfs->make<TH1F>("fEvtQEPurity10","Event Interaction Type",10000,0,10000);
  fEvtQEPurity1slcE          = tfs->make<TH1F>("fEvtQEPurity1slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity2slcE          = tfs->make<TH1F>("fEvtQEPurity2slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity3slcE          = tfs->make<TH1F>("fEvtQEPurity3slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity4slcE          = tfs->make<TH1F>("fEvtQEPurity4slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity5slcE          = tfs->make<TH1F>("fEvtQEPurity5slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity6slcE          = tfs->make<TH1F>("fEvtQEPurity6slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity7slcE          = tfs->make<TH1F>("fEvtQEPurity7slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity8slcE          = tfs->make<TH1F>("fEvtQEPurity8slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity9slcE          = tfs->make<TH1F>("fEvtQEPurity9slcE","Event Interaction Type",10000,0,10000);
  fEvtQEPurity10slcE         = tfs->make<TH1F>("fEvtQEPurity10slcE","Event Interaction Type",10000,0,10000);
  fMuonPurityGood            = tfs->make<TH1F>("fMuonPurityGood","Selected Muon Purity By Truth ID",6000,-3000,3000);
  fProtonPurityGood          = tfs->make<TH1F>("fProtonPurityGood","Selected Proton Purity By Truth ID",6000,-3000,3000);
  fMuonPurityGoodSlcE        = tfs->make<TH1F>("fMuonPurityGoodSlcE","Selected Muon Purity By Truth ID",6000,-3000,3000);
  fProtonPurityGoodSlcE      = tfs->make<TH1F>("fProtonPurityGoodSlcE","Selected Proton Purity By Truth ID",6000,-3000,3000);
  fPrDelMomFracDiff          = tfs->make<TH1F>("fPrDelDiff","(True Proton Momentum - Reco Proton Momentum)/True",100,-1,1);
  //  fMuDelMomFracDiff          = tfs->make<TH1F>("fMomDiffTrue","(Expected Proton Momentum - Reco Proton Momentum)/Expected",100,-1,1);
  fPrCosTheta                = tfs->make<TH1F>("fPrCosTheta","Proton Direction Cos(#theta)",200,-1,1);
  fMuVPrCosTheta             = tfs->make<TH2F>("fMuVPrCosTheta","Muon Cos(#theta) Vs. Proton Cos(#theta)",200,-1,1,200,-1,1);
  fPrjxnDiff                 = tfs->make<TH1F>("fPrjxnDiff","Difference In Proton Projection (slice energy vs true neutrino energy)",200,-1,1);
}

void bpfit::BreakPointProtonAna::endSubRun(const art::SubRun& sr)
{
  art::Handle<sumdata::POTSum> pot;
  
  if(fIsMC) sr.getByLabel("generator",    pot);
  else      sr.getByLabel("ifdbspillinfo",pot);
  
  if(!pot.failedToGet()){
    if(!fIsMC) fTotalPOT += (pot->totgoodpot*1.0e12);
    else       fTotalPOT += pot->totgoodpot;
  }  
}

void bpfit::BreakPointProtonAna::endJob()
{
  std::cout<<" \n\n\n Total POT:  "<<fTotalPOT<<std::endl;
  // Fill histogram containing POT for file/subrun
  fSRPOT->Fill(3,fTotalPOT); 
}

void bpfit::BreakPointProtonAna::analyze(art::Event const & e)
{
  ///////////////////////////////////////////////////////////////////////
  /// For reference purposes InteractionType 1001(1002) is CCQE(NCQE).///
  ///////////////////////////////////////////////////////////////////////
  
  // get slices
  art::Handle< std::vector< rb::Cluster > > hslice;
  e.getByLabel(fSlicerLabel, hslice);

  // get associations between slices and tracks
  art::FindManyP< rb::Track > fSlTrk(hslice, e, fTrackLabel);

  // get associations between slices and vertices.
  art::FindManyP<rb::Vertex> fSlVrtx(hslice, e, fVertexLabel);

  // get associations between slices and 3D prongs
  art::FindManyP<rb::Prong>
    fSlPrng(hslice, e, art::InputTag(fProngLabel,fProngInstance));

  // Get slices
  art::PtrVector< rb::Cluster > slices;
  for(unsigned int i=0;i<hslice->size();i++){
    slices.push_back(art::Ptr<rb::Cluster>(hslice,i));
  }

  // Get truth neutrino stuff
  std::vector< std::vector< cheat::NeutrinoEffPur > > NuEffPur;
  if(fIsMC) NuEffPur = bt->SlicesToMCTruthsTable(slices);

  // Loop over number of slices 
  for(unsigned int islice=0; islice<slices.size();islice++){
    // get vertices
    std::vector < art::Ptr< rb::Vertex > > vtx;
    if(fSlVrtx.isValid()) vtx  = fSlVrtx.at(islice);
    bool isCntdVtx = false;
    art::Ptr<rb::Vertex> vtx_0;
    // by design have a max of 1 vertex per slice
    if(vtx.size()==1){
      vtx_0 = vtx.at(0);
      isCntdVtx = fabs(vtx_0->GetX())<150 
	&& fabs(vtx_0->GetY())<150 
	&& vtx_0->GetZ()>50;
    }
    
    // get prongs
    std::vector < art::Ptr< rb::Prong > > prongs;
    if(fSlPrng.isValid()) prongs = fSlPrng.at(islice);
    // get associations between fuzzyk 3d prongs and tracks
    art::FindManyP< rb::Track > fPrngTrk(prongs, e, fTrackLabel);   

    TLorentzVector Muon, Proton, TruMuon;
    bool haveMuon    = false;
    bool haveProton  = false;
    bool isTruMuon   = false;
    bool isTruProton = false;
    int proton_pdg = 0;
    int muon_pdg   = 0;
    float proton_track_length = -10.;
    float muon_track_length   = -10.;
    float true_proton_mom     = -10.;
    
    // Make sure the vertex is contained
    if(!isCntdVtx) continue;

    // Calculate calibrated slice energy
    art::PtrVector<rb::CellHit> slchits = slices.at(islice)->AllCells();
    double slice_GeV = 1.0E-3; 
    // Make reco hit for every cell hit in order to get calibrated cell energy.
    for(unsigned int j=0;j<slchits.size();j++){
      const rb::CellHit *cellhit = slices.at(islice)->Cell(j).get();
      double Wtemp = slices.at(islice)->W(cellhit);
      rb::RecoHit rhit(hCal->MakeRecoHit(*cellhit,Wtemp));
      if(rhit.IsCalibrated()) slice_GeV += rhit.GeV();
    }

    for(unsigned int iprong=0; iprong<prongs.size();iprong++){
      // Grab the current prong to get truth information
      // art::Ptr< rb::Prong > prongi = prongs.at(iprong);
      // get tracks associated with prongs
      std::vector < art::Ptr< rb::Track > > pr_bp_trks;
      if(fPrngTrk.isValid()){pr_bp_trks = fPrngTrk.at(iprong);}
      else std::cout<<" Prong Track isn't valid!!!!"<<std::endl;
      art::FindManyP< rb::FitSum > fKin(pr_bp_trks, e, fFitSumLabel);
      unsigned int index = -1;
      unsigned int lngst_trk_index = -1;

      for(unsigned int itrk=0; itrk<pr_bp_trks.size();itrk++){
	art::Ptr< rb::Track > tracki = pr_bp_trks.at(itrk);
	// Get the kinematic information for each track
	std::vector<art::Ptr<rb::FitSum>> ikinsum;
	if(fKin.isValid()) ikinsum = fKin.at(itrk);

	art::Ptr< rb::FitSum > ifit = ikinsum.at(0);

	for(unsigned int jtrk=itrk+1; jtrk<pr_bp_trks.size();jtrk++){
	  art::Ptr< rb::Track > trackj = pr_bp_trks.at(jtrk);
	  // Get the kinematic information for each track
	  std::vector<art::Ptr<rb::FitSum>> jkinsum;
	  if(fKin.isValid()) jkinsum = fKin.at(jtrk);
	  // Grab kinematics of compared track
	  art::Ptr< rb::FitSum > jfit = jkinsum.at(0);

	  if(itrk<1){
	    // Determine which particle assumption is best fit
	    float long_trk    = tracki->TotalLength();
	    float cur_trk_len = trackj->TotalLength();
	    float min_X2      = (ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4);
	      //+(ifit->Chi2(0)+ifit->Chi2(1))/(ifit->Ndof(0)+ifit->Ndof(1)-4);
	    float cur_X2      = (jfit->Chi2(2)+jfit->Chi2(3))/(jfit->Ndof(2)+jfit->Ndof(3)-4);
	      //+(jfit->Chi2(0)+jfit->Chi2(1))/(jfit->Ndof(0)+jfit->Ndof(1)-4);
	    if(cur_X2 < min_X2){
	      min_X2 = cur_X2;
	      index = jtrk;
	    }
	    else index = itrk;	
	    // Store index of longest track.
	    if(cur_trk_len>long_trk){
	      long_trk = cur_trk_len;
	      lngst_trk_index = jtrk;
	    }
	    else lngst_trk_index = itrk;
	  }
	} // end loop over jtrk
	
	if(fIsMC){
	  if(itrk==index){
	    const std::vector< const sim::Particle *> trupart = 
	      bt->HitsToParticle((*tracki).AllCells());
	    if(ifit->PDG()==13){
	      fMuonTrkLngth->Fill(tracki->TotalLength());
	      if(fabs(tracki->Stop().X())<150 && fabs(tracki->Stop().Y())<150 
		 && tracki->Stop().Z()<1200 
		 && ((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4)) < 0.1
		 && itrk==lngst_trk_index){		
		Muon.SetPxPyPzE(ifit->FourMom().Px(), ifit->FourMom().Py(), ifit->FourMom().Pz(), ifit->FourMom().E());
		fMuonPurity->Fill(trupart.at(0)->PdgCode());
		fPmuFracDiff->Fill((Muon.P()-trupart.at(0)->P())/trupart.at(0)->P());
		haveMuon  = true;
		muon_pdg  = trupart.at(0)->PdgCode();
		muon_track_length = tracki->TotalLength();
		
		if(fabs(trupart.at(0)->PdgCode())==13){
		  fPmuFracDiffTrue->Fill((Muon.P()-trupart.at(0)->P())/trupart.at(0)->P());
		  isTruMuon = true;
		}
		if(fabs(trupart.at(0)->PdgCode())==22){
		  if(trupart.at(0)->Mother()!=0){
		    const sim::Particle *motherpart = bt->TrackIDToParticle(trupart.at(0)->Mother());
		    fPhotonAsMuonMotherType->Fill(motherpart->PdgCode());
		  }
		}

		break;  // now that we have a muon we want to skip to the next prong and associated BP tracks.
	      }
	    } // end if
	    else if(ifit->PDG()==2212){
	      fProtonTrkLngth->Fill(tracki->TotalLength());
	      if(tracki->TotalLength() < 125.0 
		   && ((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4)) < 0.02){
		fProtonPurity->Fill(trupart.at(0)->PdgCode());
		fPprFracDiff->Fill((Muon.P()-trupart.at(0)->P())/trupart.at(0)->P());
		Proton.SetPxPyPzE(ifit->FourMom().Px(), ifit->FourMom().Py(), ifit->FourMom().Pz(), ifit->FourMom().E());
		haveProton = true;
		proton_pdg = trupart.at(0)->PdgCode();
		proton_track_length = tracki->TotalLength();
		true_proton_mom     = trupart.at(0)->P();
		if(fabs(trupart.at(0)->PdgCode())==2212){
		  fPprFracDiffTrue->Fill((Proton.P()-trupart.at(0)->P())/trupart.at(0)->P());
		  isTruProton = true;
		}
		if(fabs(trupart.at(0)->PdgCode())==22){
		  if(trupart.at(0)->Mother()!=0){
		    const sim::Particle *motherpart = bt->TrackIDToParticle(trupart.at(0)->Mother());
		    fPhotonAsProtonMotherType->Fill(motherpart->PdgCode());
		  }
		}

		break; // now that we have a proton we want to skip to next prong and associated BP tracks.
	      }
	    } 
	  } // end if best fit track index matches current track index statement
	} //end IsMC
	else{ // Data 
	  if(itrk==index){
	    if(ifit->PDG()==13){
	      fMuonTrkLngth->Fill(tracki->TotalLength());
	      if(fabs(tracki->Stop().X())<150 && fabs(tracki->Stop().Y())<150 
		 && tracki->Stop().Z()<1200 
		 && ((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4)) < 0.1
		 && itrk==lngst_trk_index){
		Muon.SetPxPyPzE(ifit->FourMom().Px(), ifit->FourMom().Py(), ifit->FourMom().Pz(), ifit->FourMom().E());
		haveMuon = true;
		muon_track_length = tracki->TotalLength();
		break;  // now that we have a muon we want to skip to the next prong and associated BP tracks.
	      }
	    }
	    if(ifit->PDG()==2212){
	      fProtonTrkLngth->Fill(tracki->TotalLength());
	      if(tracki->TotalLength() < 125.0 
		 && ((ifit->Chi2(2)+ifit->Chi2(3))/(ifit->Ndof(2)+ifit->Ndof(3)-4)) < 0.02){
		haveProton = true;
		Proton.SetPxPyPzE(ifit->FourMom().Px(), ifit->FourMom().Py(), ifit->FourMom().Pz(), ifit->FourMom().E());
		proton_track_length = tracki->TotalLength();
		
		break;
	      }
	    }
	  }
	}
	
      } // end loop over itrk
    } // end loop over 3d prongs

    if(prongs.size()==2 && haveMuon && haveProton && slices[islice]->NCell()>=20){
      if(fIsMC){
	// Sort neutrinos associated with each slice and sort them according
	// to purity.
	std::vector< cheat::NeutrinoEffPur > iNuEffPur = NuEffPur[islice];
	//int purest_index = 0;
	for(int ipur=0; ipur<int(iNuEffPur.size())-1;ipur++){
	  if(iNuEffPur[ipur].purity<iNuEffPur[ipur+1].purity){
	    cheat::NeutrinoEffPur temp = iNuEffPur[ipur];
	    iNuEffPur[ipur]            = iNuEffPur[ipur+1];
	    iNuEffPur[ipur+1]          = temp;
	  }
	}	
	// Get most pure neutrino
	simb::MCNeutrino nu     = iNuEffPur[0].neutrinoInt->GetNeutrino();
	simb::MCParticle nupart = nu.Nu();

	TVector3 neu, mu, expected_proton, proton, reconu;
	// Instead of using truth neutrino energy use QE reconstructed neutrino energy.
	// This allows the same values to be used in data and MC.
	// Also, initially we set the neutrino to have unit magnitude as we need the angle
	// between the neutrino and the reco muon to calculate the neutrino energy.
	neu.SetXYZ(0,0,1);
	mu.SetXYZ(Muon.Px(), Muon.Py(), Muon.Pz());
	double mp      = 0.938;
	double mn      = 0.9396;
	double cosmunu = mu.Dot(neu)*1./(mu.Mag()*neu.Mag());
	double reconuE = 0.5*(mp*mp - mn*mn + 2.*mn*Muon.E() - 0.1057*0.1057)*1./(mn - Muon.E() + mu.Mag()*cosmunu);
	// Now set the neutrino 3 momentum magnitude equal to the reconstructed neutrino energy.
	neu.SetXYZ(0,0,reconuE);      
	// Determine expected proton 3 momentum.
	expected_proton = neu - mu; 
	//slcE_proton = slcE_neu - mu/;
	proton.SetXYZ(Proton.Px(), Proton.Py(), Proton.Pz());
	reconu = proton + mu;

	float projection      = proton.Dot(expected_proton)*1./proton.Mag()*1./expected_proton.Mag();
	// float slcEprojection  = proton.Dot(slcE_proton)*1./proton.Mag()*1./slcE_proton.Mag();

	// fPrjxnDiff->Fill(projection - slcEprojection);
	fPrDotXPr->Fill(projection);
	// fPrDotXPrSlcE->Fill(slcEprojection);
	fPrMomFracBias->Fill((expected_proton.Mag()-proton.Mag())/expected_proton.Mag());
	if(projection > 0.99){
	  fProtonPurityGoodSlcE->Fill(proton_pdg);
	  fMuonPurityGoodSlcE->Fill(muon_pdg);
	  fRecoNuE->Fill(reconuE);
	  fProtonTrkLngthGood->Fill(proton_track_length);
	  fMuonTrkLngthGood->Fill(muon_track_length);
	  fMuonMomentum->Fill(mu.Mag());
	  fProtonMomentum->Fill(proton.Mag());
	  fPrDelMomFracDiff->Fill((true_proton_mom - proton.Mag())/true_proton_mom);
	  fPrCosTheta->Fill(proton.Z()/proton.Mag());
	  fMuVPrCosTheta->Fill(mu.Z()/mu.Mag(), proton.Z()/proton.Mag());
	  // Print to screen (out file) to create a list of events with protons
	  // std::cout<<"bubbles : "<< e.run()
	  //<<" "<< e.subRun()
	  //	   <<" "<< e.id().event()
	  //	   <<" "<< islice      
	  //	   <<std::endl;
	}
	fNuMomentum->Fill(reconu.Mag());
	
	//===================================================================
	// The projection of the two proton vectors could be used to help ID 
	// CCQE events.
	//===================================================================
	if(projection>0.90)fEvtQEPurity1->Fill(nu.InteractionType());
	if(projection>0.91)fEvtQEPurity2->Fill(nu.InteractionType());
	if(projection>0.92)fEvtQEPurity3->Fill(nu.InteractionType());
	if(projection>0.93)fEvtQEPurity4->Fill(nu.InteractionType());
	if(projection>0.94)fEvtQEPurity5->Fill(nu.InteractionType());
	if(projection>0.95)fEvtQEPurity6->Fill(nu.InteractionType());
	if(projection>0.96)fEvtQEPurity7->Fill(nu.InteractionType());
	if(projection>0.97)fEvtQEPurity8->Fill(nu.InteractionType());
	if(projection>0.98)fEvtQEPurity9->Fill(nu.InteractionType());
	if(projection>0.99)fEvtQEPurity10->Fill(nu.InteractionType());
	/*if(slcEprojection>0.90)fEvtQEPurity1slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.91)fEvtQEPurity2slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.92)fEvtQEPurity3slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.93)fEvtQEPurity4slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.94)fEvtQEPurity5slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.95)fEvtQEPurity6slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.96)fEvtQEPurity7slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.97)fEvtQEPurity8slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.98)fEvtQEPurity9slcE->Fill(nu.InteractionType());
	if(slcEprojection>0.99)fEvtQEPurity10slcE->Fill(nu.InteractionType());*/
	//===================================================================
	
	if(nu.InteractionType()==1001){	  
	  fPrjxnVsRecoNuE->Fill(reconuE,projection);
	  fTrueNuE->Fill(nupart.E());
	  fRecoNuETrueQE->Fill(reconuE);
	  fPrDotXPrTrueQE->Fill(projection);
	  fPrMomFracBiasTrueQE->Fill((expected_proton.Mag()-proton.Mag())/expected_proton.Mag());
	  fNuMomentumTrueQE->Fill(reconu.Mag());
	  fMuonMomentumTrueQE->Fill(mu.Mag());
	  fProtonMomentumTrueQE->Fill(proton.Mag());
	  
	  // Fill particle purity plots allowing for purity measurements when
	  // selecting only true CCQE events. (fPurityGood histograms)
	  fMuonPurityGood->Fill(muon_pdg);
	  fProtonPurityGood->Fill(proton_pdg);
	  if(isTruMuon && isTruProton){
	    fPrDotXPrTrue->Fill(projection);
	    fPrMomFracBiasTrue->Fill((expected_proton.Mag()-proton.Mag())/expected_proton.Mag());
	    fRecoNuETrue->Fill(reconuE);
	    fMuonMomentumTrue->Fill(mu.Mag());
	    fProtonMomentumTrue->Fill(proton.Mag());
	    fNuMomentumTrue->Fill(reconu.Mag());
	  }
	}

      } // end IsMC
      else{ // Data
	// Instead of using truth neutrino energy use QE reconstructed neutrino energy.
	// This allows the same values to be used in data and MC.
	// Also, initially we set the neutrino to have unit magnitude as we need the angle
	// between the neutrino and the reco muon to calculate the neutrino energy.
	TVector3 neu, mu, expected_proton, proton, reconu;
	neu.SetXYZ(0,0,1);
	mu.SetXYZ(Muon.Px(), Muon.Py(), Muon.Pz());
	double mp      = 0.938;
	double mn      = 0.9396;
	double cosmunu = mu.Dot(neu)*1./(mu.Mag()*neu.Mag());
	double reconuE = 0.5*(mp*mp - mn*mn + 2.*mn*Muon.E() - 0.1057*0.1057)*1./(mn - Muon.E() + mu.Mag()*cosmunu);
	// Now set the neutrino 3 momentum magnitude equal to the reconstructed neutrino energy.
	neu.SetXYZ(0,0,reconuE);      
	// Determine expected proton 3 momentum.
	expected_proton = neu - mu; 
	//slcE_proton = slcE_neu - mu/;
	proton.SetXYZ(Proton.Px(), Proton.Py(), Proton.Pz());
	reconu = proton + mu;
	float projection = proton.Dot(expected_proton)*1./proton.Mag()*1./expected_proton.Mag();
	fPrDotXPr->Fill(projection);

	if(projection > 0.99){
	  fRecoNuE->Fill(reconuE);
	  fProtonTrkLngthGood->Fill(proton_track_length);
	  fMuonTrkLngthGood->Fill(muon_track_length);
	  fMuonMomentum->Fill(mu.Mag());
	  fProtonMomentum->Fill(proton.Mag());
	  fPrCosTheta->Fill(proton.Z()/proton.Mag());
	  fMuVPrCosTheta->Fill(mu.Z()/mu.Mag(), proton.Z()/proton.Mag());
	}
      }
      
    } // end event selection 

    
  } // end loop over slices
  
} // end analyze module

DEFINE_ART_MODULE(bpfit::BreakPointProtonAna)
 
	
		  /*
		    art::PtrVector<rb::CellHit> hits = track->AllCells();
  double proton_GeV = 1.0E-3; 
  int n_uncalib_hits = 0;
  // Make reco hit for every cell hit in order to get calibrated cell energy.
  for(unsigned int j=0;j<hits.size();j++){
    const rb::CellHit *cellhit = track->Cell(j).get();
    double Wtemp = track->W(cellhit);
    rb::RecoHit rhit(hCal->MakeRecoHit(*cellhit,Wtemp));
    if(rhit.IsCalibrated()) proton_GeV += rhit.GeV();
    else n_uncalib_hits++;
  }
		  */
