////////////////////////////////////////////////////////////////////////
//
// BPException.cxx
// Exception handler for track reconstruction failures specific to
// BreaPointFitter.
// author:  mbaird42@fnal.gov
//
////////////////////////////////////////////////////////////////////////
#include "BreakPointFitter/func/BPException.h"

namespace bpfit {
  //--------------------------------------------------------------------
  BPException::BPException(const char* file, unsigned int line,
			   unsigned int flag, double value) :
    fFile(file),
    fLine(line),
    fFlag(flag),
    fValue(value)
  { }

  //--------------------------------------------------------------------
  void BPException::Print()
  {
    std::cout << "\nBPException ";

    switch(fFlag) {
    case kBAD_KE:
      std::cout << "bad KE";
      break;
    case kBAD_RADL:
      std::cout << "bad radiation length";
      break;
    case kNn:
      std::cout << "num scat surf <= num measurements";
      break;
    case kINDEX:
      std::cout << "index problem";
      break;
    case kNAN:
      std::cout << "nan encountered";
      break;
    case kSIGX:
      std::cout << "bad sigx";
      break;
    case kSIGSJ:
      std::cout << "bad sigSJ";
      break;
    case kBAD_P:
      std::cout << "bad value for p";
      break;
    case kBAD_BETA:
      std::cout << "bad value for beta";
      break;
    case kBAD_NDOF:
      std::cout << "track NDOF is too small";
      break;
    case kNPLANES:
      std::cout << "too few planes in one view";
      break;
    case kBAD_TRACK:
      std::cout << "tracking failure";
      break;
    default:
      std::cout << "UNKNOWN";
    }

    std::cout << " with value = " << fValue
	      << " found in " << fFile
	      << " at line " << fLine << "."
	      << std::endl;
  }

}
