#include "BreakPointFitter/func/dEdxTable.h"
#include <cmath>
#include <iostream>
#include "TH1F.h"
#include "TFile.h"

static const double gsMmu = 105.6583568E-3;
static const double gsMp  = 0.938;

int main(void) 
{
  unsigned int i;
  unsigned int nstep  = 600;
  double logtlo = -3.0;
  double logthi =  3.0;
  double t;

  TH1F* pvc   = new TH1F("pvc",   ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* steel = new TH1F("steel", ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* glue  = new TH1F("glue",  ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* scint = new TH1F("scint", ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);

  TH1F* pvcp   = new TH1F("pvcp",   ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* steelp = new TH1F("steelp", ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* gluep  = new TH1F("gluep",  ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);
  TH1F* scintp = new TH1F("scintp", ";log10(T/GeV);dEdx(MeV/cm)",nstep,logtlo,logthi);

  bpfit::dEdxTable dedx_pvc  ("PVC");
  bpfit::dEdxTable dedx_steel("Steel");
  bpfit::dEdxTable dedx_glue ("Glue");
  bpfit::dEdxTable dedx_scint("Scintillator");

  for (i=0; i<nstep; ++i) {
    t = logtlo + (i+0.5)*(logthi-logtlo)/nstep;
    t = pow(10.0, t);
    
    pvc->  SetBinContent(i+1,dedx_pvc  (t,gsMmu)*1e3);
    steel->SetBinContent(i+1,dedx_steel(t,gsMmu)*1e3);
    glue-> SetBinContent(i+1,dedx_glue (t,gsMmu)*1e3);
    scint->SetBinContent(i+1,dedx_scint(t,gsMmu)*1e3);

    pvcp->  SetBinContent(i+1,dedx_pvc  (t,gsMp)*1e3);
    steelp->SetBinContent(i+1,dedx_steel(t,gsMp)*1e3);
    gluep-> SetBinContent(i+1,dedx_glue (t,gsMp)*1e3);
    scintp->SetBinContent(i+1,dedx_scint(t,gsMp)*1e3);
  }

  TFile* f = new TFile("dedx.root","RECREATE");
  pvc->  Write();
  steel->Write();
  glue-> Write();
  scint->Write();

  pvcp->  Write();
  steelp->Write();
  gluep-> Write();
  scintp->Write();

  f->Close();
}
