#ifdef __CINT__
void calculate_avg_nslcs()
{
  std::cout << "Sorry, you must run in compiled mode" << std::endl;
}
#else

#include "CAFAna/Vars/Vars.h"
#include "CAFAna/Cuts/Cuts.h"
#include "CAFAna/Core/Spectrum.h"
#include "CAFAna/Core/HistAxis.h"
#include "CAFAna/Core/SpectrumLoader.h"

#include "Utilities/func/MathUtil.h"

#include "TH1.h"
#include "TF1.h"
#include "TFile.h"

#include <iostream>
#include <cmath>

using namespace ana;

double PoiFit(double* xs, double* ps)
{
  if(ps[1] <= 0) return 0;

  // We're passed the bin centers for the fit. Correct down to the integer
  return ps[0]*TMath::Poisson(xs[0]-.5, ps[1]);
}

/// This Macro will calculated the average number of slices in a spill
/// I'm using this number in a systematic - you can't get around hard-coding
/// in a value, so this macro will calculate the value when needed
void calculate_avg_nslcs()
{
  const Var     kNSlcs   = SIMPLEVAR(slc.nnonnoise);
  // We'll need to weight each slice by 1 / nnonnoise, since this number 
  // is filled nnonnoise times per spill
  const Var     kSlcWei  ({"slc.nnonnoise"},
                          [](const caf::StandardRecord* sr){
                            if (sr->slc.nnonnoise == 0)
                              return (double)0.;
                            return pow(sr->slc.nnonnoise, -1);});
  const Binning  NSlcBins = Binning::Simple(20, 0, 20);
  const HistAxis NSlcAxis("N Slcs per Spill", NSlcBins, kNSlcs);

  // We don't have to run over many files, just take 
  const std::string nearNonS   = "$NOVA_PROD/mc/FA14-12-29/genie/nd/caf/000103/*/*s101*.caf.root";
  SpectrumLoader loaderMC  (nearNonS);

  Spectrum NSlcsInMC(loaderMC, NSlcAxis, kNoCut, kNoShift, kSlcWei);
  
  loaderMC.Go();

  TH1* h = NSlcsInMC.ToTH1(NSlcsInMC.POT());
  h->SetMarkerStyle(kFullCircle);
  h->Draw("ep");
  h->Sumw2();
  TF1* fit = new TF1("fit", &PoiFit, 1, 20, 2);
  fit->SetParameter(1, 3);
  fit->SetNpx(1000);
  h->Fit(fit, "R");

  std::cout << std::endl;
  std::cout << std::endl;
  std::cout << "Poisson mean slices/spill is " << fit->GetParameter(1) << std::endl;
  std::cout << std::endl;

}
#endif
