#ifdef __CINT__
void reduce_nue_or_numu_fa(std::string fname, std::string outname)
{
  std::cout << "Sorry, you must run in compiled mode" << std::endl;
}
#else

#include "CAFAna/Cuts/Cuts.h"
#include "CAFAna/Core/FileReducer.h"
#include "CAFAna/Vars/Vars.h"
#include "CAFAna/Cuts/NueCutsFirstAna.h"
#include "CAFAna/Cuts/NumuCuts.h"

using namespace ana;

// TODO: something like this should probably be in Cut.h
Cut DetectorSwitcher(const Cut& ndCut, const Cut& fdCut)
{
  std::set<std::string> ndreqs = ndCut.Requires();
  std::set<std::string> fdreqs = fdCut.Requires();
  ndreqs.insert(fdreqs.begin(), fdreqs.end());
  ndreqs.insert("hdr.det");

  return Cut(ndreqs,
	     [&ndCut, &fdCut](const caf::StandardRecord* sr)
	     {
	       if(sr->hdr.det == caf::kFARDET)
		 return fdCut(sr);
	       else
		 return ndCut(sr);
	     });
}

// Produce decaf files for events that pass either the nue or numu cuts
void reduce_nue_or_numu_fa(std::string fname, std::string outname)
{
  FileReducer reduce(fname, outname);
  reduce.SetMetadata("nova.special", "nue_or_numu_contain");

  // require that there is at least reconstructed vertex and 3D prong
  // in the event
  Cut nueNDCut = kNueNDFirstAnaRecoQuality;

  // DQ cut only cuts out FEB flashers for now
  nueNDCut = nueNDCut && kNueNDFirstAnaDQ;

  // require that the vertex is within the fiducial volume
  nueNDCut = nueNDCut && kNueNDFirstAnaFiducial;

  // contain showers in the slice
  nueNDCut = nueNDCut && kNueNDFirstAnaContain;

  // require no hits in within the first few planes from the 
  // detector front face
  nueNDCut = nueNDCut && kNueNDFirstAnaDQ;


  // From what I can make out in NueCutsFirstAna.h these are the pre-cuts
  // shared by LID and LEM
  const Cut nueFDCut = kNueFirstAnaDQ && kNueFirstAnaContainment && kNueFirstAnaCosRej;

  const Cut eitherNueCut = DetectorSwitcher(nueNDCut, nueFDCut);


  const Cut numuCut = DetectorSwitcher(kNumuDecafCutND, kNumuDecafCutFD);


  reduce.AddEventCut(eitherNueCut || numuCut);

  // Clear out unnecessary truth info 
  reduce.AddReductionStep(ClearMultiNuInfo);
  reduce.AddReductionStep(ClearGeneratorInfo);

  reduce.Go();
}

#endif
