//Example macro for demonstrating the implementation of caf based reweighting
//First run CAFReweight/loadlibs.C 
//Currently I don't think that this will run under the cafe.  It doesn't appear 
//that the cafe loads the correct libraries that are needed for reweighting.  That 
//can certainly be done if there is a desire to do it.  So if reweighting is 
//needed for a macro run as a bare root using the above loadlibs.C macro.

// C/C++ includes
#include <math.h>
#include <vector>
#include <fstream>

#include "TTree.h"
#include "TH1D.h"
#include "TFile.h"
#include "TLorentzVector.h"
#include "TMath.h"
#include "TVector.h"
#include "TCanvas.h"

#include "StandardRecord/StandardRecord.h"
#include "StandardRecord/SRGenerator.h"
#include "StandardRecord/SRTruthBranch.h"
#include "CAFReweight/CAFReweight.h"



void CAFReweightExample(char *nIn, char *nOut) {

  cafrwgt::CAFReweight* fGrwgt[3];
  TH1D *wgt_dist[3];
  for(int i = 0; i < 3; i++) {

    double sigma = (double)(i+1);
    fGrwgt[i] = new cafrwgt::CAFReweight();
    fGrwgt[i]->AddReweightValue(rwgt::fReweightMaCCQE, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightMaCCRES, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightMaNCRES, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvpCC1pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvnCC1pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvpCC2pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvnCC2pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvpNC1pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvnNC1pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvpNC2pi, sigma);
    fGrwgt[i]->AddReweightValue(rwgt::fReweightRvnNC2pi, sigma);
    
    fGrwgt[i]->Configure();
    
  }
  wgt_dist[0] = new TH1D("h1", "Weight 3#sigma", 200, -2, 2);
  wgt_dist[1] = new TH1D("h2", "Weight 2#sigma", 200, -2, 2);
  wgt_dist[2] = new TH1D("h3", "Weight 3#sigma", 200, -2, 2);

  TFile *fIn = new TFile(nIn, "read");  
  TTree *cafT = (TTree*)fIn->Get("recTree");
  
  caf::StandardRecord* sr = 0;
  cafT->SetBranchAddress("rec", &sr);
  
  int nEntries = cafT->GetEntries();
    
  for(int iEntry = 0; iEntry < nEntries; iEntry++) {
    cafT->GetEntry(iEntry);
    std::cout << "Loop over tree entries" << std::endl;
    
    if(iEntry % 50000 == 0) {
      std::cout << iEntry << " of " << nEntries << std::endl;
    }

    
    double wgt[3];
    for(int i = 0; i < 3; i++) {
      wgt[i] = fGrwgt[i]->CalcWeight(sr->mc.gen);
      if(sr->mc.gen.size() > 0) {
	std::cout << "Found a Neutrino Interaction" << std::endl;
	wgt_dist[i]->Fill(wgt[i]);
      }
    }
    
  }
  
  TFile *fOut = new TFile(nOut, "recreate");
  fOut->cd();
  for(int i = 0; i < 3; i++) {
    wgt_dist[i]->Write();
  }
  fOut->Close();


}
