////////////////////////////////////////////////////////////////////////
/// \brief   Service to store calibration data products (CDP) in the 
///          SQLite3 metadatabase of a file                         
/// \author  brebel@fnal.gov
////////////////////////////////////////////////////////////////////////
#ifndef CDPSTORAGE_H
#define CDPSTORAGE_H

// ART includes
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "fhiclcpp/ParameterSet.h"

#include "CalibrationDataProducts/DriftResponse.h"
#include "CalibrationDataProducts/AttenProfiles.h"
#include "CalibrationDataProducts/TCTrack.h"

namespace art {
  class SQLite3Wrapper;
  class OutputFileInfo;
}

///CDPStorage service
namespace calib {

  class CDPStorage {

  public:
    CDPStorage(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg);
    ~CDPStorage();

    void reconfigure(const fhicl::ParameterSet& pset);
    
    void postCloseOutputFile(art::OutputFileInfo const& ofi);
    void postOpenFile(std::string const& fn);

    void DriftResponseToStore(std::vector<caldp::DriftResponse>& drvec);
    void WriteDriftResponse(art::SQLite3Wrapper& sql3w);
    void RetrieveDriftCalFromFile(art::SQLite3Wrapper& sql3w);

    std::vector<caldp::DriftResponse> const& DriftRepsonses() const { return fDriftResponseVec; }

  private:

    std::vector<caldp::DriftResponse> fDriftResponseVec;         ///< vector of DriftResponse objects to store
    bool                              fRetrieveDriftCalFromFile; ///< get the response from the SQLite database
                                                                 ///< after opening the file
  };
} // end namespace calib
DECLARE_ART_SERVICE(calib::CDPStorage, LEGACY)
#endif
///////////////////////////////////////////////////////////////////////////  
