////////////////////////////////////////////////////////////////////////
// Class:       DCMTimingOffset
// Module Type: producer
// File:        DCMTimingOffset_module.cc
//
// Generated at Mon Feb 11 09:01:32 2013 by Evan Niner using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

//NOvASoft includes
#include "Geometry/Geometry.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "Utilities/func/MathUtil.h"
#include "CalibrationDataProducts/PCHit.h"
#include "CalibrationDataProducts/TCTrack.h"
#include "CalibrationDataProducts/DCMSummary.h"
#include "CalibrationDataProducts/DCMOffset.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "RunHistory/service/RunHistoryService.h"
#include "DAQChannelMap/DAQChannelMap.h"

//framework includes
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Core/FindManyP.h"
#include "Utilities/AssociationUtil.h"
#include "cetlib/exception.h"
#include "art/Persistency/Provenance/Timestamp.h"

#include <stdint.h>
#include <sstream>
#include <map>
#include <iterator>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <string>

//ROOT includes
#include "TTree.h"
#include "TH2F.h"
#include "TH1F.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include "TDecompSVD.h"
#include "TDecompLU.h"
#include "TDirectory.h"

namespace calib {

  class DCMTimingOffset : public art::EDFilter {
  public:
    explicit DCMTimingOffset(fhicl::ParameterSet const & p);
    virtual ~DCMTimingOffset();

    //void produce(art::Event & e) override;
    virtual bool filter(art::Event& evt) {return false;}

    void reconfigure(const fhicl::ParameterSet& pset);
    void beginJob();
    void endJob();
    virtual bool beginRun(art::Run& run);
    virtual bool endSubRun(art::SubRun& srun);
    virtual bool endRun(art::Run & r);
    //virtual void respondToCloseOutputFiles(art::FileBlock const& fb);


  private:

    std::string   fDCMLabel;        ///< Where to find dcm stats
    int           fNumDCM;          ///< Number of DCMs calibration will be performed on
    int           fMinDCM;          ///< Number of lowest DCM in calibration set
    int           fMaxDCM;          ///< Number of highest DCM in calibration set
    int           fMinCount;        ///< Minimum number of counts for a DCM pair
    bool          fInitialized = false; ///<Flag if initialization has happened
    bool          fOverrideRef; ///<override default of fixing calibration of dcm 6 with an offset of 0
    int           fRefDCMTop = 6;    ///< DCM number to use as reference for top chain, default is 6
    int           fRefDCMSide = 7;   ///< DCM number to use as reference for side chain, default is 7 
   float         fRefOffsetTop = 0.0;   ///<absolute offset, in ns, to fix refence DCM to for top chain, default is 0.0
    float         fRefOffsetSide = 0.0;  ///<absolute offset, in ns, to fix refence DCM to for side chain, default is 0.0
    bool          fIsMC;
    bool          fSaveRunCalibration;
    int           fMinSubrun;

    fhicl::ParameterSet fPSetND, fPSetFD;
    

    //keep stats for entire group of runs
    std::vector<std::vector<int> >    countDCMOffsetArray;
    std::vector<std::vector<float> > sumSquareDCMOffsetArray;
    std::vector<std::vector<float> > sumDCMOffsetArray;
    std::vector<std::vector<float> > meanDCMOffsetArray;
    std::vector<std::vector<float> > weightDCMOffsetArray;
    std::vector<float>               absoluteDCMOffset;
    std::vector<float>               absoluteDCMErrors;
    std::vector<int>                  countDCMUse;
    uint32_t minTime;
    uint32_t maxTime;

    //keep stats for an individual run
    std::vector<std::vector<int> >    countDCMOffsetArrayRun;
    std::vector<std::vector<float> > sumSquareDCMOffsetArrayRun;
    std::vector<std::vector<float> > sumDCMOffsetArrayRun;
    std::vector<std::vector<float> > meanDCMOffsetArrayRun;
    std::vector<std::vector<float> > weightDCMOffsetArrayRun;
    std::vector<float>               absoluteDCMOffsetRun;
    std::vector<float>               absoluteDCMErrorsRun;
    std::vector<int>                  countDCMUseRun;
    uint32_t minTimeRun;
    uint32_t maxTimeRun;

    int subrunCount;
    
    std::map<int, std::vector<caldp::DCMOffset> > offsetMap;

  };
}

namespace calib {

  //define speed of light in cm/ns
  static const double sCmPerNsec = 29.9792458;

  DCMTimingOffset::DCMTimingOffset(fhicl::ParameterSet const & pset)
  // :
  // Initialize member data here.
  {
    this->reconfigure(pset); 

    produces< std::vector<caldp::DCMOffset>, art::InRun >();
    produces< caldp::DCMSummary, art::InRun >();

  }

//---------------------------------------------------------------------------
  void DCMTimingOffset::reconfigure(const fhicl::ParameterSet& pset)
  {
    fDCMLabel    = pset.get< std::string >("DCMLabel");
    fMinSubrun = pset.get< int>("MinSubrun");
    fSaveRunCalibration = pset.get< bool >("SaveRunCalibration");
    fOverrideRef = false;
    fMinDCM      = 0;
    fMaxDCM      = 0;
    fMinCount    = 0;
    fInitialized = false;
    fPSetND         = pset.get<fhicl::ParameterSet>("nd");
    fPSetFD         = pset.get<fhicl::ParameterSet>("fd");
    
  }  

  //---------------------------------------------------------------------------
  DCMTimingOffset::~DCMTimingOffset()
  {
    // Clean up dynamic memory and other resources here.
  }

  //---------------------------------------------------------------------------
  void DCMTimingOffset::beginJob()
  {




    //We want to be able to handle any time possible.
    minTime =  std::numeric_limits<uint32_t>::max()-1;
    maxTime =  std::numeric_limits<uint32_t>::min();
  }

  //---------------------------------------------------------------------------
  bool DCMTimingOffset::beginRun(art::Run& run)
  {
    //for first run determine detector, initialize arrays, do not repeat
    if(!fInitialized){
      fInitialized = true;
      art::ServiceHandle<nova::dbi::RunHistoryService> rh;
      fIsMC = (rh->GetDataType() == nova::dbi::kMCOnly);

      minTimeRun =  std::numeric_limits<uint32_t>::max()-1;
      maxTimeRun =  std::numeric_limits<uint32_t>::min();

      //get fcl parameters based on detector
      art::ServiceHandle<geo::Geometry> geom;

      fhicl::ParameterSet pset;
      switch(geom->DetId()){
      case novadaq::cnv::kNEARDET:
        pset = fPSetND;
        break;
      case novadaq::cnv::kFARDET:
        pset = fPSetFD;
        break;
      default:
        assert(0 && "Unknown detector");
      }

      fOverrideRef = pset.get< bool >("OverrideRef");
      fMinDCM    = pset.get< int         >("MinDCM");
      fMaxDCM    = pset.get< int         >("MaxDCM");
      fMinCount  = pset.get< int         >("MinCount");
      if (fOverrideRef){
        fRefDCMTop = pset.get< int >("RefDCMTop");
        fRefDCMSide = pset.get< int >("RefDCMSide");
        fRefOffsetTop = pset.get< float >("RefOffsetTop");
        fRefOffsetSide = pset.get< float >("RefOffsetSide");
      }

      //assert parameters make sense
      assert(fMaxDCM >= fMinDCM);
      assert((fRefDCMTop >= fMinDCM) && (fRefDCMTop <= fMaxDCM));
      assert((fRefDCMSide >= fMinDCM) && (fRefDCMSide <= fMaxDCM));
      fNumDCM = fMaxDCM - fMinDCM + 1;


      countDCMOffsetArray.resize(fNumDCM);
      sumSquareDCMOffsetArray.resize(fNumDCM);
      sumDCMOffsetArray.resize(fNumDCM);
      meanDCMOffsetArray.resize(fNumDCM);
      weightDCMOffsetArray.resize(fNumDCM);
      absoluteDCMOffset.resize(fNumDCM);
      absoluteDCMErrors.resize(fNumDCM);
      countDCMUse.resize(fNumDCM);

      for (int i=0; i<fNumDCM; ++i){
        countDCMOffsetArray[i].resize(fNumDCM);
        sumSquareDCMOffsetArray[i].resize(fNumDCM);
        sumDCMOffsetArray[i].resize(fNumDCM);
        meanDCMOffsetArray[i].resize(fNumDCM);
        weightDCMOffsetArray[i].resize(fNumDCM);
      }
    }

    //clear arrays for the run to be safe
    countDCMOffsetArrayRun.clear();
    sumSquareDCMOffsetArrayRun.clear();
    sumDCMOffsetArrayRun.clear();
    meanDCMOffsetArrayRun.clear();
    weightDCMOffsetArrayRun.clear();
    absoluteDCMOffsetRun.clear();
    absoluteDCMErrorsRun.clear();
    countDCMUseRun.clear();

    //resize run arrays
    countDCMOffsetArrayRun.resize(fNumDCM);
    sumSquareDCMOffsetArrayRun.resize(fNumDCM);
    sumDCMOffsetArrayRun.resize(fNumDCM);
    meanDCMOffsetArrayRun.resize(fNumDCM);
    weightDCMOffsetArrayRun.resize(fNumDCM);
    absoluteDCMOffsetRun.resize(fNumDCM);
    absoluteDCMErrorsRun.resize(fNumDCM);
    countDCMUseRun.resize(fNumDCM);

    for (int i=0; i<fNumDCM; ++i){
      countDCMOffsetArrayRun[i].resize(fNumDCM);
      sumSquareDCMOffsetArrayRun[i].resize(fNumDCM);
      sumDCMOffsetArrayRun[i].resize(fNumDCM);
      meanDCMOffsetArrayRun[i].resize(fNumDCM);
      weightDCMOffsetArrayRun[i].resize(fNumDCM);
    }

    subrunCount = 0;

    return true;
  }

  //---------------------------------------------------------------------------
  bool DCMTimingOffset::endSubRun(art::SubRun& srun)
  {
    //get out summary dataproduct
    //get the cosmic tracks associated with each event
    art::Handle<caldp::DCMSummary > dcmsum;
    srun.getByLabel(fDCMLabel,dcmsum);

    subrunCount++;

    //keep running tally of relative dcm timing offsets for the run and overall
    for (int j=0; j<fNumDCM; ++j){
      int jabs = j + fMinDCM - 1;
      countDCMUseRun[j] += dcmsum->DCMUseIndx(jabs);
      countDCMUse[j] += dcmsum->DCMUseIndx(jabs);
      for (int k=j+1; k<fNumDCM; ++k){
        int kabs = k +fMinDCM - 1;
        sumDCMOffsetArrayRun[j][k]+= dcmsum->DCMOffsetSumIndx(jabs,kabs);
        sumDCMOffsetArray[j][k]+= dcmsum->DCMOffsetSumIndx(jabs,kabs);
        sumSquareDCMOffsetArrayRun[j][k] += dcmsum->DCMOffsetSumSquareIndx(jabs,kabs);
        sumSquareDCMOffsetArray[j][k] += dcmsum->DCMOffsetSumSquareIndx(jabs,kabs);
        countDCMOffsetArrayRun[j][k] += dcmsum->DCMOffsetCountIndx(jabs,kabs);
        countDCMOffsetArray[j][k] += dcmsum->DCMOffsetCountIndx(jabs,kabs);
      }
    }
 
    //adjust validity time
    uint32_t min = dcmsum->StartTime();
    uint32_t max = dcmsum->EndTime();
    if (max>maxTime) maxTime = max;
    if (min<minTime) minTime = min;
    if (max>maxTimeRun) maxTimeRun = max;
    if (min<minTimeRun) minTimeRun = min;
      

    return false;
  }



  //---------------------------------------------------------------------------
  bool DCMTimingOffset::endRun(art::Run& run)
  {
    //perform timing calibration on a run basis if desired
    if (!fSaveRunCalibration){
      countDCMOffsetArrayRun.clear();
      sumSquareDCMOffsetArrayRun.clear();
      sumDCMOffsetArrayRun.clear();
      meanDCMOffsetArrayRun.clear();
      weightDCMOffsetArrayRun.clear();
      absoluteDCMOffsetRun.clear();
      absoluteDCMErrorsRun.clear();
      countDCMUseRun.clear();
      return false;
    }

    if (subrunCount < fMinSubrun){
      countDCMOffsetArrayRun.clear();
      sumSquareDCMOffsetArrayRun.clear();
      sumDCMOffsetArrayRun.clear();
      meanDCMOffsetArrayRun.clear();
      weightDCMOffsetArrayRun.clear();
      absoluteDCMOffsetRun.clear();
      absoluteDCMErrorsRun.clear();
      countDCMUseRun.clear();
      return false;
    }
    //Ptr to dcm timing offsets for the run
    std::unique_ptr<std::vector<caldp::DCMOffset > >runOffsets(new std::vector<caldp::DCMOffset >);

    caldp::DCMSummary runsum;
    runsum.SetStartTime(minTimeRun);
    runsum.SetEndTime(maxTimeRun);

    //compute mean and standard deviation for each relative DCM pair in the matrix
    for(int i=0; i<fNumDCM-1; ++i){
      int iabs = i + fMinDCM - 1;
      runsum.SetDCMUseIndx(iabs, countDCMUseRun[i]);
      for(int j=i+1; j<fNumDCM; ++j){
        int jabs = j + fMinDCM - 1;
        float sum = sumDCMOffsetArrayRun[i][j];
        int count = countDCMOffsetArrayRun[i][j];
        float sumSqr = sumSquareDCMOffsetArrayRun[i][j];
        runsum.SetDCMOffsetSumIndx(iabs,jabs,sum);
        runsum.SetDCMOffsetSumSquareIndx(iabs,jabs,sumSqr);
        runsum.SetDCMOffsetCountIndx(iabs,jabs,count);
        if (count >= fMinCount){
          meanDCMOffsetArrayRun[i][j] = sum/count;
          float sigma = sqrt((count*sumSqr - sum*sum)/(count*(count-1.0)));
          weightDCMOffsetArrayRun[i][j] = 1.0/(sigma*sigma);
        }
        else {
          meanDCMOffsetArrayRun[i][j] = 0;
          weightDCMOffsetArrayRun[i][j] = 0;
        }
      }
    }

    const int num= fNumDCM;
    int refIndxTop = fRefDCMTop - fMinDCM;
    int refIndxSide = fRefDCMSide - fMinDCM;
    TMatrixD A;
    A.ResizeTo(fNumDCM, fNumDCM);
    double y[num];
    for (int x=0; x<num; ++x) y[x]=0.;

    //see docdb-XXXX, solve a system of linear equations that corresponds to minimizing the chhi squared for the absolute offsets.  Fill matrix in this step
    for(int i=0; i<fNumDCM; ++i){
      double temp[num];
      for (int x=0; x<num; ++x) temp[x]=0.;
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i){
          temp[i] += weightDCMOffsetArrayRun[j][i]*2.0;
          temp[j] += -weightDCMOffsetArrayRun[j][i]*2.0;
          y[i] += meanDCMOffsetArrayRun[j][i]*weightDCMOffsetArrayRun[j][i]*2.0; 
        }
        if (j>i){
          temp[i] += weightDCMOffsetArrayRun[i][j]*2.0;
          temp[j] += -weightDCMOffsetArrayRun[i][j]*2.0;
          y[i] += -meanDCMOffsetArrayRun[i][j]*weightDCMOffsetArrayRun[i][j]*2.0;
        }
      }

      if (i == refIndxTop){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndxTop]=1.0;
      } 
      if (i == refIndxSide){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndxSide]=1.0;
      } 

      y[i]*=-1.0;
      TVectorD tempvec;
      tempvec.Use(fNumDCM,temp);
      TMatrixDRow(A,i) = tempvec;
    }

    y[refIndxTop] = fRefOffsetTop;
    y[refIndxSide] = fRefOffsetSide;
    TVectorD yvec;
    yvec.Use(fNumDCM,y);

    //calculate uncertainties.  Errors are highly correlated, this is an over estimate from the diagonal elements
    for(int i=0; i<fNumDCM; ++i){
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i) absoluteDCMErrorsRun[i] += 2.0*weightDCMOffsetArrayRun[j][i];
        else if (j>i) absoluteDCMErrorsRun[i] += 2.0*weightDCMOffsetArrayRun[i][j];
      }
      absoluteDCMErrorsRun[i] = sqrt(2.0/absoluteDCMErrorsRun[i]);
    }
    absoluteDCMErrorsRun[refIndxTop] = 0.0;
    absoluteDCMErrorsRun[refIndxSide] = 0.0;

    //solve for absolute offsets
    TDecompLU lu(A);
    bool inv = lu.Decompose();
    if (inv) lu.Invert(A);
    if (inv) yvec *= A;

    for (int i=0; i<fNumDCM; ++i){
      if (!inv) break;
      caldp::DCMOffset dcmOffset;
      dcmOffset.SetDCM(i+fMinDCM);
      dcmOffset.SetOffset(yvec[i]);
      dcmOffset.SetSigma(absoluteDCMErrorsRun[i]);
      dcmOffset.SetStats(countDCMUseRun[i]);
      dcmOffset.SetStartTime(minTimeRun);
      dcmOffset.SetEndTime(maxTimeRun);
      runOffsets->push_back(dcmOffset);
      offsetMap[run.run()].push_back(dcmOffset);
    }

    countDCMOffsetArrayRun.clear();
    sumSquareDCMOffsetArrayRun.clear();
    sumDCMOffsetArrayRun.clear();
    meanDCMOffsetArrayRun.clear();
    weightDCMOffsetArrayRun.clear();
    absoluteDCMOffsetRun.clear();
    absoluteDCMErrorsRun.clear();
    countDCMUseRun.clear();

    std::unique_ptr<caldp::DCMSummary> dsum(new caldp::DCMSummary(runsum));
    run.put(std::move(dsum));

    run.put(std::move(runOffsets));

    return true;
  }

  //---------------------------------------------------------------------------
  void DCMTimingOffset::endJob()
  {

    //figure out dcms per diblock from geometry 
    art::ServiceHandle<geo::Geometry>         geom;
    daqchannelmap::DAQChannelMap* cmap = daqchannelmap::DAQChannelMap::getInstance(geom->DetId());
    const int NDCMDB = cmap->getNumberOfDCM(daqchannelmap::AFULLAFULL_DIBLOCK);

    //make summary plots of calibration performance
    art::ServiceHandle<art::TFileService> tfs;

    const int runs = offsetMap.size();
    const int num= fNumDCM;
    TGraphErrors* grAbsDCM[num];
    TGraphErrors* grAbsRuns[runs];
    TGraph* grStatsDCM[num];
    TGraph* grStatsRuns[runs];
    TGraphErrors* grAbs;
    TGraphErrors* grAbsDiff;
    TGraph* grStats;
    TGraphErrors* grRelDiff;
    TH2F* fHistoRelMap;
    TH1F* fHistoRelDiff;
    TH2F* fHistoRelDiffMap;

    double runDCM[num][runs];
    double runDCMOffset[num][runs];
    double runDCMOffsetE[num][runs];
    double runDCMStats[num][runs];

    grRelDiff = tfs->make<TGraphErrors>();
    gDirectory->Append(grRelDiff);
    grRelDiff->SetName("grRelDiff");
    grRelDiff->SetTitle("Relative offset drift vs diblock gap");
    grRelDiff->Set(55);

    grAbsDiff = tfs->make<TGraphErrors>();
    gDirectory->Append(grAbsDiff);
    

    grAbs = tfs->make<TGraphErrors>();
    gDirectory->Append(grAbs);
    grStats = tfs->make<TGraph>();
    gDirectory->Append(grStats);
    fHistoRelMap = tfs->make<TH2F>("fHistoRelMap","Relative offsets (ns), all runs, DCM Y- DCM X;dcm X;dcm Y",
                                   fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5,fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5);
    fHistoRelDiffMap = tfs->make<TH2F>("fHistoRelDiffMap","Relative offsets differences (ns), all runs, DCM Y- DCM X - expected;dcm X;dcm Y",
                                   fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5,fNumDCM,(double)fMinDCM-0.5,(double)fMaxDCM+0.5);
    fHistoRelDiff = tfs->make<TH1F>("fHistoRelDiff","DCM i - DCM j - expected;diff (ns);count",201,-100.5,100.5);

    for (int i=0; i< runs; ++i){
      grAbsRuns[i] = tfs->make<TGraphErrors>();
      gDirectory->Append(grAbsRuns[i]); 
      grStatsRuns[i] = tfs->make<TGraph>();
      gDirectory->Append(grStatsRuns[i]); 
    }
    for (int i=0; i< num; ++i){
      grAbsDCM[i] = tfs->make<TGraphErrors>();
      gDirectory->Append(grAbsDCM[i]); 
      grStatsDCM[i] = tfs->make<TGraph>();
      gDirectory->Append(grStatsDCM[i]); 
    }

    //expected pattern of DCM offsets within each FD diblock based on measured cable delays.  This is used to plot accuracy of calibration.  If 
    //the expected offsets change this should be updated.
    double dcmoffsetExpectedFDDiblock[12] = {-203.125,-164.0625,-125.0,-78.125,-39.0625,0.0,0.0,-39.0625,-78.125,-125.0,-164.0625,-203.125};

    //expected pattern of DCM offsets for all ND DCMs based on measured cable delays.  There is not a repeatable diblock pattern so all
    //DCMs are listed.  This is used to plot accuracy of calibration.  If the expected offsets change this should be updated.
    double dcmoffsetExpectedND[14] = {-101.5625,-70.3125,-30.25,0.0,-101.5625,-70.3125,-30.25,0.0,-101.5625,-70.3125,-30.25,0.0,-101.5625,-62.5};

    //initialize bins of the relative offset map so that empty bins can be distinguished from 0
    for(int i=0;i<fNumDCM;++i){
      for(int j=0;j<fNumDCM;++j){
        fHistoRelMap->SetBinContent(j+1,i+1,-5000.0);
      }
    }

    //calculate mean and standard deviation for all relative offset pairs
    for(int i=0; i<fNumDCM-1; ++i){
      for(int j=i+1; j<fNumDCM; ++j){
        float sum = sumDCMOffsetArray[i][j];
        int count = countDCMOffsetArray[i][j];
        float sumSqr = sumSquareDCMOffsetArray[i][j];
        if (count >= fMinCount){
          meanDCMOffsetArray[i][j] = sum/count;
          float sigma = sqrt((count*sumSqr - sum*sum)/(count*(count-1.0)));
          weightDCMOffsetArray[i][j] = 1.0/(sigma*sigma);
          fHistoRelMap->SetBinContent(j+1,i+1,sum/count);
          double expected = 0.0;
          if (geom->DetId() == novadaq::cnv::kFARDET) expected = dcmoffsetExpectedFDDiblock[i%12]-dcmoffsetExpectedFDDiblock[j%12];
          else if (geom->DetId() == novadaq::cnv::kNEARDET) expected = dcmoffsetExpectedND[i]-dcmoffsetExpectedND[j];
          fHistoRelDiffMap->SetBinContent(j+1,i+1,(sum/count)-expected);
          fHistoRelDiff->Fill((sum/count)-expected);
        }
        else {
          meanDCMOffsetArray[i][j] = 0;
          weightDCMOffsetArray[i][j] = 0;
        }
      }
    }


    int refIndxTop = fRefDCMTop - fMinDCM;
    int refIndxSide = fRefDCMSide - fMinDCM;
    TMatrixD A;
    A.ResizeTo(fNumDCM, fNumDCM);
    double y[num];
    for (int x=0; x<num; ++x) y[x]=0.;

    //see docdb-XXXX, solve a system of linear equations that corresponds to minimizing the chhi squared for the absolute offsets.  Fill matrix in this step
    for(int i=0; i<fNumDCM; ++i){
      double temp[num];
      for (int x=0; x<num; ++x) temp[x]=0.;
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i){
          temp[i] += weightDCMOffsetArray[j][i]*2.0;
          temp[j] += -weightDCMOffsetArray[j][i]*2.0;
          y[i] += meanDCMOffsetArray[j][i]*weightDCMOffsetArray[j][i]*2.0; 
        }
        if (j>i){
          temp[i] += weightDCMOffsetArray[i][j]*2.0;
          temp[j] += -weightDCMOffsetArray[i][j]*2.0;
          y[i] += -meanDCMOffsetArray[i][j]*weightDCMOffsetArray[i][j]*2.0;
        }
      }

      if (i == refIndxTop){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndxTop]=1.0;
      } 
      if (i == refIndxSide){
        for (int j=0; j<fNumDCM; ++j) temp[j] = 0.0;
        temp[refIndxSide]=1.0;
      } 

      y[i]*=-1.0;
      TVectorD tempvec;
      tempvec.Use(fNumDCM,temp);
      TMatrixDRow(A,i) = tempvec;
    }

    y[refIndxTop] = fRefOffsetTop;
    y[refIndxSide] = fRefOffsetSide;
    TVectorD yvec;
    yvec.Use(fNumDCM,y);

    //calculate uncertainties
    for(int i=0; i<fNumDCM; ++i){
      for(int j=0; j<fNumDCM; ++j){
        if (i==j) continue;
        if (j<i) absoluteDCMErrors[i] += 2.0*weightDCMOffsetArray[j][i];
        else if (j>i) absoluteDCMErrors[i] += 2.0*weightDCMOffsetArray[i][j];
      }
      absoluteDCMErrors[i] = sqrt(2.0/absoluteDCMErrors[i]);
    }
    absoluteDCMErrors[refIndxTop] = 0.0;
    absoluteDCMErrors[refIndxSide] = 0.0;

    //solve for absolute offsets
    A.Invert();
    yvec *= A;
  
    //write results to csv file
    const TString mcStr = fIsMC ? "_mc" : "";
    FILE* fConstsCSV = fopen("calib_abs_dcm_delay_consts"+mcStr+".csv","w");

    fprintf(fConstsCSV, "#dcmoffset,sigma,stats,channel,tv\n");

    //there is a drift in the dcm offsets in diblocks 7-14.  The cause is unknown for now offsets of each diblock are adjusted by hand  to put them in 
    //the right place.  This should not be necessary in the future when the cause in known and can be removed
    double dbcorrectionFD[14] = {0.0,
                                 0.0,
                                 0.0,
                                 0.0,
                                 0.0,
                                 0.0,
                                 4.06,
                                 3.88,
                                 3.82,
                                 4.24,
                                 6.31,
                                 7.29,
                                 10.23,
                                 11.47};

    //same style correction for the ND
    double dbcorrectionND[14] = {15.55,
                                 8.2,
                                 0.0,
                                 0.0};

    //Multipy by -1 since the correction facto is the opposite sign of the offset
    //store channel as diblock*1000+dcm
    //make CVS file with constants to put in DB
    for (int i=0; i<fNumDCM; ++i){
      int db = ((i+fMinDCM)/NDCMDB)+1;
      int dcm = (i+fMinDCM)%NDCMDB;
      if ((i+fMinDCM)%NDCMDB == 0){
        db--;
        dcm = NDCMDB;
      }
      if (geom->DetId() == novadaq::cnv::kFARDET) absoluteDCMOffset[i] = yvec[i] + dbcorrectionFD[db-1];
      else if (geom->DetId() == novadaq::cnv::kNEARDET) absoluteDCMOffset[i] = yvec[i] + dbcorrectionND[db-1];
      int chan = db*1000 + dcm;
      fprintf(fConstsCSV, "%g,%g,%d,%d,%d\n",
              (-1*absoluteDCMOffset[i]), absoluteDCMErrors[i], countDCMUse[i], chan,  minTime);
    }

    fclose(fConstsCSV);

    //save summary calibration histograms on run-by-run basis
    if (fSaveRunCalibration){
      int runcount= 0;
      for (std::map<int, std::vector<caldp::DCMOffset> >::iterator itr = offsetMap.begin(); itr != offsetMap.end(); ++itr){
        int rnum = itr->first;
        std::vector<caldp::DCMOffset> offsets = itr->second;
      
        TString dummy;
        std::ostringstream convert;
        convert << rnum;
        dummy = convert.str();
        grAbsRuns[runcount]->SetName("grAbsOffset_run_"+dummy);
        grAbsRuns[runcount]->SetTitle("Absolute DCM timing offsets (ns) for run: "+dummy);
        grAbsRuns[runcount]->Set(offsets.size());
        grStatsRuns[runcount]->SetName("grStatsOffset_run_"+dummy);
        grStatsRuns[runcount]->SetTitle("Number of track segments used by dcm for run: "+dummy);
        grStatsRuns[runcount]->Set(offsets.size());
        for (unsigned int i=0; i<offsets.size(); ++i){
          int tdcm = offsets[i].DCM();
          double toffset = offsets[i].Offset();
          double tsigma = offsets[i].Sigma(); 
          int tstats = offsets[i].Stats();
          grStatsRuns[runcount]->SetPoint(i,tdcm,tstats);
          grAbsRuns[runcount]->SetPoint(i,tdcm,toffset);
          grAbsRuns[runcount]->SetPointError(i,0.0,tsigma);
          runDCM[tdcm-fMinDCM][runcount]=rnum;
          runDCMOffset[tdcm-fMinDCM][runcount]=toffset;
          runDCMOffsetE[tdcm-fMinDCM][runcount]=tsigma;
          runDCMStats[tdcm-fMinDCM][runcount]=tstats;
        }
        runcount++;
      }

      for(int i=0; i<num; ++i){
        int tdcm = i + fMinDCM;
        TString dummy;
        std::ostringstream convert;
        convert << tdcm;
        dummy = convert.str();
        grAbsDCM[i]->SetName("grAbsOffset_dcm_"+dummy);
        grAbsDCM[i]->SetTitle("Absolute DCM timing offsets (ns) across all run for dcm: "+dummy);
        grAbsDCM[i]->Set(runs);
        grStatsDCM[i]->SetName("grStatsOffset_dcm_"+dummy);
        grStatsDCM[i]->SetTitle("Number of track segments used for all runs by dcm: "+dummy);
        grStatsDCM[i]->Set(runs);
        for(int j=0; j<runs; ++j){
          grAbsDCM[i]->SetPoint(j,runDCM[i][j],runDCMOffset[i][j]);
          grAbsDCM[i]->SetPointError(j,0.0,runDCMOffsetE[i][j]);
          grStatsDCM[i]->SetPoint(j,runDCM[i][j],runDCMStats[i][j]);
        }
      }
    }

    grAbs->SetName("grAbsOffset_total");
    grAbs->SetTitle("Absolute DCM timing offsets (ns), all runs");
    grAbs->Set(num);
    grStats->SetName("grStatsOffset_total");
    grStats->SetTitle("Number of track segments used by dcm, all runs");
    grStats->Set(num);
    grAbsDiff->SetName("grAbsDiff_total");
    grAbsDiff->SetTitle("Absolute DCM timing offset - expected value (ns), all runs");
    grAbsDiff->Set(num);

    for(int i=0; i< fNumDCM; ++i){
      if (geom->DetId() == novadaq::cnv::kFARDET) grAbsDiff->SetPoint(i,i+fMinDCM,absoluteDCMOffset[i]-dcmoffsetExpectedFDDiblock[i%12]);
      else if (geom->DetId() == novadaq::cnv::kNEARDET) grAbsDiff->SetPoint(i,i+fMinDCM,absoluteDCMOffset[i]-dcmoffsetExpectedND[i]);
      grAbsDiff->SetPointError(i,0.0,absoluteDCMErrors[i]);
      grAbs->SetPoint(i,i+fMinDCM,absoluteDCMOffset[i]);
      grAbs->SetPointError(i,0.0,absoluteDCMErrors[i]);
      grStats->SetPoint(i,i+fMinDCM,countDCMUse[i]);
    }
    

  }

} //end namespace

//-----------------------------------------------------------------------------

DEFINE_ART_MODULE(calib::DCMTimingOffset)
