////////////////////////////////////////////////////////////////////////
// Class:       MuondEdxAna
// Module Type: filter
// File:        MuondEdxAna_module.cc
//
// Generated at Tue May  7 16:23:55 2013 by Abigail Waldron using artmod
// from cetpkgsupport v1_02_00.
////////////////////////////////////////////////////////////////////////

// framework includes....
#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Core/FindManyP.h" 
#include "Geometry/LiveGeometry.h"
#include "GeometryObjects/GeometryBase.h"

// novasoft stuff
#include "CalibrationDataProducts/PCHit.h"
#include "RecoBase/Track.h"
#include "RecoBase/CellHit.h"
#include "Calibrator/Calibrator.h"
#include "NovaTimingUtilities/TimingUtilities.h"

// C++
#include <memory>
#include <string>
#include <iostream>
#include <vector>

// ROOT
#include "TH3D.h"
#include "TH2D.h"
#include "TProfile.h"
#include "TProfile2D.h"
#include "TFile.h"
#include "TVector3.h"
#include "TTree.h"

namespace caldp {
  class PCHit;
}

namespace calib {
  class MuondEdxAna;
  
  class MuondEdxAna : public art::EDFilter {
  public:
    explicit MuondEdxAna(fhicl::ParameterSet const & p);
    virtual ~MuondEdxAna();
    
    bool filter(art::Event & e) override;
    void reconfigure(fhicl::ParameterSet const & p) override;    
    void beginJob() override;
    void endJob() override;
    bool beginRun(art::Run & r) override;
    bool endRun(art::Run & r) override;
    
  private:
    double getPECorr(art::Event & e,caldp::PCHit const &pchit);
    double getPECorrToGeV(art::Event & e, caldp::PCHit const &pchit);

    void fillRespVar(double PECorr, double PE, double trueE, double RecoGeV, double path, double w, int plane, int cell, 
		     int view, double tns, double EventTimeHigh, int run, double dCosX, double dCosY, double dCosZ, 
		     int diblock);    
    void fillLowLevel(double PECorr, double PE, double trueE, double RecoGeV, double path, double w, int plane, int cell, 
		      int view, int diblock, double x, double trackLength, double FlightLength, double TotalTrackLength, 
		      double dCosX, double dCosY, double dCosZ);

    TH2D* fdEdx;
    TH2D* fdEdx_MuonCatch;
    TH2D* fdEdx_minus;
    TH2D* fdEdx_plus;
    TH2D* fdEdx_true; 
    TH2D* fdEdx_true_MuonCatch; 

    TH2D* fdEdx_NearReadout;
    TH2D* fdEdx_true_NearReadout; 
    
    TH1D* fPECorr;    
    TH1D* fPE;    
    TH1D* fTrueE;
    TH1D* fPECorr_X;    
    TH1D* fPE_X;    
    TH1D* fTrueE_X;
    TH1D* fPECorr_Y;    
    TH1D* fPE_Y;    
    TH1D* fTrueE_Y;
    TH1D* fRecoGeV;
      
    TH1D* fDiblock;
    TProfile* fPECorrDiblock;
    TProfile* fMevDiblock;

    TH1D* fPlane;
    TH1D* fPlane_X;
    TH1D* fPlane_Y;
    TProfile* fPECorrPlane;
    TProfile* fPECorrPlane_X;
    TProfile* fPECorrPlane_Y;
    TProfile* fMevPlane;
    TProfile* fMevPlane_X;
    TProfile* fMevPlane_Y;

    TH1D* fPathLength;
    TH1D* fPathLength_X;
    TH1D* fPathLength_Y;
    TProfile* fPECorrPath;
    TProfile* fPECorrPath_X;
    TProfile* fPECorrPath_Y;
    TProfile* fMevPath;
    TProfile* fMevPath_X;
    TProfile* fMevPath_Y;

    TH1D* fW;
    TH1D* fW_X;
    TH1D* fW_Y;
    TProfile* fPECorrW;
    TProfile* fPECorrW_X;
    TProfile* fPECorrW_Y;
    TProfile* fMevW;
    TProfile* fMevW_X;
    TProfile* fMevW_Y;

    TH2D* fPathW_X;
    TH2D* fPathW_Y;
    TProfile2D* fPECorrPathW_X;
    TProfile2D* fPECorrPathW_Y;
    TProfile2D* fMeVPathW_X;
    TProfile2D* fMeVPathW_Y;

    TH1D* fCell_X;
    TH1D* fCell_Y;
    TProfile* fPECorrCell_X;
    TProfile* fPECorrCell_Y;
    TProfile* fMevCell_X;
    TProfile* fMevCell_Y;

    TH3D* fWvsCell_PECorr_0;
    TH3D* fWvsCell_PECorr_1;
    TH3D* fWvsCell_PE_0;
    TH3D* fWvsCell_PE_1;
    TH3D* fWvsCell_MEV_0;
    TH3D* fWvsCell_MEV_1;
    TH2D* fWvsCell_0;
    TH2D* fWvsCell_1;
    
    TH2D* fdPEdx;
    TH1D* fTrackEndDist;
    TH1D* fTrackEndDistCustom;
    TH1D* fFlightLength;
    TH1D* fTrackLength;
    TH1D* fTrackLengthCustom;

    TH1D* fStopPlane;
    TH1D* fStopPlane_liveGeomCut;
    TH1D* fTrackEndPlane;
    TH1D* ftrackEndPlaneWithinDiblock;

    TH1D* fDistToFront_Stop;
    TH1D* fDistToBack_Stop;

    TH1D* fTNS;
    TH1D* fRun;
    TH1D* fEventTime;
    TProfile* fPECorrTNS;
    TProfile* fPECorrRun;
    TProfile* fPECorrEventTime;

    // track level plots:
    TH1D* fWindowTricellsPerTrack;
    TH1D* fWindowHitsPerTrack;
    TH1D* fWindowPlanesPerTrack;
    TH1D* fWindowViewsPerTrack;
    TH1D* fWindowOneViewTracks;

    TH2D* fWinTricellsHitsPerTrack;
    TH2D* fWinTricellsPlanesPerTrack;
    TH2D* fWinHitsPlanesPerTrack;
    TH2D* fWinTricellsViewsPerTrack;

    TH1D* fDCosX;
    TH1D* fDCosY;
    TH1D* fDCosZ;
    TProfile* fRespDCosX;
    TProfile* fRespDCosY;
    TProfile* fRespDCosZ;

    TH1D* fEvtTime;
    TProfile* fDetRespEvtTime;

    TH1D* fDeltaStartPlane;
    TH1D* fDeltaEndPlane;
    TH1D* fPlaneAsymmetry;
    TH2D* fPlaneAsymmetry2D;

    TTree* fTree;

    int diblock;
    int view;
    double FlightLength;
    double TotalTrackLength;
    double x;
    double xCust;
    double plane;
    double path;
    double trueE;
    double PE;
    double PECorr;
    double tns;
    uint32_t EventTimeHigh;
    double cell;
    double w;
    double PECorrToGeV;
    double RecoGeV;
    double distanceToLiveDetEdgeZ;
    int run;
    int evt_year;
    int evt_month;
    int evt_day;
    int evt_hour;
    int evt_min;
    int evt_sec;
    int APD;
    int sameAPDhitsEvent;
    int sameAPDhitDiblockEvent;
    int nhits;
    int nXYHits;
    bool badTNS;
    double dCosX;
    double dCosY;
    double dCosZ;

    std::string fPCHitLabel;
    std::string fQualXYName;
    std::string fQualZName;
    std::string fQualAvgName;
    std::string fStopperLabel;

    double fSysShift;

    // *time_t* of start of Nova epoch, 01-Jan-2010 00:00:00, UTC
    // This is the value subtracted from the UNIX time_t, timeval or timespec
    // seconds field.
    // Using unsigned long long rather than uint64_t to ensure consistent behavior
    // on OSX and LINUX
    const uint32_t           NOVA_EPOCH       = 1262304000;
    const unsigned long long NOVA_TIME_FACTOR = 64000000;

  };
  
  
  MuondEdxAna::MuondEdxAna(fhicl::ParameterSet const & p)
  {
    this->reconfigure(p);
    produces< TH2D, art::InRun >("dEdx");
    produces< TH2D, art::InRun >("dEdxMinus");
    produces< TH2D, art::InRun >("dEdxPlus");
    produces< TH2D, art::InRun >("dEdxTrue");
  }
  
  MuondEdxAna::~MuondEdxAna()
  {
    // Clean up dynamic memory and other resources here.
  }
  
  bool MuondEdxAna::filter(art::Event & e)
  {

    run = (int)(e.run());

    art::Timestamp ts = e.time();
    EventTimeHigh = ts.timeHigh();
    const time_t timeSec = ts.timeHigh();
    struct tm* timeStruct = localtime(&timeSec);
    evt_year=timeStruct->tm_year+1900;
    evt_month=timeStruct->tm_mon+1;
    evt_day=timeStruct->tm_mday;
    evt_hour=timeStruct->tm_hour+1;
    evt_min=timeStruct->tm_min;
    evt_sec=timeStruct->tm_sec;

    /// Get the reconstructed tracks                                           
    art::Handle<std::vector<rb::Track> > tracks;
    e.getByLabel(fStopperLabel, tracks);
    if( tracks->size() == 0 ) return false;

    art::FindManyP<caldp::PCHit> PCHitGetter(tracks, e, art::InputTag(fPCHitLabel,fQualXYName)); 
    art::FindManyP<caldp::PCHit> PCHitAvgGetter(tracks, e, art::InputTag(fPCHitLabel,fQualAvgName)); 
    art::FindManyP<caldp::PCHit> PCHitZGetter(tracks, e, art::InputTag(fPCHitLabel,fQualZName)); 
    if( !PCHitGetter.isValid() ) return false;    

    // loop over stopping tracks in event
    for( unsigned int i=0; i<tracks->size(); ++i ){

      //track level counters:
      int windowTricells = 0;
      int windowAvgHits = 0;
      int windowZHits = 0;
      int windowPlanes = 0;
      std::vector<bool> trackWindowPlanes (1000, false);  //vector to prevent double counting planes
            
      bool windowXView = false;
      bool windowYView = false;

      //make custom track length to check against inbuilt length
      TVector3 startXYZ = (*tracks)[i].Start();
      TVector3 stopXYZ  = (*tracks)[i].Stop();
      double trackLength = (startXYZ-stopXYZ).Mag();
      
      // needed for track window, but good generally to reject protons
      if( (*tracks)[i].TotalLength() < 200. ) continue;  

      art::ServiceHandle<geo::Geometry> fGeo;
      geo::CellUniqueId cid;
      int stopPlaneID = 10000;
      int firstMuCPlane = -1;
      bool isNearDet = false;

      //remove tracks that stop outside/ near edge of the active detector. New files won't need this as the selection is in the updated pchitslist_module.
      art::ServiceHandle<geo::LiveGeometry>   fLiveGeom;
      distanceToLiveDetEdgeZ = fLiveGeom->DistToEdgeZ((*tracks)[i].Stop());
      
      //continue if track ends in the muon catcher
      if(fGeo->DetId() == novadaq::cnv::kNEARDET){
	isNearDet = true;
	cid = fGeo->CellId(stopXYZ[0], stopXYZ[1], stopXYZ[2], 1.5, 1.5, 6., 1.0);
	if(cid) stopPlaneID = fGeo->getPlaneID(cid);
 	firstMuCPlane = fGeo->FirstPlaneInMuonCatcher();
	if(stopPlaneID >= firstMuCPlane) continue;
      }

      art::ServiceHandle<geo::LiveGeometry> livegeom;
      fDistToFront_Stop -> Fill(livegeom->DistToFront(stopXYZ));
      fDistToBack_Stop  -> Fill(livegeom->DistToBack(stopXYZ));

      // loop over PCHits associated to stopping track
      std::vector<art::Ptr<caldp::PCHit> > hits = PCHitGetter.at(i);
      std::vector<art::Ptr<caldp::PCHit> > avghits = PCHitAvgGetter.at(i);
      std::vector<art::Ptr<caldp::PCHit> > zhits = PCHitZGetter.at(i);

      nhits = hits.size() + avghits.size() + zhits.size();

      // ----------------  Fill vector containing hits per APD  ---------------------- //
      std::vector<int> APDhits (100,0);
      std::vector< std::vector<int> > DiblockAPDhits (100, std::vector<int> (100,0));
      badTNS = false; //temporary handle for showery events in old data

      // tricell hits
      if(hits.size() != 0)
	for( unsigned int n=0; n<hits.size(); ++n ){
	  APDhits[hits[n]->APD()] += 1;
	  DiblockAPDhits.at(hits[n]->Diblock()).at(hits[n]->APD()) += 1;
	  if(hits[n]->TNS() < 0 || hits[n]->TNS() > 1400) badTNS = true;
	}
      //avg. hits
      if(avghits.size() != 0)
	for( unsigned int p=0; p<avghits.size(); ++p ) {
	  APDhits[avghits[p]->APD()] += 1;
	  DiblockAPDhits.at(avghits[p]->Diblock()).at(avghits[p]->APD()) += 1;
	  if(avghits[p]->TNS() < 0 || avghits[p]->TNS() > 1400) badTNS = true; 
	  FlightLength     = avghits[p]->FlightLen();
	  TotalTrackLength = (*tracks)[i].TotalLength();
	  x	        = trackLength - FlightLength;
	  if(x<100 || x>200) continue;
	  windowAvgHits ++; 	  //track level counters
	}
      //z hits
      if(zhits.size() != 0)
	for( unsigned int q=0; q<zhits.size(); ++q ){
	  APDhits[zhits[q]->APD()] += 1;
	  DiblockAPDhits.at(zhits[q]->Diblock()).at(zhits[q]->APD()) += 1;
	  if(zhits[q]->TNS() < 0 || zhits[q]->TNS() > 1400) badTNS = true; 
	  FlightLength     = zhits[q]->FlightLen();
	  TotalTrackLength = (*tracks)[i].TotalLength();
	  x	        = trackLength - FlightLength;
	  if(x<100 || x>200) continue;
	  windowZHits ++; 	  //track level counters
	}
      // ----------------  Filled APD vector  -------------------------------------------- //

      //cosine giving initial direction of track
      dCosX = (*tracks)[i].Dir().X()/(*tracks)[i].Dir().Mag();
      dCosY = (*tracks)[i].Dir().Y()/(*tracks)[i].Dir().Mag();
      dCosZ = (*tracks)[i].Dir().Z()/(*tracks)[i].Dir().Mag();

      for( unsigned int j=0; j<hits.size(); ++j ){

	if(isNearDet)
	  if(hits[j]->Plane() >= firstMuCPlane) continue;

	//test poisson lambda:
	double poissonLambda = hits[j]->PoissonLambda();
	std::cout<< "poissonLambda: " << poissonLambda <<std::endl;

	nXYHits         = hits.size();
	diblock	        = hits[j]->Diblock();
	view	        = (int)hits[j]->View();
	FlightLength    = hits[j]->FlightLen();
	TotalTrackLength= (*tracks)[i].TotalLength();
	x	        = trackLength - FlightLength;
	plane	        = hits[j]->Plane();
	path	        = hits[j]->Path();
	trueE	        = hits[j]->TrueMeV();
	PE	        = hits[j]->PE();
	PECorr	        = this->getPECorr(e,*hits[j]);  
	PECorrToGeV     = this->getPECorrToGeV(e,*hits[j]);  
	RecoGeV         = PECorr * PECorrToGeV;
	tns	        = hits[j]->TNS();
	cell	        = hits[j]->Cell();
	w	        = hits[j]->W();
	APD             = hits[j]->APD();
	sameAPDhitsEvent= APDhits[APD];
	sameAPDhitDiblockEvent = DiblockAPDhits[diblock][APD];

	fTree->Fill(); //fill ttree

	fdPEdx	   ->Fill(x,PE/path);
	fdEdx	   ->Fill(x,PECorr/path);
	fdEdx_minus->Fill(x-fSysShift,PECorr/path);
	fdEdx_plus ->Fill(x+fSysShift,PECorr/path);
	fdEdx_true ->Fill(x,trueE/path);

	if(w > 0){
	  fdEdx_NearReadout->Fill(x,PECorr/path);
	  fdEdx_true_NearReadout->Fill(x,trueE/path);
	}

	if(x<100 || x>200) continue;

	//track level counters
	windowTricells ++;
	if(view == 0) windowXView = true;
	if(view == 1) windowYView = true;
	//check if plane already counted
	if(trackWindowPlanes[plane] == false) windowPlanes ++;
	trackWindowPlanes[plane] = true;
	
	fillRespVar(PECorr, PE, trueE, RecoGeV, path, w, plane, cell, view, tns, EventTimeHigh, run, dCosX, dCosY, dCosZ, diblock);
	fillLowLevel(PECorr, PE, trueE, RecoGeV, path, w, plane, cell, view, diblock, x, trackLength, FlightLength, TotalTrackLength, dCosX, dCosY, dCosZ);
	
      }

      fWindowTricellsPerTrack		 -> Fill(windowTricells);
      fWindowPlanesPerTrack		 -> Fill(windowPlanes);
      fWindowHitsPerTrack		 -> Fill(windowTricells + windowAvgHits + windowZHits);
      fWinTricellsHitsPerTrack		 -> Fill(windowTricells,windowTricells + windowAvgHits + windowZHits);
      fWinTricellsPlanesPerTrack	 -> Fill(windowTricells,windowPlanes);
      fWinHitsPlanesPerTrack		 -> Fill(windowTricells + windowAvgHits + windowZHits, windowPlanes);

      int nViewsInWindow = 0;
      if(windowXView == true) nViewsInWindow++;
      if(windowYView == true) nViewsInWindow++;

      fWinTricellsViewsPerTrack -> Fill(windowTricells,nViewsInWindow);
      fWindowViewsPerTrack      -> Fill(nViewsInWindow);
      if(nViewsInWindow == 1){
	if(windowXView) fWindowOneViewTracks -> Fill(0);
	if(windowYView) fWindowOneViewTracks -> Fill(1);
      }
    }

    return true;
  }

  void MuondEdxAna::fillLowLevel(double PECorr, double PE, double trueE, double RecoGeV, double path, double w, int plane, int cell, 
				 int view, int diblock, double x, double trackLength, double FlightLength, double TotalTrackLength, 
				 double dCosX, double dCosY, double dCosZ){

    fDCosX	 -> Fill(dCosX);
    fDCosY	 -> Fill(dCosY);
    fDCosZ	 -> Fill(dCosZ);
    fDiblock	 -> Fill(diblock);
    fPECorr	 -> Fill(PECorr/path);
    fPE		 -> Fill(PE/path);
    fTrueE	 -> Fill(trueE/path);
    fRecoGeV	 -> Fill(1000 * RecoGeV/path);

    if(view == 0){
      fPECorr_X -> Fill(PECorr/path);
      fPE_X	-> Fill(PE/path);
      fTrueE_X	-> Fill(trueE/path);
    }

    if(view == 1){
      fPECorr_Y -> Fill(PECorr/path);
      fPE_Y	-> Fill(PE/path);
      fTrueE_Y	-> Fill(trueE/path);
    }

    fTrackEndDist      -> Fill(x);    
    fTrackEndDistCustom-> Fill(trackLength - FlightLength);
    fFlightLength      -> Fill(FlightLength);
    fTrackLength       -> Fill(TotalTrackLength);
    fTrackLengthCustom -> Fill(trackLength);

  }

  void MuondEdxAna::fillRespVar(double PECorr, double PE, double trueE, double RecoGeV, double path, double w, 
				int plane, int cell, int view, double tns, double EventTimeHigh, int run, double dCosX,
				double dCosY, double dCosZ, int diblock){

    fPathLength	    -> Fill(path);
    fPlane	    -> Fill(plane);
    fW		    -> Fill(w);
    fPECorrPlane    -> Fill(plane, PECorr/path);
    fMevPlane	    -> Fill(plane, trueE/path);
    fPECorrW	    -> Fill(w, PECorr/path);
    fMevW	    -> Fill(w, trueE/path);
    fPECorrPath	    -> Fill(path,PECorr/path);
    fMevPath	    -> Fill(path,trueE/path);
    fPECorrDiblock  -> Fill(diblock, PECorr/path);
    fMevDiblock     -> Fill(diblock, trueE/path);
    fTNS            -> Fill(tns);
    fPECorrTNS      -> Fill(tns,PECorr/path);
    fRun            -> Fill(run);
    fPECorrRun      -> Fill(run,PECorr/path);
    fEventTime      -> Fill(EventTimeHigh);
    fPECorrEventTime-> Fill(EventTimeHigh,PECorr/path);
    fRespDCosX -> Fill(dCosX, PECorr/path);
    fRespDCosY -> Fill(dCosY, PECorr/path);
    fRespDCosZ -> Fill(dCosZ, PECorr/path);

    if(view == 0){
      fPathW_X	        -> Fill(w, path);
      fPECorrPathW_X    -> Fill(w, path,PECorr/path);
      fMeVPathW_X       -> Fill(w, path,trueE/path);
      fPathLength_X     -> Fill(path); 
      fPECorrPath_X     -> Fill(path,PECorr/path);
      fMevPath_X        -> Fill(path,trueE/path);
      fPlane_X	        -> Fill(plane);
      fW_X	        -> Fill(w);
      fPECorrPlane_X    -> Fill(plane, PECorr/path);
      fMevPlane_X       -> Fill(plane, trueE/path);
      fPECorrW_X        -> Fill(w, PECorr/path);
      fMevW_X	        -> Fill(w, trueE/path);
      fCell_X	        -> Fill(cell);
      fPECorrCell_X     -> Fill(cell, PECorr/path);
      fMevCell_X        -> Fill(cell, trueE/path);
      fWvsCell_PECorr_0 -> Fill(cell, w, PECorr/path);
      fWvsCell_MEV_0    -> Fill(cell, w, trueE/path);
      fWvsCell_0        -> Fill(cell, w);

    }

    if(view == 1){
      fPathW_Y         -> Fill(w, path);
      fPECorrPathW_Y   -> Fill(w, path,PECorr/path);
      fMeVPathW_Y      -> Fill(w, path,trueE/path);
      fPathLength_Y    -> Fill(path); 
      fPECorrPath_Y    -> Fill(path,PECorr/path);
      fMevPath_Y       -> Fill(path,trueE/path);
      fPlane_Y	       -> Fill(plane);
      fW_Y	       -> Fill(w);
      fPECorrPlane_Y   -> Fill(plane, PECorr/path);
      fMevPlane_Y      -> Fill(plane, trueE/path);
      fPECorrW_Y       -> Fill(w, PECorr/path);
      fMevW_Y	       -> Fill(w, trueE/path);
      fCell_Y	       -> Fill(cell);
      fPECorrCell_Y    -> Fill(cell, PECorr/path);
      fMevCell_Y       -> Fill(cell, trueE/path);
      fWvsCell_PECorr_1-> Fill(cell, w, PECorr/path);
      fWvsCell_MEV_1   -> Fill(cell, w, trueE/path);
      fWvsCell_1       -> Fill(cell, w);

    }

  }

  double MuondEdxAna::getPECorr(art::Event & e, caldp::PCHit const &pchit){
   
    //Calibrator necessary to get Atten information                             
    art::ServiceHandle<calib::Calibrator> calibrator;

    //Make mock cellhit
    rb::CellHit cellhit;
    cellhit.SetPlane(pchit.Plane());
    cellhit.SetCell(pchit.Cell());
    cellhit.SetView(pchit.View());
    cellhit.SetTNS(pchit.TNS(),pchit.GoodTime());
    if(!e.isRealData())cellhit.SetMC(); // assume only filled for MC...
    cellhit.SetPE(pchit.PE());

    double pecorr = calibrator->GetPECorr(cellhit, pchit.W());
    return pecorr;
  }

  double MuondEdxAna::getPECorrToGeV(art::Event & e, caldp::PCHit const &pchit){
   
    //Calibrator necessary to get Atten information                             
    art::ServiceHandle<calib::Calibrator> calibrator;
    rb::CellHit cellhit;
    if(!e.isRealData())cellhit.SetMC(); // assume only filled for MC...

    double pecorrtogev = calibrator->GetPECorrToGeVScale(cellhit);
    return pecorrtogev;
  }

  void MuondEdxAna::reconfigure(fhicl::ParameterSet const & pset)
  {
    fPCHitLabel		 = pset.get< std::string >("PCHitLabel"); //Label of PCHitList module            
    fQualXYName		 = pset.get< std::string >("QualXYName"); //Instance label, "XY" quality hits    
    fQualZName		 = pset.get< std::string >("QualZName"); //Instance label, "Z" quality hits    
    fQualAvgName	 = pset.get< std::string >("QualAvgName"); //Instance label, "Avg" quality hits    
    fStopperLabel	 = pset.get<std::string>("StopperLabel");
    fSysShift		 = pset.get<double>("SysShift"); // track end point systematic shift
  }
  
  void MuondEdxAna::beginJob()
  {
    // histograms for output
    art::ServiceHandle<art::TFileService> tfs;

    fTree = tfs->make<TTree>("fTree","tree to hold muondedx variables");
    fTree->Branch("nXYHits",&nXYHits);
    fTree->Branch("nhits",&nhits);
    fTree->Branch("dCosX",&dCosX);
    fTree->Branch("dCosY",&dCosY);
    fTree->Branch("dCosZ",&dCosZ);
    fTree->Branch("x",&x);
    fTree->Branch("badTNS",&badTNS);
    fTree->Branch("run",&run);
    fTree->Branch("diblock",&diblock);
    fTree->Branch("view",&view);
    fTree->Branch("FlightLength",&FlightLength);
    fTree->Branch("TotalTrackLength",&TotalTrackLength);
    fTree->Branch("xCust",&xCust);
    fTree->Branch("distanceToLiveDetEdgeZ",&distanceToLiveDetEdgeZ);
    fTree->Branch("plane",&plane);
    fTree->Branch("path",&path);
    fTree->Branch("trueE",&trueE);
    fTree->Branch("PE",&PE);
    fTree->Branch("PECorr",&PECorr);
    fTree->Branch("tns",&tns);
    fTree->Branch("cell",&cell);
    fTree->Branch("w",&w);
    fTree->Branch("PECorrToGeV",&PECorrToGeV);
    fTree->Branch("RecoGeV",&RecoGeV);
    fTree->Branch("EventTimeHigh",&EventTimeHigh);
    fTree->Branch("evt_year",&evt_year);
    fTree->Branch("evt_month",&evt_month);
    fTree->Branch("evt_day",&evt_day);
    fTree->Branch("evt_hour",&evt_hour);
    fTree->Branch("evt_min",&evt_min);
    fTree->Branch("evt_sec",&evt_sec);
    fTree->Branch("APD",&APD);
    fTree->Branch("sameAPDhitsEvent",&sameAPDhitsEvent);
    fTree->Branch("sameAPDhitDiblockEvent",&sameAPDhitDiblockEvent);

    fDeltaStartPlane	 = tfs->make<TH1D>("DeltaStartPlane",";startPlane diff. (x,y);",100,0,100);
    fDeltaEndPlane	 = tfs->make<TH1D>("DeltaEndPlane",";endPlane diff. (x,y);",100,0,100);
    fPlaneAsymmetry	 = tfs->make<TH1D>("PlaneAsymmetry",";plane asymm. (x,y);",100,0,100);
    fPlaneAsymmetry2D	 = tfs->make<TH2D>("PlaneAsymmetry2D",";diff in view-planes(x-y); sum of view-planes (x+y);",100,0,100,1000,0,1000);

    fEvtTime             = tfs->make<TH1D>("EvtTime",";EvtTime (YYYYMMDD);",std::pow(10,6),0,std::pow(10,6));
    fDetRespEvtTime      = tfs->make<TProfile>("DetRespEvtTime",";EvtTime (YYYYMMDD); PECorr/cm",std::pow(10,6),0,std::pow(10,6));
    
    fDCosX       = tfs->make<TH1D>("DCosX",";dCosX;",200,-1,1);
    fDCosY       = tfs->make<TH1D>("DCosY",";dCosY;",200,-1,1);
    fDCosZ       = tfs->make<TH1D>("DCosZ",";dCosZ;",200,-1,1);
    fRespDCosX   = tfs->make<TProfile>("RespDCosX",";dCosX;",200,-1,1);
    fRespDCosY   = tfs->make<TProfile>("RespDCosY",";dCosY;",200,-1,1);
    fRespDCosZ   = tfs->make<TProfile>("RespDCosZ",";dCosZ;",200,-1,1);

    fdPEdx			 = tfs->make<TH2D>("dPEdx",";Distance to track end (cm);PE / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx			 = tfs->make<TH2D>("dEdx",";Distance to track end (cm);PECorr / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx_minus			 = tfs->make<TH2D>("dEdx_minus",";Distance to track end (cm);PECorr / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx_plus			 = tfs->make<TH2D>("dEdx_plus",";Distance to track end (cm);PECorr / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx_true			 = tfs->make<TH2D>("dEdx_true",";Distance to track end (cm);True MeV / cm;",
						  50,0.,500.,200,0.,5.);

    fdEdx_MuonCatch		 = tfs->make<TH2D>("dEdx_MuonCatch",";Distance to track end (cm);PECorr / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx_true_MuonCatch	 = tfs->make<TH2D>("dEdx_true_MuonCatch",";Distance to track end (cm);True MeV / cm;",
						  50,0.,500.,200,0.,5.);

    fdEdx_NearReadout		 = tfs->make<TH2D>("dEdx_NearReadout",";Distance to track end (cm);PECorr / cm;",
						  50,0.,500.,200,0.,200.);
    fdEdx_true_NearReadout	 = tfs->make<TH2D>("dEdx_true_NearReadout",";Distance to track end (cm);True MeV / cm;",
						  50,0.,500.,200,0.,5.);

    fPECorr	 = tfs->make<TH1D>("PECorr",";PECorr/cm;",1000,0,1000);
    fPE		 = tfs->make<TH1D>("PE",";PE/cm;",1000,0,1000);
    fTrueE	 = tfs->make<TH1D>("TrueE",";MeV/cm;",1000,0,25);    
    fRecoGeV	 = tfs->make<TH1D>("RecoGeV",";MeV/cm;",1000,0,25);    
    fPECorr_X	 = tfs->make<TH1D>("PECorr_X",";PECorr/cm;",1000,0,1000);
    fPE_X	 = tfs->make<TH1D>("PE_X",";PE/cm;",1000,0,1000);
    fTrueE_X	 = tfs->make<TH1D>("TrueE_X",";MeV/cm;",1000,0,25);    
    fPECorr_Y	 = tfs->make<TH1D>("PECorr_Y",";PECorr/cm;",1000,0,1000);
    fPE_Y	 = tfs->make<TH1D>("PE_Y",";PE/cm;",1000,0,1000);
    fTrueE_Y	 = tfs->make<TH1D>("TrueE_Y",";MeV/cm;",1000,0,25);    

    fFlightLength		 = tfs->make<TH1D>("FlightLength",";distance from track start (cm);",1000,0,10000);
    fTrackLength		 = tfs->make<TH1D>("TrackLength",";Total track length (cm);",1000,0,10000);
    fTrackEndDist		 = tfs->make<TH1D>("TrackEndDist",";Distance to end of track (cm)",1000,0,10000);
    fTrackLengthCustom		 = tfs->make<TH1D>("TrackLengthCustom",";Total track length (cm);",1000,0,10000);
    fTrackEndDistCustom		 = tfs->make<TH1D>("TrackEndDistCustom",";Distance to end of track (cm)",1000,0,10000);
    fTrackEndPlane		 = tfs->make<TH1D>("TrackEndPlane",";plane of track end;",1000,0,1000);
    fStopPlane			 = tfs->make<TH1D>("StopPlane",";start and stop z positions;",1000,0,1000);
    fStopPlane_liveGeomCut	 = tfs->make<TH1D>("StopPlane_liveGeomCut",";start and stop z positions;",1000,0,1000);
    fDistToFront_Stop		 = tfs->make<TH1D>("DistToFront_Stop",";distance to front (cm);",1000,0,1000);
    fDistToBack_Stop		 = tfs->make<TH1D>("DistToBack_Stop",";distance to back (cm);",1000,0,1000);

    fTNS		        = tfs->make<TH1D>("TNS",";TNS (nova time);",std::pow(10,3),-5*std::pow(10,4),5*std::pow(10,4));
    fPECorrTNS		        = tfs->make<TProfile>("PECorrTNS",";TNS (nova time); PECorr/cm;",std::pow(10,3),-5*std::pow(10,4),5*std::pow(10,4));
    fRun		        = tfs->make<TH1D>("Run",";Run;",10000,0,100000);
    fPECorrRun		        = tfs->make<TProfile>("PECorrRun",";Run; PECorr/cm;",10000,0,100000);

    fEventTime		        = tfs->make<TH1D>("EventTime",";date and time (seconds);",std::pow(10,6),0,std::pow(10,10));
    fPECorrEventTime	        = tfs->make<TProfile>("PECorrEventTime",";date and time (seconds); PECorr/cm;",std::pow(10,6),0,std::pow(10,10));

    fPathW_X		        = tfs->make<TH2D>("PathW_X",";W (cm); Path (cm)",160,-800,800, 100, 2, 12);
    fPathW_Y		        = tfs->make<TH2D>("PathW_Y",";W (cm); Path (cm)",160,-800,800, 100, 2, 12);
    fPECorrPathW_X	        = tfs->make<TProfile2D>("PECorrPathW_X",";W (cm); Path (cm); PECorr/cm",160,-800,800, 100, 2, 12);
    fPECorrPathW_Y	        = tfs->make<TProfile2D>("PECorrPathW_Y",";W (cm); Path (cm); PECorr/cm",160,-800,800, 100, 2, 12);
    fMeVPathW_X		        = tfs->make<TProfile2D>("MeVPathW_X",";W (cm); Path (cm); MeV/cm",160,-800,800, 100, 2, 12);
    fMeVPathW_Y		        = tfs->make<TProfile2D>("MeVPathW_Y",";W (cm); Path (cm); MeV/cm",160,-800,800, 100, 2, 12);

    fPathLength		        = tfs->make<TH1D>("PathLength",";path length (cm);",200,0,20);
    fPathLength_X	        = tfs->make<TH1D>("PathLength_X",";path length (cm);",200,0,20);
    fPathLength_Y	        = tfs->make<TH1D>("PathLength_Y",";path length (cm);",200,0,20);
    fPECorrPath		        = tfs->make<TProfile>("PECorrPath",";path length X-view (cm); PECorr/cm;",200,0,20);
    fPECorrPath_X	        = tfs->make<TProfile>("PECorrPath_X",";path length X-view (cm); PECorr/cm;",200,0,20);
    fPECorrPath_Y	        = tfs->make<TProfile>("PECorrPath_Y",";path length X-view (cm); PECorr/cm;",200,0,20);
    fMevPath		        = tfs->make<TProfile>("MevPath",";path length X-view (cm); Mev/cm;",200,0,20);
    fMevPath_X		        = tfs->make<TProfile>("MevPath_X",";path length X-view (cm); Mev/cm;",200,0,20);
    fMevPath_Y		        = tfs->make<TProfile>("MevPath_Y",";path length X-view (cm); Mev/cm;",200,0,20);

    fPlane		        = tfs->make<TH1D>("Plane",";plane;",1000,0,1000);
    fPlane_X		        = tfs->make<TH1D>("Plane_X",";plane;",1000,0,1000);
    fPlane_Y		        = tfs->make<TH1D>("Plane_Y",";plane;",1000,0,1000);
    fPECorrPlane	        = tfs->make<TProfile>("PECorrPlane",";plane; PECorr;",1000,0,1000);
    fPECorrPlane_X	        = tfs->make<TProfile>("PECorrPlane_X",";plane; PECorr;",1000,0,1000);
    fPECorrPlane_Y	        = tfs->make<TProfile>("PECorrPlane_Y",";plane; PECorr;",1000,0,1000);
    fMevPlane		        = tfs->make<TProfile>("MevPlane",";plane; Mev;",1000,0,1000);
    fMevPlane_X		        = tfs->make<TProfile>("MevPlane_X",";plane; Mev;",1000,0,1000);
    fMevPlane_Y		        = tfs->make<TProfile>("MevPlane_Y",";plane; Mev;",1000,0,1000);

    fW			        = tfs->make<TH1D>("W",";w;",160,-800,800);
    fW_X		        = tfs->make<TH1D>("W_X",";w;",160,-800,800);
    fW_Y		        = tfs->make<TH1D>("W_Y",";w;",160,-800,800);
    fPECorrW		        = tfs->make<TProfile>("PECorrW",";w; PECorr/cm;",160,-800,800);
    fPECorrW_X		        = tfs->make<TProfile>("PECorrW_X",";w; PECorr/cm;",160,-800,800);
    fPECorrW_Y		        = tfs->make<TProfile>("PECorrW_Y",";w; PECorr/cm;",160,-800,800);
    fMevW		        = tfs->make<TProfile>("MevW",";w; Mev/cm;",160,-800,800);
    fMevW_X		        = tfs->make<TProfile>("MevW_X",";w; Mev/cm;",160,-800,800);
    fMevW_Y		        = tfs->make<TProfile>("MevW_Y",";w; Mev/cm;",160,-800,800);

    fCell_X		        = tfs->make<TH1D>("Cell_X",";cell;",1000,0,1000);
    fCell_Y		        = tfs->make<TH1D>("Cell_Y",";cell;",1000,0,1000);
    fPECorrCell_X	        = tfs->make<TProfile>("PECorrCell_X",";cell; PECorr/cm;",1000,0,1000);
    fPECorrCell_Y	        = tfs->make<TProfile>("PECorrCell_Y",";cell; PECorr/cm;",1000,0,1000);
    fMevCell_X		        = tfs->make<TProfile>("MevCell_X",";cell; Mev/cm;",1000,0,1000);
    fMevCell_Y		        = tfs->make<TProfile>("MevCell_Y",";cell; Mev/cm;",1000,0,1000);

    //track level histograms
    fWindowTricellsPerTrack	 = tfs->make<TH1D>("WindowTricellsPerTrack",";tricells per track in track window;",50,0,50);
    fWindowHitsPerTrack		 = tfs->make<TH1D>("WindowHitsPerTrack",";hits per track in track window;",1000,0,1000);
    fWindowPlanesPerTrack	 = tfs->make<TH1D>("WindowPlanesPerTrack",";planes per track in track window;",1000,0,1000);
    fWindowViewsPerTrack	 = tfs->make<TH1D>("WindowViewsPerTrack",";views per track in track window;",10,0,10);
    fWindowOneViewTracks	 = tfs->make<TH1D>("WindowOneViewTracks",";View in window when only one present;",10,0,10);

    fWinTricellsHitsPerTrack     = tfs->make<TH2D>("WinTricellsHitsPerTrack",";tricells per track in track window;hits per track in window;",
						      50,0,50,60,0,60);
    fWinTricellsPlanesPerTrack   = tfs->make<TH2D>("WinTricellsPlanesPerTrack",";tricells per track in track window;planes hit per track in window;",
							50,0,50,10,0,10);
    fWinHitsPlanesPerTrack       = tfs->make<TH2D>("WinHitsPlanesPerTrack",";hits per track in window;planes per track in window",
						    60,0,60,10,0,10);
    fWinTricellsViewsPerTrack    = tfs->make<TH2D>("WinTricellsViewsPerTrack",";tricells per track in track window;views per track in window;",
						       50,0,50,4,0,4);

    fWvsCell_0		        = tfs->make<TH2D>("WvsCell_0",";Cell;W;",
						  40,0.,400.,160,-800.,800.);
    fWvsCell_1		        = tfs->make<TH2D>("WvsCell_1",";Cell;W;",
						  40,0.,400.,160,-800.,800.);
    fWvsCell_PECorr_0	        = tfs->make<TH3D>("WvsCell_PECorr_0",";Cell;W;",
						  40,0.,400.,160,-800.,800.,100,0.,100.);
    fWvsCell_PECorr_1	        = tfs->make<TH3D>("WvsCell_PECorr_1",";Cell;W;",
						  40,0.,400.,160,-800.,800.,100,0.,100.);
    fWvsCell_MEV_0	        = tfs->make<TH3D>("WvsCell_MEV_0",";Cell;W;",
						  40,0.,400.,160,-800.,800.,100,0.,10.);
    fWvsCell_MEV_1	        = tfs->make<TH3D>("WvsCell_MEV_1",";Cell;W;",
						  40,0.,400.,160,-800.,800.,100,0.,10.);

    fDiblock		        = tfs->make<TH1D>("Diblock","diblock",20,0,20);
    fPECorrDiblock              = tfs->make<TProfile>("PECorrDiblock",";diblock; PECorr/cm",20,0,20);
    fMevDiblock                 = tfs->make<TProfile>("MevDiblock",";diblock; MeV/cm",20,0,20);

    ftrackEndPlaneWithinDiblock = tfs->make<TH1D>("trackEndPlaneWithinDiblock", ";plane of track end within diblocks;",100,0,100);

  }
  
  bool MuondEdxAna::beginRun(art::Run &r){
    return true;
  }

  bool MuondEdxAna::endRun(art::Run &r){

    art::ServiceHandle<art::TFileService> tfs;
    std::unique_ptr<TH2D> pdEdx(tfs->make<TH2D>(*fdEdx));
    std::unique_ptr<TH2D> pdEdx_minus(tfs->make<TH2D>(*fdEdx_minus));
    std::unique_ptr<TH2D> pdEdx_plus(tfs->make<TH2D>(*fdEdx_plus));
    std::unique_ptr<TH2D> pdEdx_true(tfs->make<TH2D>(*fdEdx_true));

    r.put(std::move(pdEdx),"dEdx");
    r.put(std::move(pdEdx_minus),"dEdxMinus");
    r.put(std::move(pdEdx_plus),"dEdxPlus");
    r.put(std::move(pdEdx_true),"dEdxTrue");
    return true;
  }

  void MuondEdxAna::endJob()
  {

  }
}// end namespace
DEFINE_ART_MODULE(calib::MuondEdxAna)
