////////////////////////////////////////////////////////////////////////
// Class:       StopperFilter
// Module Type: filter
// File:        StopperFilter_module.cc
//
// Generated at Tue Nov  4 12:40:35 2014 by Brian Rebel using artmod
// from cetpkgsupport v1_07_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include <memory>

#include "Geometry/LiveGeometry.h"
#include "RecoBase/Track.h"

namespace calib {
  class StopperFilter;
}

class calib::StopperFilter : public art::EDFilter {
public:
  explicit StopperFilter(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  StopperFilter(StopperFilter const &) = delete;
  StopperFilter(StopperFilter &&) = delete;
  StopperFilter & operator = (StopperFilter const &) = delete;
  StopperFilter & operator = (StopperFilter &&) = delete;

  // Required functions.
  bool filter(art::Event & e) override;

  // Selected optional functions.
  void reconfigure(fhicl::ParameterSet const & p) override;

private:

  // Declare member data here.
  art::ServiceHandle<geo::LiveGeometry> fLiveGeo;       ///< handle to live geometry service
  std::string                           fTrackLabel;    ///< label of module making the tracks
  double                                fMinDistToEdge; ///< minimum distance to the edge of the detector for stopper
};

//------------------------------------------------------------------------------
calib::StopperFilter::StopperFilter(fhicl::ParameterSet const & p)
{
  this->reconfigure(p);

  produces< art::PtrVector<rb::Track> >();
}

//------------------------------------------------------------------------------
bool calib::StopperFilter::filter(art::Event & e)
{

  // get the tracks that were made by CosmicTrack
  art::Handle< std::vector<rb::Track> > trks;
  e.getByLabel(fTrackLabel,trks);

  std::unique_ptr< art::PtrVector<rb::Track> > trackcol(new art::PtrVector<rb::Track>);

  // loop over the tracks and store art::Ptrs to the stoppers
  // if there are no stoppers, return false
  for(size_t t = 0; t < trks->size(); ++t){
    if(fLiveGeo->DistToEdgeXY((*trks)[t].Stop()) > fMinDistToEdge &&
       fLiveGeo->DistToEdgeZ ((*trks)[t].Stop()) > fMinDistToEdge ) 
      trackcol->push_back(art::Ptr<rb::Track>(trks, t));
  }
  
  if(trackcol->size() < 1) return false;

  e.put(std::move(trackcol));

  return true;
}

//------------------------------------------------------------------------------
void calib::StopperFilter::reconfigure(fhicl::ParameterSet const & p)
{
  fTrackLabel    = p.get< std::string >("TrackLabel",    "windowtrack");
  fMinDistToEdge = p.get< double      >("MinDistToEdge", 50.);
}


DEFINE_ART_MODULE(calib::StopperFilter)
