#include "TCanvas.h"
#include "TF1.h"
#include "TFile.h"
#include "TH2.h"
#include "TLegend.h"
#include "TLine.h"
#include "TProfile.h"
#include "TString.h"
#include "TTree.h"

const int kStatLimit = 10000000; // 2 minutes

void plot(TTree* tr, TString var, int view, TString xlabel, int nx, double x0, double x1)
{
  new TCanvas;

  const TString viewStr = view ? "Y view" : "X view";
  TH2* axes = new TH2F("", viewStr+", true deposits >15MeV;"+xlabel+";Reco / True", nx, x0, x1, 100, 0, 2);
  axes->Draw();

  TLine* one = new TLine(x0, 1, x1, 1);
  one->SetLineWidth(2);
  one->SetLineStyle(7);
  one->Draw();

  const TString hName = TString::Format("h%s%d", var.Data(), view);
  TH2* h = new TH2F(hName, ";"+xlabel+";Reco / True", nx, x0, x1, 100, 0, 5);
  tr->Draw("gev/trueE:"+var+">>"+hName,
	   TString::Format("view==%d && gev>0"" && trueE>15e-3", view), "goff", kStatLimit);

  h->Write(TString::Format("th2_%s_%s", var.Data(), view ? "Y" : "X"));

  TProfile* p = h->ProfileX();
  p->SetLineColor(kBlue);
  p->SetMarkerColor(kBlue);
  p->Draw("same");

  p->Write(TString::Format("prof_%s_%s", var.Data(), view ? "Y" : "X"));

  TH2* hu = new TH2F(hName+"_uncal", ";"+xlabel+";Uncalibrated / True", nx, x0, x1, 100, 0, 5);
  tr->Draw("5e-5*pe/trueE:"+var+">>"+hName+"_uncal",
	   TString::Format("view==%d && gev>0"" && trueE>15e-3", view), "goff", kStatLimit);

  hu->Write(TString::Format("th2_uncorr_%s_%s", var.Data(), view ? "Y" : "X"));

  TProfile* pu = hu->ProfileX();
  pu->SetLineColor(kRed);
  pu->SetMarkerColor(kRed);
  pu->Draw("same");

  pu->Write(TString::Format("prof_uncorr_%s_%s", var.Data(), view ? "Y" : "X"));

  gPad->Print(TString::Format("plots/assess_vs_%s_%s_nd.eps", var.Data(), view ? "Y" : "X"));

  if(var == "W"){
    new TCanvas;
    for(int i = 0; i < 5; ++i){
      TH1* proj = h->ProjectionY("proj", (i*nx)/5, ((i+1)*nx)/5);
      proj->SetTitle(h->GetTitle());
      proj->Scale(1./proj->Integral());
      proj->SetLineColor(1+i);
      proj->DrawClone(i ? "same" : "");

      proj->Write(TString::Format("slice_%s_%d", view ? "Y" : "X", i));
    }

    gPad->Print(TString::Format("plots/assess_slices_%s_nd.eps", view ? "Y" : "X"));

    new TCanvas;
    for(int i = 0; i < 5; ++i){
      TH1* proj = hu->ProjectionY("proj", (i*nx)/5, ((i+1)*nx)/5);
      proj->SetTitle(hu->GetTitle());
      proj->Scale(1./proj->Integral());
      proj->SetLineColor(1+i);
      proj->DrawClone(i ? "same" : "");

      proj->Write(TString::Format("slice_uncorr_%s_%d_nd.eps", view ? "Y" : "X", i));
    }

    gPad->Print(TString::Format("plots/assess_slices_uncorr_%s_nd.eps", view ? "Y" : "X"));
  }
}

void plot_assess_nd()
{
  TFile* f = new TFile("assess.root");

  TTree* tr = (TTree*)f->Get("assess/tree");

  TFile* fout = new TFile("plots/plot_assess.root", "RECREATE");

  for(int view = 0; view < 2; ++view){
    plot(tr, "W", view, "W (cm)", 110, -220, +220);

    plot(tr, "cell", view, "Cell", 96, 0, 96);

    plot(tr, "plane", view, "Plane", 110, 0, 220);
  }
}
