///////////////////////////////////////////////////////////////////////////
/// \brief   A DCMOffset contains the resulting offset for a dcm needed for calibration.
/// \author  eniner@indiana.edu
/// \date    May 2014
///////////////////////////////////////////////////////////////////////////
#ifndef DCMOffset_H
#define DCMOffset_H

#include <stdint.h>

/// Calibration data products
namespace caldp
{
  class DCMOffset
  {
  public:
    DCMOffset();
    
    ~DCMOffset();

    /// Return DCM
    int         DCM    () const {return dcm;     }; 
    /// return the timing offset (ns)
    float      Offset () const {return offset;  };
    /// return uncertaint on offset (ns)
    float      Sigma  () const {return sigma;   };
    /// return number of track segments for dcm in analysis, quality measure
    int         Stats  () const {return stats;   };
    /// return start time offset is valid for
    uint32_t    StartTime () const {return sTime; };
    /// return end time offset is valid for
    uint32_t    EndTime () const {return eTime; };

    /// Set DCM
    void         SetDCM    (int aDCM)  {dcm = aDCM;     }; 
    /// Set the timing offset (ns)
    void      SetOffset (float aOffset)  {offset = aOffset;  };
    /// Set uncertaint on offset (ns)
    void      SetSigma  (float aSigma)  {sigma = aSigma;   };
    /// Set number of track segments for dcm in analysis, quality measure
    void         SetStats  (int aStats)  {stats = aStats;   };
    /// Set start time offset is valid for
    void  SetStartTime (uint32_t aStart) {sTime = aStart; };
    /// Set end time offset is valid for
    void SetEndTime (uint32_t aEnd) {eTime = aEnd; };



  protected:    
    int    dcm; ///< dcm number
    float offset; ///< dcm timing offset
    float sigma;  ///< uncertainty in offset 
    float stats;  ///< number of track segments for dcm
    uint32_t sTime; ///< start time offset is valid for
    uint32_t eTime; ///< end time offset is valid for
  };
  
} // end namespace

#endif // DCMOffset_H
