////////////////////////////////////////////////////////////////////////
/// \brief   Hold timing constants 
/// \author bckhouse@caltech.edu
/// \date
////////////////////////////////////////////////////////////////////////
#ifndef CALIB_TIMECACHE_H
#define CALIB_TIMECACHE_H

#include <map>

#include "NovaDAQConventions/DAQConventions.h"

namespace calib
{
  class TimingCache
  {
  public:

    TimingCache();
    virtual ~TimingCache();

    void SetCSVSource(const std::string& src){fCSVSource = src;}
    void UseCSVsFromUPS(bool use = true){fUseCSVsFromUPS = use;}
    void SetTag(const std::string& tag){fTag = tag;}

    double GetOffset(int dcm, int diblock, std::uint32_t time, bool isData);
    double GetSigma(int dcm,  int diblock, std::uint32_t time, bool isData);

    void SetVldTime(novadaq::cnv::DetId det, std::uint32_t minTime, std::uint32_t maxTime);

  protected:
    struct DelayRec
    {
      DelayRec() : dcmoffset(0), sigma(0), stats(0) {}
      // The values from the DB
      double dcmoffset, sigma, stats;      
    };

    void LoadFromDatabase(bool isData);

    bool fInitialized;

    std::string fCSVSource;
    bool fUseCSVsFromUPS;
    std::string fTag;

    novadaq::cnv::DetId fCurDet;
    std::uint32_t fMinTime, fMaxTime;

    std::map<int, DelayRec> fMap;
  };

} // end namespace calib

#endif
////////////////////////////////////////////////////////////////////////
