////////////////////////////////////////////////////////////////////////
// Class:       BadChanValidate
// Module Type: analyzer
// File:        BadChanValidate_module.cc
//
// Generated at Fri Jan 30 20:02:58 2015 by Kanika Sachdev using artmod
// from cetpkgsupport v1_07_00.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include <art/Framework/Services/Optional/TFileService.h>
#include "art/Framework/Core/FileBlock.h"

#include "ChannelInfo/BadChanList.h"
#include "Geometry/Geometry.h"
#include "RunHistory/service/RunHistoryService.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "DAQChannelMap/HardwareDisplay.h"
#include "RawData/RawDigit.h"

#include "TH2F.h"
#include "TFile.h"
#include "TMath.h"
#include "TTree.h"

#include <iomanip>
#include <iostream>

namespace chaninfo {
  class BadChanValidate;
}

class chaninfo::BadChanValidate : public art::EDAnalyzer {
public:
  explicit BadChanValidate(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  BadChanValidate(BadChanValidate const &) = delete;
  BadChanValidate(BadChanValidate &&) = delete;

  
  // Required functions.
  void analyze(art::Event const & e) override;

  // Selected optional functions.
  void beginRun(art::Run const & r) override;
  void beginSubRun(art::SubRun const & sr) override;
  void endSubRun(art::SubRun const & sr) override;
  void reconfigure(fhicl::ParameterSet const & p) override;

private:

  // Declare member data here.

  int fADCLowCut;
  int fADCHighCut;
  int fNOnmonReleases;

  unsigned int fRun;
  unsigned int fSubrun;

  int fNpass = 0;
  std::string fOnmonFilename;
  std::string fRawDigitLabel;

  TH2F* fDBMaskHist;
  TH2F* fOMMaskHist;
  TH2F* fDBMinusOM;

  void BadChansFromDB();
  void BadChansFromOM();
  void OnmonNameFor( std::string fname);

  novadaq::cnv::DetId fDetID;
  art::ServiceHandle<geo::Geometry> fGeom;
  
};

///////////////////////////////////////////////////////////////////

chaninfo::BadChanValidate::BadChanValidate(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p)  
{
  this->reconfigure(p);
}

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::reconfigure(fhicl::ParameterSet const & p)
{
  fADCLowCut     = p.get<int>("ADCLowCut");
  fADCHighCut    = p.get<int>("ADCHighCut");
  fOnmonFilename = p.get<std::string>("OnmonFilename");
  fRawDigitLabel = p.get<std::string>("RawDigitLabel");
}

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::beginRun(art::Run const & r)
{
  fDetID = fGeom->DetId();
}

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::beginSubRun(art::SubRun const & sr)
{

  fRun    = sr.run();
  fSubrun = sr.subRun();

  art::ServiceHandle<art::TFileService> tfs;

  char *htitledb = new char[100];
  sprintf(htitledb, "DB Channel Map For Run %d, Subrun %d;Plane;Cell", 
	  fRun, fSubrun);

  char *htitleom = new char[100];
  sprintf(htitleom, "Onmon Channel Map For Run %d, Subrun %d;Plane;Cell", 
	  fRun, fSubrun);
  
  int nplanes = fGeom->NPlanes();
  int ncells  = fGeom->Plane(0)->Ncells();

  char *hnamedb = new char[100];
  sprintf(hnamedb, "db_mask_%d_%d", fRun, fSubrun);

  char *hnameom = new char[100];
  sprintf(hnameom, "om_mask_%d_%d", fRun, fSubrun);

  fDBMaskHist  = tfs->make<TH2F>(hnamedb, htitledb,
				nplanes+1, -1, nplanes,
				ncells +1, -1, ncells);
  
  fOMMaskHist  = tfs->make<TH2F>(hnameom, htitleom,
				nplanes+1, -1, nplanes,
				ncells +1, -1, ncells);
  
}


///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::analyze(art::Event const & e)
{

  art::Handle< std::vector<rawdata::RawDigit> > rawdigs;
  e.getByLabel(fRawDigitLabel, rawdigs);
    
  if( rawdigs->empty() || fNpass > 0 )
    return;
  
  BadChansFromDB();
  BadChansFromOM();    
  fNpass += 1;
}

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::BadChansFromDB()
{
  
  art::ServiceHandle<geo::Geometry> geom;
  art::ServiceHandle<chaninfo::BadChanList> bad;

  unsigned int nplanes = fGeom->NPlanes();
  for(unsigned int iplane = 0; iplane < nplanes; ++iplane){
    unsigned int ncells  = fGeom->Plane(iplane)->Ncells();
    for(unsigned int icell = 0; icell < ncells; ++icell){
      fDBMaskHist->Fill(iplane, icell, !bad->IsBad(iplane, icell));
    }// end of loop over cells
  }// end of loop over planes
}// end of BadChansFromDB

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::BadChansFromOM()
{
  art::ServiceHandle<nova::dbi::RunHistoryService> rh;
  art::ServiceHandle<geo::Geometry> geom;

  std::cout<<"\n";
  std::cout<<"Name of OnMon File:  "<<fOnmonFilename<<std::endl;
  std::cout<<"\n";


  TFile *fOnmonFile = new TFile(fOnmonFilename.c_str(),"READ");

  // Check that we have the right onmon file
  TTree *header = (TTree*)(fOnmonFile->Get("Header"));
  assert( header->GetEntries() > 0 );

  unsigned int omrun, omsubrun;
  header->SetBranchAddress("Run", &omrun);
  header->SetBranchAddress("Subrun", &omsubrun);
  header->GetEntry(0);
  
  assert( omrun    == fRun &&
	  omsubrun == fSubrun);

  int ndcm  = rh->NDCMs();
  int ndb  = rh->NDiBlocks();
  TH2F *hitmap[ndb][ndcm];

  for(int idb = 0; idb < ndb; ++idb){
    int indcm = rh->GetDiBlock(idb, false).dcm.size();

    int rhidb = rh->GetDiBlock(idb, false).num;
    if(fDetID == novadaq::cnv::kNEARDET && idb == 3)
      indcm = 2;

    for(int idcm = 0; idcm < indcm; ++idcm){
      char hname[256];
      sprintf(hname,"PixelsRateDCM_%02d_%02d",rhidb,idcm+1);    

      fOnmonFile->cd();
      hitmap[idb][idcm] = (TH2F*)(fOnmonFile->Get(hname));
      
    }// end loop over dcms 
  }//end loop over diblock
  
  daqchannelmap::DAQChannelMap* daqcmap = 
    daqchannelmap::DAQChannelMap::getInstance(fDetID);

  daqchannelmap::HardwareDisplay hwd;
  hwd.SetupDet(fDetID);
  
  for( int idb = 0; idb < ndb ; ++idb ){

    std::vector<nova::dbi::RunHistory::DCM> dcms = rh->GetDiBlock(idb, false).dcm;
    int ndcm = dcms.size();
    int rhidb = rh->GetDiBlock(idb, false).num;
    if(fDetID == novadaq::cnv::kNEARDET && idb == 3)
      ndcm = 2;
    
    for( int idcm = 0; idcm < ndcm; ++idcm){
      
      std::vector<nova::dbi::RunHistory::FEB> febs = dcms[idcm].feb;
      // unsigned int nfebs = febs.size();
      
      // In case the subrun is short, there isn't enough
      // info to compute rates. These hists don't exist then.
      if(!hitmap[idb][idcm])
	continue;

      int nx = hitmap[idb][idcm]->GetXaxis()->GetNbins();
      int ny = hitmap[idb][idcm]->GetYaxis()->GetNbins();
      
      for (int ix=1; ix<=nx; ++ix) {
  	for (int iy=1; iy<=ny; ++iy) {

	  unsigned int ifeb, ipixel;
	  hwd.XYFEB(&ifeb,ix-1,iy-1);
	  hwd.XYPix(&ipixel,ix-1,iy-1);

  	  // map to plane and cell in detector
  	  daqchannelmap::dchan daqChan = daqcmap->encodeDChan(fDetID,rhidb,
  							      idcm+1,ifeb,
  							      ipixel);
	  // not so logical chan :/
  	  daqchannelmap::lchan logChan = daqcmap->encodeLChan(daqChan);

  	  int plane = daqcmap->getPlane(logChan);
  	  int cell = daqcmap->getCell(logChan);

  	  // check that this FEB was instrumented
  	  // bool hasAPD = ( ifeb < nfebs && febs[ifeb].hasAPD);
  	  float rate  = hitmap[idb][idcm]->GetBinContent(ix,iy);

  	  if (rate>0.) 
  	    rate = TMath::Log10(rate);
  	  else
  	    rate = -5.;

	  if(rate <= 3.5 && rate >= 0.5)
	    fOMMaskHist->Fill( plane, cell);
	  
  	}// end loop over y bins
      }// end loop over x bins

    }// end loop over dcms
  }// end loop over diblocks
   
}// end of BadChansFromOM

///////////////////////////////////////////////////////////////////

void chaninfo::BadChanValidate::endSubRun(art::SubRun const & sr)
{
  fDBMinusOM = (TH2F*)fDBMaskHist->Clone("fDBMinusOM");
  fDBMinusOM->Add(fOMMaskHist,-1);
  fDBMinusOM->GetZaxis()->SetRangeUser(-1,1);

  char *hnamedf = new char[100];
  sprintf(hnamedf, "diff_mask_%d_%d", fRun, fSubrun);
  fDBMinusOM->SetName(hnamedf);

  char *htitledf = new char[100];
  sprintf(htitledf, "DB minus Onmon Channel Map For Run %d, Subrun %d;Plane;Cell", 
	  fRun, fSubrun);
  fDBMinusOM->SetTitle(htitledf);

  art::ServiceHandle<art::TFileService> tfs;
  tfs->file().cd();
  fDBMinusOM->Write();

  int nbins = fDBMinusOM->GetSize();
  int nfill = 0;
  for(int i = 0; i < nbins ; i++){
    if(fDBMinusOM->GetBinContent(i) != 0 )
      nfill+=1;
  }

  //double nfill = fDBMinusOM->GetEntries();
  double perc = nfill*100/(double)nbins;
  
  std::cout<<"\n";
  std::cout<<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
  std::cout<<"\n";
  std::cerr<<"Difference between DB and OnMon masks : "<< std::setprecision(2) <<  std::fixed << perc << "%\n";
  std::cout<<"\n";
  std::cout<<"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
  std::cout<<"\n";

}

///////////////////////////////////////////////////////////////////



DEFINE_ART_MODULE(chaninfo::BadChanValidate)
