#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include <map>
#include <cstdlib>
#include <ctime>
#include <getopt.h>
#include <boost/filesystem.hpp>

#include "NovaDAQConventions/DAQConventions.h"
#include "DAQChannelMap/DAQChannelMap.h"

std::string detId = "";
std::string hwId = "";

//------------------------------------------------------------

void PrintUsage()
{
  std::cout << "Usage: NovaChannelConvert [options]" << std::endl;
  std::cout << "options:\n";
  std::cout << "\t -d (--detector) [detector]: REQUIRED" << std::endl;
  std::cout << "\t -e (--dcm) [db]-dcm]-[feb]-[pixel]: convert [db]-[dcm]-[feb]-[pixel] to [plane]-[cell]" << std::endl;
  std::cout << "\t -g (--geo) [plane]-[cell]: convert [plane]-[cell] to [db]-[dcm]-[feb]-[pixel]" << std::endl;
}

//------------------------------------------------------------

bool ParseCLArgs(int argc, char* argv[])
{
  if (argc == 1) {
    PrintUsage();
    exit(0);
  }

  struct option long_options[] = {
    {"detector", 0, 0, 'd'},
    {"dcm", 0, 0, 'e'},
    {"geo", 0, 0, 'g'},
    {0,0,0,0}
  };
  
  while (1) {
    int optindx;
    int c = getopt_long(argc,argv,"d:e:g:",long_options,&optindx);
        
    if (c==-1) break;
    
    switch(c) {
    case 'd':
      detId = optarg;
      break;
    case 'e':
      hwId = optarg;
      break;
    case 'g':
      hwId = optarg;
      break;
    default:
      break;
    }
  }
  
  if (detId.empty() || hwId.empty()) {
    std::cerr << "Missing arguments." << std::endl;
    PrintUsage();
    exit(0);
  }
  
  return true;
}

//------------------------------------------------------------

int main(int argc, char *argv[])
{
  ParseCLArgs(argc, argv);

  int idet = novadaq::cnv::DetInfo::GetId(detId);
  if (idet != novadaq::cnv::kFARDET && idet != novadaq::cnv::kNEARDET) {
    std::cout << "No bad channels available for " << detId << " at this time"
	      << std::endl;
    exit(0);
  }

  int ndashes=0;
  for (size_t i=0; i<hwId.length(); ++i) {
    if (hwId[i] == '-') ndashes++;
  }

  if (ndashes != 1 && ndashes != 3) {
    std::cout << ndashes << ": Invalid hardware id." << std::endl;
    PrintUsage();
    exit(0);
  }

  daqchannelmap::DAQChannelMap* daqcmap = daqchannelmap::DAQChannelMap::getInstance(idet);

  int plane, cell;
  int db, dcm, feb, pixel;

  if (ndashes == 1) {
    sscanf(hwId.c_str(),"%d-%d",&plane,&cell);
    daqchannelmap::lchan logChan = daqcmap->encodeLChan(idet,plane,cell);
    daqchannelmap::dchan daqChan = daqcmap->encodeDChan(logChan);
    
    db = daqcmap->getDiBlock(daqChan);
    dcm = daqcmap->getDCM(daqChan);
    feb = daqcmap->getFEB(daqChan);
    pixel = daqcmap->getPixel(daqChan);
    std::cout << db << "-" << dcm << "-" << feb << "-" << pixel << std::endl;
  }
  else {
    sscanf(hwId.c_str(),"%d-%d-%d-%d",&db,&dcm,&feb,&pixel);    
    daqchannelmap::dchan daqChan = daqcmap->encodeDChan(idet,db,dcm,feb,pixel);
    daqchannelmap::lchan logChan = daqcmap->encodeLChan(daqChan);
    
    plane = daqcmap->getPlane(logChan);
    cell = daqcmap->getCell(logChan);
    std::cout << plane << "-" << cell << std::endl;
  }

  return 0;
}
