///////////////////////////////////////////////////////////////////
/// \file    LeanaNt.h
/// \brief   Summary ntuple for low energy (<100 MeV) events
/// \version $Id: LeanaNt.h,v 1.3 2011-10-20 03:59:21 gsdavies Exp $
/// \author  messier@indiana.edu
//////////////////////////////////////////////////////////////////
#ifndef COMMISSIONING_LEANANT_H
#define COMMISSIONING_LEANANT_H

namespace comi {
  /// Ntuple produced by the leana module
  class LeanaNt {
  public:
    int   run;     ///< run number
    int   subrun;  ///< subrun number
    int   evt;     ///< event number
    int   iwin;    ///< timing window number inside event
    int   nx;      ///< number of hits in x view
    int   ny;      ///< number of hits in y view
    int   dcellx;  ///< range of cells in x view
    int   dcelly;  ///< range of cells in y view
    int   dplanex; ///< range of cells in x view
    int   dplaney; ///< range of cells in y view
    float compx;   ///< "completeness" in x view
    float compy;   ///< "completeness" in y view
    float x;       ///< location of collection (cm)
    float y;       ///< location of collection (cm)
    float z;       ///< location of collection (cm)
    float qx;      ///< charge of collection in x view (ADC)
    float qy;      ///< charge of collection in y view (ADC)
    float t;       ///< time of collection (usec)
    float dperp;   ///< minimum perpendicular distance to a muon track
    float tperp;   ///< time difference to muon track IDed above
    int   isstop;  ///< 1 if the above two used track end point
    float dstop;   ///< minimum distance to a muon stop point
    float tstop;   ///< time difference to muon stop point IDed above
    //
    // Each of the variables below are computed wrt. muons that are
    // not associated ("na") with the current event
    //
    float dperpna; ///< minimum perpendicular distance to a muon track
    float tperpna; ///< time difference to muon track IDed above
    int   isstona; ///< 1 if the above two used track end point
    float dstopna; ///< minimum distance to a muon stop point
    float tstopna; ///< time difference to muon stop point IDed above
  public:
    LeanaNt();
  };
} // end namespace comi
#endif
////////////////////////////////////////////////////////////////////////
