////////////////////////////////////////////////////////////////////////
// Class:       MakeNueCosRej
// Module Type: producer
// \file        MakeNueCosRej_module.cc
// \brief       A module to write out the NueCosRej object
//
// Generated at Mon Jul 28 16:46:22 2014 by Kanika Sachdev using artmod
// from cetpkgsupport v1_06_02.
////////////////////////////////////////////////////////////////////////

// art includes

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Core/FindManyP.h"
#include "art/Framework/Core/FindMany.h"
#include "art/Persistency/Common/Assns.h"

#include <memory>
#include "TMath.h"
#include "TParticlePDG.h"
#include "TLorentzVector.h"
// novasoft includes

#include "CosRej/NueCosRej.h"
#include "ShowerLID/ParticleIDAlg.h"
#include "ShowerLID/ShowerLID.h"
#include "Geometry/LiveGeometry.h"
#include "Geometry/Geometry.h"
#include "Utilities/AssociationUtil.h"

#include "RecoBase/FilterList.h"
#include "RecoBase/Prong.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Shower.h"
#include "RecoBase/Vertex.h"


namespace cosrej
{
  class MakeNueCosRej : public art::EDProducer {
  public:

    explicit MakeNueCosRej(fhicl::ParameterSet const & p);
    ~MakeNueCosRej(){};

    void reconfigure(const fhicl::ParameterSet& pset);
    void produce(art::Event & e) override;


  private:

    std::string fProngLabel;
    std::string fShowerLIDLabel;
    std::string fSliceLabel;
    std::string fProngInstance;
    std::string fVertexLabel;

    /// Particle ID alorithm (loglikelihoods and dE/dx)
    slid::ParticleIDAlg* fParticleIDAlg;
    /// FCL parameters for particle ID alorithm (loglikelihoods and dE/dx)
    fhicl::ParameterSet fParticleIDAlgPSet;

    /// Return transverse momentum fraction of the event. Calculation based on 
    /// reconstructed prongs.
    double TransMomFraction(const std::vector
			    < art::Ptr<rb::Prong> >& prongs) const;

    /// Compute transverse momentum fraction of the event. Based on showers
    /// One calculation assumes 0 mass in computing particle momentum,
    /// the other assigns mass based on which particle hypothesis has
    /// maximum summed longitudinal and transverse log-likelihood.
    void TransMomFractionShower(std::vector< const slid::ShowerLID* > shwlids,
				art::FindOneP<rb::Shower> fos,
				double & nuPtFraction,
				double & nuPt0Fraction);

    /// Return particle mass based on particle type in 
    /// \ref slid::DedxParticleType
    double Mass(const int type);
  };

  bool CompareByCalEnergy( const art::Ptr<rb::Prong> a, 
			   const art::Ptr<rb::Prong> b);

}

//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

cosrej::MakeNueCosRej::MakeNueCosRej(fhicl::ParameterSet const & p)
  :fParticleIDAlg(0),
   fParticleIDAlgPSet(p.get< fhicl::ParameterSet >("ParticleIDAlgPSet"))
{
  produces< std::vector<cosrej::NueCosRej> > ();
  produces< art::Assns< cosrej::NueCosRej, rb::Cluster> > ();
  this->reconfigure(p);
}


//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

void cosrej::MakeNueCosRej::reconfigure(fhicl::ParameterSet const & p)
{
  fSliceLabel        = p.get< std::string >("SliceLabel");
  fVertexLabel       = p.get< std::string >("VertexLabel");
  fProngLabel        = p.get< std::string >("ProngLabel");
  fProngInstance     = p.get< std::string >("ProngInstance");
  fShowerLIDLabel    = p.get< std::string >("ShowerLIDLabel");
  fParticleIDAlgPSet = p.get< fhicl::ParameterSet >("ParticleIDAlgPSet");
}

//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

void cosrej::MakeNueCosRej::produce(art::Event & evt)
{

  std::unique_ptr< std::vector< cosrej::NueCosRej > >
    nueCosRejjies(new std::vector< cosrej::NueCosRej >);

  std::unique_ptr< art::Assns< cosrej::NueCosRej, rb::Cluster > >
    sliceAssn(new art::Assns< cosrej::NueCosRej, rb::Cluster >);
  
  art::ServiceHandle<geo::LiveGeometry> livegeo;

  // get the slices in the event

  art::Handle<std::vector<rb::Cluster> > sliceHandle;
  evt.getByLabel( fSliceLabel, sliceHandle);
  std::vector<art::Ptr<rb::Cluster> > slices;
  art::fill_ptr_vector(slices, sliceHandle);

  // get things associated with slices

  art::FindManyP<rb::Prong>       
    prongAssn(sliceHandle,evt, art::InputTag(fProngLabel, fProngInstance));

  art::FindMany<slid::ShowerLID>  
    showerLidAssn(sliceHandle, evt, fShowerLIDLabel);

  art::FindManyP<rb::Vertex>      
    vertexAssn(sliceHandle, evt, fVertexLabel);

  const int nslices = slices.size();
  for( int isli = 0; isli< nslices; ++isli){
    art::Ptr<rb::Cluster> thisSlice = slices.at(isli);
    if( thisSlice->IsNoise())
      continue;

    if(!prongAssn.isValid())
      continue;

    cosrej::NueCosRej nuecosrej;

    std::vector<art::Ptr<rb::Prong>> prongs = prongAssn.at(isli);
    if(prongs.empty()) continue;

    nuecosrej.SetProngTransMom( TransMomFraction(prongs) );

    double mintopStart   = 9999;
    double minbotStart   = 9999;
    double mineastStart  = 9999;
    double minwestStart  = 9999;
    double minfrontStart = 9999;
    double minbackStart  = 9999;

    double mintopStop   = 9999;
    double minbotStop   = 9999;
    double mineastStop  = 9999;
    double minwestStop  = 9999;
    double minfrontStop = 9999;
    double minbackStop  = 9999;

    const int npng = prongs.size();
    for(int ipng = 0; ipng < npng; ++ipng){

      TVector3 start = prongs[ipng]->Start();
      TVector3 stop  = prongs[ipng]->Start() +
        (prongs[ipng]->TotalLength() * prongs[ipng]->Dir() );

      mintopStart = std::min(mintopStart, livegeo->DistToTop(start));
      mintopStop  = std::min(mintopStop,  livegeo->DistToTop(stop ));

      minbotStart = std::min(minbotStart, livegeo->DistToBottom(start));
      minbotStop  = std::min(minbotStop,  livegeo->DistToBottom(stop ));

      mineastStart = std::min(mineastStart, livegeo->DistToEdgeXMinus(start));
      mineastStop  = std::min(mineastStop,  livegeo->DistToEdgeXMinus(stop ));

      minwestStart = std::min(minwestStart, livegeo->DistToEdgeXPlus(start));
      minwestStop  = std::min(minwestStop,  livegeo->DistToEdgeXPlus(stop ));

      minfrontStart = std::min(minfrontStart, livegeo->DistToFront(start));
      minfrontStop  = std::min(minfrontStop,  livegeo->DistToFront(stop ));

      minbackStart = std::min(minbackStart, livegeo->DistToBack(start));
      minbackStop  = std::min(minbackStop,  livegeo->DistToBack(stop ));
    } // end for ipng

    nuecosrej.SetStartMinDistToTop( mintopStart );
    nuecosrej.SetStartMinDistToBottom( minbotStart );
    nuecosrej.SetStartMinDistToFront( minfrontStart );
    nuecosrej.SetStartMinDistToBack( minbackStart );
    nuecosrej.SetStartMinDistToEast( mineastStart );
    nuecosrej.SetStartMinDistToWest( minwestStart );

    nuecosrej.SetStopMinDistToTop( mintopStop );
    nuecosrej.SetStopMinDistToBottom( minbotStop );
    nuecosrej.SetStopMinDistToFront(  minfrontStop );
    nuecosrej.SetStopMinDistToBack( minbackStop );
    nuecosrej.SetStopMinDistToEast( mineastStop );
    nuecosrej.SetStopMinDistToWest(  minwestStop );

    // Find the most energetic prong
    std::sort( prongs.begin(),
               prongs.end(),
               CompareByCalEnergy);

    art::Ptr< rb::Prong> leadingProng = prongs[0];

    TVector3 start = leadingProng->Start();
    TVector3 stop  = leadingProng->Start() +
      (leadingProng->TotalLength() * leadingProng->Dir() );

    nuecosrej.SetStartDistToTop(    livegeo->DistToTop(start) );
    nuecosrej.SetStartDistToBottom( livegeo->DistToBottom(start) );
    nuecosrej.SetStartDistToFront(  livegeo->DistToFront(start) );
    nuecosrej.SetStartDistToBack(   livegeo->DistToBack(start) );
    nuecosrej.SetStartDistToEast(   livegeo->DistToEdgeXMinus(start) );
    nuecosrej.SetStartDistToWest(   livegeo->DistToEdgeXPlus(start) );

    nuecosrej.SetStopDistToTop(     livegeo->DistToTop(stop) );
    nuecosrej.SetStopDistToBottom(  livegeo->DistToBottom(stop) );
    nuecosrej.SetStopDistToFront(   livegeo->DistToFront(stop) );
    nuecosrej.SetStopDistToBack(    livegeo->DistToBack(stop) );
    nuecosrej.SetStopDistToEast(    livegeo->DistToEdgeXMinus(stop) );
    nuecosrej.SetStopDistToWest(    livegeo->DistToEdgeXPlus(stop) );

    nuecosrej.SetProngMaxX( std::max(start.X(), stop.X()) );
    nuecosrej.SetProngMaxY( std::max(start.Y(), stop.Y()) );
    nuecosrej.SetProngMaxZ( std::max(start.Z(), stop.Z()) );
    nuecosrej.SetProngMinX( std::min(start.X(), stop.X()) );
    nuecosrej.SetProngMinY( std::min(start.Y(), stop.Y()) );
    nuecosrej.SetProngMinZ( std::min(start.Z(), stop.Z()) );

    nuecosrej.SetHitsPerPlane((float)thisSlice->NCell()/
                              leadingProng->ExtentPlane());

    if(vertexAssn.isValid()){
      const std::vector<art::Ptr<rb::Vertex>> vertCol =
        vertexAssn.at(isli);
      //currently only one vertex per slice
      TVector3 vtx = vertCol[0]->GetXYZ();
      // TODO: so far as I (CJB) can see fParticleIDAlg is never initialized,
      // so presumably this code is never reached and can be removed.
      nuecosrej.SetProngDistToVtx( fParticleIDAlg->PointDoca(vtx,
                                                             start,
                                                             stop) );
    }

    if(prongs.size() > 1){
      // some cuts need the presence of two or more showers
      art::Ptr<rb::Prong> png2 = prongs[1];

      nuecosrej.SetCosAngleToNextProng( leadingProng->Dir()
                                        .Dot( png2->Dir() ) );
    }
    else{
      nuecosrej.SetCosAngleToNextProng(-5);
    }

    if( showerLidAssn.isValid() ){
      std::vector< const slid::ShowerLID* > shwlids = showerLidAssn.at(isli);
      if( shwlids.empty() )
        continue;

      sort(shwlids.begin(), shwlids.end(), slid::CompareByEnergy);

      double nuPtFraction, nuPt0Fraction;
      art::FindOneP<rb::Shower> fos(shwlids, evt, fShowerLIDLabel);
      TransMomFractionShower( shwlids, fos, nuPtFraction, nuPt0Fraction);

      nuecosrej.SetParticleShowerTransMom( nuPtFraction);
      nuecosrej.SetPhotonShowerTransMom( nuPt0Fraction);

      // No shower associated with the shwlids
      if(!fos.at(0)) continue;

      nueCosRejjies->push_back(nuecosrej);
      util::CreateAssn( *this, evt, *nueCosRejjies, thisSlice ,*sliceAssn);
    }
  }// end loop over slices

  evt.put( std::move(sliceAssn) );
  evt.put( std::move(nueCosRejjies) );

}// end of producer

//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

double cosrej::MakeNueCosRej::TransMomFraction(const std::vector< art::Ptr<rb::Prong> >& prongs) const
{
  if(prongs.empty()) return 0;
  
  TVector3 ret;
  for(const art::Ptr<rb::Prong>& prong: prongs){
    if(!prong->Is3D()) continue;
    const double w = prong->CalorimetricEnergy();
    ret += w*prong->Dir();
  }

  ret = ret.Unit();
  
  art::ServiceHandle<geo::Geometry> geom;
  const TVector3 beamDir = geom->NuMIBeamDirection();
  const double beamProj = ret.Dot(beamDir);
  
  return (ret-beamProj*beamDir).Mag();
}

//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

void cosrej::MakeNueCosRej::TransMomFractionShower(std::vector< const slid::ShowerLID* > shwlids,
						   art::FindOneP<rb::Shower> fos,
						   double & nuPtFraction,
						   double & nuPt0Fraction)
{

  art::ServiceHandle<geo::Geometry>     geom;  
      
  int nsh = shwlids.size();

  TLorentzVector nuP, nuP0;

  for(int ish =0; ish < nsh; ++ish){
    cet::maybe_ref<art::Ptr<rb::Shower> const> shwref(fos.at(ish));
    art::Ptr<rb::Shower> shw = shwref.ref();
    if(!shw)
      continue;
    
    // figure out for which particle the LL is maximum
    std::map<int, float> partLongLL  = shwlids[ish]->fPartLongLL;
    std::map<int, float> partTransLL = shwlids[ish]->fPartTransLL;
    
    int maxllType = 0;
    int maxll   = -1;
    
    for( auto const & itype : partLongLL){
      if(itype.second + partTransLL[itype.first]  > maxll){
	maxllType = itype.first;
	maxll     = itype.second + partTransLL[itype.first];
      }
    }
	
    
    TVector3 dir  = shw->Dir();
    double showerE = shwlids[ish]->ShowerEnergy();
    double mass    = Mass( maxllType );
    // if the mass is non zero, the measured energy
    // is the kineticenergy of the particle.
    // so the momentum is p = (T^2 + 2Tm )^1/2
    double momMagMass  = sqrt( (showerE*showerE) + (2*showerE*mass) ); 
    
    // If mass is 0, showerE = KE = p = E
    TLorentzVector mom0mass( dir.X()*showerE,
			     dir.Y()*showerE,
			     dir.Z()*showerE,
			     showerE);
    
    // If mass in non-zero, total E = T + m
    TLorentzVector mommass( dir.X()*momMagMass,
			    dir.Y()*momMagMass,
			    dir.Z()*momMagMass,
			    showerE+mass);
	
    nuP  += mommass;
    nuP0 += mom0mass;
  }// end loop over showerlids
  
  TVector3 numiDir     = geom->NuMIBeamDirection();
  double nu0AngleWrtBeam = nuP.Angle(numiDir);
  double nuAngleWrtBeam  = nuP.Angle(numiDir);
  
  double nuPt  = nuP.Vect().Mag()*sin(nuAngleWrtBeam);
  double nuPt0 = nuP0.Vect().Mag()*sin(nu0AngleWrtBeam);
  
  nuPtFraction  = nuPt/nuP.Vect().Mag();  
  nuPt0Fraction = nuPt0/nuP0.Vect().Mag();  
}


//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 

double cosrej::MakeNueCosRej::Mass(const int type)
{
  int pdg;
  if (type == slid::DedxParticleType::kELECTRON)
    pdg = 11;
  else if (type == slid::DedxParticleType::kPHOTON)
    pdg = 22;
  else if (type == slid::DedxParticleType::kMUON)
    pdg = 13;
  else if (type == slid::DedxParticleType::kPI0)
    pdg = 111;
  else if (type == slid::DedxParticleType::kPION)
    pdg = 211;
  else if (type == slid::DedxParticleType::kPROTON)
    pdg = 2212; 
  else if (type == slid::DedxParticleType::kNEUTRON)
    pdg = 2112;
  else
    return 0;

  TParticlePDG part(pdg);
  return part.Mass();
}

////////////////////////////////////////////////////////////////////////      

bool cosrej::CompareByCalEnergy( const art::Ptr<rb::Prong> a, 
				 const art::Ptr<rb::Prong> b)
{
  return a->CalorimetricEnergy() > b->CalorimetricEnergy();
}

//////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////// 


DEFINE_ART_MODULE(cosrej::MakeNueCosRej)
