////////////////////////////////////////////////////////////////////////
// Class:       HitsAna
// Module Type: analyzer
// File:        HitsAna_module.cc
//
// Generated at Thu Apr 10 02:56:14 2014 by Matthew Tamsett using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/CompareDAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"

#include "RawData/RawTrigger.h"

#include "art/Framework/Services/Optional/TFileService.h"
#include "TH1.h"
#include "TH2.h"
#include "TVector3.h"

// using unsigned long long rather than uint64_t to ensure consistent behavior on OSX and LINUX

//---------------------------------------------------------------------
namespace novaddt {
    class HitsAna;
}
//---------------------------------------------------------------------
class novaddt::HitsAna : public art::EDAnalyzer {
    public:
        explicit HitsAna(fhicl::ParameterSet const & p);
        virtual ~HitsAna();
        void analyze(art::Event const & e) override;
        void endJob() override;
        std::vector<novaddt::HitList> HitFormatConsolidation(art::Event const & event);
        void AnalyzeHitLists(std::vector<novaddt::HitList> hit_lists, unsigned long long event_time);
        void FillHitDensitiesInTDCWindow(std::vector<int> TDC_vals, int minTDC, int maxTDC, int window_size, int overlap, TH1F* histogram);
    private:
        // Declare member data here.
        // FHiCL parameters
        std::string  _hitsModuleLabel;   ///< label of module making the Hits
        std::string  _hitsInstanceLabel; ///< instance label making the Hits
        std::string  _hitsFormat;        ///< The format to expect the hits in, can be one of HitList, Grouped, Vector
        std::string  _rawDigitLabel;     ///< label of where to find Raw Digits
        bool         _verbose;           ///< Verbose output
        // counters
        unsigned int _nEvents = 0;
        unsigned int _nSlices = 0;
        unsigned int _nHits   = 0;
        // Plots
        TH1F*   hCutFlow;
        TH1F*   hNSlices;
        TH1F*   hNSlices_wide;
        TH1F*   hAllNHits;
        TH1F*   hAllNHitsXZ;
        TH1F*   hAllNHitsYZ;
        TH1F*   hNHits;
        TH1F*   hNHits_wide;
        TH1F*   hNHitsXZ;
        TH1F*   hNHitsYZ;
        TH1F*   hX;
        TH1F*   hY;
        TH1F*   hYZ;
        TH1F*   hXZ;
        TH1F*   hdX;
        TH1F*   hdY;
        TH1F*   hdZ;
        TH1F*   hdXUnit;
        TH1F*   hdYUnit;
        TH1F*   hdZUnit;
        TH1F*   hXMin;
        TH1F*   hXMax;
        TH1F*   hYMin;
        TH1F*   hYMax;
        TH1F*   hZMin;
        TH1F*   hZMax;
        TH1F*   hADC;
        TH1F*   hSumADC;
        TH1F*   hMeanADC;
        TH1F*   hTDC;
        TH1F*   hTDC_wide;
        TH1F*   hdTDC;
        TH1F*   hMeanTDC;
        TH1F*   hHit_max_dTDC;
        TH1F*   hHit_max_dZ;
        TH1F*   hHit_max_dX;
        TH1F*   hHit_max_dY;
        TH1F*   hHit_dTDC;
        TH1F*   hHit_dZ;
        TH1F*   hHit_dX;
        TH1F*   hHit_dY;
        TH1F*   hHitDensity_per_50TDC;
        TH1F*   hHitDensity_per_70TDC;
        TH1F*   hHitDensity_per_100TDC;
        TH2F*   hNHitsZ_vs_X;
        TH2F*   hNHitsZ_vs_Y;
};
//---------------------------------------------------------------------
novaddt::HitsAna::HitsAna(fhicl::ParameterSet const & p):
  EDAnalyzer(p)
  , _hitsModuleLabel    (p.get<std::string >("HitsModuleLabel"    ))
  , _hitsInstanceLabel  (p.get<std::string >("HitsInstanceLabel"))
  , _hitsFormat         (p.get<std::string >("HitsFormat"))
  , _rawDigitLabel      (p.get< std::string >("RawDigitLabel"))
  , _verbose            (p.get< bool >("Verbose"))
{
    std::cout << "--- novaddt::HitsAna instantiate  " << std::endl;
    std::cout << "\t HitsModuleLabel:               " << _hitsModuleLabel   << std::endl;
    std::cout << "\t HitsInstanceLabel:             " << _hitsInstanceLabel << std::endl;
    std::cout << "\t HitsFormat:                    " << _hitsFormat        << std::endl;
    std::cout << "\t RawDigitLabel:                 " << _rawDigitLabel     << std::endl;
    std::cout << "\t Verbose:                       " << _verbose           << std::endl;
    
    assert( (_hitsFormat == "HitList") ||
            (_hitsFormat == "Grouped") ||
            (_hitsFormat == "Vector")
    );
    
    art::ServiceHandle<art::TFileService> tfs;
    hCutFlow        = tfs->make<TH1F>("CutFlow",    ";Cut number;Entries",                  21,-.5,20.5);
    hNSlices        = tfs->make<TH1F>("NSlices",    ";Number of slices;Entries",            301,-.5,300.5);
    hNSlices_wide   = tfs->make<TH1F>("NSlices_wide", ";Number of slices;Entries",          201,-.5,3000.5);
    hAllNHits       = tfs->make<TH1F>("AllNHits",   ";Number of hits per event;Entries",    100,0.,200000.);
    hAllNHitsXZ     = tfs->make<TH1F>("AllNHitsXZ", ";Number of hits XZ per event;Entries", 100,0.,100000);
    hAllNHitsYZ     = tfs->make<TH1F>("AllNHitsYZ", ";Number of hits YZ per event;Entries", 100,0.,100000);
    hNHits          = tfs->make<TH1F>("NHits",      ";Number of hits;Entries",              500,-0.5,499.5);
    hNHits_wide     = tfs->make<TH1F>("NHits_wide", ";Number of hits;Entries",              100,0.,1000.);
    hNHitsXZ        = tfs->make<TH1F>("NHitsXZ",    ";Number of hits XZ;Entries",           500,-0.5,499.5);
    hNHitsYZ        = tfs->make<TH1F>("NHitsYZ",    ";Number of hits YZ;Entries",           500,-0.5,499.5);
    hX              = tfs->make<TH1F>("X",          ";X [Cell];Entries",                    401,-0.5,400.5);
    hY              = tfs->make<TH1F>("Y",          ";Y [Cell];Entries",                    401,-0.5,400.5);
    hXZ             = tfs->make<TH1F>("XZ",         ";X planes Z;Entries",                  102,-2.5,201.5);
    hYZ             = tfs->make<TH1F>("YZ",         ";Y planes Z;Entries",                  102,-2.5,201.5);
    hdX             = tfs->make<TH1F>("dX",         ";dX;Entries",                          401,-0.5,401.);
    hdY             = tfs->make<TH1F>("dY",         ";dY;Entries",                          401,-0.5,401.);
    hdZ             = tfs->make<TH1F>("dZ",         ";dZ;Entries",                          226,-2.5,449.5);
    hdXUnit         = tfs->make<TH1F>("dXUnit",     ";dX unit vector;Entries",              101,-0.01,1.01);
    hdYUnit         = tfs->make<TH1F>("dYUnit",     ";dY unit vector;Entries",              101,-0.01,1.01);
    hdZUnit         = tfs->make<TH1F>("dZUnit",     ";dZ unit vector;Entries",              101,-0.01,1.01);
    hXMin           = tfs->make<TH1F>("XMin",";Slice box x-min;Entries",                    403,-2.5,400.5);
    hXMax           = tfs->make<TH1F>("XMax",";Slice box x-max;Entries",                    403,-2.5,400.5);
    hYMin           = tfs->make<TH1F>("YMin",";Slice box y-min;Entries",                    403,-2.5,400.5);
    hYMax           = tfs->make<TH1F>("YMax",";Slice box y-max;Entries",                    403,-2.5,400.5);
    hZMin           = tfs->make<TH1F>("ZMin",";Slice box z-min;Entries",                    226,-2.5,449.5);
    hZMax           = tfs->make<TH1F>("ZMax",";Slice box z-max;Entries",                    226,-2.5,449.5);
    hADC            = tfs->make<TH1F>("ADC",        ";ADC;Entries",                         256,0.5,4096.5);
    hSumADC         = tfs->make<TH1F>("SumADC",     ";Sum ADC;Entries",                     100,0.,50000.);
    hMeanADC        = tfs->make<TH1F>("MeanADC",    ";Mean ADC;Entries",                    256,0.5,4096.5);
    hTDC            = tfs->make<TH1F>("TDC",        ";TDC of each hit in slice;Entries",    1120,-3220. ,326400.);
    hTDC_wide       = tfs->make<TH1F>("TDC_wide",   ";TDC of each hit in slice;Entries",    100, -10000.,400000.);
    hdTDC           = tfs->make<TH1F>("dTDC",       ";dTDC of hits in slice;Entries",       129,-0.5,1024.5);
    hMeanTDC        = tfs->make<TH1F>("MeanTDC",    ";Mean TDC of hits in slice;Entries",   1120,-3220.,326400);
    hHit_max_dTDC   = tfs->make<TH1F>("Hit_max_dTDC",   ";Maximum dTDC between hits in slice;Entries",  200,-0.5,199.5);
    hHit_max_dX     = tfs->make<TH1F>("Hit_max_dX",     ";Maximum dX between hits in slice;Entries",    100,-0.5,99.5);
    hHit_max_dY     = tfs->make<TH1F>("Hit_max_dY",     ";Maximum dY between hits in slice;Entries",    100,-0.5,99.5);
    hHit_max_dZ     = tfs->make<TH1F>("Hit_max_dZ",     ";Maximum dZ between hits in slice;Entries",    100,-0.5,99.5);
    hHit_dTDC       = tfs->make<TH1F>("Hit_dTDC",       ";dTDC between hits in slice;Entries",  200,-0.5,199.5);
    hHit_dX         = tfs->make<TH1F>("Hit_dX",         ";dX between hits in slice;Entries",    100,-0.5,99.5);
    hHit_dY         = tfs->make<TH1F>("Hit_dY",         ";dY between hits in slice;Entries",    100,-0.5,99.5);
    hHit_dZ         = tfs->make<TH1F>("Hit_dZ",         ";dZ between hits in slice;Entries",    100,-0.5,99.5);
    hHitDensity_per_50TDC  =  tfs->make<TH1F>("HitDensity_per_50TDC",  ";Number of hits per 50 TDC;Entries",  100,-0.5,499.5);
    hHitDensity_per_70TDC  =  tfs->make<TH1F>("HitDensity_per_70TDC",  ";Number of hits per 70 TDC;Entries",  100,-0.5,499.5);
    hHitDensity_per_100TDC =  tfs->make<TH1F>("HitDensity_per_100TDC", ";Number of hits per 100 TDC;Entries", 100,-0.5,499.5);
    
    hNHitsZ_vs_X    = tfs->make<TH2F>("NHitsZ_vs_X",";Z [plane];X [Cell];Entries",          226,-2.5,449.5,403,-2.5,400.5);
    hNHitsZ_vs_Y    = tfs->make<TH2F>("NHitsZ_vs_Y",";Z [plane];Y [Cell];Entries",          226,-2.5,449.5,403,-2.5,400.5);
}
//---------------------------------------------------------------------
novaddt::HitsAna::~HitsAna()
{
    // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
void novaddt::HitsAna::analyze(art::Event const & event)
{
    // Implementation of required member function here.
    if (_verbose) std::cout << "--- novaddt::HitsAna analyze. Event: " 
              << event.id().event() 
              << std::endl;
    _nEvents++;
    
    // Translate whichever format we get our hits list in into a vector of HitLists
    std::vector<novaddt::HitList> hit_lists = HitFormatConsolidation(event);

    // Get the event time from the raw trigger
    art::Handle< std::vector<rawdata::RawTrigger> > RawTriggers; 
    event.getByLabel(_rawDigitLabel, RawTriggers);
    unsigned long long event_time = RawTriggers->at(0).fTriggerTimingMarker_TimeStart;
    
    // Now analyse our standard format
    AnalyzeHitLists(hit_lists, event_time);
}
//---------------------------------------------------------------------
void novaddt::HitsAna::AnalyzeHitLists(std::vector<novaddt::HitList> hit_lists, unsigned long long event_time)
{
    if (_verbose){
        std::cout << "\tAnalysing hit lists\n";
        std::cout << "\tconsolidated data into " << hit_lists.size()
                  << " hit lists\n";
        //unsigned int _nHitList = 0;
        //for (auto hit_list : hit_lists){
            //std::cout << "\t\thit list[" << _nHitList
                      //<< "] size: "      << hit_list.size()
                      //<< std::endl;
            //_nHitList++;
        //} // end of loop on hit lists
    } // end of verbose
    
    // Fill histograms
    int AllNHits = 0;
    int AllNHitsXZ = 0;
    int AllNHitsYZ = 0;
    for (auto hit_list : hit_lists){
        _nSlices++;
        int NHitsXZ = 0;
        int NHitsYZ = 0;
        int minX    = 0;
        int minY    = 0;
        int minZ    = 0;
        int maxX    = 0;
        int maxY    = 0;
        int maxZ    = 0;
        int sumADC  = 0;
        int minTDC  = 0;
        int maxTDC  = 0;
        int sumTDC  = 0;
        bool first  = true;
        bool firstX = true;
        bool firstY = true;
        std::vector<novaddt::DAQHit> x_hits;
        std::vector<novaddt::DAQHit> y_hits;
        std::vector<int> TDC_vals;
        for (auto hit : hit_list){
            _nHits++;
            AllNHits++;
            // fill hit level histograms
            hADC->Fill(hit.ADC().val);
            sumADC = sumADC + hit.ADC().val;
            
            int TDC_diff = 0;
            if (hit.TDC().val<event_time){
                TDC_diff = -1*(event_time - hit.TDC().val);
            } else {
                TDC_diff = (hit.TDC().val - event_time);
            }
            TDC_vals.push_back(TDC_diff);
            hTDC->Fill(TDC_diff);
            hTDC_wide->Fill(TDC_diff);
            sumTDC = sumTDC + TDC_diff;
            
            if (first){
                minZ = hit.Plane().val;
                maxZ = hit.Plane().val;
                minTDC = TDC_diff;
                first = false;
            }
            if (TDC_diff < minTDC) minTDC = TDC_diff;
            if (TDC_diff > maxTDC) maxTDC = TDC_diff;
            if (hit.Plane().val < minZ) minZ = hit.Plane().val;
            if (hit.Plane().val > maxZ) maxZ = hit.Plane().val;
            
            if( hit.View().val == daqchannelmap::X_VIEW ){
                x_hits.push_back(hit);
                NHitsXZ++;
                AllNHitsXZ++;
                hX->Fill(hit.Cell().val);
                hXZ->Fill(hit.Plane().val);
                hNHitsZ_vs_X->Fill(hit.Plane().val, hit.Cell().val);
                if (firstX){
                    minX = hit.Cell().val;
                    maxX = hit.Cell().val;
                    firstX = false;
                }
                if (hit.Cell().val < minX) minX = hit.Cell().val;
                if (hit.Cell().val > maxX) maxX = hit.Cell().val;
            } else {
                y_hits.push_back(hit);
                NHitsYZ++;
                AllNHitsYZ++;
                hY->Fill(hit.Cell().val);
                hYZ->Fill(hit.Plane().val);
                hNHitsZ_vs_Y->Fill(hit.Plane().val, hit.Cell().val);
                if (firstY){
                    minY = hit.Cell().val;
                    maxY = hit.Cell().val;
                    firstY = false;
                }
                if (hit.Cell().val < minY) minY = hit.Cell().val;
                if (hit.Cell().val > maxY) maxY = hit.Cell().val;
            }
        } // end of loop on hits
        // fill slice level histograms
        hNHits->Fill(hit_list.size());
        hNHits_wide->Fill(hit_list.size());
        assert(int(x_hits.size())==NHitsXZ);
        assert(int(y_hits.size())==NHitsYZ);
        hNHitsXZ->Fill(NHitsXZ);
        hNHitsYZ->Fill(NHitsYZ);
        int dX = maxX - minX;
        int dY = maxY - minZ;
        int dZ = maxZ - minZ;
        hdX->Fill(dX);
        hdY->Fill(dY);
        hdZ->Fill(dZ);
        TVector3 vect(dX,dY,dZ);
        TVector3 unit = vect.Unit();
        hdXUnit->Fill(unit.X());
        hdYUnit->Fill(unit.Y());
        hdZUnit->Fill(unit.Z());
        hXMin->Fill(minX);
        hXMax->Fill(maxX);
        hYMin->Fill(minY);
        hYMax->Fill(maxY);
        hZMin->Fill(minZ);
        hZMax->Fill(maxZ);
        hSumADC->Fill(sumADC);
        hMeanADC->Fill( ((double)sumADC) / ((double)hit_list.size()) );
        int dTDC = maxTDC - minTDC;
        hdTDC->Fill(dTDC);
        hMeanTDC->Fill( ((double)sumTDC) / ((double)hit_list.size()) );
        // Fill density variables, these often rely on sorting
        // sort by TDC
        int max_dTDC = 0;
        sort(hit_list.begin(), hit_list.end(), CompareDAQHit<TDC>());
        for (unsigned int i_hit = 0; i_hit < hit_list.size(); ++i_hit){
            if (i_hit ==0) continue;
            int this_dTDC = hit_list[i_hit].TDC().val - hit_list[i_hit-1].TDC().val;
            if (this_dTDC > max_dTDC) max_dTDC = this_dTDC;
            hHit_dTDC->Fill(this_dTDC);
        }
        hHit_max_dTDC->Fill(max_dTDC);
        // TDC hit density
        // this is time consuming, so skip it if we don't need it
        sort(TDC_vals.begin(), TDC_vals.end());
        FillHitDensitiesInTDCWindow(TDC_vals, minTDC, maxTDC, 50,  10, hHitDensity_per_50TDC);
        //FillHitDensitiesInTDCWindow(TDC_vals, minTDC, maxTDC, 70,  10, hHitDensity_per_70TDC);
        //FillHitDensitiesInTDCWindow(TDC_vals, minTDC, maxTDC, 100, 10, hHitDensity_per_100TDC);
        // sort by Plane
        int max_dZ = 0;
        sort(hit_list.begin(), hit_list.end(), CompareDAQHit<Plane>());
        for (unsigned int i_hit = 0; i_hit < hit_list.size(); ++i_hit){
            if (i_hit ==0) continue;
            int this_dZ = hit_list[i_hit].Plane().val - hit_list[i_hit-1].Plane().val;
            if (this_dZ > max_dZ) max_dZ = this_dZ;
            hHit_dZ->Fill(this_dZ);
        }
        hHit_max_dZ->Fill(max_dZ);
        // sort by X Cell
        int max_dX = 0;
        sort(x_hits.begin(), x_hits.end(), CompareDAQHit<Cell>());
        for (unsigned int i_hit = 0; i_hit < x_hits.size(); ++i_hit){
            if (i_hit ==0) continue;
            int this_dX = x_hits[i_hit].Cell().val - x_hits[i_hit-1].Cell().val;
            if (this_dX > max_dX) max_dX = this_dX;
            hHit_dX->Fill(this_dX);
        }
        hHit_max_dX->Fill(max_dX);
        // sort by Y Cell
        int max_dY = 0;
        sort(y_hits.begin(), y_hits.end(), CompareDAQHit<Cell>());
        for (unsigned int i_hit = 0; i_hit < y_hits.size(); ++i_hit){
            if (i_hit ==0) continue;
            int this_dY = y_hits[i_hit].Cell().val - y_hits[i_hit-1].Cell().val;
            if (this_dY > max_dY) max_dY = this_dY;
            hHit_dY->Fill(this_dY);
        }
        hHit_max_dY->Fill(max_dY);
    } // end of loop on hit lists
    
    // fill event level histograms
    hNSlices->Fill(hit_lists.size());
    hNSlices_wide->Fill(hit_lists.size());
    hAllNHits->Fill(AllNHits);
    hAllNHitsXZ->Fill(AllNHitsXZ);
    hAllNHitsYZ->Fill(AllNHitsYZ);
}
//---------------------------------------------------------------------
std::vector<novaddt::HitList> novaddt::HitsAna::HitFormatConsolidation(art::Event const & event)
{
    std::vector<novaddt::HitList> hit_lists;
    
    if (_hitsFormat == "HitList"){
        if (_verbose) std::cout << "\tgetting single hit list" << std::endl;
        art::Handle<novaddt::HitList> hits;
        event.getByLabel(_hitsModuleLabel, _hitsInstanceLabel, hits);
        if (_verbose) std::cout << "\tgot " << hits->size() << " hits\n";
        // copy these out
        HitList frag(hits->begin(), hits->end());
        hit_lists.emplace_back(frag);
    } else if (_hitsFormat == "Grouped"){
        if (_verbose) std::cout << "\tgetting grouped hit list" << std::endl;
        art::Handle<novaddt::GroupedHitList<novaddt::TDC> > ghits;
        event.getByLabel(_hitsModuleLabel, _hitsInstanceLabel, ghits);
        if (_verbose) std::cout << "\tgot " << ghits->n_groups() << " groups\n";
        // copy these out
        for(unsigned int i=0; i<ghits->n_groups(); ++i){
            HitList frag = ghits->get(i);
            hit_lists.emplace_back(frag);
        } // end of loop over groups
    } else {
        if (_verbose) std::cout << "\tgetting vector of hit lists" << std::endl;
        art::Handle<std::vector<HitList> > hit_lists_handle;
        event.getByLabel(_hitsModuleLabel, _hitsInstanceLabel, hit_lists_handle);
        if (_verbose) std::cout << "\tgot " << hit_lists_handle->size() << " hit lists\n";
        // copy these out
        for (unsigned int i=0; i<hit_lists_handle->size(); ++i){
            HitList frag(hit_lists_handle->at(i).begin(), hit_lists_handle->at(i).end()); 
            hit_lists.emplace_back(frag);
        } // end of loop over hit lists
    } // end of hit format consolidation
    
    return hit_lists;
}
//---------------------------------------------------------------------
void novaddt::HitsAna::FillHitDensitiesInTDCWindow(std::vector<int> TDC_vals, int minTDC, int maxTDC, int window_size, int overlap, TH1F* histogram)
{
    assert(window_size > overlap);
    
    int running_TDC  = minTDC;
    int window_slide = window_size - overlap;
    
    while (running_TDC<maxTDC){
        int this_N_hits = 0;
        for(auto TDC: TDC_vals){
            if (TDC < running_TDC) continue;
            if (TDC > (running_TDC + window_size))    break;
            this_N_hits++;
        }
        histogram->Fill(this_N_hits);
        running_TDC = running_TDC + window_slide;
    } // end of while

    return;
}
//---------------------------------------------------------------------
void novaddt::HitsAna::endJob()
{
    std::cout << "--- novaddt::HitsAna endJob       " << std::endl;
    std::cout << "\tNumber of events seen:          " << _nEvents   << std::endl;
    std::cout << "\tNumber of slices seen:          " << _nSlices   << std::endl;
    std::cout << "\tNumber of hits seen:            " << _nHits     << std::endl;
    hCutFlow->SetBinContent(1, _nEvents);
    hCutFlow->SetBinContent(2, _nSlices);
    hCutFlow->SetBinContent(3, _nHits);
    
}
//---------------------------------------------------------------------
DEFINE_ART_MODULE(novaddt::HitsAna)
