#include <iostream>
#include <fstream>
#include <sys/stat.h>
#include <cstdlib>
#include <cctype>
#include <ctime>
#include <algorithm>

#ifdef NOVACMAKE
#include <Database/cxx/include/Database.h>
#include <Database/cxx/include/Column.h>
#include <Database/cxx/include/ColumnDef.h>
#include <Database/cxx/include/Util.h>
#else
#include <Database/Database.h>
#include <Database/Column.h>
#include <Database/ColumnDef.h>
#include <Database/Util.h>
#endif
using namespace std;

//************************************************************
namespace nova {
  namespace dbi {
    Column::Column(const dBColumn_t &c) :
	fModified(false)
    {
      fValue = 0;
      fType = kIntLike;//"string";
      if (c.hasColType()) {
	std::string t = c.getColType();
	if (t == "timestamp") fType = kTimeStamp;
	else if (t == "date") fType = kDateStamp;
	else if (t == "bool") fType = kBool;
	else if (t == "float" || t == "double") fType = kFloatLike;
	else if (t == "string" || t == "text") fType = kString;
	else if (t == "autoincr") fType = kAutoIncr;
      }

      if (c.hasValue()) {
	std::string vStr = c.getValue();
	if (vStr == "" || vStr=="NULL") {
	  fValue=0;
	}
	else if (fType == kBool) {
	  fValue = new char[2];
	  if (vStr == "TRUE" || vStr == "t" || vStr == "true" || 
	      vStr == "y" || vStr == "yes" || vStr == "1" || vStr == "on") 
	    fValue[0] = '1';
	  else 
	    fValue[0] = '0';
          // Null terminate
          fValue[1] = '\0';
	}
	else {
	  fValue = new char[16];
	  strncpy(fValue,vStr.c_str(),16);
	  fValue[15]='\0';
	}
      }
    }
    
    //************************************************************

    Column::Column(const ColumnDef &c) :
	fModified(false)
    {
      fValue = 0;
      fType = kIntLike;

      std::string t = c.Type();
      if (t == "timestamp") fType = kTimeStamp;
      else if (t == "date") fType = kDateStamp;
      else if (t == "bool") fType = kBool;
      else if (t == "float" || t == "double") fType = kFloatLike;
      else if (t == "string" || t == "text") fType = kString;
      else if (t == "autoincr") fType = kAutoIncr;

    }
    
    //************************************************************
    
    Column::Column(const Column& c)
    {
      if (!c.fValue)
	fValue=0;
      else {
	fValue = new char[strlen(c.fValue)+1];
	strcpy(fValue,c.fValue);
      }
      fType = c.fType;
      fModified = c.fModified;

    }

    //************************************************************
    
    Column::~Column()
    {
      if (fValue) delete[] fValue;
    }
    
    //************************************************************
    void Column::Clear() 
    {
      if (fValue) {
	delete fValue;
	fValue = 0;
      }
      //      fIsNull = true;
      fModified = false; 
    }
    
    //************************************************************
    bool Column::operator >= (const Column& c) const
    {
      if (c.Type() != fType) return false;
      
      std::string val1 = fValue;
      std::string val2 = c.fValue;
      
      if (fType == kBool) {
	bool a = (val1 == "1");
	bool b = (val2 == "1");
	return ((a == b) || (a&!b));
      }
      
      if (fType == kIntLike) {
	long a = boost::lexical_cast<long>(val1);
	long b = boost::lexical_cast<long>(val2);
	return (a >= b);
      }
      
      if (fType == kFloatLike) {
	double a = boost::lexical_cast<double>(val1);
	double b = boost::lexical_cast<double>(val2);
	return (a >= b);
      }
      
      if (fType == kString) 
	return (val1 >= val2);
      
      if (fType == kTimeStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::TimeAsStringToTime_t(a,tta) && 
	      Util::TimeAsStringToTime_t(b,ttb) ) 
	    return (tta >= ttb);
	}
	else 
	  return false;
      }
      
      if (fType == kDateStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::DateAsStringToTime_t(a,tta) && 
	      Util::DateAsStringToTime_t(b,ttb) ) 
	    return (tta >= ttb);
	}
	else 
	  return false;
      }
      return false;
      
    }
    
    //************************************************************
    bool Column::operator > (const Column& c) const
    {
      if (c.Type() != fType) return false;
      
      std::string val1 = fValue;
      std::string val2 = c.fValue;
      
      if (fType == kBool) {
	bool a = (val1 == "1");
	bool b = (val2 == "1");
	return (a&!b);
      }
      
      if (fType == kIntLike) {
	long a = boost::lexical_cast<long>(val1);
	long b = boost::lexical_cast<long>(val2);
	return (a > b);
      }
      
      if (fType == kFloatLike) {
	double a = boost::lexical_cast<double>(val1);
	double b = boost::lexical_cast<double>(val2);
	return (a > b);
      }
      
      if (fType == kString) 
	return (val1 > val2);
      
      if (fType == kTimeStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::TimeAsStringToTime_t(a,tta) && 
	      Util::TimeAsStringToTime_t(b,ttb) ) 
	    return (tta > ttb);
	}
	else 
	  return false;
      }
      
      if (fType == kDateStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::DateAsStringToTime_t(a,tta) && 
	      Util::DateAsStringToTime_t(b,ttb) ) 
	    return (tta > ttb);
	}
	else 
	  return false;
      }
      return false;
      
    }
    
    //************************************************************
    bool Column::operator <= (const Column& c) const
    {
      if (c.fType != fType) return false;
      
      std::string val1 = fValue;
      std::string val2 = c.fValue;
      
      if (fType == kBool) {
	bool a = (val1 == "1");
	bool b = (val2 == "1");
	return ((a == b) || (!a&b));
      }
      
      if (fType == kIntLike) {
	long a = boost::lexical_cast<long>(val1);
	long b = boost::lexical_cast<long>(val2);
	return (a <= b);
      }
      
      if (fType == kFloatLike) {
	double a = boost::lexical_cast<double>(val1);
	double b = boost::lexical_cast<double>(val2);
	return (a <= b);
      }
      
      if (fType == kString) 
	return (val1 <= val2);
      
      if (fType == kTimeStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::TimeAsStringToTime_t(a,tta) && 
	      Util::TimeAsStringToTime_t(b,ttb) ) 
	    return (tta <= ttb);
	}
	else 
	  return false;
      }
      
      if (fType == kDateStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::DateAsStringToTime_t(a,tta) && 
	      Util::DateAsStringToTime_t(b,ttb) ) 
	    return (tta <= ttb);
	}
	else 
	  return false;
      }
      
      return false;
      
    }

    //************************************************************
    bool Column::operator < (const Column& c) const
    {
      if (c.fType != fType) return false;
      
      std::string val1 = fValue;
      std::string val2 = c.fValue;
      
      if (fType == kBool) {
	bool a = (val1 == "1");
	bool b = (val2 == "1");
	return (!a&b);
      }
      
      if (fType == kIntLike) {
	long a = boost::lexical_cast<long>(val1);
	long b = boost::lexical_cast<long>(val2);
	return (a < b);
      }
      
      if (fType == kFloatLike) {
	double a = boost::lexical_cast<double>(val1);
	double b = boost::lexical_cast<double>(val2);
	return (a < b);
      }
      
      if (fType == kString) 
	return (val1 < val2);
      
      if (fType == kTimeStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::TimeAsStringToTime_t(a,tta) && 
	      Util::TimeAsStringToTime_t(b,ttb) ) 
	    return (tta < ttb);
	}
	else 
	  return false;
      }
      
      if (fType == kDateStamp) {
	string a, b;
	if (this->Get(a) && c.Get(b)) {
	  time_t tta, ttb;
	  if (Util::DateAsStringToTime_t(a,tta) && 
	      Util::DateAsStringToTime_t(b,ttb) ) 
	    return (tta < ttb);
	}
	else 
	  return false;
      }
      
      return false;
      
    }
    //************************************************************
    bool Column::operator == (const Column& c) const
    {
      if (c.fType != fType) return false;

      if (fValue == 0 && (fValue==c.fValue)) return true;

      return (strcmp(fValue,c.fValue)==0);
      
    }
  }
}
