////////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  bckhouse@hep.caltech.edu
////////////////////////////////////////////////////////////////////////

#include <string>

// Framework includes
#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"

// NOvASoft includes
#include "MCCheater/BackTracker.h"
#include "Simulation/ParticleNavigator.h"


namespace tut
{
  class TutFilter : public art::EDFilter
  {
  public:
    explicit TutFilter(const fhicl::ParameterSet& pset);
    ~TutFilter();

    bool filter(art::Event& evt);

    void reconfigure(const fhicl::ParameterSet& pset);

    void beginJob();

  protected:
    std::string fGeantLabel;
  };
}

////////////////////////////////////////////////////////////////////////
namespace tut
{
  //.......................................................................
  TutFilter::TutFilter(const fhicl::ParameterSet& pset)
  {
    reconfigure(pset);
  }

  //......................................................................
  TutFilter::~TutFilter()
  {
  }

  //......................................................................
  void TutFilter::reconfigure(const fhicl::ParameterSet& pset)
  {
    fGeantLabel = pset.get<std::string>("GeantLabel");
  }

  //......................................................................
  void TutFilter::beginJob()
  {
  }

  //......................................................................
  bool TutFilter::filter(art::Event& evt)
  {
    // get the BackTracker service and use that to obtain a 
    // ParticleNavigator
    art::ServiceHandle<cheat::BackTracker> bt;
    const sim::ParticleNavigator& nav = bt->ParticleNavigator();

    // pizero should be a primary
    for(int primIdx = 0; primIdx < nav.NumberOfPrimaries(); ++primIdx){
      const sim::Particle* pizero = nav.Primary(primIdx);
      // look for pi0's only
      if(pizero->PdgCode() != 111) continue;

      if(pizero->NumberDaughters() != 2) continue;

      // Check the daughters are what they should be
      const sim::Particle* phot0 = nav[pizero->Daughter(0)];
      if(phot0->PdgCode() != 22) continue;
      const sim::Particle* phot1 = nav[pizero->Daughter(1)];
      if(phot1->PdgCode() != 22) continue;

      // Know we have at least one 2-gamma pi0
      return true;
    }

    // Didn't find any
    return false;
  }
} // end namespace tut

////////////////////////////////////////////////////////////////////////
namespace tut
{
  DEFINE_ART_MODULE(TutFilter);
}


