////////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  shanahan@fnal.gov
/// \date    March 2012
////////////////////////////////////////////////////////////////////////
#include "DetectorPerformance/RawFebPlots.h"

#include "TH1F.h"
#include "TH1I.h"
#include "TH1D.h"

#include "art/Framework/Services/Optional/TFileService.h"

#include "DAQChannelMap/DAQChannelMapConstants.h"

namespace dprf {

  
 RawFebPlots::RawFebPlots(std::string label, std::string suffix, bool detailed,fhicl::ParameterSet const &pset):
 fLabel(label),
 fSuffix(suffix),
 hitsPerEvent(std::vector<unsigned int>(daqchannelmap::NUM_PIXELS+1)),
 hADC(std::vector<TH1F*>(daqchannelmap::NUM_PIXELS+1)),
 hADC_MZ(std::vector<TH1F*>(daqchannelmap::NUM_PIXELS+1)),
 hADC_Z(std::vector<TH1F*>(daqchannelmap::NUM_PIXELS+1)),
 hDigitsPerEvent(std::vector<TH1F*>(daqchannelmap::NUM_PIXELS+1)),
 hStats(std::vector<TH1F*>(daqchannelmap::NUM_PIXELS+1)),
 fDetailed(detailed)
 {
    /*
    unsigned int hits_per_event_n=pset.get<unsigned int>("HitsPerEvtN"); 
    float hits_per_event_l=pset.get<float>("HitsPerEvtXmin"); 
    float hits_per_event_h=pset.get<float>("HitsPerEvtXmax"); 
    */
    unsigned int hits_per_event_n=100;
    float hits_per_event_l=-0.5;
    float hits_per_event_h=99.5;

    art::ServiceHandle <art::TFileService> tfs;
    
    for (unsigned int ip=detailed ? 0 : daqchannelmap::NUM_PIXELS;
	 ip<=daqchannelmap::NUM_PIXELS; ip++) {
      std::string pixelstring,pix;
      if (ip<daqchannelmap::NUM_PIXELS)
	{
	  pixelstring=Form(", px=%2.2d",ip);
	  pix=Form("_%2.2d",ip);
	} else {
	pixelstring=" ";
	pix="_F";
      }
      
      
      std::string label_suffix(Form("%s%s_%s",
				    fLabel.c_str(),pix.c_str(),fSuffix.c_str()));
      std::string title_suffix(Form("%s%s (%s)",
				    fLabel.c_str(),pixelstring.c_str(),fSuffix.c_str()));
      hADC[ip]=tfs->make<TH1F>(Form("hADC_%s",label_suffix.c_str()),
			       Form("ADC Distribution for %s",title_suffix.c_str()),
			       100,-.5,4999.5);
      hADC_MZ[ip]=tfs->make<TH1F>(Form("hADC_MZ_%s",label_suffix.c_str()),
				  Form("ADC Distribution (Mid-Zoomed) for %s",title_suffix.c_str()),
				  100,-.5,999.5);
      hADC_Z[ip]=tfs->make<TH1F>(Form("hADC_Z_%s",label_suffix.c_str()),
				 Form("ADC Distribution (Zoomed) for %s",title_suffix.c_str()),
				 200,-.5,199.5);
      hDigitsPerEvent[ip]=tfs->make<TH1F>(Form("hDigitsPerEvent_%s",label_suffix.c_str()),
					  Form("Digits per event for %s",title_suffix.c_str()),
					  hits_per_event_n,hits_per_event_l,hits_per_event_h);
    } // pixel
    
    std::string label_suffix(Form("%s_%s",
				  fLabel.c_str(),fSuffix.c_str()));
    std::string title_suffix(Form("%s (%s)",
				  fLabel.c_str(),fSuffix.c_str()));
    hSumADC=tfs->make<TH1D>(Form("hSumADC_%s",label_suffix.c_str()),
			    Form("Sum of ADC vs. Pixel, %s",title_suffix.c_str()),daqchannelmap::NUM_PIXELS+1,
			    -.5,(float) daqchannelmap::NUM_PIXELS+.5);
    hSumADC2=tfs->make<TH1D>(Form("hSumADC2_%s",label_suffix.c_str()),
			     Form("Sum of ADC^2 vs. Pixel, %s",title_suffix.c_str()),daqchannelmap::NUM_PIXELS+1,
			     -.5,(float) daqchannelmap::NUM_PIXELS+.5);
    hSumHits=tfs->make<TH1I>(Form("hSumHits_%s",label_suffix.c_str()),
			     Form("Sum of Hits vs. Pixel, %s",title_suffix.c_str()),daqchannelmap::NUM_PIXELS+1,
			     -.5,(float) daqchannelmap::NUM_PIXELS+.5);
    
    this->Reset();
    
 }
  void RawFebPlots::AddHit(float adc,unsigned int pixel) 
  {
    
    //fill for the actual pixel, plus the "33rd" pixel for the entire FEB
    
    if (fDetailed) {
      hitsPerEvent[pixel]++;
      hADC[pixel]->Fill(adc);
      hADC_MZ[pixel]->Fill(adc);
      hADC_Z[pixel]->Fill(adc);
    }
    
    hitsPerEvent[daqchannelmap::NUM_PIXELS]++;
    hADC[daqchannelmap::NUM_PIXELS]->Fill(adc);
    hADC_MZ[daqchannelmap::NUM_PIXELS]->Fill(adc);
    hADC_Z[daqchannelmap::NUM_PIXELS]->Fill(adc);
    
    hSumADC->Fill(pixel,adc);
    hSumADC2->Fill(pixel,adc*adc);
    hSumHits->Fill(pixel);
    
    hSumADC->Fill(daqchannelmap::NUM_PIXELS,adc);
    hSumADC2->Fill(daqchannelmap::NUM_PIXELS,adc*adc);
    hSumHits->Fill(daqchannelmap::NUM_PIXELS);
    
  } //AddHit
  
  void RawFebPlots::Reset() {
    for (unsigned int ip=fDetailed ? 0 : daqchannelmap::NUM_PIXELS;
	 ip<=daqchannelmap::NUM_PIXELS; ip++) {
      hitsPerEvent[ip]=0;
    }
  } //Reset
  
  void RawFebPlots::FinishEvent() 
  {
    for (unsigned int ip=fDetailed ? 0 : daqchannelmap::NUM_PIXELS;
	 ip<=daqchannelmap::NUM_PIXELS; ip++) {
      hDigitsPerEvent[ip]->Fill(hitsPerEvent[ip]);
      
    }
  }
  
}//namespace
