////////////////////////////////////////////////////////////////////////
// \file    Chain.h
// \version $Id: Chain.h,v 1.15 2012-10-31 00:05:29 bckhouse Exp $
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef DTCHAIN_H
#define DTCHAIN_H

#include <list>

#include "art/Persistency/Common/PtrVector.h"

#include "DiscreteTracker/Types.h"
#include "DiscreteTracker/View.h" // To get ChunkMap

#include "GeometryObjects/CellGeo.h"
#include "Geometry/Geometry.h"

namespace rb{class CellHit; class Track;}

namespace dt
{
  class Cand;
  class Chunk;
  class Segment;

  /// Collection of overlapping Cands forming a 2D track
  class Chain
  {
  public:
    Chain() {}
    Chain(const Cand& c);
    rb::Track ToTrack() const;
    std::vector<rb::Track> ToDebugTracks() const;
    void Add(const Cand& c);
    void Add(const Cand& c, Direction dir);
    void AddFront(const Cand& c);
    unsigned int FirstPlane() const;
    unsigned int LastPlane() const;
    unsigned int ExtremalPlane(Direction dir) const;
    unsigned int ExtremalHitPlane(Direction dir) const;
    unsigned int ExtentPlane() const {return LastPlane()-FirstPlane();}
    unsigned int NCands() const {return fCands.size();}
    const Cand& FirstCand() const {return fCands.front();}
    const Cand& LastCand() const {return fCands.back();}
    const Cand& ExtremalCand(Direction dir) const
    {
      if(dir == kUpstream) return FirstCand();
      return LastCand();
    }
    const Chunk& FirstChunk() const;
    const Chunk& LastChunk() const;
    const Segment& FirstSeg() const;
    const Segment& LastSeg() const;
    const Segment& ExtremalSeg(Direction dir) const
    {
      if(dir == kUpstream) return FirstSeg();
      return LastSeg();
    }
    art::PtrVector<rb::CellHit> AllHits() const;
    std::vector<Chunk> AllChunks() const;
    int NHitPlanes() const;

    /// Remove Cands that add no actual hits. After building but before writing
    void TrimEnds();

    // Truncates up to but not including \a plane
    void Truncate(unsigned int plane, Direction end);

    void Add(const Chain& c, Direction dir)
    {
      if(dir == kDownstream)
        fCands.insert(fCands.end(), c.fCands.begin(), c.fCands.end());
      else
        fCands.insert(fCands.begin(), c.fCands.begin(), c.fCands.end());
    }
    dt::View::ChunkMap AsChunkMap() const;
  protected:
    std::list<Cand> fCands;
  };

  bool compareByLength(const Chain& a, const Chain& b);
  bool compareByStart(const Chain& a, const Chain& b);
  bool compareByEnd(const Chain& a, const Chain& b);
} // namespace

#endif
