////////////////////////////////////////////////////////////////////////
// \file    View.h
// \version $Id: View.h,v 1.13 2012-10-31 00:05:29 bckhouse Exp $
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef DTVIEW_H
#define DTVIEW_H

#include "art/Persistency/Common/PtrVector.h"

#include "DiscreteTracker/Channel.h"
#include "DiscreteTracker/Chunk.h"
#include "DiscreteTracker/Types.h"

#include <set>
#include <vector>

namespace dt
{
  /// Representation of the channels in one view
  class View
  {
  public:
    View(art::PtrVector<rb::CellHit> chits, bool considerBadChans);
    geo::View_t GeoView() const {return fGeoView;}
    typedef std::map<int, std::vector<Chunk> > ChunkMap;
    ChunkMap MakeChunks() const;
    std::set<unsigned int> GetPlanes() const {return fPlanes;}
    // -1 on failure
    int PrevPlane(unsigned int plane) const;
    int NextPlane(unsigned int plane) const;
    int AdjacentPlane(unsigned int plane, Direction dir) const;
    // Convert to physical plane and cell number. Provide padding outside det
    dt::Channel Channel(int plane, int cell) const;
    dt::Channel& Channel(int plane, int cell);
  protected:
    // Note convention: begin <= i < end
    std::vector<Chunk> MakeChunkCombos(int plane, int begin, int end) const;

    bool Outside(int plane, int cell) const;

    geo::View_t fGeoView;

    unsigned int fMinCell; // Offset to apply to cell index
    /// Should access this via \ref Channel function
    std::vector<std::vector<dt::Channel> > fChans;

    std::set<unsigned int> fPlanes;
  };

} // namespace

#endif
