void plot(TFile* f, std::string plotname, double x0 = 0, double x1 = 0, int rebin = 0)
{
  TH1* h = (TH1*)f->Get(plotname.c_str());
  h->SetTitle(TString::Format("%s %s", plotname.c_str(), h->GetTitle()));
  if(rebin) h->Rebin(rebin);
  if(x0 || x1) h->GetXaxis()->SetRangeUser(x0, x1);
  h->Draw();
  gPad->SetLogy(false);
  gPad->Print(("plots/valid/"+plotname+".eps").c_str());

  gPad->SetLogy(true);
  gPad->Print(("plots/valid/"+plotname+"_log.eps").c_str());
}

void plot2d(TFile* f, std::string plotname, int rebin = 0)
{
  TH2* h = (TH2*)f->Get(plotname.c_str());
  if(!h){
    std::cerr << "Didn't find " << plotname << std::endl;
    //    return;
  }
  h->SetTitle(plotname.c_str());
  if(rebin) h->Rebin2D(rebin, rebin);
  h->Draw("colz");
  gPad->SetLogy(false);
  gPad->SetLogz(false);
  gPad->Print(("plots/valid/"+plotname+".eps").c_str());

  gPad->SetLogz(true);
  gPad->Print(("plots/valid/"+plotname+"_log.eps").c_str());
}

void plot_validation()
{
  gStyle->SetOptStat(1111);

  TFile* f = new TFile("dtvalidation_hist.root");

  plot(f, "kaldt/trackLength2d", 0, 200);
  plot(f, "kaldt/hitsPerTrack2d", 0, 100);
  plot(f, "kaldt/cosX2d");
  plot(f, "kaldt/cosY2d");
  plot(f, "kaldt/cosZ2d");
  plot(f, "kaldt/xPosStart", 0, 0, 5);
  plot(f, "kaldt/yPosStart", 0, 0, 5);
  plot(f, "kaldt/zPosStart", 0, 0, 10);
  plot(f, "kaldt/xPosEnd", 0, 0, 5);
  plot(f, "kaldt/yPosEnd", 0, 0, 5);
  plot(f, "kaldt/zPosEnd", 0, 0, 10);
  plot(f, "kaldt/2dTracksPerEvent");

  plot(f, "kaldtm/trackLength", 0, 1e3);
  plot(f, "kaldtm/hitsPerTrack", 0, 200);
  plot(f, "kaldtm/cosX");
  plot(f, "kaldtm/cosY");
  plot(f, "kaldtm/cosZ");
  plot(f, "kaldtm/xPosStart", 0, 0, 5);
  plot(f, "kaldtm/yPosStart", 0, 0, 5);
  plot(f, "kaldtm/zPosStart", 0, 0, 10);
  plot(f, "kaldtm/xPosEnd", 0, 0, 5);
  plot(f, "kaldtm/yPosEnd", 0, 0, 5);
  plot(f, "kaldtm/zPosEnd", 0, 0, 10);

  plot2d(f, "kaldtm/XvYStart", 5);
  plot2d(f, "kaldtm/XvYEnd", 5);
  plot2d(f, "kaldtm/ZvXReco", 5);
  plot2d(f, "kaldtm/ZvYReco", 5);
  plot2d(f, "kaldtm/ZvX", 5);
  plot2d(f, "kaldtm/ZvY", 5);

  plot(f, "kaldtm/tracksPerEvent");
  plot(f, "kaldtm/xHitsPerPlane", 0, 0, 2);
  plot(f, "kaldtm/yHitsPerPlane", 0, 0, 2);
  plot(f, "kaldtm/hitsPerPlane");
  plot(f, "kaldtm/xHitsPerCell");
  plot(f, "kaldtm/yHitsPerCell");
  plot(f, "kaldtm/xPos", 0, 0, 5);
  plot(f, "kaldtm/yPos", 0, 0, 5);
  plot(f, "kaldtm/xRecoPos");
  plot(f, "kaldtm/yRecoPos");
  plot2d(f, "kaldtm/xPlaneVCell");
  plot2d(f, "kaldtm/yPlaneVCell");
  plot2d(f, "kaldtm/planeVCell");


  plot(f, "fuzzydt/fHistoNt");
  plot(f, "fuzzydt/fHistoLen", 0, 200);
  plot(f, "fuzzydt/fHistoSZ");
  plot(f, "fuzzydt/fHistoNCells", 0, 100);
  plot(f, "fuzzydt/fHistoPECorr", 0, 1e4);

  plot(f, "fuzzydtm/fHistoNt");
  plot(f, "fuzzydtm/fHistoLen", 0, 1000);
  plot(f, "fuzzydtm/fHistoSX");
  plot(f, "fuzzydtm/fHistoSY");
  plot(f, "fuzzydtm/fHistoSZ", 0, 0, 2);
  plot(f, "fuzzydtm/fHistoDX");
  plot(f, "fuzzydtm/fHistoDY");
  plot(f, "fuzzydtm/fHistoDZ");
  plot(f, "fuzzydtm/fHistoNCells");
  plot(f, "fuzzydtm/fHistoNCellsX");
  plot(f, "fuzzydtm/fHistoNCellsY");
  plot2d(f, "fuzzydtm/fHistoNCellsXY");
  plot(f, "fuzzydtm/fHistoPECorr");
  plot2d(f, "fuzzydtm/fHistoPECorrYvX");
  plot(f, "fuzzydtm/fHistoEff");
  plot(f, "fuzzydtm/fHistoPur");
  plot(f, "fuzzydtm/fHistoAngle");
  plot(f, "fuzzydtm/fHistoMatched");
  plot(f, "fuzzydtm/fHistoUnMatched");
  plot(f, "fuzzydtm/fHistoLeft");
  plot(f, "fuzzydtm/fHistoLeptonEff");
  plot(f, "fuzzydtm/fHistoLeptonPur");
  plot(f, "fuzzydtm/fHistoLeptonAngle");
  plot2d(f, "fuzzydtm/fHistoUvL");
  plot(f, "fuzzydtm/fHistoUnVisE");
  plot(f, "fuzzydtm/fHistoMVisE");
  plot2d(f, "fuzzydtm/fHistoEffvEvis");
  plot2d(f, "fuzzydtm/fHistoPurvEvis");
  plot2d(f, "fuzzydtm/fHistoAnglevEvis");
  plot2d(f, "fuzzydtm/fHistoEffvPur");
}
