/////////////////////////////////////////////////////////////////////////////
/// \file    HardwareView.cxx
/// \brief   View the data by hardware addresses
/// \version $Id: HardwareView.cxx,v 1.2 2011-10-21 20:37:50 gsdavies Exp $
/// \author  messier@indiana.edu
/////////////////////////////////////////////////////////////////////////////
#include "TCanvas.h"

#include "EventDisplay/HardwarePad.h"
#include "EventDisplay/HardwareView.h"
#include "EventDisplay/HeaderPad.h"
#include "EventDisplay/TQPad.h"

namespace evd
{
  //................................................................
  HardwareView::HardwareView(TGMainFrame* mf) : evdb::Canvas(mf)
  {
    // X and Y locations which determine the layout
    double x0 = 0.0;
    double x1 = 0.15;
    double x3 = 1.0;
    double x2 = 0.5*(x1+x3);
  
    double y0 = 0.0;
    double y2 = 0.15;
    double y4 = 1.0;

    evdb::Canvas::fCanvas->cd();
    fHeaderPad = new HeaderPad("fHeaderPad","Header",x0,y0,x1,y2,"");
    fHeaderPad->Draw();

    evdb::Canvas::fCanvas->cd();
    fRawT = new TQPad("fTPad", "Hit Times",x1,y0,x2,y2,"T");
    fRawT->Draw();

    evdb::Canvas::fCanvas->cd();
    fRawQ = new TQPad("fQPad", "Hit Charges",x2,y0,x3,y2,"Q");
    fRawQ->Draw();

    evdb::Canvas::fCanvas->cd();
    fHardwarePad = new HardwarePad("fHardwarePad","Hardware Pad",
				   x0, y2, x3, y4, 0);
    fHardwarePad->Draw();

    evdb::Canvas::fCanvas->Update();
  }

  //......................................................................

  HardwareView::~HardwareView() 
  {
    if (fRawT)  { delete fRawT;  fRawT  = 0; }
    if (fRawQ)  { delete fRawQ;  fRawQ  = 0; }
  }
    
  //......................................................................
  
  void HardwareView::Draw(const char* opt) 
  {
    evdb::Canvas::fCanvas->cd();
  
    fHeaderPad->Draw();
  
    fRawT->Draw();
    fRawQ->Draw();
  
    fHardwarePad->Draw();
  
    evdb::Canvas::fCanvas->Update();
  }

} // end namespace evd
////////////////////////////////////////////////////////////////////////
