///
/// \file    HitTower.h
/// \brief   Plot hits (Raw, Calib, ...) as a tower in 3D
/// \author  messier@indiana.edu
/// \version $Id: HitTower.h,v 1.1.1.1 2010-11-10 19:44:54 p-novaart Exp $
///
#ifndef EVD_HITTOWER_H
#define EVD_HITTOWER_H
#include <vector>
namespace evdb { class View3D; }
namespace geo  { class Geometry; }

namespace evd {
  /// Some glue so that hits of various types call all use the HitTower class.
  class HitData 
  {
  public:
    HitData(int v, int p, int c, float tp, float zp, float q, float t);
  public:
    int   fView;  ///< Which view is the hit in? (X or Y)
    int   fPlane; ///< Which plane?
    int   fCell;  ///< Which cell?
    float fTpos;  ///< Transverse position (X or Y, cm)
    float fZpos;  ///< Longitudinal postiion (Z, cm)
    float fQ;     ///< Charge (arb. units)
    float fT;     ///< Time (arb. units)
  };
  
  /// Allow for rendering of detector hits as towers on the detector
  class HitTower 
  {
  public:
    void AddHit(int v, int p, int c, float tp, float zp, float q, float t);
    void Draw(evdb::View3D* view);
  public:
    double               fQscale;  ///< Charge scale factor
    std::vector<HitData> fHitData;
  };
}
#endif
////////////////////////////////////////////////////////////////////////
