////////////////////////////////////////////////////////////////////////
/// \file    OfflineChan.cxx
/// \brief   Simple object representing a (plane, cell) pair
/// \version $Id: OfflineChan.cxx,v 1.2 2012-09-20 21:42:43 greenc Exp $
/// \author  bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#include "GeometryObjects/OfflineChan.h"

namespace geo
{
  //....................................................................
  OfflineChan OfflineChan::FromDBValidityChan(int chan)
  {
    return OfflineChan(chan/1000, chan%1000);
  }

  //....................................................................
  int OfflineChan::ToDBValidityChan() const
  {
    // Cell can never be more than 383.
    // And this format is fairly human-readable
    return 1000*fPlane+fCell;
  }

  //....................................................................
  bool OfflineChan::operator<(const OfflineChan& rhs) const
  {
    if(fPlane < rhs.fPlane) return true;
    if(fPlane > rhs.fPlane) return false;
    return fCell < rhs.fCell;
  }

  //....................................................................
  bool OfflineChan::operator==(const OfflineChan& rhs) const
  {
    return fPlane == rhs.fPlane && fCell == rhs.fCell;
  }

  //....................................................................
  std::ostream& operator<<(std::ostream& os, const OfflineChan& chan)
  {
    os << "Plane " << chan.Plane() << " Cell " << chan.Cell();
    return os;
  }

} // namespace

////////////////////////////////////////////////////////////////////////
