////////////////////////////////////////////////////////////////////////
/// \file  OfflineChan.h
/// \brief Simple object representing a (plane, cell) pair
///
/// \version $Id: OfflineChan.h,v 1.1 2012-08-21 23:10:42 bckhouse Exp $
/// \author  bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef GEO_OFFLINECHAN_H
#define GEO_OFFLINECHAN_H

#include <ostream>

namespace geo
{
  /// A (plane, cell) pair
  class OfflineChan
  {
  public:
    OfflineChan() : fPlane(0), fCell(0) {}

#ifndef __GCCXML__
    OfflineChan(unsigned short plane, unsigned short cell)
      : fPlane(plane), fCell(cell)
    {
    }

    static OfflineChan FromDBValidityChan(int chan);
    int ToDBValidityChan() const;

    unsigned short Plane() const {return fPlane;}
    unsigned short Cell() const {return fCell;}

    bool operator==(const OfflineChan& rhs) const;
#endif
    // Allow this object to be stored in various sorted containers
    bool operator<(const OfflineChan& rhs) const;
  protected:
    unsigned short fPlane, fCell;
  };

  std::ostream& operator<<(std::ostream&, const OfflineChan&);
};

#ifndef __GCCXML__
// This enables OfflineChans to be put in unsorted_map etc
namespace std {
  template<>
  struct hash<geo::OfflineChan>
  {
  public:
    inline size_t operator()(const geo::OfflineChan& c) const
    {
      return c.ToDBValidityChan();
    }
  };
}
#endif

#endif // GEO_OFFLINECHAN_H

////////////////////////////////////////////////////////////////////////
