///////////////////////////////////////////////
/// \brief  TODO
/// \author mbaird42@fnal.gov
/// \date
/////////////////////////////////////////////
#include "HoughTransform/IslandsTH2.h"

#include "TMath.h"

namespace hough
{
  //.....................................
  IslandsTH2::IslandsTH2(const char* n,
			 const char* t,
			 const TH2* h,
			 double q, 
			 bool wrapx,
			 bool wrapy) :
    TH2S(n,
	 t,
	 h->GetNbinsX(),
	 h->GetXaxis()->GetXmin(),
	 h->GetXaxis()->GetXmax(),
	 h->GetNbinsY(),
	 h->GetYaxis()->GetXmin(),
	 h->GetYaxis()->GetXmax()),
    fPco(q),
    fWrapX(wrapx),
    fWrapY(wrapy),
    fNx(h->GetNbinsX()),
    fNy(h->GetNbinsY()),
    fNislands(0)
  { }
  
  //......................................................................
  
  void IslandsTH2::LabelIslands(const TH2* h)
  {
    
    register int a;
    register int b;
    double Pave = 0.0;     // average peak height
    double Ps = 0.0;       // standard deviation of average peak height
    double th = 0.0;       // threshold to use for peak cutoff
    int nb = 0;            // number of bins counted for avergae peak height
    
    // calculate average peak height to determine threshold
    // empty bins are NOT counted
    for (a = 1; a <= fNx; ++a) {
      for (b = 1; b <= fNy; ++b) {
	if (h->GetBinContent(a,b) != 0) {
	  Pave += h->GetBinContent(a,b);
	  Ps += (h->GetBinContent(a,b))*(h->GetBinContent(a,b));
	  nb++;
	} // end if
      } // end for b
    } // end for a
    
    Pave = Pave/nb;
    Ps = sqrt(Ps/nb - Pave*Pave);
    
    th = Pave + fPco*Ps;
    
    
    register int i;
    register int j;
    for (i=1; i<=fNx; ++i) {
      for (j=1; j<=fNy; ++j) {
	if (this->GetBinContent(i,j) == 0 && h->GetBinContent(i,j) >= th)	{
	  ++fNislands;
	  this->Mark(h,i,j,fNislands,th);
	} // end if
      } // end for j
    } // end for i
    
  } // end LabelIslands
  
  //......................................................................
  
  int IslandsTH2::Neighbor(int i, int m, int n, bool wrap) 
  {
    int ix = i+m;
    if (ix==0) {
      if (wrap) return n;
      else      return -1;
    }
    if (ix>n) {
      if (wrap) return 1;
      else      return -1;
    }
    return ix;
  }
  
  //......................................................................
  
  void IslandsTH2::Mark(const TH2* h, int i, int j, int isl_indx, double th) 
  {
    int m,  n;
    int ix, iy;
    
    this->SetBinContent(i,j,isl_indx);
    for (m=-1; m<=1; ++m) {
      ix = this->Neighbor(i,m,fNx,fWrapX);
      if (ix==-1) continue;
      for (n=-1; n<=1; ++n) {
	if (m==0 && n==0) continue;
	iy = this->Neighbor(j,n,fNy,fWrapY);
	if (iy==-1) continue;
	if (this->GetBinContent(ix,iy) == 0 && h->GetBinContent(ix,iy) >= th) {
	  this->Mark(h,ix,iy,isl_indx,th);
	}
      }
    }
  }
} // end namespace hough  
////////////////////////////////////////////////////////////////////////
