////////////////////////////////////////////////////////////////////////
/// \brief  Find event vertex using a Hough transform
/// \author Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef HV_HOUGHVERTEXALG_H
#define HV_HOUGHVERTEXALG_H

#include "TVector3.h"

#include <tuple>
#include <vector>

namespace hv
{
  struct Vector2D
  {
    Vector2D(float _z, float _v, float _dz, float _dv)
      : z(_z), v(_v), dz(_dz), dv(_dv) {}
    float z, v;
    float dz, dv;

    bool operator<(const Vector2D& rhs) const
    {
      // Compare by z first
      return (std::make_tuple(z, v, dz, dv) <
	      std::make_tuple(rhs.z, rhs.v, rhs.dz, rhs.dv));
    }
  };

  class LiteTH2;

  /// Find event vertex using a Hough transform
  class HoughVertexAlg
  {
  public:
    HoughVertexAlg();

    TVector3 FindVertex(const std::vector<Vector2D>& xhits,
                        const std::vector<Vector2D>& yhits) const;
  protected:
    /// Fill the Hough space
    void FillMaps(LiteTH2& houghx, LiteTH2& houghy,
                  const std::vector<Vector2D>& xhits,
                  const std::vector<Vector2D>& yhits) const;

    /// Search the Hough space for the highest peak
    TVector3 FindMaximum(const LiteTH2& houghx, const LiteTH2& houghy) const;
  };
} // namespace

#endif
