////////////////////////////////////////////////////////////////////////
// Class:       SupernovaAna
// Module Type: analyzer
// File:        SupernovaAna_module.cc
//
// Generated at Fri Oct 11 08:06:27 2013 by Matthew Tamsett using artmod
// from cetpkgsupport v1_02_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "RawData/RawDigit.h"
#include "RawData/RawTrigger.h"
#include "CMap/CMap.h"
#include "DAQChannelMap/DAQChannelMap.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/CellHit.h"
#include "Utilities/func/MathUtil.h"
// Root includes
#include "TH1.h"
#include "TH2.h"
//----------------------------------------------------------------------------
namespace evgen
{
    class SupernovaAna;
}
//----------------------------------------------------------------------------
class evgen::SupernovaAna : public art::EDAnalyzer
{
    public:
        explicit SupernovaAna(fhicl::ParameterSet const & p);
        virtual ~SupernovaAna();        
        void analyze(art::Event const & e) override;
        void fillPlaneSeperations(rb::Cluster slice,int & min_cell_sep, double & min_plane_sep, double & min_sep);
        void beginJob();
        void endJob();
    private:
        // Declare member data here.
        // labels
        std::string  _rawDigitModuleLabel;   ///< label of module making the raw digits
        std::string  _rawDigitInstanceLabel; ///< instance label making the raw digits
        std::string  _slicerLabel;           ///< instance label making the slices
        // options
        bool _verbose;
        // counters
        int n_events  = 0;
        // histograms
        TH1F*   hNSlices;
        TH1F*   hSlicesNHits;
        TH1F*   hSlicesSumADC;
        TH1F*   hSlicesMeanADC;
        TH1F*   hSlicesMinInPlaneSeperation;
        TH1F*   hSlicesMinCrossPlaneSeperation;
        TH1F*   hSlicesMinSeperation;
        TH1F*   hNNoiseSlices;
        TH1F*   hNoiseSlicesNHits;
        TH1F*   hNoiseSlicesSumADC;
        TH1F*   hNoiseSlicesMeanADC;
        TH1F*   hNoiseSlicesMinInPlaneSeperation;
        TH1F*   hNoiseSlicesMinCrossPlaneSeperation;
        TH1F*   hNoiseSlicesMinSeperation;
        // time dependent 1-D
        TH1F*   h_t_SumADC;
        TH1F*   h_t_Nhits;
        TH1F*   h_t_MeanADC;
        TH1F*   h_t_NSlices;
        TH1F*   h_t_SliceNHits;
        TH1F*   h_t_SliceMeanHits;
        TH1F*   h_t_NNoiseSlices;
        TH1F*   h_t_NoiseNHits;
        TH1F*   h_t_NoiseMeanHits;
        // 2-D
        TH2F*   hADC_vs_mb;
        TH2F*   hSlice_hits_vs_mb;
        TH2F*   hMeanADC_vs_Slice_hits;
        TH2F*   hNoiseSlice_hits_vs_mb;
        TH2F*   hNoiseMeanADC_vs_Slice_hits;
};
//----------------------------------------------------------------------------
evgen::SupernovaAna::SupernovaAna(fhicl::ParameterSet const & p):
  EDAnalyzer(p),
  _rawDigitModuleLabel     (p.get<std::string >("raw_digit_module_label"     "")),
  _rawDigitInstanceLabel   (p.get<std::string >("raw_digit_instance_label",  "")),
  _slicerLabel             (p.get<std::string >("slicer_label",              "")),
  _verbose                 (p.get<bool >("verbose", ""))
    
// Initialize member data here.
{
    std::cout << "--- evgen::SupernovaAna instantiate" << std::endl;
    std::cout << "\t RawDigitModuleLabel:    " << _rawDigitModuleLabel    << std::endl;
    std::cout << "\t RawDigitInstanceLabel:  " << _rawDigitInstanceLabel  << std::endl;
    std::cout << "\t SlicerLabel:            " << _slicerLabel            << std::endl;
    std::cout << "\t verbose:                " << _verbose                << std::endl;
    
    art::ServiceHandle<art::TFileService> tfs;
    hNSlices= tfs->make<TH1F>("NSlices",  ";NSlices;Entries",200,0,7000);
    hSlicesNHits= tfs->make<TH1F>("SlicesNHits",  ";SlicesNHits;Entries",200,0,200);
    hSlicesSumADC= tfs->make<TH1F>("SlicesSumADC",  ";SlicesSumADC;Entries",200,0,10000);
    hSlicesMeanADC= tfs->make<TH1F>("SlicesMeanADC",  ";SlicesMeanADC;Entries",410,0.5,4100.5);
    hSlicesMinInPlaneSeperation= tfs->make<TH1F>("SlicesMinInPlaneSeperation",  ";hSlicesMinInPlaneSeperation;;Entries",11,-0.5,10.5);
    hSlicesMinCrossPlaneSeperation= tfs->make<TH1F>("SlicesMinCrossPlaneSeperation",  ";hSlicesMinCrossPlaneSeperation;Entries",100,-0.5,10.5);
    hSlicesMinSeperation= tfs->make<TH1F>("SlicesMinSeperation",  ";hSlicesMinSeperation;Entries",100,-0.5,10.5);
    
    hNNoiseSlices= tfs->make<TH1F>("NNoiseSlices",  ";NSlices;Entries",200,0,7000);
    hNoiseSlicesNHits= tfs->make<TH1F>("NoiseSlicesNHits",  ";SlicesNHits;Entries",200,0,200);
    hNoiseSlicesSumADC= tfs->make<TH1F>("NoiseSlicesSumADC",  ";SlicesSumADC;Entries",200,0,10000);
    hNoiseSlicesMeanADC= tfs->make<TH1F>("NoiseSlicesMeanADC",  ";SlicesMeanADC;Entries",410,0.5,4100.5);
    hNoiseSlicesMinInPlaneSeperation= tfs->make<TH1F>("NoiseSlicesMinInPlaneSeperation",  ";hSlicesMinInPlaneSeperation;;Entries",11,-0.5,10.5);
    hNoiseSlicesMinCrossPlaneSeperation= tfs->make<TH1F>("NoiseSlicesMinCrossPlaneSeperation",  ";hSlicesMinCrossPlaneSeperation;Entries",100,-0.5,10.5);
    hNoiseSlicesMinSeperation= tfs->make<TH1F>("NoiseSlicesMinSeperation",  ";hSlicesMinSeperation;Entries",100,-0.5,10.5);
    
    // time dependent
    h_t_SumADC = tfs->make<TH1F>("t_SumADC",   "SumADC;Event;SumADC",  200,-.5,199.5);
    h_t_Nhits  = tfs->make<TH1F>("t_Nhits",    "Nhits;Event;Nhits",    200,-.5,199.5);
    h_t_MeanADC= tfs->make<TH1F>("t_MeanADC",  "MeanADC;Event;MeanADC",200,-.5,199.5);
    h_t_NSlices= tfs->make<TH1F>("t_NSlices",  "NSlices;Event;NSlices",200,-.5,199.5);
    h_t_SliceNHits= tfs->make<TH1F>("t_SliceNHits",  "SliceNHits;Event;NSliceHits",200,-.5,199.5);
    h_t_SliceMeanHits= tfs->make<TH1F>("t_SliceMeanHits",  "SliceMeanHits;Event;SliceMeanHits",200,-.5,199.5);
    h_t_NNoiseSlices= tfs->make<TH1F>("t_NNoiseSlices",  "NNoiseSlices;Event;NNoiseSlices",200,-.5,199.5);
    h_t_NoiseNHits= tfs->make<TH1F>("t_NoiseNHits",  "NoiseNHits;Event;NoiseNHits",200,-.5,199.5);
    h_t_NoiseMeanHits= tfs->make<TH1F>("t_NoiseMeanHits",  "NoiseMeanHits;Event;NoiseMeanHits",200,-.5,199.5);
    
    hADC_vs_mb = tfs->make<TH2F>("ADC_vs_mb", "ADC_vs_mb;mb;ADC;entries",200,-.5,199.5,410,0.5,4100.5);
    
    hSlice_hits_vs_mb = tfs->make<TH2F>("Slice_hits_vs_mb", "Slice_hits_vs_mb;mb;Slice Hits;entries",200,-.5,199.5,101,-0.5,100.5);
    hMeanADC_vs_Slice_hits = tfs->make<TH2F>("MeanADC_vs_Slice_hits", "MeanADC_vs_Slice_hits;MeanADC;Slice Hits;entries",410,0.5,4100.5,101,-0.5,100.5);
    
    hNoiseSlice_hits_vs_mb = tfs->make<TH2F>("NoiseSlice_hits_vs_mb", "NoiseSlice_hits_vs_mb;mb;Noise Slice Hits;entries",200,-.5,199.5,101,-0.5,100.5);
    hNoiseMeanADC_vs_Slice_hits = tfs->make<TH2F>("NoiseMeanADC_vs_Slice_hits", "NoiseMeanADC_vs_Slice_hits;MeanADC;Slice Hits;entries",410,0.5,4100.5,101,-0.5,100.5);
}
//----------------------------------------------------------------------------
evgen::SupernovaAna::~SupernovaAna()
{
    // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
void evgen::SupernovaAna::beginJob()
{
    std::cout << "--- evgen::SupernovaAna begin job" << std::endl;
}
//---------------------------------------------------------------------
void evgen::SupernovaAna::endJob()
{
    std::cout << "--- evgen::SupernovaAna end job" << std::endl;
    std::cout << "\tevents:                       " << n_events                   << std::endl;
}
//----------------------------------------------------------------------------
void evgen::SupernovaAna::analyze(art::Event const & event)
{
    // Implementation of required member function here.
    std::cout << "--- evgen::SupernovaAna analyzer. Event: " 
              << event.id().event() 
              << ", event counter: " << n_events
              << std::endl;
    n_events++;
    // get the RawDigit list
    art::Handle< std::vector<rawdata::RawDigit> > RawDigits;
    event.getByLabel(_rawDigitModuleLabel, _rawDigitInstanceLabel, RawDigits);
    const unsigned int nDigits = RawDigits->size();
    if (_verbose) std::cout << "\tgot " << nDigits << " raw digits" << std::endl;
    int sum_adc = 0;
    int min_TDC = 0;
    int max_TDC = 0;
    if (RawDigits->size()>0){
        min_TDC = RawDigits->at(0).TDC();
        max_TDC = RawDigits->at(0).TDC();
    }
    int min_x = 999;
    int max_x = 0;
    int min_y = 999;
    int max_y = 0;
    int min_z = 999;
    int max_z = 0;
    // loop over digits
    // Map between channel numbers and cell and plane number
    art::ServiceHandle<cmap::CMap> cmap;
    for(unsigned int iDigit=0; iDigit < nDigits; ++iDigit){
        const rawdata::RawDigit* digit = &(*RawDigits)[iDigit];
        if (digit->TDC() < min_TDC) min_TDC = digit->TDC();
        if (digit->TDC() > max_TDC) max_TDC = digit->TDC();
        const int plane = cmap->GetPlane(digit);
        if (plane < min_z) min_z = plane;
        if (plane > max_z) max_z = plane;
        const int cell  = cmap->GetCell(digit);
        //const daqchannelmap::lchan channel       = digit->Channel();
        //const daqchannelmap::dchan daqchannel    = cmap->encodeDChan(channel);
        // WARNING - THIS NEEDS TO BE CROSS CHECKED TO CONFIRM THE RIGHT
        // CHANNEL TYPE IS BEING GIVEN
        std::string view = "y";
        if (cmap->Map()->getView(digit->Channel()) == daqchannelmap::X_VIEW){
            view = "x";
            if (cell < min_x) min_x = cell;
            if (cell > max_x) max_x = cell;
        } else {
            if (cell < min_y) min_y = cell;
            if (cell > max_y) max_y = cell;
        }
        if(_verbose){
            //std::cout << "\t\t digit["     << iDigit
                      //<< "], plane: "      << plane
                      //<< ", cell: "        << cell
                      //<< ", Channel: "     << digit->Channel()        
                      //<< ", DAQ Channel: " << daqchannel
                      //<< ", nADC hits: "   << digit->NADC()
                      //<< ", nTDC hits: "   << digit->NTDC()
                      //<< std::endl;
            //for (unsigned int iADC=0; iADC<digit->NADC(); ++iADC){
                //std::cout << "\t\t\t ADC[" << iADC
                          //<< "]: "         << digit->fADC[iADC]
                          //<< std::endl;
            //} // end of loop on ADC
            //for (unsigned int iTDC=0; iTDC<digit->NTDC(); ++iTDC){
                //std::cout << "\t\t\t TDC["      << iTDC
                          //<< "]: "              << digit->TDC()
                          ////<< ", + event time: " << digit->TDC() + event_time
                          //<< std::endl;
            //} // end of loop on TDC
            //std::cout << "\t\thit["   << iDigit
                      //<< "]: TDC: "   << digit->TDC()
                      //<< ", ADC: "    << digit->fADC[0]
                      //<< ", plane: "  << plane
                      //<< ", "         << view
                      //<< "-cell: "    << cell
                      //<< std::endl;         

        } // end of verbose
        sum_adc = sum_adc + int(digit->ADC());
        hADC_vs_mb->Fill(n_events,digit->ADC());
    } // end of loop on raw digits
    if (_verbose){
        std::cout << "\tTDC: " << min_TDC
                  << " -> "    << max_TDC
                  << ", dTDC: "<< max_TDC - min_TDC
                  << std::endl;
        std::cout << "\tX: " << min_x
                  << " -> "  << max_x
                  << ", Y: " << min_y
                  << " -> "  << max_y
                  << ", Z: " << min_z
                  << " -> "  << max_z
                  <<std::endl;
        std::cout << "\tADC sum: " << sum_adc << std::endl;
    } // end of if on verbose
    h_t_SumADC->SetBinContent(n_events,sum_adc);
    h_t_Nhits->SetBinContent(n_events,nDigits);
    if (nDigits>0){ 
        h_t_MeanADC->SetBinContent(n_events,(float(sum_adc)/nDigits));
    } else {
        h_t_MeanADC->SetBinContent(n_events,0);
    }
    // get the slices
    art::Handle< std::vector< rb::Cluster > > slices;
    event.getByLabel(_slicerLabel, slices);
    
    double n_noise_slices = 0.;
    double n_noise_hits   = 0.;
    double n_hits         = 0.;
    
    for(unsigned int i = 0; i < slices->size(); ++i){
        rb::Cluster slice = slices->at(i);
        int    min_cell_sep  = 999;
        double min_plane_sep = 999;
        double min_sep       = 999;
        if(slice.IsNoise()){
            n_noise_slices++;
            n_noise_hits = n_noise_hits + slice.NCell();
            
            hNoiseSlice_hits_vs_mb->Fill(n_events,slice.NCell());
            hNoiseMeanADC_vs_Slice_hits->Fill((slice.TotalADC()/(double)slice.NCell()),slice.NCell());
            hNoiseSlicesNHits->Fill(slice.NCell());
            hNoiseSlicesSumADC->Fill(slice.TotalADC());
            hNoiseSlicesMeanADC->Fill((slice.TotalADC()/(double)slice.NCell()));
            //fillPlaneSeperations(slice, min_cell_sep, min_plane_sep, min_sep);
            hNoiseSlicesMinInPlaneSeperation->Fill(min_cell_sep);
            hNoiseSlicesMinCrossPlaneSeperation->Fill(min_plane_sep);
            hNoiseSlicesMinSeperation->Fill(min_sep);
        } else {
            n_hits = n_hits + slice.NCell();
            
            hSlice_hits_vs_mb->Fill(n_events,slice.NCell());
            hMeanADC_vs_Slice_hits->Fill((slice.TotalADC()/(double)slice.NCell()),slice.NCell());
            hSlicesNHits->Fill(slice.NCell());
            hSlicesSumADC->Fill(slice.TotalADC());
            hSlicesMeanADC->Fill((slice.TotalADC()/(double)slice.NCell()));
            fillPlaneSeperations(slice, min_cell_sep, min_plane_sep, min_sep);
            hSlicesMinInPlaneSeperation->Fill(min_cell_sep);
            hSlicesMinCrossPlaneSeperation->Fill(min_plane_sep);
            hSlicesMinSeperation->Fill(min_sep);
        }
    } // end loop on slices
    if (_verbose){
        std::cout << "\tSlices:             " << slices->size() << std::endl;
        std::cout << "\tNoise slices:       " << n_noise_slices << std::endl;
        std::cout << "\tMean hits:          " << n_noise_hits/n_noise_slices << std::endl;
        std::cout << "\tNon-noise slices:   " << slices->size() - n_noise_slices << std::endl;
        std::cout << "\tMean hits:          " << n_hits/(slices->size() - n_noise_slices) << std::endl;
    } // end of if on verbose
    // slices
    hNSlices->Fill(slices->size() - n_noise_slices);
    h_t_NSlices->SetBinContent(n_events, slices->size() - n_noise_slices);
    h_t_SliceNHits->SetBinContent(n_events, n_hits);
    if( (slices->size() - n_noise_slices) > 0 ){
        h_t_SliceMeanHits->SetBinContent(n_events, n_hits/(slices->size() - n_noise_slices));
    } else {
        h_t_SliceMeanHits->SetBinContent(n_events, 0);
    }
    // noise slice
    hNNoiseSlices->Fill(n_noise_slices);
    h_t_NNoiseSlices->SetBinContent(n_events,n_noise_slices);
    h_t_NoiseNHits->SetBinContent(n_events,n_noise_hits);
    if (n_noise_slices>0){
        h_t_NoiseMeanHits->SetBinContent(n_events,n_noise_hits/n_noise_slices);
    } else {
        h_t_NoiseMeanHits->SetBinContent(n_events,0);
    }
    
}
//----------------------------------------------------------------------------
void evgen::SupernovaAna::fillPlaneSeperations(rb::Cluster slice, 
            int & min_cell_sep, double & min_plane_sep, double & min_sep)
{
    // loop over the hits and find the minimum seperation for each
    //if (_verbose) std::cout << "--- Fill plane seperations\n";
    for(unsigned int i=0; i<slice.NCell();++i){
      const art::Ptr<rb::CellHit>& ihit = slice.Cell(i);
        //double min_sep = 999;
        //unsigned int min_index =999;
        
        //int min_cell_sep = 999;
        //unsigned int min_cell_index =999;
        
        //double min_plane_sep = 999;
        //unsigned int min_plane_index =999;
        
        for (unsigned int j=(i+1); j<(slice.NCell()-1);++j){
            const art::Ptr<rb::CellHit>& jhit = slice.Cell(j);
            if (ihit->Plane() == jhit->Plane()){
                int sep = fabs(ihit->Cell() - jhit->Cell());
                if (sep < min_cell_sep){
                    min_cell_sep = sep;
                    //min_cell_index = j;
                }
                if (sep < min_sep){
                    min_sep = sep;
                    //min_index = j;
                }                
            } else {
                if (ihit->View()  == jhit->View()){
                    double p_s = ihit->Plane() - jhit->Plane();
                    double c_s = ihit->Cell() - jhit->Cell();
                    double sep = util::pythag(p_s, c_s);
                    if (sep < min_plane_sep){
                        min_plane_sep = sep;
                        //min_plane_index = j;
                    }
                    if (sep < min_sep){
                        min_sep = sep;
                        //min_index = j;
                    }
                }
            }
        } // end of inner loop
        //if (min_cell_sep != 999){
            ////if (_verbose){
                ////std::cout << "\tCell["              << i
                          ////<< "]: "                  << ihit->Cell()
                          ////<< ", nearest to Cell["   << min_index
                          ////<< "]: "                  << hits[min_index]->Cell()
                          ////<< ", seperation: "       << min_seperation
                          ////<< std::endl;
            ////}
            //hSlicesMinInPlaneSeperation->Fill(min_cell_sep);
        //}
        //if (min_plane_sep != 999){
            ////if (_verbose){
                ////std::cout << "\tPlane["             << i
                          ////<< "]: "                  << ihit->Plane()
                          ////<< ", nearest to Plane["  << min_plane_index
                          ////<< "]: "                  << hits[min_plane_index]->Plane()
                          ////<< ", seperation: "       << min_plane_sep
                          ////<< std::endl;
            ////}
            //hSlicesMinCrossPlaneSeperation->Fill(min_plane_sep);
        //}
        //if (min_sep != 999){
            //hSlicesMinSeperation->Fill(min_sep);
        //}
    } // end of outer loop
}
//----------------------------------------------------------------------------
DEFINE_ART_MODULE(evgen::SupernovaAna)
