////////////////////////////////////////////////////////////////////////
// \file    MichelECluster.cxx
// \brief   A MichelECluster is a cluster of hits that arise from a Michel                                                             
//          electron cluster at the end of stopping muon track.  
// \version $Id: MichelECluster.cxx,v 1.5 2012-09-20 21:42:47 greenc Exp $ 
// \author  Kanika Sachdev (ksachdev@physics.umn.edu)
////////////////////////////////////////////////////////////////////////

#include "MichelEFilter/MichelECluster.h"
#include "Utilities/func/MathUtil.h"
#include "RecoBase/Track.h"

#include <TVector3.h>

//----------------------------------------------------------------------
me::MichelECluster::MichelECluster()
  : rb::Cluster()
{
}

  
//----------------------------------------------------------------------
me::MichelECluster::~MichelECluster()
{
}
  
//----------------------------------------------------------------------
double me::MichelECluster::DistanceToTrackEnd() const{
  
  TVector3 trkEnd;
  if( this->AtTrackEnd() )
    trkEnd = fParentTrack->Stop();
  else
    trkEnd = fParentTrack->Start();

  double deltaX = this->MeanX() - trkEnd.X();
  double deltaY = this->MeanY() - trkEnd.Y();
  double deltaZ = this->MeanZ() - trkEnd.Z();
  
  return util::pythag( deltaX, deltaY, deltaZ);

}// End of DistanceToTrackEnd

//------------------------------------------------------------

double me::MichelECluster::DeltaT() const{
  return this->MeanTNS() - fParentTrack->MeanTNS();
  
}// End of DeltaT

//------------------------------------------------------------
  
void me::MichelECluster::SetParentTrack(art::Ptr<rb::Track> parentTrack) {
  fParentTrack = parentTrack;
}

//------------------------------------------------------------

art::Ptr<rb::Track> me::MichelECluster::ParentTrack() const{
  return fParentTrack;
}

//------------------------------------------------------------

bool me::MichelECluster::AtTrackEnd() const{
  // Compare which end of the track is the michel E cluster closer to.

  bool closerToEnd = false;
  
  TVector3 end   = fParentTrack->Stop();
  TVector3 start = fParentTrack->Start();
  TVector3 mePos = this->MeanXYZ();
  
  if( (end - mePos).Mag() < (start - mePos).Mag())
    closerToEnd = true;
  
  return closerToEnd;
}

////////////////////////////////////////////////////////////////////////////
