/* 
 * File:   BufferNodeSimulator.cpp
 * Author: denis
 * 
 * Created on February 14, 2012, 2:50 PM
 */

#ifdef NOVACMAKE
#include "MockDataDAQ/include/BufferNodeSimulator.h"
#include "MockDataDAQ/include/Configuration.h"
#else
#include "MockDataDAQ/BufferNodeSimulator.h"
#include "MockDataDAQ/Configuration.h"
#endif

#include <DAQDataFormats/RawMilliSlice.h>
#include <DAQDataFormats/RawTrigger.h>



namespace mcdatadaq{

//------------------------------------------------------------------------------
BufferNodeSimulator::BufferNodeSimulator(uint32_t buffer_node_id):_bufferNodeID(buffer_node_id) {
    _dataBlock.setBuffId(_bufferNodeID);
    _dataBlock.setMonteCarloFlag(true);
}

//------------------------------------------------------------------------------
BufferNodeSimulator::~BufferNodeSimulator() { }

//------------------------------------------------------------------------------
daqdataformats::RawDataBlock* BufferNodeSimulator::getDataBlock(const std::vector<daqdataformats::RawMilliSlice*>& millislices,
                                                                daqdataformats::RawTrigger* trigger){

    /// Set Trigger Number taken from Trigger Object
    _dataBlock.setTrigNum(trigger->getTriggerHeader()->getTriggerNumber());

    /// Loop over MilliSlices and fill DataBlock
    for(uint32_t imillislice = 0; imillislice < millislices.size(); imillislice++ ){
        addMilliSlice(millislices[imillislice]);
    }// end of loop over MilliSlices

    _dataBlock.close(); ///< Need to close datablock - produces CRC


    return &_dataBlock;
}

//------------------------------------------------------------------------------
bool BufferNodeSimulator::addMilliSlice(daqdataformats::RawMilliSlice* millislice){

    const Configuration* config = Configuration::getInstance();

    const daqdataformats::version_t microblock_version = config->getMicroBlockVersion();

    //millislice->print();
    
    /// Loop over microslices. Fill DataBlock with MicroBlocks.
    const uint32_t number_of_microslices = millislice->getMicrosliceNumber();
    for(uint32_t imicroslice = 0; imicroslice < number_of_microslices; imicroslice++ ){

        millislice->setFloatingMicroSlice(imicroslice);
        daqdataformats::RawMicroSlice* microslice = millislice->getFloatingMicroSlice();

        _dataBlock.addMicroBlock(millislice->getBuffer(), microslice->getBuffer(), microblock_version);
    }// end of loop over microslices

    return true;
}

//------------------------------------------------------------------------------
uint32_t BufferNodeSimulator::getBufferNodeID() const{ return _bufferNodeID;}

//------------------------------------------------------------------------------
bool BufferNodeSimulator::nextEvent(){

    /// This is a fast but very dangerous to clear out the datablock
    _dataBlock.getHeader()->setCRCCalculationUsed(false);
    _dataBlock.getHeader()->setNumMicroBlocks(0);
    _dataBlock.getHeader()->setDataSize(_dataBlock.getHeader()->sizeofdata());

    return true;
}

}// end of namespace
