/* 
 * File:   DataLoggerSimulator.cpp
 * Author: denis
 * 
 * Created on February 15, 2012, 10:27 AM
 */

#ifdef NOVACMAKE
#include "MockDataDAQ/include/DataLoggerSimulator.h"
#include "MockDataDAQ/include/Configuration.h"
#else
#include "MockDataDAQ/DataLoggerSimulator.h"
#include "MockDataDAQ/Configuration.h"
#endif
#include "DAQDataFormats/RawDataBlock.h"
#include "DAQDataFormats/RawTrigger.h"



/// Keyword in the file name for Events's file
#define EVENT_KEYWORD_IN_FILE_NAME "Events"
#define RUN_KEYWORD_IN_FILE_NAME   "Run"

namespace mcdatadaq {

//------------------------------------------------------------------------------
DataLoggerSimulator::DataLoggerSimulator():
    fOutputEventFile(0),
    fOutputRunFile(0)
{

   const Configuration* config = Configuration::getInstance();

   /// Setup the Event Object
   fCurrentEvent.setRunNumber  (1);
   fCurrentEvent.setSubrun     (1);
   fCurrentEvent.setEventNumber(1);


   /// Open event files
   if(!config->isProducingJustMicroSlices()){
            const std::string final_file_name_event = config->getFullFileName(EVENT_KEYWORD_IN_FILE_NAME);
            fOutputEventFile = fopen (final_file_name_event.c_str(),"wb");

            /// Open run file if needed
            if(config->isProducingRunObject()){
                const std::string final_file_name_run   = config->getFullFileName(RUN_KEYWORD_IN_FILE_NAME);
                fOutputRunFile   = fopen (final_file_name_run.c_str(),"wb");
            }// end of opening run file
   }// end of opening event and run files

}


//------------------------------------------------------------------------------
DataLoggerSimulator::~DataLoggerSimulator() {

    std::cout<<"Destructing DataLoggerSimulator\n";

    /// Close output files
    if(fOutputEventFile) {
        fclose(fOutputEventFile);
    }

    if(fOutputRunFile){
        fCurrentRunHeader.close();
        writeRun();
        fclose(fOutputRunFile);
    }
}

//------------------------------------------------------------------------------
bool DataLoggerSimulator::produceEvent(const std::vector<daqdataformats::RawDataBlock*>& datablocks,
                                       daqdataformats::RawTrigger* trigger){

    const Configuration* config = Configuration::getInstance();

   /// Set Trigger for the event
    fCurrentEvent.setTrigger(trigger->getBuffer());

    /// Loop over Datablocks and add them to the event
    for(uint32_t idatablock = 0; idatablock < datablocks.size(); idatablock++){

        daqdataformats::RawDataBlock* current_datablock = datablocks[idatablock];
        fCurrentEvent.addDataBlock(current_datablock->getBuffer());
    }// end of loop over datablocks
    fCurrentEvent.close();

    //fCurrentEvent.print();

    /// Write out the event
    writeEvent();


    /// Add event to the run object if it is configured to do so.
    /// Otherwise, don't bother
    if(config->isProducingRunObject())
        writeRun();

    return true;
}

//------------------------------------------------------------------------------
bool DataLoggerSimulator::writeEvent(){
        const Configuration* config = Configuration::getInstance();
        return config->writeToFile(fOutputEventFile, fCurrentEvent);
}

//------------------------------------------------------------------------------
bool DataLoggerSimulator::writeRun(){
        fCurrentRunHeader.addRunSize(fCurrentEvent.sizeofdata());
        fCurrentRunHeader.advanceEventNumber();
        const Configuration* config = Configuration::getInstance();
        return config->writeToFile(fOutputRunFile, fCurrentEvent);
}

//------------------------------------------------------------------------------
bool DataLoggerSimulator::nextEvent(){

    fCurrentEvent.getHeader()->setDataBlockNumber(0);
    fCurrentEvent.getHeader()->setEventSize(  fCurrentEvent.getHeader()->sizeofdata()
                                             +fCurrentEvent.getTrigger()->sizeofdata()
                                           );
    fCurrentEvent.getHeader()->advanceEventNumber();
    fCurrentEvent.getHeader()->setCRCCalculationUsed(false);

    return true;
}


}// end of namespace
