/* 
 * File:   GlobalTriggerSimulator.cpp
 * Author: denis
 * 
 * Created on February 14, 2012, 6:01 PM
 */

#ifdef NOVACMAKE
#include "MockDataDAQ/include/GlobalEventTime.h"
#include "MockDataDAQ/include/GlobalTriggerSimulator.h"
#include "MockDataDAQ/include/Configuration.h"
#else
#include "MockDataDAQ/GlobalEventTime.h"
#include "MockDataDAQ/GlobalTriggerSimulator.h"
#include "MockDataDAQ/Configuration.h"
#endif

#include <DAQDataFormats/RawTrigger.h>
#include <DAQDataFormats/RawTriggerMask.h>
#include <DAQDataFormats/RawTriggerTime.h>

/// Keyword in the file name for Trigger's file
#define TRIGGER_KEYWORD_IN_FILE_NAME "TriggerBlocks"

namespace mcdatadaq {

//------------------------------------------------------------------------------
GlobalTriggerSimulator::GlobalTriggerSimulator():fOutputTriggerFile(0) {
   // Setup the pointer to the Trigger inside the fRawEvent
   fTrigger.getTriggerHeader()->setSource(daqdataformats::TRIGSRC_SIMULATED);

   fTrigger.getTriggerTime()->setClockSource(daqdataformats::CLOCKSRC_DUMMY);


   fTrigger.getTriggerRange()->setTriggerLength(4000); //500 microseconds
   //fTrigger.getTriggerRange()->setTriggerCont(0); //Not infinity

   fTrigger.getTriggerMask()->setMonteCarloFlag(1);
   fTrigger.getTriggerMask()->setTrigMod(0);// who know what this is
   fTrigger.getTriggerMask()->setPrescale(1); //?

   const Configuration* config                  = Configuration::getInstance();
   const std::string    final_file_name_trigger = config->getFullFileName(TRIGGER_KEYWORD_IN_FILE_NAME);
   fOutputTriggerFile                           = fopen (final_file_name_trigger.c_str(),"wb");

   /// Now we need to write DAQ Simulation version in the beginning of the file
   config->writeDAQSimulatorVersionIntoFile(fOutputTriggerFile);

}

//------------------------------------------------------------------------------
GlobalTriggerSimulator::~GlobalTriggerSimulator() {

    /// Close output files
    if(fOutputTriggerFile) {
        fclose(fOutputTriggerFile);
    }

}

//------------------------------------------------------------------------------
daqdataformats::RawTrigger* GlobalTriggerSimulator::getNewTrigger() {

   const GlobalEventTime* global_event_time = GlobalEventTime::getInstance();

   fTrigger.getTriggerTime()->setTriggerTimeStamp(global_event_time->getGlobalEventTime());
   fTrigger.getTriggerHeader()->setTriggerNumber(fTrigger.getTriggerHeader()->getTriggerNumber() + 1);

   /// Write the Trigger into the output triggerblocks file
   writeTrigger();

   return getTrigger();
}

//------------------------------------------------------------------------------
/// Gets current trigger
daqdataformats::RawTrigger* GlobalTriggerSimulator::getTrigger() { return &fTrigger;}

//------------------------------------------------------------------------------
bool GlobalTriggerSimulator::writeTrigger(){
        const Configuration* config = Configuration::getInstance();
        return config->writeToFile(fOutputTriggerFile, fTrigger);
}

}// end of namespace
