////////////////////////////////////////////////////////////////////////
// $Id: MockDataDAQAna_module.cc,v 1.2 2012-09-14 18:22:14 denis Exp $
//
// This module inputs the simulated offline ART files
// and produces raw files of the DAQ
// that would be produced by different DAQ applications
// FEBs, DCMs, Buffer Nodes, DataLogger, GlobalTrigger
// that correspond to the simulated event
//
// denis@fnal.gov
//
////////////////////////////////////////////////////////////////////////

// Framework includes
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Event.h"

/// Something conflicts with RandomNumberGenerator. Keep DAQSimulator.h include below Art Framework includes
#ifdef NOVACMAKE
#include "MockDataDAQ/include/DAQSimulator.h"
#include "MockDataDAQ/include/Configuration.h"
#else
#include "MockDataDAQ/DAQSimulator.h"
#include "MockDataDAQ/Configuration.h"
#endif

#include <string>

namespace mcdatadaq {

///////////////////// Definitions //////////////////////////////////////////////

  class MockDataDAQAna : public art::EDAnalyzer {

  public:
    explicit MockDataDAQAna(fhicl::ParameterSet const& pset);
    virtual ~MockDataDAQAna();

    void analyze(art::Event const& evt);
    void beginJob();
    void endJob();

  private:
    /// Initializes detector and stuff
    bool init();

  private:
    std::string                                 fRawDataLabel; ///< Label name for RawDigits
    DAQSimulator                                fDAQSimulator; ///< DAQSimulator instance
  };// end of class MockDataDAQAna definition

  ///////////////////// End of Definitions /////////////////////////////////////

//----------------------------------------------------------------------------
  MockDataDAQAna::MockDataDAQAna(fhicl::ParameterSet const& pset):
    EDAnalyzer(pset),
    fRawDataLabel(pset.get< std::string >("RawDataLabel"))
{
    /// Initialize Configuration with parameters from the fcl file
    Configuration::setInstance(pset);

    std::cout<<"RawDataLabel:"<<fRawDataLabel<<"\n";
}

//----------------------------------------------------------------------------
MockDataDAQAna::~MockDataDAQAna(){}

//----------------------------------------------------------------------------
void MockDataDAQAna::beginJob(){ }

//----------------------------------------------------------------------------
void MockDataDAQAna::analyze(art::Event const& evt){

    const uint32_t event = evt.event();
    std::cout<<"Processing event "<<event<<"\n";

    const Configuration* config = Configuration::getInstance();

    /// If Detector is unknown, try to initialize it
    if(config->isDetectorUnknown()){
          /// If initialization returned failed, then something is wrong
          if(!init()){
              std::cout<<"From MockDataDAQAna::analyze Initialization failed\n";
              exit(1);
          }// end of checking whether the init was not good
    }// end of checking if Detector is unknown

    /// Pull the Raw Digits information out
    art::Handle<std::vector<rawdata::RawDigit> > diglist;
    try{
      evt.getByLabel(fRawDataLabel, diglist);
    }// end of try
    catch(...){
    }// end of catch

    /// Simulate the current event in the DAQ
    fDAQSimulator.processEvent(*diglist);

}// end of MockDataDAQAna::analyze

//......................................................................
// Initializes detector and stuff
bool MockDataDAQAna::init(){

      /// Initializing job configuration
      Configuration* config = Configuration::getInstance();
      if(!config->init())return false;

      /// Initialize DAQSimulator
      if(!fDAQSimulator.init()) return false;

      return true;
}// end of init()

//----------------------------------------------------------------------------
void MockDataDAQAna::endJob(){}


//----------------------------------------------------------------------------
  DEFINE_ART_MODULE(MockDataDAQAna);
}// end of namespace MockDataDAQAna
