/* 
 * File:   NanoSliceOrder.cpp
 * Author: denis
 * 
 * Created on February 13, 2012, 4:04 PM
 */

#ifdef NOVACMAKE
#include "MockDataDAQ/include/NanoSliceOrder.h"
#else
#include "MockDataDAQ/NanoSliceOrder.h"
#endif

#include <iostream>

#define BITFIELDGET(block, mask, shift)  (((mask_t(block) )& mask) >> (shift))
#define BITFIELDSET(block, data, mask, shift)  block = (((block) & ~mask)| ((mask_t(data) << shift ) & mask))

namespace mcdatadaq{


NanoSliceOrder* NanoSliceOrder::finstanceID = NULL;

//------------------------------------------------------------------------------
NanoSliceOrder* NanoSliceOrder::getInstance(){
  if(finstanceID == NULL){	               // Check if there is an instance active
    finstanceID = new NanoSliceOrder();       // If there is not, make a new one
  }
  return finstanceID;
}


//------------------------------------------------------------------------------
NanoSliceOrder::NanoSliceOrder() {

    /// Calculating shifts
    shift_diblock = calculateShiftFromMask(mask_diblock);
    shift_dcm     = calculateShiftFromMask(mask_dcm);
    shift_feb     = calculateShiftFromMask(mask_feb);
    shift_time    = calculateShiftFromMask(mask_time);
    shift_pixel   = calculateShiftFromMask(mask_pixel);

    print();
}

//------------------------------------------------------------------------------
NanoSliceOrder::~NanoSliceOrder() { }

//------------------------------------------------------------------------------
shift_t NanoSliceOrder::calculateShiftFromMask(const mask_t mask) const {

    mask_t floating_digit = 1;

    for(shift_t idigit = 0; idigit < 64; idigit++){
        if(mask & floating_digit) return idigit;
        floating_digit <<= 1;
    }// end of loop over digits

    return 0;
}

//------------------------------------------------------------------------------
mask_t NanoSliceOrder::getNumberWithShiftsAndMask(const uint32_t number,
						  const shift_t  shift,
						  const mask_t   mask) const {
    mask_t out_number = 0;
    BITFIELDSET(out_number, number, shift, mask);

    return out_number;
}

//------------------------------------------------------------------------------
mask_t NanoSliceOrder::calculateOrderingWeight(const daqchannelmap::diblock_t diblock,
					       const daqchannelmap::dcm_id_t dcm,
					       const daqchannelmap::feb_t feb,
					       const daqchannelmap::pixel_t pixel,
					       const feb_timestamp time) const
{
    mask_t weight = 0;

    BITFIELDSET(weight, time   , mask_time,    shift_time);
    BITFIELDSET(weight, diblock, mask_diblock, shift_diblock);
    BITFIELDSET(weight, dcm    , mask_dcm,     shift_dcm);
    BITFIELDSET(weight, feb    , mask_feb,     shift_feb);
    BITFIELDSET(weight, pixel  , mask_pixel,   shift_pixel);

    return weight;
}

//------------------------------------------------------------------------------
bool NanoSliceOrder::print() const {

    std::cout<<"shifts "<<shift_diblock;
    std::cout<<"  "<<shift_dcm;
    std::cout<<"  "<<shift_feb;
    std::cout<<"  "<<shift_time;
    std::cout<<"  "<<shift_pixel<<"\n";

    return true;
}

//------------------------------------------------------------------------------
bool NanoSliceOrder::print(const mask_t number) const {


    const daqchannelmap::diblock_t  diblock = BITFIELDGET(number, mask_diblock, shift_diblock);
    const daqchannelmap::dcm_id_t   dcm     = BITFIELDGET(number, mask_dcm    , shift_dcm);
    const daqchannelmap::feb_t      feb     = BITFIELDGET(number, mask_feb    , shift_feb);
    const daqchannelmap::pixel_t    pixel   = BITFIELDGET(number, mask_pixel  , shift_pixel);
    const feb_timestamp             time    = BITFIELDGET(number, mask_time   , shift_time);

    std::cout<<std::hex<<number<<"x";
    std::cout<<std::dec;
    std::cout<<" diblock="<<diblock;
    std::cout<<" dcm="<<dcm;
    std::cout<<" feb="<<feb;
    std::cout<<" pixel="<<pixel;
    std::cout<<" time="<<time<<std::endl;

    return true;
}

}// end of namespace
