#include <iostream>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <string>
#include <cstring>
#include "TFile.h"
#include "TTree.h"
#include "TChain.h"

void CreateKNNTrainingTrees(){

  // output file with signal and background trees that can be used by TMVA for training
  TFile* outfile = new TFile("outputfilename.root","RECREATE");

  // input file that holds the output of the ReMIdTrain_module histo file
  const std::string filename = "inputfilename.root" ;
  
  TChain* sigtree = new TChain("remidtrain/fTreeS");
  sigtree->Add(filename.c_str());
  
  TChain* backtree = new TChain("remidtrain/fTreeB");
  backtree->Add(filename.c_str());
  
  // make the signal output trees
  float fTrackLength, fDedxSep, fScatSep, fMeasFrac, fnuE; 
  TTree* SigTree = new TTree("SigTree","Input Variables for the Signal Tree");
  SigTree->Branch("trackLength", &fTrackLength, "trackLength/F");
  SigTree->Branch("dedxSep", &fDedxSep, "dedxSep/F");
  SigTree->Branch("scatSep", &fScatSep, "scatSep/F");
  SigTree->Branch("measFrac", &fMeasFrac, "measFrac/F");
  SigTree->Branch("nuE", &fnuE, "nuE/F");

  // setup reading in the signal tree
  float trklensig, ESepsig, ScatSepsig, nuEsig, measfracsig;
  sigtree->SetBranchAddress("trackLength",&trklensig);
  sigtree->SetBranchAddress("dedxSep",&ESepsig);
  sigtree->SetBranchAddress("scatSep",&ScatSepsig);
  sigtree->SetBranchAddress("measFrac", &measfracsig);
  sigtree->SetBranchAddress("visE", &nuEsig);

  int nsigevts = sigtree->GetEntries();
  int nbackevts = backtree->GetEntries();

  int nmaxevts = nsigevts;
  if(nsigevts > nbackevts){ nmaxevts = nbackevts; }
  std::cout<<"Number of signal events: "<<sigtree->GetEntries()<<std::endl;
  std::cout<<"Number of background events: "<<backtree->GetEntries()<<std::endl;


  for(unsigned int idx = 0; idx < nmaxsigevts; ++idx){
    // get the energy of this event
    sigtree->GetEntry(idx);
    float energy = nuEsig;
    fnuE = energy;
    fTrackLength = trklensig;
    fDedxSep = ESepsig;
    fScatSep = ScatSepsig;
    fMeasFrac = measfracsig;
    SigTree->Fill();
  } // loop over the signal tree events


  // make the background output trees
  TTree* BackTree = new TTree("BackTree","Input Variable for the Background Tree");
  BackTree->Branch("trackLength", &fTrackLength, "trackLength/F");
  BackTree->Branch("dedxSep", &fDedxSep, "dedxSep/F");
  BackTree->Branch("scatSep", &fScatSep, "scatSep/F");
  BackTree->Branch("measFrac", &fMeasFrac, "measFrac/F");
  BackTree->Branch("nuE", &fnuE, "nuE/F");

  // setup reading in the background tree
  float trklenback, ESepback, ScatSepback, nuEback, measfracback;
  backtree->SetBranchAddress("trackLength",&trklenback);
  backtree->SetBranchAddress("dedxSep",&ESepback);
  backtree->SetBranchAddress("scatSep",&ScatSepback);
  backtree->SetBranchAddress("visE", &nuEback);
  backtree->SetBranchAddress("measFrac", &measfracback);

  for(unsigned int idx = 0; idx < nmaxsigevts; ++idx){
    // get the energy of this event
    backtree->GetEntry(idx);
    float energy = nuEback;
    fnuE = energy;
    fTrackLength = trklenback;
    fDedxSep = ESepback;
    fScatSep = ScatSepback;
    fMeasFrac = measfracback;
    BackTree->Fill();
  } // loop over the background tree events

  outfile->cd();
  SigTree->Write();
  BackTree->Write();

}
