#include <iostream.h>
#include <fstream>
#include <iomanip.h>
#include <sstream>
#include <string>
#include <cstring>


void MakeSampleHistograms()
{

  // input filename of histogram file created from running ReMIdTrain_module.cc
  const std::string filename = "inputfilename.root";

  TChain* dedxch = new TChain("remidtrain/fDedxSample");
  TChain* scatch = new TChain("remidtrain/fScatSample");

  // Add file to the signal dedx chain
  dedxch->Add((filename).c_str());

  // Add file to the signal scat chain
  scatch->Add((filename).c_str());

  TCut cutSDedx = "abs(pdg) == 13 && intType < 4";
  TCut UsedNonVert = "vert == 0 && used == 1";
  TCut cutBDedxPion = "abs(pdg) == 211";
  TCut cutBDedxProton = "pdg == 2212";

  // setup some binning
  int ndedxbins = 25;
  double dedxlow = 0.0;
  double dedxhigh = 0.005;
  double lhigh = 3000;
  double llow = 0;
  int nlbins = floor((lhigh-llow)/5.0);
  double lbinwidth = 5.0;

  // signal x-axis binning
  const int xbinsizemu = 201;
  Double_t xbinsmu[xbinsizemu];
  xbinsmu[0] = llow;
  xbinsmu[xbinsizemu-1] = lhigh;
  for(unsigned int i = 1; i < (xbinsizemu-1); ++i){
    if(i <= 100){
      xbinsmu[i] = 0 + 5.0*(double)i;
    }
    else if(i <= 150){
      xbinsmu[i] = 500 + 10*(double)(i-100);
    }
    else{
      xbinsmu[i] = 1000 + 40*(double)(i-150);
    }
  }

  // background x-axis binning
  double llowpi = 0.0;
  double lhighpi = 2000.0;
  const int xbinsizepi = 161; 
  Double_t xbinspi[xbinsizepi];
  xbinspi[0] = llowpi;
  xbinspi[xbinsizepi-1] = lhighpi;
  for(unsigned int i = 1; i < (xbinsizepi-1); ++i){
    if(i <= 100){
      xbinspi[i] = 0 + 5.0*(double)i;
    }
    else if(i <= 150){
      xbinspi[i] = 500 + 10*(double)(i-100);
    }
    else{
      xbinspi[i] = 1000 + 100*(double)(i-150);
    }
  }


  // dedx (y-axis) binnning
  double dedxbinwidth = 0.0002;
  const int ydedxbinsize = ndedxbins+1;
  Double_t ydedxbins[ydedxbinsize];
  ydedxbins[0] = dedxlow;
  ydedxbins[ndedxbins] = dedxhigh;
  for(unsigned int i = 1; i< ndedxbins; ++i){
    ydedxbins[i] = dedxlow + dedxbinwidth*(double)i;
  }


  // Create dEdx histograms
  TH2F* DedxVxS = new TH2F("DedxVxS","Signal;Distance from End of Track (cm); dE/dx (GeV/cm)",
			   (xbinsizemu-1),xbinsmu,ndedxbins,ydedxbins);
  TH2F* DedxVxB = new TH2F("DedxVxB","Background;Distance from End of Track (cm); dE/dx (GeV/cm)",
			   (xbinsizepi-1),xbinspi,ndedxbins,ydedxbins);

//   TH2F* DedxVxBProton = new TH2F("DedxVxBProton","Background;Distance from End of Track (cm); dE/dx (GeV/cm)",
// 				 (xbinsizepi-1),xbinspi,ndedxbins,ydedxbins);


  // scat (y-axis) binning
  int nscatbins = 30;
  double scatlow = 0.0;
  double scathigh = 0.003;
  double scatbinwidth = 0.0001;

  const int yscatbinsize = nscatbins+1;
  Double_t yscatbins[yscatbinsize];
  yscatbins[0] = scatlow;
  yscatbins[nscatbins] = scathigh;
  for(unsigned int i = 1; i< nscatbins; ++i){
    yscatbins[i] = scatlow + scatbinwidth*(double)i;
  }

  // Create scattering histograms
  TH2F* ScatVxS = new TH2F("ScatVxS","Signal;Distance from End of Track (cm); Scat",(xbinsizemu-1),xbinsmu,nscatbins,yscatbins);
  TH2F* ScatVxB = new TH2F("ScatVxB","Background;Distance from End of Track (cm); Scat",(xbinsizepi-1),xbinspi,nscatbins,yscatbins);


  // Draw signal dedx v x histogram
  TCanvas* c1 = new TCanvas("c1","c1");
  dedxch->Draw("dedx:distFromEnd>>DedxVxS",cutSDedx&&UsedNonVert,"colz");

  // Draw background dedx v x histogram
  TCanvas* c2 = new TCanvas("c2","c2");
  dedxch->Draw("dedx:distFromEnd>>DedxVxB",cutBDedxPion&&UsedNonVert,"colz");

  // Draw signal scattering v x histogram
  TCanvas* c3 = new TCanvas("c3","c3");
  scatch->Draw("scatVar:distFromEnd>>ScatVxS",cutSDedx,"colz");

  // Draw background scattering v x histogram
  TCanvas* c4 = new TCanvas("c4","c4");
  scatch->Draw("scatVar:distFromEnd>>ScatVxB",cutBDedxPion,"colz");

//   // Draw background dedx v x histogram
//   TCanvas* c5 = new TCanvas("c5","c5");
//   dedxch->Draw("dedx:distFromEnd>>DedxVxBProton",cutBDedxProton&&UsedNonVert,"colz");

}
