////////////////////////////////////////////////////////////////////////
// \file    FilterList.h
// \brief   A simple list of "filtered out" products
// \version $Id: FilterList.h,v 1.1 2012-09-25 02:46:08 bckhouse Exp $
// \author  $Author: bckhouse $
// \date    $Date: 2012-09-25 02:46:08 $
////////////////////////////////////////////////////////////////////////
#ifndef RBFILTERLIST_H
#define RBFILTERLIST_H

#ifndef __GCCXML__
#include "art/Framework/Principal/Event.h"
#endif
#include "art/Persistency/Common/PtrVector.h"

namespace rb
{
#ifndef __GCCXML__
  /// Does \a x exist in any of the FilterLists in the Event?
  template<class T> bool IsFiltered(const art::Event& evt,
                                    art::Ptr<T> x);

  /// Does \a x exist in any of the FilterLists in the Event?
  template<class T> bool IsFiltered(const art::Event& evt,
                                    art::Handle<std::vector<T> > h,
                                    int idx);
#endif

  /// A simple list of products that have been marked "filtered out"
  template<class T> class FilterList
  {
  public:
#ifndef __GCCXML__
    friend bool IsFiltered<>(const art::Event&, art::Ptr<T>);

    void Add(art::Ptr<T> prod)
    {
      fFiltered.push_back(prod);
    }
    void Add(art::Handle<std::vector<T> > h, int idx)
    {
      fFiltered.push_back(art::Ptr<T>(h, idx));
    }
#endif
  protected:
    art::PtrVector<T> fFiltered;
  };

#ifndef __GCCXML__
  template<class T> bool IsFiltered(const art::Event& evt,
                                    art::Ptr<T> x)
  {
    std::vector<art::Handle<rb::FilterList<T> > > filts;
    evt.getManyByType(filts);

    std::set<art::Ptr<T> > vetoed;
    const unsigned int I = filts.size();
    for(unsigned int i = 0; i < I; ++i){
      const unsigned int J = filts[i]->fFiltered.size();
      for(unsigned int j = 0; j < J; ++j){
        vetoed.insert(filts[i]->fFiltered[j]);
      }
    }

    return vetoed.find(x) != vetoed.end();
  }

  template<class T> bool IsFiltered(const art::Event& evt,
                                    art::Handle<std::vector<T> > h,
                                    int idx)
  {
    return IsFiltered(evt, art::Ptr<T>(h, idx));
  }
#endif
}

#endif // RBFILTERLIST_H
////////////////////////////////////////////////////////////////////////
