/////////////////////////////////////////////////////////////////////////
/// \file    HitList.h
/// \brief   A convenient packaging of cell information required for
///          simple geometric track finding and fitting
/// \version $Id: HitList.h,v 1.4 2011-11-21 19:25:42 edniner Exp $
/// \author  messier@indiana.edu
/////////////////////////////////////////////////////////////////////////
#ifndef RB_HITLIST_H
#define RB_HITLIST_H
#include <vector>

#include "GeometryObjects/PlaneGeo.h"

namespace rb {
  class Cluster;
  
  /// Hits from the x or y view
  class Hit2D {
  public:
    Hit2D(geo::View_t v =geo::kXorY, 
	  double      xy =0, 
	  double      z  =0, 
	  double      t  =0, 
          double      cxy=0,
          double      cz =0,
	  double      w  =1.0) :
      fView(v), fXY(xy), fZ(z), fT(t), fCXY(cxy), fCZ(cz), fW(w)
      { }
  public:      
    geo::View_t fView;   ///< View hit belongs in (x or y)
    double      fXY;     ///< x or y coordinate of hit
    double      fZ;      ///< z coordinate of hit
    double      fT;      ///< time coordinate of hit
    double      fCXY;    ///< half width of cell, will be on x or y axis
    double      fCZ;     ///< half depth of cell, will be on z axis
    double      fW;      ///< Initial weight of hit
  };
  
  /// Just the essential information required for track fitting
  class HitList : public std::vector<Hit2D> {
  public:
    HitList();
    HitList(unsigned int n);
    ///
    /// \brief Construct a hit list from a cluster
    ///
    /// \param clust - The cluster of hits
    /// \param wbyq  - Assign weights by charge, else all weights are 1
    ///
    HitList(const rb::Cluster& clust, bool wbyq=false);
  public:
    void SetHit(unsigned int i, 
		geo::View_t, 
		double xy,
		double z,
		double t,
                double cxy,
                double cz,
		double w=1.0);
    void AddHit(geo::View_t,
		double xy, 
		double z, 
		double t, 
                double cxy,
                double cz,
		double w=1.0);
  };
}

#endif
////////////////////////////////////////////////////////////////////////
