/////////////////////////////////////////////////////////////////////////////
/// \file    HoughResult.cxx
/// \brief   Data resulting from a Hough transform on the cell hit positions
/// \version $Id: HoughResult.cxx,v 1.4 2011-12-15 18:42:19 mbaird42 Exp $
/// \author  messier@indiana.edu
/////////////////////////////////////////////////////////////////////////////
#include "RecoBase/HoughResult.h"

#include <algorithm>
#include <cassert>
#include <cmath>

namespace rb
{
  //.....................................................................
  HoughPeak::HoughPeak() :
    fRho(0),
    fTheta(0),
    fH(0),
    fThr(0.0),
    fSR(0),
    fST(0)
  { }
  
  //......................................................................
  
  HoughPeak::HoughPeak(double r, double q, double h, double thr, double sr, double st) : 
    fRho(r),
    fTheta(q),
    fH(h),
    fThr(thr),
    fSR(sr),
    fST(st)
  { }
  
  //......................................................................
  
  HoughResult::HoughResult() : 
    fView(geo::kXorY),
    fTNSlo(0),
    fTNShi(0),
    fXY0(0),
    fZ0(0)
  { }
  
  //......................................................................
  
  HoughResult::HoughResult(geo::View_t v,
			   double      tlo,
			   double      thi,
			   double      xy0,
			   double      z0) :
    fView(v),
    fTNSlo(tlo),
    fTNShi(thi),
    fXY0(xy0),
    fZ0(z0)
  { }
  
  //......................................................................
  
  void HoughResult::SlopeIcept(unsigned int i, double* m, double* b) const 
  {
    assert(i<fPeak.size());
    double s = sin(fPeak[i].fTheta);
    double c = cos(fPeak[i].fTheta);
    if (s==0.0) s = 1.0E-9;
    //
    // Put the slope and intercepts into detector coordinates
    //
    *m = -c/s;
    *b = fPeak[i].fRho/s - (*m)*fZ0 + fXY0;
  }
  
  //......................................................................
  
  static bool compare_peaks(const HoughPeak& a, const HoughPeak& b) 
  {
    return a.fH>b.fH;
  }
  //......................................................................
  
  void HoughResult::SortPeaks() 
  {
    sort(fPeak.begin(), fPeak.end(), compare_peaks);
  }
  
} // end namespace rb
////////////////////////////////////////////////////////////////////////
