////////////////////////////////////////////////////////////////////////
// \file    Shower.cxx
// \brief   A shower is a \ref rb::Prong with a length, other variables which can be commonly used by analysis showers will be added.
// \version $Id: Shower.h,v 1.00 2013-07-12 21:31:55 bianjm Exp $
// \author  Jianming Bian - bianjm@physics.umn.edu
// \date    $Date: 2013-07-12 21:31:55 $
////////////////////////////////////////////////////////////////////////

#include "RecoBase/Shower.h"

namespace rb
{
  //......................................................................
  Shower::Shower(const rb::Prong& prong,
                 int id,
                 double length)
    : rb::Prong(prong)
  {
    SetID(id);
    SetTotalLength(length);
  }

  //......................................................................
  Shower::~Shower()
  {
  }

  //......................................................................
  double Shower::TotalLength() const
  {
    return fTotalLength;
  }

  //......................................................................
  void Shower::SetTotalLength(double length)
  {
    fTotalLength = length;
  }

  //......................................................................
  TVector3 Shower::Stop() const
  {
    return fStart + fDir * TotalLength();
  }

} // end namespace rb
//////////////////////////////////////////////////////////////////////////////
