////////////////////////////////////////////////////////////////////////////////
/// \file    Vertex.cxx
/// \brief   Defining a Vertex Position
/// \version $Id: Vertex.cxx,v 1.7 2011-11-18 01:52:11 bckhouse Exp $
/// \author  eniner@indiana.edu
////////////////////////////////////////////////////////////////////////////////
#include "RecoBase/Vertex.h"

#include "RecoBase/Track.h"

#include <cassert>

namespace rb
{
  //.....................................................................

  Vertex::Vertex() :
    fX(0),
    fY(0),
    fZ(0),
    fT(0)
  {
  }

  //......................................................................

  Vertex::Vertex(TVector3 xyz, double t) :
    fX(xyz.X()),
    fY(xyz.Y()),
    fZ(xyz.Z()),
    fT(t)
  {
  }

  //......................................................................

  Vertex::Vertex(double x, double y, double z, double t) :
    fX(x),
    fY(y),
    fZ(z),
    fT(t)
  {
  }

  //......................................................................

  void Vertex::Add(art::Ptr<rb::Cluster> c)
  {
    fClusters.push_back(c);
  }

  //......................................................................

  void Vertex::Add(art::Ptr<rb::Prong> p)
  {
    fProngs.push_back(p);
  }

  //......................................................................

  void Vertex::Add(art::Ptr<rb::Track> t)
  {
    fTracks.push_back(t);
  }

  //......................................................................

  TVector3 Vertex::GetXYZ() const
  {
    return TVector3(fX, fY, fZ);
  }

  //......................................................................

  void Vertex::SetXYZ(double x, double y, double z)
  {
    fX = x;
    fY = y;
    fZ = z;
  }

  //......................................................................

  void Vertex::SetXXYZ(TVector3 xyz)
  {
    fX = xyz.X();
    fY = xyz.Y();
    fZ = xyz.Z();
  }

  //......................................................................

  unsigned int Vertex::NClusters() const
  {
    return fClusters.size();
  }

  //......................................................................

  art::Ptr<rb::Cluster> Vertex::Cluster(unsigned int idx) const
  {
    assert(idx < NClusters());
    return fClusters[idx];
  }

  //......................................................................

  unsigned int Vertex::NProngs() const
  {
    return fProngs.size();
  }

  //......................................................................

  art::Ptr<rb::Prong> Vertex::Prong(unsigned int idx) const
  {
    assert(idx < NProngs());
    return fProngs[idx];
  }

  //......................................................................

  unsigned int Vertex::NTracks() const
  {
    return fTracks.size();
  }

  //......................................................................

  art::Ptr<rb::Track> Vertex::Track(unsigned int idx) const
  {
    assert(idx < NTracks());
    return fTracks[idx];
  }

  //......................................................................

  unsigned int Vertex::NProngsAndTracks() const
  {
    return fProngs.size()+fTracks.size();
  }

  //......................................................................

  const rb::Prong* Vertex::ProngsAndTracks(unsigned int idx) const
  {
    assert(idx < NProngsAndTracks());
    if(idx < NTracks()) return Track(idx).get();
    return Prong(idx-NTracks()).get();
  }

  //......................................................................

  unsigned int Vertex::NClustersProngsAndTracks() const
  {
    return fClusters.size()+fProngs.size()+fTracks.size();
  }

  //......................................................................

  const rb::Cluster* Vertex::ClustersProngsAndTracks(unsigned int idx) const
  {
    assert(idx < NClustersProngsAndTracks());
    if(idx < NProngsAndTracks()) return ProngsAndTracks(idx);
    return Cluster(idx-NProngsAndTracks()).get();
  }

} // end namespace rb
////////////////////////////////////////////////////////////////////////
