////////////////////////////////////////////////////////////////////////
/// \brief   A jmshower is a \ref rb::Shower with energy and PID information
/// \author  bianjm@umn.physics.edu
////////////////////////////////////////////////////////////////////////
#ifndef JMSHOWER_H
#define JMSHOWER_H

#include "GeometryObjects/OfflineChan.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Shower.h"

#include <vector>

class TVector3;
class TLorentzVector;

namespace jmshower
{
  class PCluster : public rb::Cluster
  {
  public:
    PCluster();
    virtual ~PCluster();
    void SetPlane(unsigned ip) {fP = ip;}
    unsigned int  Plane() {return fP;}

  private:
    unsigned int fP;//Plane
  };

  class TCCluster : public rb::Cluster
  {
  public:
    TCCluster();
    virtual ~TCCluster();
    void SetTCell(int itc) {fTc = itc;}
    int  TCell() {return fTc;}
  private:
    unsigned int fTc;//cell to shower core
  };

  enum PidType
  {
    kNull = -1,

    kElectron,
    kGamma,
    kMuon,
    kPi0,
    kHad,
    kProton,
    kNeutron,
    kPion,
    kElectronQE,
    kElectronRES,
    kElectronDIS,
    kElectronCOH,

    kLast
  };


  /// \brief TODO
  class JMShower : public rb::Shower
  {
  public:
    
    JMShower(){}
    virtual ~JMShower(){}
#ifndef __GCCXML__
    JMShower(const rb::Prong& prong,
          int id = 0,
          double length = 0);

    //************************************
    void SetWindowSize(unsigned int nplane);
    void SetHitColSize(unsigned int ncell);

    void SetIsFiducial(bool isfiducial);
    bool IsFiducial() const;

    void SetDistToEdge(double disttoedge);
    double DistToEdge()const;

    void SetBackTag(bool backtag);
    bool BackTag() const;

    void SetGap(double gap);
    double Gap() const;

    void SetContGap(double gap);
    double ContGap() const;

    void SetVtxDoca(double vtxdoca);
    double VtxDoca() const;


    void SetDepositEnergy(double depositenergy);
    double DepositEnergy() const;

    void SetExclEnergy(double exclenergy);
    double ExclEnergy() const;


    void SetEnergy(double energy);
    double Energy() const;
    void SetVtxGeV(double gev);
    double VtxGeV() const;

    void SetVtxCenterGeV(double gev);
    double VtxCenterGeV() const;
    void SetPi0Mgg(double pi0mass);
    double Pi0Mgg() const;

    void SetPi0G2ID(int id);
    int Pi0G2ID() const;


    void SetTheta(double theta);
    double Theta() const;

    void SetTrkGeV(double gev);
    double TrkGeV() const;

    void SetSliceGeV(double gev);
    double SliceGeV() const;

    void SetSliceEtot(double etot);
    double SliceEtot() const;

    void SetNueEnergy(double nueenergy);
    double NueEnergy() const;


    void SetSliceEdge10GeV(double gev);
    double SliceEdge10GeV() const;
    void SetSliceEdge5GeV(double gev);
    double SliceEdge5GeV() const;
    void SetSliceEdge2GeV(double gev);
    double SliceEdge2GeV() const;


    void SetNCellToEdge(int nce);
    int NCellToEdge() const;
    void SetClosetEdge(int edge);
    int ClosetEdge() const;
    void SetEdge20GeV(double gev);
    double Edge20GeV() const;
    void SetEdge15GeV(double gev);
    double Edge15GeV() const;
    void SetEdge10GeV(double gev);
    double Edge10GeV() const;
    void SetEdge5GeV(double gev);
    double Edge5GeV() const;
    void SetEdge2GeV(double gev);
    double Edge2GeV() const;

    void SetSliceExclGeV(double gev);
    double SliceExclGeV() const;

    void SetRadius(double radius);
    double Radius() const;

    void SetSumDRadius(double dradius);
    double SumDRadius() const;

    void SetSum10DRadius(double dradius);
    double Sum10DRadius() const;


    void SetCentroid(TVector3 centroid);
    TVector3 Centroid() const;


    void SetStartPlane(unsigned int startplane);
    unsigned int StartPlane() const;
    void SetStopPlane(unsigned int stopplane);
    unsigned int StopPlane() const;
    void SetNPlane(unsigned int nplane);
    unsigned int NPlane() const;

    void SetXNPlane(unsigned int xnplane);
    unsigned int XNPlane() const;
    void SetYNPlane(unsigned int ynplane);
    unsigned int YNPlane() const;

    void SetXMinPlane(unsigned int xminplane);
    unsigned int XMinPlane() const;
    void SetYMinPlane(unsigned int yminplane);
    unsigned int YMinPlane() const;

    void SetXMaxPlane(unsigned int xmaxplane);
    unsigned int XMaxPlane() const;
    void SetYMaxPlane(unsigned int ymaxplane);
    unsigned int YMaxPlane() const;




    void SetStop(TVector3 stop);
    TVector3 Stop() const;


    void SetPlaneHitXYZ(unsigned int plane, TVector3 xyz);
    TVector3 PlaneHitXYZ(unsigned int plane) const;
    void SetPlaneHitCell(unsigned int plane, unsigned int cell);
    unsigned int PlaneHitCell(unsigned int plane) const;
    void SetPlaneE1Cell(unsigned int plane, int cell);
    int PlaneE1Cell(unsigned int plane) const;
    void SetPlaneE2Cell(unsigned int plane, int cell);
    int PlaneE2Cell(unsigned int plane) const;
    void SetPlaneCentroidCell(unsigned int plane, int cell);
    int PlaneCentroidCell(unsigned int plane) const;

    void SetPlaneRadius(unsigned int plane,double planerad);
    double PlaneRadius(unsigned int plane) const;

    void SetPlaneDedx(unsigned int plane,double planededx);
    double PlaneDedx(unsigned int plane) const;

    void SetDedxLongLL(double dedxlongll, int type);
    double DedxLongLL(int type) const;

    void SetDedxInvLongLL(double dedxlongll, int type);
    double DedxInvLongLL(int type) const;


    void SetPlaneProb(unsigned int plane, double planeprob, int type);
    double PlaneProb(unsigned int plane, int type) const;

    void SetCellTransDedx(unsigned int tcell,double celldedx);
    double CellTransDedx(unsigned int tcell) const;

    void SetDedxTransLL(double dedxtransll, int type);
    double DedxTransLL(int type) const;

    void SetIsMuon(bool ismuon);
    bool IsMuon() const;

    void SetIsInvPhoton(bool isinvphoton);
    bool IsInvPhoton() const;



    void SetNMIPPlane(int nmipplane);
    int NMIPPlane() const;

    void SetISlice(unsigned int islice);
    unsigned int ISlice() const;

    void SetHitDist(unsigned int cell, double dist);
    double HitDist(unsigned int cell) const;


    void SetHitDeconvE(unsigned int cell, double deconve);
    double HitDeconvE(unsigned int cell) const;

    void SetHitSumE(unsigned int cell, double sume);
    double HitSumE(unsigned int cell) const;

    void SetNodes(std::vector<TVector3> nodespos, std::vector<double> nodese, std::vector<geo::OfflineChan> xid, std::vector<geo::OfflineChan> yid);
    int  NNode() const;
    TVector3 NodePos(int node) const;
    double NodeE(int node) const;
    geo::OfflineChan NodeXID(int node)const;
    geo::OfflineChan NodeYID(int node)const;

    void SetIxyz(TVector3 ixyz);
    TVector3 Ixyz()const;

    void SetAsym(double asym);
    double Asym()const;


    void SetANN(unsigned int mode, double ann);
    double ANN(unsigned int mode) const;

    double Efrac_2slides() const;//maximal energy fraction in 2 sliding window
    void SetEfrac_2slides(double efrac_2slides);
    double Efrac_4slides() const;//maximal energy fraction in 4 sliding window
    void SetEfrac_4slides(double efrac_4slides);
    double Efrac_6slides() const;//maximal energy fraction in 6 sliding window
    void SetEfrac_6slides(double efrac_6slides);
    double Efrac_20slides() const;//energy fraction in first 20 slides
    void SetEfrac_20slides(double efrac_20slides);
    double Efrac_2sigmaHOR() const;//energy fraction in 2 sigma road
    void SetEfrac_2sigmaHOR(double efrac_2sigmaHOR);
    double Efrac_4sigmaHOR() const;//energy fraction in 4 sigma road
    void SetEfrac_4sigmaHOR(double efrac_4sigmaHOR);

    double Eiso_2sigmaHOR() const;//energy isolation of 2 sigma road
    void SetEiso_2sigmaHOR(double eiso_2sigmaHOR);
    double Eiso_4sigmaHOR() const;//energy isolation of 4 sigma road
    void SetEiso_4sigmaHOR(double eiso_4sigmaHOR);
    double Eiso_6sigmaHOR() const;//energy isolation 6 sigma road
    void SetEiso_6sigmaHOR(double eiso_6sigmaHOR);


    /// \brief Returns a map where the first element is
    /// plane number and the second is vector of global
    /// cell indices in a shower of cells that belong to
    /// that plane.
    std::map< int, std::vector< int > > GetPlaneHits();

    /// \brief Returns a map where the first index is the
    /// number of cells away from the track core, and the
    /// second is a vector of global cell indices in a
    /// shower that are that many cells from the track
    /// core. It callapses the transverse profile of the
    /// shower to a single plane.
    std::map< int, std::vector< int > > GetTransHits();

#endif // __GCCXML__

  private:
    std::vector<double> fHitDistCol;
    std::vector<double> fHitDeconvECol;
    std::vector<double> fHitSumECol;

    std::vector<TVector3> fNodesPos;
    std::vector<double> fNodesE;
    std::vector<geo::OfflineChan> fNodesXID;
    std::vector<geo::OfflineChan> fNodesYID;
    int fNNode;

    TVector3 fStop;
    bool fIsFiducial;
    double fDistToEdge;
    double fDepositEnergy;
    double fExclEnergy;
    double fEnergy;
    double fVtxGeV;
    double fVtxCenterGeV;
    double fPi0Mgg;
    int fPi0G2ID;
    double fTheta;
    double fTrkGeV;
    double fSliceGeV;
    double fSliceEtot;
    double fNueEnergy;
    double fSliceEdge10GeV;
    double fSliceEdge5GeV;
    double fSliceEdge2GeV;

    int fNCellToEdge;
    int fClosetEdge;
    double fEdge20GeV;
    double fEdge15GeV;
    double fEdge10GeV;
    double fEdge5GeV;
    double fEdge2GeV;


    double fSliceExclGeV;
    double fRadius;
    double fSumDRadius;
    double fSum10DRadius;
    unsigned fStartPlane;
    unsigned fStopPlane;
    unsigned fNPlane;
    unsigned fXNPlane;
    unsigned fYNPlane;
    unsigned fXMinPlane;
    unsigned fYMinPlane;
    unsigned fXMaxPlane;
    unsigned fYMaxPlane;

    int fNMIPPlane;
    unsigned int fISlice;
    std::vector<double> fPlaneDedx;
    std::vector<double> fPlaneRadius;
    std::vector<double> fPlaneProbE;
    std::vector<double> fPlaneProbEqe;
    std::vector<double> fPlaneProbEres;
    std::vector<double> fPlaneProbEdis;
    std::vector<double> fPlaneProbEcoh;

    std::vector<double> fPlaneProbG;
    std::vector<double> fPlaneProbMu;
    std::vector<double> fPlaneProbPi0;
    std::vector<double> fPlaneProbHad;
    std::vector<double> fPlaneProbProton;
    std::vector<double> fPlaneProbNuetron;
    std::vector<double> fPlaneProbPion;

    std::vector<TVector3> fPlaneHitXYZ;

    std::vector<int> fPlaneE1Cell;
    std::vector<int> fPlaneE2Cell;
    std::vector<int> fPlaneCentroidCell;
    std::vector<unsigned int> fPlaneHitCell;


    double fDedxLongLL[20];
    double fDedxInvLongLL[20];
    double  fCelltransDedx[20];
    double fDedxTransLL[20];
    bool fIsMuon;
    bool fIsInvPhoton;
    bool fBackTag;
    double fGap;
    double fContGap;
    double fVtxDoca;
    TVector3 fCentroid;
    TVector3 fIxyz;
    double fAsym;

    double fANN[20];

    double fEfrac_2slides;
    double fEfrac_4slides;
    double fEfrac_6slides;
    double fEfrac_20slides;
    double fEfrac_2sigmaHOR;
    double fEfrac_4sigmaHOR;
    double fEiso_2sigmaHOR;
    double fEiso_4sigmaHOR;
    double fEiso_6sigmaHOR;
  };
}

#endif // JMSHOWER_H
