////////////////////////////////////////////////////////////////////////
/// \brief  Modified from TrackMerge
/// \author Jianming Bian
////////////////////////////////////////////////////////////////////////
#include <cmath>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>

#include "TH1F.h"
#include "TTree.h"
#include "TVector3.h"

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h" 
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h" 
#include "art/Persistency/Common/Ptr.h"
#include "art/Persistency/Common/PtrVector.h"	
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "Calibrator/Calibrator.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "Utilities/func/MathUtil.h"


namespace jmshower {
  class JMTrackMerge : public art::EDProducer
    {
    public:
      JMTrackMerge(fhicl::ParameterSet const &pset); 
      ~JMTrackMerge();                        
      void produce(art::Event &evt);
      void reconfigure(const fhicl::ParameterSet& pset);
      void beginJob();

      
    private:
      std::string pSliceDir;
      std::string fTrackLabel;
      std::string fOutputFolder;        ///< Output folder for Clusters, Prongs, Tracks
      double      fSigmaStart;          ///<
      double      fSigmaStop;           ///<
      double      fSigmaChi;            ///<
      double      fMaxThresh;           ///<
      double      fMinThresh;           ///<
      double      fMaxScore;            ///<
      int         fMinPlaneLength;      ///<
      bool        fMCCheater;           ///< True/false MCCheater SimHit/SimTrack input
      TH1F*       fHistoStart;          ///<
      TH1F*       fHistoStop;           ///<
      TH1F*       fHistoStartLog;       ///< Easier to read in the reco validation.
      TH1F*       fHistoStopLog;        ///< Easier to read in the reco validation.
      TH1F*       fHistoChi;            ///<
      TH1F*       fHistoTrack;          ///<
      TH1F*       fHistoChiZoom;        ///<
      TH1F*       fHistoTrackScore;     ///<
      TH1F*       fHistoTrackScoreLog;  ///< Easier to read in the reco validation.
      TH1F*       fHistoPurity;         ///<
      TH1F*       fHistoRemainingCells; ///<
      TH1F*       fHistoEfficiency;     ///<

      TTree*      fTrack;
      int    m_trackTrueNuCCNC;
      int    m_trackTrueNuMode;
      int    m_trackTrueNuPdg;
      double m_trackTrueNuEnergy;
      int    m_trackTrueNuIsFid;
      double m_trackTrueNuP4[4];
      double m_trackTrueNuVtx[3];
      int    m_trackEvtTruePdg[20];
      double m_trackEvtTrueEnergy[20];
      double m_trackEvtTrueElecPhotons;
      
      
      int m_trackTrueElecTag;
      int m_trackTrueElecTag1;
      int m_trackTrueElecTag2;
      
      double m_trackTrueDang;
      double m_trackTrueEDang;
      int m_trackTruePdg;
      int m_trackTrueId;
      double m_trackTrueVtx[3];
      double m_trackTrueP4[4];
      double m_trackTruePartHit[10];
      double m_trackTruePartEnergy[10];
      
      double m_trackTrueDepEnergy;
      double m_trackTrueElecPhotons;
      double m_trackTrueTotPhotons;
      double m_trackTrueTrkElecPhotons;
      double m_trackTrueTrkElecPhotons1;
      double m_trackTrueTrkElecPhotons2;
      double m_trackTrueTrkTotPhotons;
      double m_trackTrueTrkTotPhotons1;
      double m_trackTrueTrkTotPhotons2;
      double m_trackTrueRWElecPhotons;
      double m_trackTrueRWTotPhotons;
      double m_trackTrueTrkRWElecPhotons;
      double m_trackTrueTrkRWTotPhotons;
      
      double    m_trackEvtMinStart;
      int    m_trackNHit1;
      int    m_trackNHit2;
      int    m_trackNPlane1;
      int    m_trackNPlane2;
      double    m_trackChisq1;
      double    m_trackChisq2;
      double    m_trackChisq;
      double m_trackxzSlope;
      double m_trackyzSlope;
      double m_trackxzIntercept;
      double m_trackyzIntercept;
      double m_trackLength;
      
      double    m_trackGap1;
      double    m_trackGap2;
      double    m_trackOverlap;
      double    m_trackOut1;
      double    m_trackOut2;
      double    m_trackPE1;
      double    m_trackPE2;
      double    m_trackPEM1;
      double    m_trackPEM2;
      double    m_trackPEOut1;
      double    m_trackPEOut2;
      double    m_trackPESub1;
      double    m_trackPESub2;
      
      
      double    m_trackDir[3];
      double    m_trackStart1[3];
      double    m_trackStop1[3];
      double    m_trackStart2[3];
      double    m_trackStop2[3];
      
      
      int m_trackBtTrueElecTag;
      int m_trackBtTrueElecTag1;
      int m_trackBtTrueElecTag2;
      
      int m_trackBestChisqTag;
      int m_trackBestPETag;
      int m_trackBestPlaneTag;
      int m_trackBestOverlapTag;
      
      double m_trackScore; 
      double m_trackCBScore; 
      double m_trackPEMatch;
      double m_trackPEMatchM;
      double m_trackPEMatchSub;
      double m_trackPlaneMatch;
      double m_trackOverlapMatch;
      
      double m_trackBestScore1; 
      double m_trackBestCBScore1; 
      double m_trackBestPEMatch1; 
      double m_trackBestPlaneMatch1; 
      double m_trackBestOverlapMatch1; 
      
      double m_trackBestScore2; 
      double m_trackBestCBScore2; 
      double m_trackBestPEMatch2; 
      double m_trackBestPlaneMatch2; 
      double m_trackBestOverlapMatch2; 
      int m_trackMergeTag; 
      
      
      
      double m_trackBtTrueDang;
      double m_trackBtTrueEDang;
      int m_trackBtTruePdg;
      int m_trackBtTrueId;
      double m_trackBtTrueVtx[3];
      double m_trackBtTrueP4[4];
      double m_trackBtTruePartHit[10];
      double m_trackBtTruePartEnergy[10];
      
      double m_trackBtTrueDepEnergy;
      double m_trackBtTrueElecPhotons;
      double m_trackBtTrueTotPhotons;
      double m_trackBtTrueTrkElecPhotons;
      double m_trackBtTrueTrkElecPhotons1;
      double m_trackBtTrueTrkElecPhotons2;
      double m_trackBtTrueTrkTotPhotons;
      double m_trackBtTrueTrkTotPhotons1;
      double m_trackBtTrueTrkTotPhotons2;
      double m_trackBtTrueRWElecPhotons;
      double m_trackBtTrueRWTotPhotons;
      double m_trackBtTrueTrkRWElecPhotons;
      double m_trackBtTrueTrkRWTotPhotons;
      
      int    m_trackBtNHit1;
      int    m_trackBtNHit2;
      int    m_trackBtNPlane1;
      int    m_trackBtNPlane2;
      double    m_trackBtChisq1;
      double    m_trackBtChisq2;
      double    m_trackBtChisq;
      double    m_trackBtGap1;
      double    m_trackBtGap2;
      double    m_trackBtOverlap;
      double    m_trackBtPE1;
      double    m_trackBtPE2;
      
      double    m_trackBtDir[3];
      double    m_trackBtStart1[3];
      double    m_trackBtStop1[3];
      double    m_trackBtStart2[3];
      double    m_trackBtStop2[3];
  };
}


namespace jmshower 
{
  //...............................................................
  JMTrackMerge::JMTrackMerge(fhicl::ParameterSet const& pset)
  {
    reconfigure(pset);
    produces< std::vector<rb::Track>   >();
  }
  
  //......................................................................
  void JMTrackMerge::reconfigure(const fhicl::ParameterSet& pset)
  {
    pSliceDir       = pset.get< std::string >("SliceDir" );
    fTrackLabel     = pset.get< std::string >("TrackLabel" );
    fSigmaStart     = pset.get< double >("SigmaStart"    );	
    fSigmaStop      = pset.get< double >("SigmaStop"     );
    fSigmaChi       = pset.get< double >("SigmaChi"      );
    fMaxThresh      = pset.get< double >("MaxThresh"     );
    fMinThresh      = pset.get< double >("MinThresh"     );
    fMaxScore       = pset.get< double >("MaxScore"      );
    fMinPlaneLength = pset.get< int    >("MinPlaneLength");
    fMCCheater      = pset.get< bool   >("MCCheater"     );
  }
  
  //......................................................................
  JMTrackMerge::~JMTrackMerge()
  {
  }
  
  
  //......................................................................
  void JMTrackMerge::beginJob()
  {
    art::ServiceHandle<art::TFileService> tfs;
    fHistoStart = tfs->make<TH1F>("fHistoStart","StartZ Diff Squared;StartZ Diff;events", 10000,0.,10000.);
    fHistoStartLog = tfs->make<TH1F>("fHistoStartLog","StartZ Diff Squared;Log10[StartZDiff + 1];events", 50,0.,4.);
    fHistoChiZoom = tfs->make<TH1F>("fHistoChiZoom","Zoom Chi2;Chi2 diff;events", 100,0.,5.);
    fHistoStop = tfs->make<TH1F>("fHistoStop","StopZ Diff Squared;StartZ Diff;events", 10000,0.,10000.);
    fHistoStopLog = tfs->make<TH1F>("fHistoStopLog","StopZ Diff Squared;Log10[StopZ Diff + 1];events", 50,0.,4.);
    fHistoChi = tfs->make<TH1F>("fHistoChi","Chi2 Diff Squared;Chi2 Diff;events", 1000,0.,1000.);
    fHistoTrack = tfs->make<TH1F>("fHistoTrack","Tracks;Tracks;events", 30,0.,30.);
    fHistoTrackScore = tfs->make<TH1F>("fHistoTrackScore","Track Score;score;events", 1000,0.,1000.);
    fHistoPurity = tfs->make<TH1F>("fHistoPurity",
				   "Purity of Clusters;(Hits of true particle/total hits in cluster);events",101,0.,1.01);
    fHistoRemainingCells = tfs->make<TH1F>("fHistoRemainingCells",
					   "True hits not put into a 3D prong;Missing Cells;events",100,0.,200.);
    fHistoEfficiency = tfs->make<TH1F>("fHistoEfficiency",
				       "Ratio of hits of one type in a prong to total hits of that type;Ratio;events",101,0.,1.01);
    fHistoTrackScoreLog = tfs->make<TH1F>("fHistoTrackScoreLog","Track Score;Log10[score + 1];events", 50,0.,3.);
  
    fTrack = tfs->make<TTree>("fTrack","fTrack");
      fTrack->Branch("trackTrueNuCCNC", &m_trackTrueNuCCNC);
      fTrack->Branch("trackTrueNuMode", &m_trackTrueNuMode);
      fTrack->Branch("trackTrueNuPdg", &m_trackTrueNuPdg);
      fTrack->Branch("trackTrueNuEnergy", &m_trackTrueNuEnergy);
      fTrack->Branch("trackTrueNuIsFid", &m_trackTrueNuIsFid);
      fTrack->Branch("trackTrueNuP4[4]", m_trackTrueNuP4);
      fTrack->Branch("trackTrueNuVtx[3]", m_trackTrueNuVtx);
      fTrack->Branch("trackEvtTrueEnergy[20]", m_trackEvtTrueEnergy);
      fTrack->Branch("trackEvtTruePdg[20]", m_trackEvtTruePdg);
      fTrack->Branch("trackEvtTrueElecPhotons",&m_trackEvtTrueElecPhotons);

      fTrack->Branch("trackTrueDang", &m_trackTrueDang);
      fTrack->Branch("trackTrueEDang", &m_trackTrueEDang);
      fTrack->Branch("trackTruePdg", &m_trackTruePdg);
      fTrack->Branch("trackTrueId", &m_trackTrueId);
      fTrack->Branch("trackTrueVtx[3]", m_trackTrueVtx);
      fTrack->Branch("trackTrueP4[4]", m_trackTrueP4);
      fTrack->Branch("trackTruePartHit[10]", m_trackTruePartHit);
      fTrack->Branch("trackTruePartEnergy[10]", m_trackTruePartEnergy);
      fTrack->Branch("trackTrueDepEnergy", &m_trackTrueDepEnergy);
      fTrack->Branch("trackTrueElecPhotons", &m_trackTrueElecPhotons);
      fTrack->Branch("trackTrueTotPhotons", &m_trackTrueTotPhotons);
      fTrack->Branch("trackTrueTrkElecPhotons", &m_trackTrueTrkElecPhotons);
      fTrack->Branch("trackTrueTrkTotPhotons", &m_trackTrueTrkTotPhotons);
      fTrack->Branch("trackTrueTrkElecPhotons1", &m_trackTrueTrkElecPhotons1);
      fTrack->Branch("trackTrueTrkTotPhotons1", &m_trackTrueTrkTotPhotons1);
      fTrack->Branch("trackTrueTrkElecPhotons2", &m_trackTrueTrkElecPhotons2);
      fTrack->Branch("trackTrueTrkTotPhotons2", &m_trackTrueTrkTotPhotons2);
      fTrack->Branch("trackTrueElecTag",&m_trackTrueElecTag);
      fTrack->Branch("trackTrueElecTag1",&m_trackTrueElecTag1);
      fTrack->Branch("trackTrueElecTag2",&m_trackTrueElecTag2);
      fTrack->Branch("trackBestChisqTag",  &m_trackBestChisqTag);
      fTrack->Branch("trackBestPETag",  &m_trackBestPETag);
      fTrack->Branch("trackBestPlaneTag",  &m_trackBestPlaneTag);
      fTrack->Branch("trackBestOverlapTag",  &m_trackBestOverlapTag);

      fTrack->Branch("trackEvtMinStart", &m_trackEvtMinStart);
      fTrack->Branch("trackNHit1", &m_trackNHit1);
      fTrack->Branch("trackNHit2",&m_trackNHit2);;
      fTrack->Branch("trackNPlane1",&m_trackNPlane1);
      fTrack->Branch("trackNPlane2",&m_trackNPlane2);
      fTrack->Branch("trackChisq1",&m_trackChisq1);
      fTrack->Branch("trackChisq2",&m_trackChisq2);
      fTrack->Branch("trackChisq",&m_trackChisq);
      fTrack->Branch("trackxzSlope",&m_trackxzSlope);
      fTrack->Branch("trackyzSlope",&m_trackyzSlope);
      fTrack->Branch("trackxzIntercept",&m_trackxzIntercept);
      fTrack->Branch("trackyzIntercept",&m_trackyzIntercept);
      fTrack->Branch("trackLength",&m_trackLength);

      fTrack->Branch("trackGap1",&m_trackGap1);
      fTrack->Branch("trackGap2",&m_trackGap2);
      fTrack->Branch("trackOverlap",&m_trackOverlap);
      fTrack->Branch("trackOut1",&m_trackOut1);
      fTrack->Branch("trackOut2",&m_trackOut2);
      fTrack->Branch("trackPE1",&m_trackPE1);
      fTrack->Branch("trackPE2",&m_trackPE2);
      fTrack->Branch("trackPEM1",&m_trackPEM1);
      fTrack->Branch("trackPEM2",&m_trackPEM2);
      fTrack->Branch("trackPEOut1",&m_trackPEOut1);
      fTrack->Branch("trackPEOut2",&m_trackPEOut2);
      fTrack->Branch("trackPESub1",&m_trackPESub1);
      fTrack->Branch("trackPESub2",&m_trackPESub2);


      fTrack->Branch("trackDir[3]", m_trackDir);
      fTrack->Branch("trackStart1[3]", m_trackStart1);
      fTrack->Branch("trackStop1[3]", m_trackStop1);
      fTrack->Branch("trackStart2[3]", m_trackStart2);
      fTrack->Branch("trackStop2[3]", m_trackStop2);
  
      fTrack->Branch("trackScore",  &m_trackScore);
      fTrack->Branch("trackCBScore",  &m_trackCBScore);
      fTrack->Branch("trackPEMatch",  &m_trackPEMatch);
      fTrack->Branch("trackPEMatchM",  &m_trackPEMatchM);
      fTrack->Branch("trackPEMatchSub",  &m_trackPEMatchSub);
      fTrack->Branch("trackPlaneMatch",  &m_trackPlaneMatch);
      fTrack->Branch("trackOverlapMatch",  &m_trackOverlapMatch);

      fTrack->Branch("trackBestScore1",  &m_trackBestScore1);
      fTrack->Branch("trackBestCBScore1",  &m_trackBestCBScore1);
      fTrack->Branch("trackBestPEMatch1",  &m_trackBestPEMatch1);
      fTrack->Branch("trackBestPlaneMatch1",  &m_trackBestPlaneMatch1);
      fTrack->Branch("trackBestOverlapMatch1",  &m_trackBestOverlapMatch1);

      fTrack->Branch("trackBestScore2",  &m_trackBestScore2);
      fTrack->Branch("trackBestCBScore2",  &m_trackBestCBScore2);
      fTrack->Branch("trackBestPEMatch2",  &m_trackBestPEMatch2);
      fTrack->Branch("trackBestPlaneMatch2",  &m_trackBestPlaneMatch2);
      fTrack->Branch("trackBestOverlapMatch2",  &m_trackBestOverlapMatch2);
      fTrack->Branch("trackMergeTag",  &m_trackMergeTag);



      fTrack->Branch("trackBtTrueDang", &m_trackBtTrueDang);
      fTrack->Branch("trackBtTrueEDang", &m_trackBtTrueEDang);
      fTrack->Branch("trackBtTruePdg", &m_trackBtTruePdg);
      fTrack->Branch("trackBtTrueId", &m_trackBtTrueId);
      fTrack->Branch("trackBtTrueVtx[3]", m_trackBtTrueVtx);
      fTrack->Branch("trackBtTrueP4[4]", m_trackBtTrueP4);
      fTrack->Branch("trackBtTruePartHit[10]", m_trackBtTruePartHit);
      fTrack->Branch("trackBtTruePartEnergy[10]", m_trackBtTruePartEnergy);
      fTrack->Branch("trackBtTrueDepEnergy", &m_trackBtTrueDepEnergy);
      fTrack->Branch("trackBtTrueElecPhotons", &m_trackBtTrueElecPhotons);
      fTrack->Branch("trackBtTrueTotPhotons", &m_trackBtTrueTotPhotons);
      fTrack->Branch("trackBtTrueTrkElecPhotons", &m_trackBtTrueTrkElecPhotons);
      fTrack->Branch("trackBtTrueTrkTotPhotons", &m_trackBtTrueTrkTotPhotons);
      fTrack->Branch("trackBtTrueTrkElecPhotons1", &m_trackBtTrueTrkElecPhotons1);
      fTrack->Branch("trackBtTrueTrkTotPhotons1", &m_trackBtTrueTrkTotPhotons1);
      fTrack->Branch("trackBtTrueTrkElecPhotons2", &m_trackBtTrueTrkElecPhotons2);
      fTrack->Branch("trackBtTrueTrkTotPhotons2", &m_trackBtTrueTrkTotPhotons2);
      fTrack->Branch("trackBtTrueElecTag",&m_trackBtTrueElecTag);
      fTrack->Branch("trackBtTrueElecTag1",&m_trackBtTrueElecTag1);
      fTrack->Branch("trackBtTrueElecTag2",&m_trackBtTrueElecTag2);

      fTrack->Branch("trackBtNHit1", &m_trackBtNHit1);
      fTrack->Branch("trackBtNHit2",&m_trackBtNHit2);;
      fTrack->Branch("trackBtNPlane1",&m_trackBtNPlane1);
      fTrack->Branch("trackBtNPlane2",&m_trackBtNPlane2);
      fTrack->Branch("trackBtChisq1",&m_trackBtChisq1);
      fTrack->Branch("trackBtChisq2",&m_trackBtChisq2);
      fTrack->Branch("trackBtChisq",&m_trackBtChisq);
      fTrack->Branch("trackBtGap1",&m_trackBtGap1);
      fTrack->Branch("trackBtGap2",&m_trackBtGap2);
      fTrack->Branch("trackBtOverlap",&m_trackBtOverlap);
      fTrack->Branch("trackBtPE1",&m_trackBtPE1);
      fTrack->Branch("trackBtPE2",&m_trackBtPE2);
      fTrack->Branch("trackBtDir[3]", m_trackBtDir);
      fTrack->Branch("trackBtStart1[3]", m_trackBtStart1);
      fTrack->Branch("trackBtStop1[3]", m_trackBtStop1);
      fTrack->Branch("trackBtStart2[3]", m_trackBtStart2);
      fTrack->Branch("trackBtStop2[3]", m_trackBtStop2);
  }
  
  //......................................................................
  void JMTrackMerge::produce(art::Event& evt)
  {

    art::Handle<std::vector<rb::Cluster> > slicecol;
    evt.getByLabel(pSliceDir,slicecol);
    art::PtrVector<rb::Cluster> slicelist;

    for(unsigned int i = 0; i < slicecol->size(); ++i){
      art::Ptr<rb::Cluster>slice(slicecol,i);
      slicelist.push_back(slice);
    }

    art::Handle< std::vector<rb::Track> > trackhandle;
    evt.getByLabel(fTrackLabel,trackhandle);
    std::unique_ptr< std::vector<rb::Track> > trackcol(new std::vector<rb::Track> );
    std::vector<art::Ptr<rb::Track> >  track;
    std::vector<art::Ptr<rb::Track> >  xzTrack;
    std::vector<art::Ptr<rb::Track> >  yzTrack;
    
    std::vector<rb::Cluster> vCluster;
    unsigned int cc = 0;
    art::ServiceHandle<geo::Geometry> geom;   
    geo::View_t view;
    art::ServiceHandle<calib::Calibrator> cal;


    
    std::vector< int > trId;
    std::vector< int > hitcount;
    std::vector< std::vector< int > > trackeff;

    //get slices  for timing information

    std::vector< std::pair< double, double> > tMap; //contains min and max time for each slice in event, first number is min, secont max
    for (unsigned int i=0; i<slicecol->size(); ++i) {
      const rb::Cluster& s = (*slicecol)[i];
      if (s.IsNoise()) continue;
      std::pair< double, double> p(s.MinTNS(), s.MaxTNS()); //add min and max time as a pair for a given slice
      tMap.push_back(p);
    }
    
    for(unsigned int i = 0; i < trackhandle->size(); i++){
      art::Ptr<rb::Track> p(trackhandle, i);
      track.push_back(p);
    }
    for(unsigned int i=0;i<track.size();i++){
      if (track[i]->NXCell() > 0){
	xzTrack.push_back(track[i]);
      }
      if (track[i]->NYCell() > 0){
	yzTrack.push_back(track[i]);
      }
    }

    std::vector< std::vector<double> > starts;
    starts.resize(xzTrack.size()+yzTrack.size());
    std::vector< std::vector<double> > ends;
    ends.resize(xzTrack.size()+yzTrack.size());
    std::vector< std::vector<double> > dirs;
    dirs.resize(xzTrack.size()+yzTrack.size());
    std::vector<double> signal;
    signal.resize(xzTrack.size()+yzTrack.size());
   
    fHistoTrack->Fill(track.size());
    
    std::vector<int>  ntxtrkelecphotonscol;
    std::vector<int>  ntytrkelecphotonscol;
    std::vector<int>  ntxtrktotphotonscol;
    std::vector<int>  ntytrktotphotonscol;
    ntxtrkelecphotonscol.clear();
    ntytrkelecphotonscol.clear();
    ntxtrktotphotonscol.clear();
    ntytrktotphotonscol.clear();

    
    std::vector<std::vector< art::Ptr<rb::CellHit> > > xzCells(xzTrack.size());
    view = geo::kX;
    for (unsigned int i=0;i<xzTrack.size();i++) { 
      int trackElecPhotons = 0;
      int trackTotPhotons = 0;

      ntxtrkelecphotonscol.push_back(trackElecPhotons);
      ntxtrktotphotonscol.push_back(trackTotPhotons);


      for ( unsigned int nh=0;nh<xzTrack[i]->NCell();nh++) {
	xzCells[i].push_back(xzTrack[i]->Cell(view,nh));
      }
    }  
    std::vector<std::vector< art::Ptr<rb::CellHit> > > yzCells(yzTrack.size());
    view = geo::kY;
    for (unsigned int i=0;i<yzTrack.size();i++) {
      int trackElecPhotons = 0;
      int trackTotPhotons = 0;

      ntytrkelecphotonscol.push_back(trackElecPhotons);
      ntytrktotphotonscol.push_back(trackTotPhotons);

      for ( unsigned int nh=0;nh<yzTrack[i]->NCell();nh++) {
	yzCells[i].push_back(yzTrack[i]->Cell(view,nh));
      }
    } 

    double xzSlope[xzTrack.size()][yzTrack.size()];
    double xzIntercept[xzTrack.size()][yzTrack.size()];
    double yzSlope[xzTrack.size()][yzTrack.size()];
    double yzIntercept[xzTrack.size()][yzTrack.size()];

    double xyz[3] = {0.,0.,0.};

    std::vector<art::Ptr<rb::CellHit> >::iterator itr; 

    using util::sqr;

    double score[xzTrack.size()][yzTrack.size()];
    double cbscore[xzTrack.size()][yzTrack.size()];
    double pematch[xzTrack.size()][yzTrack.size()];
    double planematch[xzTrack.size()][yzTrack.size()];
    double overlapmatch[xzTrack.size()][yzTrack.size()];

    double minstart = 99999;
    for (unsigned int i=0;i<xzTrack.size();i++){
      for (unsigned int j=0;j<yzTrack.size();j++){

        double start1 = std::min(xzTrack[i]->Start().Z(), xzTrack[i]->Stop().Z());
        double stop1  = std::max(xzTrack[i]->Start().Z(), xzTrack[i]->Stop().Z());
        double start2 = std::min(yzTrack[j]->Start().Z(), yzTrack[j]->Stop().Z());
        double stop2  = std::max(yzTrack[j]->Start().Z(), yzTrack[j]->Stop().Z());

        double gap1 = (start1-start2)/double(xzTrack[i]->ExtentPlane()+yzTrack[j]->ExtentPlane()); 
        double gap2 = (stop1-stop2)/double(xzTrack[i]->ExtentPlane()+yzTrack[j]->ExtentPlane());

        if(fabs(gap1)>fabs(gap2)){
          double gaptmp = gap1;
          gap1 = gap2;
          gap2 = gaptmp; 
        }

          art::PtrVector<rb::CellHit> mergecells;
          double sig1 = 0.;
          double sig2 = 0.;
          double sig1match = 0.;
          double sig2match = 0.;

          double wx = 0, wy=0;
          if(yzTrack[j]->ExtentPlane()==1){
            double ycellXYZ[3];
            geom->Plane(yzTrack[j]->YCell(0)->Plane())->Cell(yzTrack[j]->YCell(0)->Cell())->GetCenter(ycellXYZ);
            wx = ycellXYZ[1];
          }

          if(xzTrack[i]->ExtentPlane()==1){
            double xcellXYZ[3];
            geom->Plane(xzTrack[i]->XCell(0)->Plane())->Cell(xzTrack[i]->XCell(0)->Cell())->GetCenter(xcellXYZ);
            wy = xcellXYZ[0];
          }

          itr = xzCells[i].begin();
          std::vector<double> xzViewX;
          std::vector<double> xzViewZ;
          std::vector<double> xzViewW;
          xzViewX.clear();
          xzViewZ.clear();
          xzViewW.clear();

          double xzStart[2]    = {0.,0.};
          double xzEnd[2]      = {0.,0.};

          while (itr != xzCells[i].end()){
            mergecells.push_back(*itr);
            double witrk = 0; 
            if(yzTrack[j]->ExtentPlane()==1){witrk=wx;}
              else{witrk = yzTrack[j]->W((*itr).get());}
            const double pecor = cal->GetPECorr(*(*itr), witrk);
            sig1 += pecor;
            if((int)(*itr)->Plane()>=(int)yzTrack[j]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)yzTrack[j]->MaxPlane())    sig1match +=pecor;

            geom->Plane((*itr)->Plane())->Cell((*itr)->Cell())->GetCenter(xyz);
            xzViewX.push_back(xyz[0]);
            xzViewZ.push_back(xyz[2]);
            xzViewW.push_back(pecor);

            itr++;
          }

          xzSlope[i][j]    = 999999.;
          xzIntercept[i][j] = xzEnd[1] - xzSlope[i][j]*xzEnd[0];
          if(fabs(xzStart[0] - xzEnd[0]) > 1.e-2){
            xzSlope[i][j]     = (xzEnd[1] - xzStart[1])/(xzEnd[0] - xzStart[0]);
            xzIntercept[i][j] = xzEnd[1] - xzSlope[i][j]*xzEnd[0];
          }


          itr = yzCells[j].begin();
          std::vector<double> yzViewY;
          std::vector<double> yzViewZ;
          std::vector<double> yzViewW;
          yzViewY.clear();
          yzViewZ.clear();
          yzViewW.clear();
          double yzStart[2]    = {0.,0.}; 
          double yzEnd[2]      = {0.,0.};


          while (itr != yzCells[j].end()){
            mergecells.push_back(*itr);
            double witrk = 0; 
            if(xzTrack[i]->ExtentPlane()==1){witrk=wy;}
              else{witrk = xzTrack[i]->W((*itr).get());}
            const double pecor = cal->GetPECorr(*(*itr), witrk);
            sig2 += pecor;
            if((int)(*itr)->Plane()>=(int)xzTrack[i]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)xzTrack[i]->MaxPlane())    sig2match +=pecor;

            geom->Plane((*itr)->Plane())->Cell((*itr)->Cell())->GetCenter(xyz);
            yzViewY.push_back(xyz[1]);
            yzViewZ.push_back(xyz[2]);
            yzViewW.push_back(pecor);

            itr++;
          }

          yzSlope[i][j]    = 999999.;
          yzIntercept[i][j] = yzEnd[1] - yzSlope[i][j]*yzEnd[0];
          if(fabs(yzStart[0] - yzEnd[0]) > 1.e-2){
            yzSlope[i][j]     = (yzEnd[1] - yzStart[1])/(yzEnd[0] - yzStart[0]);
            yzIntercept[i][j] = yzEnd[1] - yzSlope[i][j]*yzEnd[0];
          } 


          if(sig1+sig2<0.00001)sig1=1;
          if(sig1match+sig2match<0.00001)sig1match=1;


          double trackOverlap = -9999;
          if(start1<=start2&&stop1<=stop2){
            trackOverlap = stop1 - start2;
          }else if(start1<=start2&&stop1>stop2){
            trackOverlap = stop2 - start2;
          }else if(start1>start2&&stop1<=stop2){
            trackOverlap = stop1 - start1;
          }else if(start1>start2&&stop1>stop2){
            trackOverlap = stop2 - start1;
          }


          score[i][j] = (gap1/1.73191e-01)*(gap1/1.73191e-01) + (gap2/1.01610e+0)*(gap2/1.01610e+0);
          pematch[i][j] = fabs(sig1-sig2)/double(sig1+sig2); 
          cbscore[i][j] = log(score[i][j]+1)/1.57518e-01 + pematch[i][j]/3.45537e-02;
          planematch[i][j] = fabs((double)xzTrack[i]->ExtentPlane()-(double)yzTrack[j]->ExtentPlane())/double((int)xzTrack[i]->ExtentPlane()+(int)yzTrack[j]->ExtentPlane()); 
          overlapmatch[i][j] = trackOverlap/double(xzTrack[i]->ExtentPlane())+trackOverlap/double(yzTrack[j]->ExtentPlane()); 

        if(xzTrack[i]->IsNoise()==true&&yzTrack[j]->IsNoise()==true) score[i][j] = 999999.;

        if(start1<minstart){
          minstart = start1;
        }
        if(start2<minstart){
          minstart = start2;
        }

      }
    }

   
    double xbestscore[xzTrack.size()];
    double xbestcbscore[xzTrack.size()];
    double xbestpematch[xzTrack.size()];
    double xbestplanematch[xzTrack.size()];
    double xbestoverlapmatch[xzTrack.size()];

    double ybestscore[yzTrack.size()];
    double ybestcbscore[yzTrack.size()];
    double ybestpematch[yzTrack.size()];
    double ybestplanematch[yzTrack.size()];
    double ybestoverlapmatch[yzTrack.size()];
   
    unsigned int jxbestscore[xzTrack.size()];
    unsigned int jxbestcbscore[xzTrack.size()];
    unsigned int jxbestpematch[xzTrack.size()];
    unsigned int jxbestplanematch[xzTrack.size()];
    unsigned int jxbestoverlapmatch[xzTrack.size()];

    unsigned int iybestscore[yzTrack.size()];
    unsigned int iybestcbscore[yzTrack.size()];
    unsigned int iybestpematch[yzTrack.size()];
    unsigned int iybestplanematch[yzTrack.size()];
    unsigned int iybestoverlapmatch[yzTrack.size()];

    bool loop = true;

    while (loop) {
      for (unsigned int i=0;i<xzTrack.size();i++){
        xbestscore[i] = 999999;
        xbestcbscore[i] = 999999;
        xbestpematch[i] = 999999;
        xbestplanematch[i] = 999999;
        xbestoverlapmatch[i] = -999999;
        jxbestscore[i]=0;
        jxbestcbscore[i]=0;
        jxbestpematch[i]=0;
        jxbestplanematch[i]=0;
        jxbestoverlapmatch[i]=0;
      }

      for (unsigned int j=0;j<yzTrack.size();j++){
        ybestscore[j] = 999999;
        ybestcbscore[j] = 999999;
        ybestpematch[j] = 999999;
        ybestplanematch[j] = 999999;
        ybestoverlapmatch[j] = -999999;
        iybestscore[j]=0;
        iybestcbscore[j]=0;
        iybestpematch[j]=0;
        iybestplanematch[j]=0;
        iybestoverlapmatch[j]=0;
      }

      for (unsigned int i=0;i<xzTrack.size();i++){
        for (unsigned int j=0;j<yzTrack.size();j++){
          if (score[i][j] < xbestscore[i]){
            xbestscore[i] = score[i][j];
            jxbestscore[i] = j; 
          }
          if (score[i][j] < ybestscore[j]){
            ybestscore[j] = score[i][j];
            iybestscore[j] = i;
          }
          if (cbscore[i][j] < xbestcbscore[i]){
            xbestcbscore[i] = cbscore[i][j];
            jxbestcbscore[i] = j;
          }
          if (cbscore[i][j] < ybestcbscore[j]){
            ybestcbscore[j] = cbscore[i][j];
            iybestcbscore[j] = i;
          }

          if (pematch[i][j] < xbestpematch[i]){
            xbestpematch[i] = pematch[i][j];
            jxbestpematch[i] = j;
          }
          if (pematch[i][j] < ybestpematch[j]){
            ybestpematch[j] = pematch[i][j];
            iybestpematch[j] = i;
          }

          if (planematch[i][j] < xbestplanematch[i]){
            xbestplanematch[i] = planematch[i][j];
            jxbestplanematch[i] = j;
          }
          if (planematch[i][j] < ybestplanematch[j]){
            ybestplanematch[j] = planematch[i][j];
            iybestplanematch[j] = i;
          }

          if (overlapmatch[i][j] > xbestoverlapmatch[i]){
            xbestoverlapmatch[i] = overlapmatch[i][j];
            jxbestoverlapmatch[i] = j;
          }
          if (overlapmatch[i][j] > ybestoverlapmatch[j]){
            ybestoverlapmatch[j] = overlapmatch[i][j];
            iybestoverlapmatch[j] = i;
          }
        }
      }

      bool matchtag = false;
      for (unsigned int i=0;i<xzTrack.size();i++){
        if(matchtag == true)break;
        for (unsigned int j=0;j<yzTrack.size();j++){
          if(matchtag == true)break;
          if(i==iybestcbscore[j]||j==jxbestcbscore[i]){
            if(score[i][j]<50000.&&pematch[i][j]<10.5&&overlapmatch[i][j]>-1000.1){

              double zStart = yzTrack[j]->Start().Z();
              double zStop = yzTrack[j]->Stop().Z();
              if (xzTrack[i]->Start().Z() < yzTrack[j]->Start().Z()) zStart = xzTrack[i]->Start().Z();
              if (xzTrack[i]->Stop().Z() > yzTrack[j]->Stop().Z()) zStop = xzTrack[i]->Stop().Z();

              double xyz1[3] = {(xzSlope[i][j]*zStart +xzIntercept[i][j]), 
                		(yzSlope[i][j]*zStart +yzIntercept[i][j]), zStart};
              double xyz2[3] = {(xzSlope[i][j]*zStop +xzIntercept[i][j]), 
                		(yzSlope[i][j]*zStop +yzIntercept[i][j]), zStop};



              double length = util::pythag(zStop-zStart,
                                           xyz2[0] - xyz1[0],
                                           xyz2[1] - xyz1[1]);


              if(xzSlope[i][j]>900000||yzSlope[i][j]>900000||fabs(xyz1[0])>geom->DetHalfWidth()||fabs(xyz1[1])>geom->DetHalfHeight()||fabs(xyz2[0])>geom->DetHalfWidth()||fabs(xyz2[1])>geom->DetHalfHeight()||(length>600&&std::min(xzTrack[i]->ExtentPlane(),yzTrack[j]->ExtentPlane())<=20)){
                xyz1[0] = xzTrack[i]->Start().X();
                xyz1[1] = yzTrack[j]->Start().Y();
                xyz1[2] = zStart;
                xyz2[0] = xzTrack[i]->Stop().X();
                xyz2[1] = yzTrack[j]->Stop().Y();
                xyz2[2] = zStop;
              } 

              length = util::pythag(zStop-zStart,
                			   xyz2[0] - xyz1[0],
                			   xyz2[1] - xyz1[1]);

              double dcosx  = (xyz2[0] - xyz1[0])/length;
              double dcosy  = (xyz2[1] - xyz1[1])/length;
              double dcosz  = (zStop-zStart)/length;

              TVector3 xydir(dcosx,dcosy,dcosz);
               

              art::PtrVector<rb::CellHit> mergecells;

              double sig1 = 0.;
              double sig2 = 0.;
              double sig = 0.;
              double sig1match = 0.;
              double sig2match = 0.;

              double wx = 0, wy=0;
              if(yzTrack[j]->ExtentPlane()==1){
                double ycellXYZ[3];
                geom->Plane(yzTrack[j]->YCell(0)->Plane())->Cell(yzTrack[j]->YCell(0)->Cell())->GetCenter(ycellXYZ);
                wx = ycellXYZ[1];
              }

              if(xzTrack[i]->ExtentPlane()==1){
                double xcellXYZ[3];
                geom->Plane(xzTrack[i]->XCell(0)->Plane())->Cell(xzTrack[i]->XCell(0)->Cell())->GetCenter(xcellXYZ);
                wy = xcellXYZ[0];
              }

              itr = xzCells[i].begin();
              while (itr != xzCells[i].end()){
                mergecells.push_back(*itr);
                double witrk = 0; 
                if(yzTrack[j]->ExtentPlane()==1){witrk=wx;}
                  else{witrk = yzTrack[j]->W((*itr).get());}
		const double pecor = cal->GetPECorr(*(*itr), witrk);
                sig1 += pecor;
                if((int)(*itr)->Plane()>=(int)yzTrack[j]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)yzTrack[j]->MaxPlane())    sig1match +=pecor;

                itr++;
              }
              itr = yzCells[j].begin();
              while (itr != yzCells[j].end()){
                mergecells.push_back(*itr);
                double witrk = 0; 
                if(xzTrack[i]->ExtentPlane()==1){witrk=wy;}
                  else{witrk = xzTrack[i]->W((*itr).get());}
		const double pecor = cal->GetPECorr(*(*itr), witrk);
                sig2 += pecor;
                if((int)(*itr)->Plane()>=(int)xzTrack[i]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)xzTrack[i]->MaxPlane())    sig2match +=pecor;
                itr++;
              }

              bool SliceMatch = false;
              for( unsigned int itm = 0; itm < tMap.size(); ++itm){

                if ((xzTrack[i]->MinTNS() >= tMap[itm].first) && (xzTrack[i]->MaxTNS() <= tMap[itm].second)){
                  if ((yzTrack[j]->MinTNS() >= tMap[itm].first) && (yzTrack[j]->MaxTNS() <= tMap[itm].second)) SliceMatch = true;
                  break;
                }
              }
              if (SliceMatch == false){
                score[i][j] = 999999.;
                cbscore[i][j] = 999999.;
                pematch[i][j] = 999999.;
                continue;
              }

              rb::Cluster Clust(mergecells);

              vCluster.push_back(Clust);
              starts[cc].push_back(xyz1[0]); 
              starts[cc].push_back(xyz1[1]); 
              starts[cc].push_back(xyz1[2]);
              ends[cc].push_back(xyz2[0]); 
              ends[cc].push_back(xyz2[1]); 
              ends[cc].push_back(xyz2[2]);
              dirs[cc].push_back(dcosx);
              dirs[cc].push_back(dcosy);
              dirs[cc].push_back(dcosz);
              signal[cc] = sig;
              cc++;

                for(unsigned int ii=0;ii<xzTrack.size();ii++){
                  score[ii][j] = 999999;
                  cbscore[ii][j] = 999999;
                  pematch[ii][j] = 999999;
                }

                for(unsigned int jj=0;jj<yzTrack.size();jj++){
                  score[i][jj] = 999999;
                  cbscore[i][jj] = 999999;
                  pematch[i][jj] = 999999;
                }

              matchtag = true;

            }
          }
        }
      }
      if(matchtag==false)loop=false;
    }

    for (unsigned int i=0;i<xzTrack.size();i++){
      xbestscore[i] = 999999;
      xbestcbscore[i] = 999999;
      xbestpematch[i] = 999999;
      xbestplanematch[i] = 999999;
      xbestoverlapmatch[i] = -999999;
      jxbestscore[i]=0;
      jxbestcbscore[i]=0;
      jxbestpematch[i]=0;
      jxbestplanematch[i]=0;
      jxbestoverlapmatch[i]=0;
    }

    for (unsigned int j=0;j<yzTrack.size();j++){
      ybestscore[j] = 999999;
      ybestcbscore[j] = 999999;
      ybestpematch[j] = 999999;
      ybestplanematch[j] = 999999;
      ybestoverlapmatch[j] = -999999;
      iybestscore[j]=0;
      iybestcbscore[j]=0;
      iybestpematch[j]=0;
      iybestplanematch[j]=0;
      iybestoverlapmatch[j]=0;
    }

    for (unsigned int i=0;i<xzTrack.size();i++){
      for (unsigned int j=0;j<yzTrack.size();j++){
        if (score[i][j] < xbestscore[i]){
          xbestscore[i] = score[i][j];
          jxbestscore[i] = j; 
        }
        if (score[i][j] < ybestscore[j]){
          ybestscore[j] = score[i][j];
          iybestscore[j] = i;
        }
        if (cbscore[i][j] < xbestcbscore[i]){
          xbestcbscore[i] = cbscore[i][j];
          jxbestcbscore[i] = j;
        }
        if (cbscore[i][j] < ybestcbscore[j]){
          ybestcbscore[j] = cbscore[i][j];
          iybestcbscore[j] = i;
        }

        if (pematch[i][j] < xbestpematch[i]){
          xbestpematch[i] = pematch[i][j];
          jxbestpematch[i] = j;
        }
        if (pematch[i][j] < ybestpematch[j]){
          ybestpematch[j] = pematch[i][j];
          iybestpematch[j] = i;
        }

        if (planematch[i][j] < xbestplanematch[i]){
          xbestplanematch[i] = planematch[i][j];
          jxbestplanematch[i] = j;
        }
        if (planematch[i][j] < ybestplanematch[j]){
          ybestplanematch[j] = planematch[i][j];
          iybestplanematch[j] = i;
        }

        if (overlapmatch[i][j] > xbestoverlapmatch[i]){
          xbestoverlapmatch[i] = overlapmatch[i][j];
          jxbestoverlapmatch[i] = j;
        }
        if (overlapmatch[i][j] > ybestoverlapmatch[j]){
          ybestoverlapmatch[j] = overlapmatch[i][j];
          iybestoverlapmatch[j] = i;
        }
      }
    }
    bool dumpmatch = false;
    if(dumpmatch==true){
      for (unsigned int i=0;i<xzTrack.size();i++){
        std::cout<<xbestscore[i]<<std::endl;
        std::cout<<xbestcbscore[i]<<std::endl;
        std::cout<<xbestpematch[i]<<std::endl;
        std::cout<<xbestplanematch[i]<<std::endl;
        std::cout<<xbestoverlapmatch[i]<<std::endl;
        std::cout<<jxbestscore[i]<<std::endl;
        std::cout<<jxbestcbscore[i]<<std::endl;
        std::cout<<jxbestpematch[i]<<std::endl;
        std::cout<<jxbestplanematch[i]<<std::endl;
        std::cout<<jxbestoverlapmatch[i]<<std::endl;
      }


      for (unsigned int j=0;j<yzTrack.size();j++){
        std::cout<<ybestscore[j]<<std::endl;
        std::cout<<ybestcbscore[j]<<std::endl;
        std::cout<<ybestpematch[j]<<std::endl;
        std::cout<<ybestplanematch[j]<<std::endl;
        std::cout<<ybestoverlapmatch[j]<<std::endl;
        std::cout<<iybestscore[j]<<std::endl;
        std::cout<<iybestcbscore[j]<<std::endl;
        std::cout<<iybestpematch[j]<<std::endl;
        std::cout<<iybestplanematch[j]<<std::endl;
        std::cout<<iybestoverlapmatch[j]<<std::endl;
      }

    }


    for (unsigned int iMin=0;iMin<0;iMin++){
      for (unsigned int jMin=0;jMin<yzTrack.size();jMin++){
        if(score[iMin][jMin]>900000)continue;
        if(cbscore[iMin][jMin]>900000)continue;
        if(pematch[iMin][jMin]>900000)continue;

        double zStart = yzTrack[jMin]->Start().Z();
        double zStop = yzTrack[jMin]->Stop().Z();
        if (xzTrack[iMin]->Start().Z() < yzTrack[jMin]->Start().Z()) zStart = xzTrack[iMin]->Start().Z();
        if (xzTrack[iMin]->Stop().Z() > yzTrack[jMin]->Stop().Z()) zStop = xzTrack[iMin]->Stop().Z();

        double xyz1[3] = {(xzSlope[iMin][jMin]*zStart +xzIntercept[iMin][jMin]), 
          		(yzSlope[iMin][jMin]*zStart +yzIntercept[iMin][jMin]), zStart};
        double xyz2[3] = {(xzSlope[iMin][jMin]*zStop +xzIntercept[iMin][jMin]), 
          		(yzSlope[iMin][jMin]*zStop +yzIntercept[iMin][jMin]), zStop};

        double length = util::pythag(zStop-zStart,
                                     xyz2[0] - xyz1[0],
                                     xyz2[1] - xyz1[1]);


        if(xzSlope[iMin][jMin]>900000||yzSlope[iMin][jMin]>900000||fabs(xyz1[0])>geom->DetHalfWidth()||fabs(xyz1[1])>geom->DetHalfHeight()||fabs(xyz2[0])>geom->DetHalfWidth()||fabs(xyz2[1])>geom->DetHalfHeight()||(length>600&&std::min(xzTrack[iMin]->ExtentPlane(),yzTrack[jMin]->ExtentPlane())<=20)){
          xyz1[0] = xzTrack[iMin]->Start().X();
          xyz1[1] = yzTrack[jMin]->Start().Y();
          xyz1[2] = zStart;
          xyz2[0] = xzTrack[iMin]->Stop().X();
          xyz2[1] = yzTrack[jMin]->Stop().Y();
          xyz2[2] = zStop;
        }

        length = util::pythag(zStop-zStart,
                              xyz2[0] - xyz1[0],
                              xyz2[1] - xyz1[1]);



        double dcosx  = (xyz2[0] - xyz1[0])/length;
        double dcosy  = (xyz2[1] - xyz1[1])/length;
        double dcosz  = (zStop-zStart)/length;


        TVector3 xydir(dcosx,dcosy,dcosz);
         

        art::PtrVector<rb::CellHit> mergecells;

        double sig1 = 0.;
        double sig2 = 0.;
        double sig = 0.;
        double sig1match = 0.;
        double sig2match = 0.;

        double wx = 0, wy=0;
        if(yzTrack[jMin]->ExtentPlane()==1){
          double ycellXYZ[3];
          geom->Plane(yzTrack[jMin]->YCell(0)->Plane())->Cell(yzTrack[jMin]->YCell(0)->Cell())->GetCenter(ycellXYZ);
          wx = ycellXYZ[1];
        }

        if(xzTrack[iMin]->ExtentPlane()==1){
          double xcellXYZ[3];
          geom->Plane(xzTrack[iMin]->XCell(0)->Plane())->Cell(xzTrack[iMin]->XCell(0)->Cell())->GetCenter(xcellXYZ);
          wy = xcellXYZ[0];
        }

        itr = xzCells[iMin].begin();
        while (itr != xzCells[iMin].end()){
          mergecells.push_back(*itr);
          double witrk = 0; 
          if(yzTrack[jMin]->ExtentPlane()==1){witrk=wx;}
            else{witrk = yzTrack[jMin]->W((*itr).get());}

	  const double pecor = cal->GetPECorr(*(*itr), witrk);
	  
          sig1 += pecor;
          if((int)(*itr)->Plane()>=(int)yzTrack[jMin]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)yzTrack[jMin]->MaxPlane())    sig1match +=pecor;

          itr++;
        }
        itr = yzCells[jMin].begin();
        while (itr != yzCells[jMin].end()){
          mergecells.push_back(*itr);
          double witrk = 0; 
          if(xzTrack[iMin]->ExtentPlane()==1){witrk=wy;}
            else{witrk = xzTrack[iMin]->W((*itr).get());}
	  const double pecor = cal->GetPECorr(*(*itr), witrk);
          sig2 += pecor;
          if((int)(*itr)->Plane()>=(int)xzTrack[iMin]->MinPlane()-1 && (int)(*itr)->Plane()<=(int)xzTrack[iMin]->MaxPlane())    sig2match +=pecor;
          itr++;
        }


        int mtag = 0; 
        if(pematch[iMin][jMin]<0.5&&score[iMin][jMin]<50.&&overlapmatch[iMin][jMin]>-1000.1&&(jMin==jxbestscore[iMin]||iMin==iybestscore[jMin]||jMin==jxbestpematch[iMin]||iMin==iybestpematch[jMin])) mtag = 1;

      if(mtag==0)continue;

      // check to make sure both track merge candidates are part of the same slice
      bool SliceMatch = false;
      for( unsigned int itm = 0; itm < tMap.size(); ++itm){
        //get slice window for xzTrack
        if ((xzTrack[iMin]->MinTNS() >= tMap[itm].first) && (xzTrack[iMin]->MaxTNS() <= tMap[itm].second)){
          //check to see if yzTrack is in same window, if so allow match
          if ((yzTrack[jMin]->MinTNS() >= tMap[itm].first) && (yzTrack[jMin]->MaxTNS() <= tMap[itm].second)) SliceMatch = true;
          break;
        }
      }
      //if score match is false, remove that combination from 3D track candidates and continue
      if (SliceMatch == false){
        score[iMin][jMin] = 999999.;
        cbscore[iMin][jMin] = 999999.;
        pematch[iMin][jMin] = 999999.;
        continue;
      }


      rb::Cluster Clust(mergecells);

      score[iMin][jMin] = 999999.;
      cbscore[iMin][jMin] = 999999.;
      pematch[iMin][jMin] = 999999.;

      vCluster.push_back(Clust);
      starts[cc].push_back(xyz1[0]); 
      starts[cc].push_back(xyz1[1]); 
      starts[cc].push_back(xyz1[2]);
      ends[cc].push_back(xyz2[0]); 
      ends[cc].push_back(xyz2[1]); 
      ends[cc].push_back(xyz2[2]);
      dirs[cc].push_back(dcosx);
      dirs[cc].push_back(dcosy);
      dirs[cc].push_back(dcosz);
      signal[cc] = sig;
      cc++;
      std::vector< int > trackcount;

      }  
    }

    
    std::unique_ptr< std::vector<rb::Cluster> > clustcol(new std::vector<rb::Cluster> );
    for(unsigned int i = 0; i < vCluster.size(); i++)
      {
	clustcol->push_back(vCluster[i]);
      }
    // put the clusters into the event
    for( size_t i = 0; i < vCluster.size(); i++)
    {
      TVector3 start(starts[i][0], starts[i][1], starts[i][2]);
      TVector3 end(ends[i][0], ends[i][1], ends[i][2]);
      TVector3 dir(dirs[i][0], dirs[i][1], dirs[i][2]);
      rb::Track track(vCluster[i], start, dir);
      track.AppendTrajectoryPoint(end);

      trackcol->push_back(track);
      
    }
    evt.put(std::move(trackcol));
    return; // kFailed if you want to fail the event
    
  }

}//end namespace jmshower

namespace jmshower
{
  DEFINE_ART_MODULE(JMTrackMerge)
}
