//////////////////////////////////////////////////////////////////////////
/// \brief  A very simple service to remember what detector we're working in
/// \author Christopher Backhouse - bckhouse@caltech.edu
//////////////////////////////////////////////////////////////////////////

#include "RunHistory/service/DetectorService.h"

#include "SummaryData/RunData.h"

#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

namespace rh
{
  //------------------------------------------------------------
  DetectorService::DetectorService(const fhicl::ParameterSet& pset,
				   art::ActivityRegistry &reg)
    : fDetId(novadaq::cnv::kUNKNOWN_DET)
  {
    // Belt and braces. Maybe there's a case where the information becomes
    // available at beginRun().
    reg.sPreBeginRun.watch(this, &DetectorService::preBeginRun);
    reg.sPostBeginRun.watch(this, &DetectorService::postBeginRun);
  }

  //------------------------------------------------------------
  DetectorService::~DetectorService()
  {
  }

  //----------------------------------------------------------
  novadaq::cnv::DetId DetectorService::DetId(const art::Run& run)
  {
    HandleRun(run);
    return fDetId;
  }

  //----------------------------------------------------------
  void DetectorService::preBeginRun(const art::Run& run)
  {
    HandleRun(run);
  }

  //----------------------------------------------------------
  void DetectorService::postBeginRun(const art::Run& run)
  {
    HandleRun(run);
  }

  //----------------------------------------------------------
  void DetectorService::HandleRun(const art::Run& run)
  {
    std::vector<art::Handle<sumdata::RunData>> rdcol;
    run.getManyByType(rdcol);
    if(!rdcol.empty()) fDetId = rdcol[0]->DetId();
  }

  DEFINE_ART_SERVICE(DetectorService)

}
////////////////////////////////////////////////////////////////////////
