////////////////////////////////////////////////////////////////////////
/// \file    DeconvolveAlg.h
/// \brief   Preforms energy deconvolution for cells that may belong to 
///          multple clusters. It does so, assuming that the shower is
///          electromagnetic
///                                                                                                                     
/// \author  ksachdev@physics.umn.edu                                                                                   
////////////////////////////////////////////////////////////////////////     

#ifndef DeconvolveAlg_h
#define DeconvolveAlg_h

#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Persistency/Common/Ptr.h"

#include "Geometry/Geometry.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Prong.h"
#include "RecoBase/Shower.h"
#include "RecoBase/Track.h"
#include "RecoBase/WeightedHit.h"
#include "ShowerLID/NuEEnergyAlg.h"

namespace rb
{ 
  class Cluster;
  class Track;
  class CellHit;
  class WeightedHit;
  class Shower;
}


namespace slid{

  class DeconvolveAlg
  {
  public:
    explicit DeconvolveAlg( fhicl::ParameterSet const& pset);
    ~ DeconvolveAlg();

    void  reconfigure( const fhicl::ParameterSet& pset );
    
    void  GetWeights( std::vector<rb::Prong> & prong,
		      const std::vector< float > & totpe);
    
    float DistanceToCore( art::Ptr<rb::CellHit> cHit,
			  rb::Prong & prong);
    
    
  private:
    art::ServiceHandle<geo::Geometry> fGeom;
    float fExpConst; ///< The constant length scale used in energy deconvolution

    NuEEnergyAlg* fNuEEnergyAlg;
    fhicl::ParameterSet fNuEEnergyAlgPSet;


  }; // end of class DeconvolveAlg

}// end of namespace jmshower


#endif // DeconvolveAlg_h
