////////////////////////////////////////////////////////////////////////
// \file    EID.h
// \brief   A pid object for electron identification based on Jianming 
//          Bian's original EID
// \version 
// \author  Alex Smith - smith@physics.umn.edu
////////////////////////////////////////////////////////////////////////
#ifndef LID_H
#define LID_H

#include <vector>
#include <string>
#include "RecoBase/PID.h"

namespace slid {

  class LID : public rb::PID {
  public:

    LID() : fVtxE(-5.0), fShwDepE(-5.0), fShwE(-5.0), fNueEnergy(-5.0), fIsMuon(false){}
    ~LID(){}
#ifndef __GCCXML__
    LID(int pdg, double val);


    /// Setters

    /// \brief set algorithm name
    void SetAlgName(const std::string algName) { fAlgName = algName; }

    /// \brief set algorithm description
    void SetAlgDescription(const std::string algDescription) { fAlgDescription = algDescription; }

    void SetVertexEnergy (float in) { fVtxE      = in; }
    void SetShwDepEnergy (float in) { fShwDepE   = in; } 
    void SetShwEnergy    (float in) { fShwE      = in; }
    void SetNueEnergy    (float in) { fNueEnergy = in; }
    void SetIsMuon       (bool in)  { fIsMuon    = in; }


    /// Getters
    
    std::string AlgName        () const  { return fAlgName;        }
    std::string AlgDescription () const  { return fAlgDescription; }
    
    float VertexEnergy () const  { return fVtxE;    }
    float ShwDepEnergy () const  { return fShwDepE; }
    float ShwEnergy    () const  { return fShwE;    }
    
    float NueEnergy()  const { return fNueEnergy; }
    bool  IsMuon()     const { return fIsMuon; }
#endif // __GCCXML__
  protected:

    /// Algorithm name 
    std::string fAlgName;
    /// Algorithm description
    std::string fAlgDescription;


  public:
    /// \brief training variables
    float fVtxE;
    float fShwDepE;
    float fShwE;
    float fNueEnergy;
    bool  fIsMuon;
  };

};
#endif
