////////////////////////////////////////////////////////////////////////
// \file    ShowerLID.h
// \brief   A Shower class that records the log likelihood information 
//          for every shower.
// \version 
// \author  Kanika Sachdev (ksachdev@physics.umn.edu)
////////////////////////////////////////////////////////////////////////

#include "ShowerLID/ShowerLID.h"
#include "ShowerLID/DedxDistribution.h"

namespace slid {

  //......................................
  ShowerLID::ShowerLID(const rb::PID & pid)
    : rb::PID( pid),
      fPIDWithE(-5.0),
      fEGLLL(-5.0), fEGLLT(-5.0),
      fEMuLLL(-5.0), fEMuLLT(-5.0),
      fEPi0LLL(-5.0), fEPi0LLT(-5.0),
      fEPLLL(-5.0), fEPLLT(-5.0),
      fENLLL(-5.0), fENLLT(-5.0),
      fEPiLLL(-5.0), fEPiLLT(-5.0),
      fInvGLLL(-5.0),
      fGap(-5.0), fPi0mass(-5.0),
      fRadius(-5.0), fIsMuon(false),
      fCosTheta(-5.0), fShwEFrac(-5.0),
      fVtxDoca(-5.0),
      fVtxE(-5.0),fShwDepE(-5.0),
      fShwE(-5.0), fNueEnergy(-5.0),
      fHadE(-5.0)
  {
  }// end of constructor

  bool ShowerLID::IsInversePhoton() const
  {
    std::map<int,float>::const_iterator itr = fPartLongLL.find(slid::DedxParticleType::kPHOTON);
    float glll = -5;
    if (itr != fPartLongLL.end()) glll = itr->second;
    //float glll = fPartLongLL(slid::DedxParticleType::kPHOTON);
    return (fInvGLLL > glll);
  }

  ////////////////////////////////////////////////////////////////////////

  bool CompareByEnergy( const slid::ShowerLID* a, const slid::ShowerLID* b)
  {
    return a->ShowerEnergy() > b->ShowerEnergy();
  }

}// end of namespace slid
