////////////////////////////////////////////////////////////////////////
// \file    SREnergyBranch.h
// \brief   An SREnergyBranch contains vectors of energy objects.                  
//          It is intended for use in the Common Analysis ROOT trees.
//
// \version $Id: SREnergyBranch.h,v 1.2 2012-11-19 17:32:22 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 17:32:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRENERGYBRANCH_H
#define SRENERGYBRANCH_H

#include "StandardRecord/SRNumuEnergy.h"
#include "StandardRecord/SREnergy.h"
#include "StandardRecord/SRJMEnergy.h"
#include "StandardRecord/SRSLidEnergy.h"
#include "StandardRecord/SRBPFEnergy.h"

#include <vector>
#include "stdlib.h"

namespace caf
{
  /// An SREnergyBranch contains vectors of energy objects.                  
  /// It is intended for use in the Common Analysis ROOT trees.
  class SREnergyBranch
    {
    public:
      ///  Default constructor
      SREnergyBranch();
      /// Default destuctor.  
      ~SREnergyBranch();
      
      SRNumuEnergy          numusimp;  ///< Numu energy estimator
      std::vector<SREnergy> mutrkE;    ///  \brief Energy estimate for each 3D
                                       ///  KalmanTrack, under the muon assumption

      SRSLidEnergy              elecid;  ///< Enery estimate from lid for slice pid (most energetic shower for now)
      std::vector<SRSLidEnergy> shwlid;  ///< Energy estimator from lid for each shower
      size_t                    nshwlid; ///< number of showers in vector

      SRBPFEnergy bpfenergy; ///< BreakPointFitter energy estimator

      void fillSizes();
      
      
    };
  
} // end namespace

#endif // SRENERGYBRANCH_H
//////////////////////////////////////////////////////////////////////////////
