////////////////////////////////////////////////////////////////////////
// \file    SRGenerator.cxx
// \brief   Object collecting Generator (GENIE) MC truth variables for caf
// \version
// \author  Nathan Mayer - nathan.mayer@tufts.edu
////////////////////////////////////////////////////////////////////////

#include "StandardRecord/SRGenerator.h"


namespace caf {

  SRGenerator::SRGenerator():
    fGint(0), 
    fGscatter(0), 

    fweight(std::numeric_limits<double>::signaling_NaN()),
    fprobability(std::numeric_limits<double>::signaling_NaN()),
    fXsec(std::numeric_limits<double>::signaling_NaN()),
    fDiffXsec(std::numeric_limits<double>::signaling_NaN()),

    fNumPiPlus(0), 
    fNumPiMinus(0),
    fNumPi0(0),    
    fNumProton(0), 
    fNumNeutron(0),
    fIsCharm(0),
    fResNum(0),  

    fgQ2(std::numeric_limits<double>::signaling_NaN()),
    fgq2(std::numeric_limits<double>::signaling_NaN()),
    fgW(std::numeric_limits<double>::signaling_NaN()),
    fgT(std::numeric_limits<double>::signaling_NaN()),
    fgX(std::numeric_limits<double>::signaling_NaN()),
    fgY(std::numeric_limits<double>::signaling_NaN()),
    fFShadSystP4(std::numeric_limits<float>::signaling_NaN(),
                 std::numeric_limits<float>::signaling_NaN(),
                 std::numeric_limits<float>::signaling_NaN(),
                 std::numeric_limits<float>::signaling_NaN()),

    fIsSeaQuark(0),
    ftgtZ(0),
    ftgtA(0),
    ftgtPDG(0),
    fHitNucP4(std::numeric_limits<float>::signaling_NaN(),
              std::numeric_limits<float>::signaling_NaN(),
              std::numeric_limits<float>::signaling_NaN(),
              std::numeric_limits<float>::signaling_NaN()),
    fProbePDG(0),
    fProbeP4(std::numeric_limits<float>::signaling_NaN(),
             std::numeric_limits<float>::signaling_NaN(),
             std::numeric_limits<float>::signaling_NaN(),
             std::numeric_limits<float>::signaling_NaN()),
    fVertex(std::numeric_limits<float>::signaling_NaN(),
            std::numeric_limits<float>::signaling_NaN(),
            std::numeric_limits<float>::signaling_NaN(),
            std::numeric_limits<float>::signaling_NaN()),
    
    fParticleList(),
    fFSLeptonIndx(0),
    fProbeIndx(0),

    fHitNucleon(0),
    fHitQuark(0),

    fVisEfraction(1.0),
    visEinslc(0)
    
  { }

  SRGenerator::~SRGenerator() { }

} // end namespace caf
////////////////////////////////////////////////////////////////////////
