////////////////////////////////////////////////////////////////////////
// \file    SRIDBranch.h
// \brief   An SRIDBranch contains event ID and selection variables.
//          It is intended for use in the Common Analysis ROOT trees.
//
// \version $Id: SRIDBranch.h,v 1.3 2012-12-07 06:37:29 gsdavies Exp $
// \author  rocco@physics.umn.edu
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////
#ifndef SRIDBRANCH_H
#define SRIDBRANCH_H

#include "StandardRecord/SRRemid.h"
#include "StandardRecord/SRQepid.h"
#include "StandardRecord/SRELid.h"
#include "StandardRecord/SRLid.h"
#include "StandardRecord/SRSPid.h"
#include "StandardRecord/SRLem.h"
#include "StandardRecord/SRNuePresel.h"
#include "StandardRecord/SRRvp.h"
#include "StandardRecord/SRCosRej.h"
#include "StandardRecord/SRNueCosRej.h"
#include "StandardRecord/SRContain.h"
#include "StandardRecord/SRVeto.h"

namespace caf
{
  /// An SRIDBranch contains event ID and selection variables.                  
  /// It is intended for use in the Common Analysis ROOT trees.
  class SRIDBranch
    {
    public:
      ///  Default constructor
      SRIDBranch();
      /// Default destuctor.  
      ~SRIDBranch();
      
      SRRemid      remid;     ///< Output from RecoMuonID (ReMId) package
      SRQepid      qepid;     ///< Output from QuasiElasticPID (QePId) package
      SRELid       elecid;    ///< Output from LIDBuilder (LID) package
      SRLem        lem;       ///< Output from Library Event Matching (LEM)
      SRNuePresel  nuepre;    ///< Official nue preselection information 
      SRRvp        rvp;       ///< Output from RecoVariablePID (RVP)
      SRCosRej     cosrej;    ///< Output from CosRej (Cosmic Rejection)
      SRNueCosRej  nuecosrej; ///< Output from NueCosRej (Nue Cosmic Rejection)
      SRContain    contain;   ///< Output from SRContain (containment related variables)
      SRVeto       veto;      ///< Output from CosmicVeto (Preliminary preselection)
    };
  
} // end namespace

#endif // SRIDBRANCH_H
//////////////////////////////////////////////////////////////////////////
