

#ifndef SRNUECOSREJ_H
#define SRNUECOSREJ_H


namespace caf
{
    /// Class for storing output from Cosmic Rejection (Nuecosrej) module
    class SRNueCosRej{
    public:
      ///  Default constructor
      SRNueCosRej();

      /// Default destuctor
      ~SRNueCosRej();
      void setDefault();

      /// Number of slice hits per prong planes. 
      ///See \ref cosrej::NueCosRej::HitsPerPlane
      float hitsperplane; 

      /// Event transverse momentum fraction, based 
      /// on prongs. See \ref cosrej::NueCosRej::ProngTransMom
      float pngptp;
       
      /// Event transverse momentum fraction, based on
      /// showers. See \ref cosrej::NueCosRej::ParticleShowerTransMom
      float partptp;      
      
      /// Event transverse momentum fraction, based on 
      /// showers. See \ref cosrej::NueCosRej::PhotonShowerTransMom
      float photptp;      

      /// Leading prong start to detector top. See 
      /// \ref cosrej::NueCosRej::StartDistToTop
      float starttop;     

      /// Leading prong start to detector bottom. See 
      /// \ref cosrej::NueCosRej::StartDistToBottom
      float startbottom;  

      /// Leading prong start to detector front. See 
      /// \ref cosrej::NueCosRej::StartDistToFront
      float startfront;   

      /// Leading prong start to detector back. See 
      /// \ref cosrej::NueCosRej::StartDistToBack
      float startback;    

      /// Leading prong start to detector west. See 
      /// \ref cosrej::NueCosRej::StartDistToWest
      float startwest;    

      /// Leading prong start to detector east. See 
      /// \ref cosrej::NueCosRej::StartDistToEast
      float starteast;    

      /// Leading prong start to detector top. See 
      /// \ref cosrej::NueCosRej::StartDistToTop
      float stoptop;      

      ///< Leading prong start to detector bottom. 
      /// See \ref cosrej::NueCosRej::StartDistToBottom
      float stopbottom;   

      /// Leading prong start to detector front. 
      /// See \ref cosrej::NueCosRej::StartDistToFront
      float stopfront;    

      /// Leading prong start to detector back. See
      /// \ref cosrej::NueCosRej::StartDistToBack
      float stopback;     

      /// Leading prong start to detector west. See 
      /// \ref cosrej::NueCosRej::StartDistToWest
      float stopwest;     

      /// Leading prong start to detector east. See 
      /// \ref cosrej::NueCosRej::StartDistToEast
      float stopeast;     

      /// Cosine of angle between 1st and 2nd leading
      /// prongs. See \ref cosrej::NueCosRej::CosAngleToNextProng
      float cosdang;      

      /// Distance of closest approach of leading prong
      /// to vertex. See \ref cosrej::NueCosRej::ProngDistToVtx
      float vtxdoca;      

      /// Leading prong max X between start and stop. See 
      /// \ref cosrej::NueCosRej::ProngMaxX
      float prongmaxx;   

      /// Leading prong max Y between start and stop. See 
      /// \ref cosrej::NueCosRej::ProngMaxY
      float prongmaxy;   

      /// Leading prong max Z between start and stop. See 
      /// \ref cosrej::NueCosRej::ProngMaxZ
      float prongmaxz;   

      /// Leading prong min X between start and stop. See 
      /// \ref cosrej::NueCosRej::ProngMinX
      float prongminx;   

      /// Leading prong min Y between start and stop. See 
      /// \ref cosrej::NueCosRej::ProngMinY
      float prongminy;   

      /// Leading prong min Z between start and stop. 
      /// See \ref cosrej::NueCosRej::ProngMinZ
      float prongminz;   
    };

} // end namespace

#endif // SRNUECOSREJ_H
//////////////////////////////////////////////////////////////////////////////
