////////////////////////////////////////////////////////////////////////
// \file    SRNueSandbox.cxx
// \brief   Class for storing information necessary for nu_e analysis. 
//          This information might not exist here forever, it may 
//          eventually be moved somewhere else. 
// \version $Id: SRNueSandbox.cxx,v 1.1 2012-12-06 20:18:33 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-12-06 20:18:33 $
////////////////////////////////////////////////////////////////////////

#include "StandardRecord/SRNueSandbox.h"


namespace caf 
{
  SRNueSandbox::SRNueSandbox():
  infid(false),
  slicemeanx(std::numeric_limits<float>::signaling_NaN()),
  slicemeany(std::numeric_limits<float>::signaling_NaN()),
  slicemeanz(std::numeric_limits<float>::signaling_NaN()),
  npi0(0),
  npip(0),
  npim(0),
  pi0E(std::numeric_limits<float>::signaling_NaN()),
  pipE(std::numeric_limits<float>::signaling_NaN()),
  pimE(std::numeric_limits<float>::signaling_NaN()),
  phot0E(std::numeric_limits<float>::signaling_NaN()),
  phot1E(std::numeric_limits<float>::signaling_NaN()),
  opencos(std::numeric_limits<float>::signaling_NaN()),
  conv0(std::numeric_limits<float>::signaling_NaN()),
  conv1(std::numeric_limits<float>::signaling_NaN()),
  mincellcosmic(std::numeric_limits<float>::signaling_NaN()),
  mincellwallcosmic(std::numeric_limits<int>::signaling_NaN()),
  anglecosmic(std::numeric_limits<float>::signaling_NaN()),
  nplanesfilled(0),
  nxplanes(0),
  nyplanes(0),
  ncellrow(0),
  ncellsum(0),
  mincellavg(std::numeric_limits<float>::signaling_NaN()),
  mincellkalman(std::numeric_limits<float>::signaling_NaN()),
  nmiphits(0),
  mipmin(std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN()),
  ecfNu(),
  mipmax(std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN()),
  evtncell(0),
  evtsumcostheta(0),
  evtsump(0),
  evtsumpt(0),
  evtsump0(0),
  evtsumpt0(0),
  evtetot(0),
  evtminx(99999),
  evtminy(99999),
  evtminz(99999),
  evtmaxx(-99999),
  evtmaxy(-99999),
  evtmaxz(-99999),
  evtgaptns(1e8),
  evtncelltoedge(10000),
  sh1energy(0),
  sh1exclenergy(0),
  sh1totalL(-5),     
  sh1start(std::numeric_limits<float>::signaling_NaN(),
           std::numeric_limits<float>::signaling_NaN(),
           std::numeric_limits<float>::signaling_NaN()),   
  sh1stop(std::numeric_limits<float>::signaling_NaN(),
          std::numeric_limits<float>::signaling_NaN(),
          std::numeric_limits<float>::signaling_NaN()),    
  sh1dir(std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN()),     
  sh1vtxdoca(0), 
  sh1gap(0), 
  sh1nplane(0), 
  sh1xnplane(0),
  sh1ynplane(0),
  sh1ncell(0),
  sh1xncell(0), 
  sh1yncell(0),
  sh1pid(-1),
  sh1ncelltoedge(10000),
  sh1sh2dang(0),
  sh2energy(0),
  sh2exclenergy(0),
  sh2totalL(-5),     
  sh2start(std::numeric_limits<float>::signaling_NaN(),
           std::numeric_limits<float>::signaling_NaN(),
           std::numeric_limits<float>::signaling_NaN()),
  sh2stop(std::numeric_limits<float>::signaling_NaN(),
          std::numeric_limits<float>::signaling_NaN(),
          std::numeric_limits<float>::signaling_NaN()),
  sh2dir(std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN(),
         std::numeric_limits<float>::signaling_NaN()),
  sh2vtxdoca(0), 
  sh2gap(0), 
  sh2nplane(0), 
  sh2xnplane(0),
  sh2ynplane(0),
  sh2ncell(0),
  sh2xncell(0), 
  sh2yncell(0),
  sh2pid(-1),
  sh2ncelltoedge(10000),
  elll(-5),          
  sh1d2edge(-5),     
  eedge2cell(-5),    
  eedge5cell(-5),    
  eedge10cell(-5),
  jm(),
  jmid(),
  vtx(std::numeric_limits<float>::signaling_NaN(),
      std::numeric_limits<float>::signaling_NaN(),
      std::numeric_limits<float>::signaling_NaN()),
  time(-5),
  E(-5),
  depE(-5),
  shwE(-5),
  fracangchanges(std::numeric_limits<float>::signaling_NaN()),
  fracangchangesloose(std::numeric_limits<float>::signaling_NaN()),
  fracmodalhits(std::numeric_limits<float>::signaling_NaN()),
  fracnoncontig(std::numeric_limits<float>::signaling_NaN()),
  fracnoncontigloose(std::numeric_limits<float>::signaling_NaN()),
  maxgap(-5),
  dedxpng1(-5),
  dedxpng2(-5)
  {  }
  
  
  SRNueSandbox::~SRNueSandbox(){  }
  
} // end namespace caf
////////////////////////////////////////////////////////////////////////
