////////////////////////////////////////////////////////////////////////
// \file    SRProng.h
// \brief   An SRProng is a high level reconstructed object.  It knows its 
//          direction and start point, but does not own its cell hits.
// \version $Id: SRProng.h,v 1.4 2012-11-13 22:06:10 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-13 22:06:10 $
////////////////////////////////////////////////////////////////////////
#ifndef SRPRONG_H
#define SRPRONG_H

#include "TVector3.h"

namespace caf
{
  typedef enum _plane_proj {
    /// This is copied directly from GeometryObjects/PlaneGeo.h
    kX,   ///< Vertical planes which measure X
    kY,   ///< Horizontal planes which measure Y
    kXorY ///< X or Y views
  } View_t;
  
  
  /// An SRProng is a simple descriptor for a prong.  This class does not contain individual  
  /// cell hits, but does know it's start point and direction.  This class is meant to be used in 
  /// Common Analysis Files as part of the StandardRecord.
  class SRProng
    {
    public:
      ///  Default constructor for SRProng. 
      SRProng();

      ~SRProng(){  };
      
      unsigned short nhit;            ///< number of hits
      unsigned short nhitx;           ///< number of hits in x-view
      unsigned short nhity;           ///< number of hits in y-view 
      unsigned short nplane;          ///< number of planes spanned 
      unsigned short maxplanecont;    ///< maximum number of contiguous planes in prong 
      unsigned short maxplanegap;     ///< maximum number of gapped planes in prong 
      float          calE;            ///< energy based on summed calibrated deposited charge [GeV]
      TVector3       start;           ///< Shower start point in detector coordinates. [cm]
      TVector3       dir;             ///< Shower direction at start point [unit vector recommended]
      float          len;             ///< track length [cm]
      View_t view;                    ///< Prong view caf::kX = 0, caf::kY = 1 or caf::kXorY = 2

    };
  
} // end namespace

#endif // SRPRONG_H
//////////////////////////////////////////////////////////////////////////////
