////////////////////////////////////////////////////////////////////////
// \file    SRTrueMichelE.h
// \brief   An SRTrueMichelE represents a Michel electron.  The SRTruth 
//          object contains a vector of these.  SRNeutrino and SRCosmic
//          inherit from SRTruth. 
// \author  rocco
// \date    June 14, 2013
////////////////////////////////////////////////////////////////////////
#ifndef SRTRUEMICHELE_H
#define SRTRUEMICHELE_H

#include "TVector3.h"
#include "TLorentzVector.h"

namespace caf
{  
  
  /// An SRTrueMichelE represents a Michel electron.  The SRTruth
  /// object contains a vector of these.  SRNeutrino and SRCosmic
  /// inherit from SRTruth. 
  class SRTrueMichelE
    {
    public:
      ///  Default constructor for SRTrueMichelE. 
      SRTrueMichelE();

      ~SRTrueMichelE(){  };
      
      float           E;            ///< True energy of electron [GeV]
      float           visE;         ///< Visible Energy in detector, all summed FLSHits that made CellHits [GeV]
      float           time;         ///< Time of first Michel electron trajectory point [GeV]

      
      TLorentzVector  mustop;       ///<  Stopping position of parent muon [cm]      
      TLorentzVector  p;            ///< Momentum 4-vector

    };
  
} // end namespace

#endif // SRTRUEMICHELE_H
//////////////////////////////////////////////////////////////////////////////
