///////////////////////////////////////////////////////////////////////
/// \file CosmicTrackAlg.h
/// \brief   Track finder for cosmic rays
/// \author  brebel@fnal.gov, gsdavies@iastate.edu
/// \date
///////////////////////////////////////////////////////////////////////
#ifndef TRACKFIT_COSMICTRACKALG_H
#define TRACKFIT_COSMICTRACKALG_H

// C++ includes
#include <vector>

// ART includes
#include "art/Persistency/Common/Ptr.h"

#include "TrackFit/TrackAlg.h"

namespace fhicl { class ParameterSet; }
namespace rb {
  class Track;
  class Cluster;
}

namespace trk {

  /// An algorithm to perform cosmic ray track fitting
  class CosmicTrackAlg : public TrackAlg {
  public:
    explicit CosmicTrackAlg(fhicl::ParameterSet const &pset);
    virtual ~CosmicTrackAlg();

    // Take a slice and return a track
    std::vector<rb::Track> MakeTrack( const rb::Cluster* slice );

  private:
    unsigned int FitView(std::vector<double> &x,
			 std::vector<double> &y,
			 std::vector<double> &w,
			 double &chiSqr,
			 double *start,
			 double *end);

    bool IsTrackDownstreamFromTiming( const rb::Cluster* slice) const;

    double fDHitGood;  ///< Maximum distance from the line for a hit to be considered part of it

  };

}

#endif // TRACKFIT_COSMICTRACKALG_H
///////////////////////////////////////////////////////////////////////
