//////////////////////////////////////////////////////////////////////////
/// \brief  Cosmic track selection code
/// \author Brian Rebel - brebel@fnal.gov
/// \date 
//////////////////////////////////////////////////////////////////////////
#ifndef TRACKFIT_COSMICTRACKSELECTION_H
#define TRACKFIT_COSMICTRACKSELECTION_H

#include "fhiclcpp/ParameterSet.h"

#include "art/Framework/Services/Registry/ServiceHandle.h"

#include "Geometry/LiveGeometry.h"
#include "TrackFit/CosmicTrackStruct.h"

namespace rb{
  class Track;
  class Cluster;
}

class TVector3;

namespace trk{

  // A class to select cosmic ray muons for analysis
  class CosmicTrackSelection {

  public:
    
    explicit CosmicTrackSelection();
    virtual ~CosmicTrackSelection();

    void reconfigure   (fhicl::ParameterSet const& pset);

    bool  Stopper      (TVector3            const& finalPoint);
    bool  GoodSteps    (rb::Track 	    const& track,
			std::vector<float>       & steps,
			float                    & median);     
    bool  GoodReco     (rb::Track 	    const& track);     
    bool  CleanSample  (CosmicTrackInfo     const& ri);
    bool  IsCalibTrack (rb::Track           const& track,
			rb::Cluster         const& slice);
    float DistToEdge   (TVector3            const& point);

  private:

    double fTrkLengthCut;      ///< Minimum length of track to accept (cm)
    double fBadStepSizeLimit;  ///< multiplicative factor to median trajectory 
                               ///< step size to indicate a too large step
    double fMaxCosTheta;       ///< reject tracks coming from larger cos(theta)
    double fMaxDeltaEndPlane;  ///< biggest allowed difference in end plane number	       
    double fMaxPlaneAsymmetry; ///< biggest allowed asymmetry in number of planes in each view 
    double fMaxCellsPerPlane;  ///< to remove shower-y events 
    double fMaxVtxDistFromEdge;///< maximum distance of the track vertex from the edge of the detector
    double fMaxEndDistFromEdge;///< maximum distance of the track end from the edge of the detector

    art::ServiceHandle<geo::LiveGeometry>  fLiveGeom;
    
  };

}

#endif // TRACKFIT_COSMICTRACKSELECTION_H
