#include "TrackFit/LocatedChan.h"

namespace trk
{
  //......................................................................
  LocatedChan::LocatedChan()
    :OfflineChan()
  {
  }
  //......................................................................
  LocatedChan::LocatedChan(int plane, int cell)
    : OfflineChan(plane, cell)
  {
  }

  //......................................................................
  void LocatedChan::SetHit(art::Ptr<rb::CellHit> chit)
  {
    fHit = chit;
  }

  //......................................................................
  art::Ptr<rb::CellHit> LocatedChan::GetHit() const
  {
    return fHit;
  }

  //......................................................................
  void LocatedChan::SetCenter(double x, double y, double z)
  {
    fcenter.SetXYZ(x,y,z);
  }

  //......................................................................
  /// Return the center position of a the channel
  /// \param xyz : 3-D array. The returned location
  void LocatedChan::GetCenter(double* xyz)
  {
    xyz[0] = fcenter.X();
    xyz[1] = fcenter.Y();
    xyz[2] = fcenter.Z();
  }

  //......................................................................
  void LocatedChan::SetHalfWidths(double dx, double dy, double dz)
  {
    fhalfwidths.SetXYZ(dx,dy,dz);
  }

  //......................................................................
  /// Return the half widths of the channel
  /// \param dxyz : 3-D array. The half width of the cell in each dimension
  void LocatedChan::GetHalfWidths(double* dxyz)
  {
    dxyz[0] = fhalfwidths.X();
    dxyz[1] = fhalfwidths.Y();
    dxyz[2] = fhalfwidths.Z();
  }

  //......................................................................
  bool CompareByPlane(trk::LocatedChan a, trk::LocatedChan b)
  {
    return a.Plane() < b.Plane();
  }

  //......................................................................
  void SortByPlane(std::vector<trk::LocatedChan>& c)
  {
    std::sort(c.begin(), c.end(), trk::CompareByPlane);
  }
  //......................................................................
  bool CompareByCell(trk::LocatedChan a, trk::LocatedChan b)
  {
    return a.Cell() < b.Cell();
  }

  //......................................................................
  void SortByCell(std::vector<trk::LocatedChan>& c)
  {
    std::sort(c.begin(), c.end(), trk::CompareByCell);
  }

}
