#!/usr/bin/perl
#
# Use "--help" to learn how this script works and what it does.
#
$dont_run      = 0;
$event_file    = "events.txt";
$trigger       = "t02";
$tier          = "reco";
$file_prefix   = "fardet";
$release       = "S14-05-08";
$tempfile      = "tempfile.root";
$endfile       = "runevtfilt-evt.root";
$data_def      = "";
$have_data_def = 0;
for ($i=0; $i<=$#ARGV; ++$i) {
    if (($ARGV[0] eq "-h") || ($ARGV[0] eq "--help")) {
	print "This script will take a list of events in [run] [subrun] [event] [slice] format.\n";
	print "The script will generate file names\n";
	print "and run an ART job to extract the listed events assuming the files are in SAM though xrootd. The event file output is hard-wired\n";
	print "to runevtfilt-evt.root\n";
	print "\n";
	print "Usage: find-events --[opt]=[value]\n";
	print "  Where [opt] is one of\n";
	print "   --event-file=[file] : File that lists the events you want\n";
	print "                         Format [run] [subrun] [event] [slice]. Default = $event_file\n";
	print "   --data-def=[data def]: a SAM data set definition eg. 'prod_caf_FA14-12-29_nd_genie_nonswap_preshutdown_downsampled'\n";
	print "   --trigger=[trig]    : Trigger selection (t00, t02, etch). Default = $trigger\n";
	print "   --tier=[tier]    : Data tier (artdaq,reco,pid,caf etc). Default = $tier\n";
	print "   --prefix=[prefix]   : File prefix (ndos,reco). Default=$file_prefix\n";
	print "   --release=[rel]     : Software release. Default=$release\n";
	print "   --dont-run          : Print what this would do, but don't actually do it\n";
	print "   --help              : Prints this message\n";
	exit(0);
    }
    if ($ARGV[0] eq "--dont-run") { $dont_run = 1; }
    
    ($opt,$value) = split('=',$ARGV[$i]);
    if ($opt eq "--event-file") { $event_file  = $value; }
    if ($opt eq "--data-def")   { $data_def    = $value; $have_data_def = 1; }
    if ($opt eq "--trigger")    { $trigger     = $value; }
    if ($opt eq "--detector")   { $detector    = $value; }
    if ($opt eq "--prefix")     { $file_prefix = $value; }
    if ($opt eq "--release")    { $release     = $value; }
}
#.......................................................................

#
# If we are given a SAM data set definition, get a list of all the
# files that match that data set definition. Our events will be there.
#
print "Finding the full data set file list...";
if ($have_data_def==1) {
    `samweb list-definition-files $data_def > samfiles.txt`;
}
print "done.\n";

$file_list  = "files-list.txt";
$event_list = "event-list.txt";

#
# Open the file listing the run, subrun, and event numbers
#
open (FILE, $event_file);
open (FILE_LIST,">$file_list");
open (EVT_LIST,">$event_list");
while ( <FILE> ) {
    chomp($_);
    ($run,$subrun,$event,$slice) = split(" ",$_);
    #
    # Construct a file name wild card which will select the file which contains 
    # the event we want
    #
    if ($have_data_def==1) {
	$run_text    = sprintf("r%.8d",$run);
	$subrun_text = sprintf("s%02d",$subrun);
	$fullname = `grep $run_text samfiles.txt | grep $subrun_text`;
    }
    else {
	$fullname = sprintf("%s_r%.8d_s%.2d_%s_%s_v1_data.%s.root\n",
			    $file_prefix,
			    $run,
			    $subrun,
			    $trigger,
			    $release,
			    $tier);
    }
    print FILE_LIST "$fullname";
    print EVT_LIST "$run $subrun $event $slice\n";
}
close(FILE);
close(FILE_LIST);
close(EVT_LIST);

#
# Make sure the file list contains files only once
#
`sort $file_list > files-list.tmp`;
open (TMP_LIST,"files-list.tmp");
open (FILE_LIST,">$file_list");
$last_line = "";
while ( <TMP_LIST> ) {
    if ($_ ne $last_line) {
	print FILE_LIST $_;
	$last_line = $_;
    }
}
close (TMP_LIST);
close (FILE_LIST);

#
# Get ticket to be able to stream files from SAM with samweb2xrootd
#
system("kx509");
system("kxlist -p");
system("voms-proxy-init --rfc --voms=fermilab:/fermilab/nova/Role=Analysis --noregen");

open (FILE_LIST, $file_list);
while ( <FILE_LIST> ) {
  my $tvar = $_;
  chomp $tvar;
  $cmd = "nova -c runeventfiltjob.fcl -o $tvar `samweb2xrootd $_`";
  system($cmd);
}
close (FILE_LIST);

#
# Need to combine small files together
#
$cmd2 = "nova -c concat_files.fcl -o $endfile -S $file_list";
system($cmd2);

#
# Remove small files now that we are done
#
open (FILE_LIST, $file_list);
while ( <FILE_LIST> ) {
  system("rm $_");
}
close (FILE_LIST);



########################################################################
