//////////////////////////////////////////////////////////////////////
/// \brief
/// \author  ricken@fnal.gov
/// \date    Sept. 2012
/////////////////////////////////////////////////////////////////////
#include "VertexFinder/DOCAInfo.h"

#include "RecoBase/Track.h"

#include <cmath>

namespace vf
{
  //...................................................................
  DOCAInfo::DOCAInfo(rb::Track track1, rb::Track track2)
  {
    TVector3 end1 = track1.Start() + track1.Dir();
    TVector3 end2 = track2.Start() + track2.Dir();

    DoCalculation(track1.Start(), track2.Start(), end1, end2);
  }

  //...................................................................
  void DOCAInfo::DoCalculation(TVector3 I1, TVector3 I2,
		      TVector3 F1, TVector3 F2)
  {
    TVector3 anyV, A, B, alpha, beta, ratio;
    double sigma1, sigma2;

    delta1_ = F1 - I1;
    delta2_ = F2 - I2;
    anyV = I2 - I1;

    ratio = (delta1_.Cross(delta2_)).Unit();

    // Distance of closest approach:
    doca_ = fabs(anyV * ratio);

    alpha = delta1_.Cross(ratio);
    beta = delta2_.Cross(ratio);

    sigma1 = -(alpha * anyV) / (alpha * delta2_);
    sigma2 = (beta * anyV) / (beta * delta1_);

    A = I1 + sigma2 * delta1_;
    B = I2 + sigma1 * delta2_;

    // Reconstructed Vertex:
    vertex_reco_ = (A + B) * 0.5;
  }

  //.....................................................................
  TVector3 DOCAInfo::VertexReco() const
  {
    return vertex_reco_;
  }

  //...................................................................
  double DOCAInfo::Doca() const
  {
    return doca_;
  }

  //...................................................................
  double DOCAInfo::AngleReco() const
  {
    return (delta1_ * delta2_) / (delta1_.Mag() * delta2_.Mag());
  }
  
} // end namespace vf
//////////////////////////////////////////////////////////////////////////////
