##############
# Setup the Job specific parameters
JOBNAME='FMM_dataRC.sh'
DATASET='whateverFmm'
#NUMFILES=4
#NUMJOBS=-1 # if <= 0, it submits all of the jobs in the project 
NUMFILES=1
NUMJOBS=1
#OUTPUT_DIR="/pnfs/nova/scratch/users/ricken/FmmRecoData/"
RECO_DIR="/pnfs/nova/scratch/users/ricken/FD_fmmdata_reco/"
ANA_DIR="/nova/ana/exotics_ana/FMMana/"
JOBFCL='${SRT_PRIVATE_CONTEXT}/job/fmmtrackerjob.fcl'

#############################################
# USER INTERVENTION ENDS HERE, IN PRINCIPLE #
#############################################
ROLE=""
DATA_TIER1="reco"
DATA_TIER2="ana"

# Setup the SAM project here
if [[ "x$PROJECTNAME" == "x" ]]; then
    DATE="`date +%Y%m%d_%H%M`"
    PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
    if [ $? -eq 0 ]; then
	export SAM_PROJECT_NAME="${PROJECTNAME}"
    fi
else
    export SAM_PROJECT_NAME="${PROJECTNAME}"
fi

if [ ${NUMJOBS} -le 0 ]; then
    export NUMPROJECTFILES=`samweb -e nova count-definition-files ${DATASET}`
    if [ ${NUMFILES} -eq 1 ]; then
	export NUMJOBS=${NUMPROJECTFILES}
    else
	export NUMJOBS=`echo "(${NUMPROJECTFILES}/ ${NUMFILES}) + 1" | bc`
    fi
fi

##########################################
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
    X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
    X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

###########################################
# Give log output names a sensible name based on the PROJECTNAME
## assuming Martin has not changed the scripts
ln -s $SCRIPT /nova/app/condor-exec/$USER/${PROJECTNAME}.sh
JOBSCRIPT=/nova/app/condor-exec/$USER/${PROJECTNAME}.sh

##########################################
# Print out some useful information
echo "NUMJOBS IS $NUMJOBS with $NUMFILES per job"
echo "DATASET: " $DATASET
echo "Project name is: $SAM_PROJECT_NAME"
echo "Station Monitor: http://samweb.fnal.gov:8480/station_monitor/nova/stations/nova/projects/$SAM_PROJECT_NAME"
echo "$SAM_PROJECT_NAME" >> $0.projectname.log

###########################################
# The Magic Jobsub command
jobsub \
    -T \
    -N ${NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -e NOVADBTIMEOUT \
    -e NOVADBUSER  \
    -e NOVADBPWDFILE \
    -g \
    ${JOBSCRIPT} \
    --multifile \
    --export EXTERNALS='/nusoft/app/externals' \
    --config $JOBFCL \
    --source '/grid/fermiapp/nova/novaart/novasvn/setup/setup_nova.sh:-r:'"${NOVA_RELEASE}:-b:maxopt" \
    --source "/nova/app/users/ricken/art/GridSetups/setup_rel.sh" \
    --source '/nova/app/users/ricken/art/GridSetups/setup_grid_db.sh' \
    --limit ${NUMFILES} \
    --postscript '/nova/app/users/ricken/art/GridSetups/copyOutFiles.sh' \
    --export DEST=${RECO_DIR} \
    -X '/nova/app/users/ricken/art/GridSetups/runNovaSAM.py' \
    --outTier=out1:${DATA_TIER1} \
    --histTier=${DATA_TIER2} \
