////////////////////////////////////////////////////////////////////////
// Class:       FmmTriggerAna
// Module Type: analyzer
// File:        FmmTriggerAna_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"

#include "MCCheater/BackTracker.h"
#include "Calibrator/Calibrator.h"
#include "Simulation/Particle.h"
#include "Simulation/ParticleNavigator.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "Utilities/func/MathUtil.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "HoughPair.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/Geo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/RecoHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"

#include "TVector3.h"
#include "TTree.h"
#include "TProfile.h"

#include <algorithm> 
#include <utility>
#include <cmath>
#include <iostream>

namespace zcl {
  class FmmTriggerAna;
}

class zcl::FmmTriggerAna : public art::EDAnalyzer {
public:
  explicit FmmTriggerAna(fhicl::ParameterSet const & pset);
  virtual ~FmmTriggerAna();

  void analyze(const art::Event & evt);
  void beginJob();
  void endJob();
  
private:
  std::string fClusterInput;    ///< Input folder from cluster reco

  //Each entry is corresponding to 1 reconstructed slice
  TTree    * RC_Tree;
  TProfile * hitTdiff;
  TProfile * hitTdiff_log;
  TProfile * DCS_beta;
  TProfile * DCS_logbeta;

  int    evtID, subRunID, RunID;

  //Trigger decision branches
  bool   _sliced;
  int    noise_hits;
  bool   _penetrated;
  int    _nSatHits;
  double _meanADC;
  int    _deltaP;
  double _deltaTNS;
  int    _nxhits_slice, _nyhits_slice;

  double _tmin, _tmax;  //The time window of the slice

  int _pmin, _pmax;

  double beta_MC;
  double Edep_MC;
  double xi_MC, xf_MC;
  double yi_MC, yf_MC;
  double zi_MC, zf_MC;
  double ti_MC, tf_MC;
  int    nxhits_MC, nyhits_MC; 

};

zcl::FmmTriggerAna::FmmTriggerAna(fhicl::ParameterSet const & p) :
  EDAnalyzer(p),
  fClusterInput      (p.get< std::string > ("ClusterInput")      )
{
}

zcl::FmmTriggerAna::~FmmTriggerAna()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::FmmTriggerAna::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;

  hitTdiff= tfs->make<TProfile>("hitTdiff","",100,0,0.2,-1000, 1000);
  hitTdiff_log= tfs->make<TProfile>("hitTdiff_log","",100,-3.7,-0.7,-1000, 1000);
  DCS_beta= tfs->make<TProfile>("DCS_beta","",100,0,0.2, 0, 4095);
  DCS_logbeta= tfs->make<TProfile>("DCS_logbeta","",100,-3.7,-0.7,0, 4095);


  hitTdiff -> SetErrorOption("s");
  hitTdiff -> Approximate(true);
  hitTdiff_log -> SetErrorOption("s");
  hitTdiff_log -> Approximate(true);
  DCS_beta -> SetErrorOption("s");
  DCS_beta -> Approximate(true);
  DCS_logbeta -> SetErrorOption("s");
  DCS_logbeta -> Approximate(true);

  //Combining cheater info and the RC info to make the data structure better
  //One RC track info per entry, if no tracks reconstructed in this event, fill in 1 entry anyway tagging the 
  //reconstruction failed
  RC_Tree = tfs->make<TTree>("RC_Tree","Information from RC track info");
  RC_Tree -> Branch("event",  &evtID,      "event/I");
  RC_Tree -> Branch("subRun", &subRunID,   "subRun/I");     
  RC_Tree -> Branch("Run",    &RunID,      "Run/I");

  //Trigger threshold parameters
  RC_Tree -> Branch("sliced",      &_sliced,      "sliced/O");
  RC_Tree -> Branch("penetrated",  &_penetrated,  "penetrated/O");
  RC_Tree -> Branch("nSatHits",    &_nSatHits,    "nSatHits/I");
  RC_Tree -> Branch("meanADC",     &_meanADC,     "meanADC/D");
  RC_Tree -> Branch("deltaP",      &_deltaP,      "deltaP/I");
  RC_Tree -> Branch("deltaTNS",    &_deltaTNS,    "deltaTNS/D");
  RC_Tree -> Branch("nxhS",        &_nxhits_slice,    "nxhS/I");
  RC_Tree -> Branch("nyhS",        &_nyhits_slice,    "nyhS/I");

  RC_Tree -> Branch("tmin",        &_tmin,    "tmin/D");
  RC_Tree -> Branch("tmax",        &_tmax,    "tmax/D");
  RC_Tree -> Branch("pmin",        &_pmin,    "pmin/I");
  RC_Tree -> Branch("pmax",        &_pmax,    "pmax/I");

  //Cheater info ntuple part
  RC_Tree -> Branch("hits_noise",    &noise_hits,     "hits_noise/I");  //Check the purity
  RC_Tree -> Branch("betaMC",  &beta_MC,  "betaMC/D");
  RC_Tree -> Branch("edepMC",  &Edep_MC,  "edepMC/D");
  RC_Tree -> Branch("tiMC",    &ti_MC,    "tiMC/D");
  RC_Tree -> Branch("xiMC",    &xi_MC,    "xiMC/D");
  RC_Tree -> Branch("yiMC",    &yi_MC,    "yiMC/D");
  RC_Tree -> Branch("ziMC",    &zi_MC,    "ziMC/D");
  RC_Tree -> Branch("tfMC",    &tf_MC,    "tfMC/D");
  RC_Tree -> Branch("xfMC",    &xf_MC,    "xfMC/D");
  RC_Tree -> Branch("yfMC",    &yf_MC,    "yfMC/D");
  RC_Tree -> Branch("zfMC",    &zf_MC,    "zfMC/D");
  RC_Tree -> Branch("xhitsMC", &nxhits_MC,  "xhitsMC/I");
  RC_Tree -> Branch("yhitsMC", &nyhits_MC,  "yhitsMC/I");
}

void zcl::FmmTriggerAna::analyze(const art::Event & evt)
{
  evtID    = evt.id().event();
  subRunID = evt.subRun();
  RunID    = evt.run();

  art::ServiceHandle<geo::Geometry> geom;
  art::ServiceHandle<cheat::BackTracker> bt;
  art::ServiceHandle<calib::Calibrator> calib;

  std::vector<int> plane_fls;
  std::vector<int> cell_fls;
  std::vector<double> t_fls;
  /*
  //Get MC info first:
  const sim::ParticleNavigator& pnav = bt->ParticleNavigator();
  for (sim::ParticleNavigator::const_iterator i = pnav.begin(); i != pnav.end(); ++i) {
    nxhits_MC = 0;
    nyhits_MC = 0;
    const sim::Particle *p = (*i).second;
    if (p->PdgCode()!=42 ) continue;
    const std::vector<sim::FLSHit>& flshits = bt->ParticleToFLSHit(p->TrackId());
    unsigned ntruehits_Tot = flshits.size();
 
    if ( ntruehits_Tot==0 ) continue;
    double momentum2 = (p->P())*(p->P());
    double mass2     = (p->Mass())*(p->Mass());
    beta_MC = sqrt(momentum2 / (momentum2+mass2) );

    ti_MC = flshits[0].GetEntryT();
    xi_MC = flshits[0].GetEntryX();
    yi_MC = flshits[0].GetEntryY();
    zi_MC = flshits[0].GetEntryZ();
    tf_MC = flshits[0].GetExitT();
    xf_MC = flshits[0].GetExitX();
    yf_MC = flshits[0].GetExitY();
    zf_MC = flshits[0].GetExitZ();
    Edep_MC = flshits[0].GetEdep();

    size_t planeID_i = flshits[0].GetPlaneID();
    size_t cellID_i  = flshits[0].GetCellID();
    size_t planeID_f = planeID_i;
    size_t cellID_f  = cellID_i;

    if (planeID_i%2) ++nxhits_MC;
    else ++nyhits_MC;

    plane_fls.emplace_back(planeID_i);
    cell_fls.emplace_back(cellID_i);
    t_fls.emplace_back(ti_MC);

    for (unsigned h =1; h!= flshits.size(); ++h) {
      double ti = flshits[h].GetEntryT();
      double tf = flshits[h].GetExitT(); 
      plane_fls.emplace_back(flshits[h].GetPlaneID());
      cell_fls.emplace_back(flshits[h].GetCellID());
      t_fls.emplace_back(ti);
      if (ti < ti_MC) {
	ti_MC = ti;
	xi_MC = flshits[h].GetEntryX();
	yi_MC = flshits[h].GetEntryY();
	zi_MC = flshits[h].GetEntryZ();
	planeID_i = flshits[h].GetPlaneID();
	cellID_i  = flshits[h].GetCellID();
      }
      if (tf > tf_MC) {
	tf_MC = tf;
	xf_MC = flshits[h].GetExitX();
	yf_MC = flshits[h].GetExitY();
	zf_MC = flshits[h].GetExitZ();
	planeID_f = flshits[h].GetPlaneID();
	cellID_f  = flshits[h].GetCellID();
      }

      if (flshits[h].GetPlaneID() % 2) ++nxhits_MC;
      else ++nyhits_MC;

      Edep_MC += flshits[h].GetEdep();
    }

    //By now, you have got the local coordinate, you have to translate it to world:
    const geo::CellGeo* startCell = geom->Plane(planeID_i)->Cell(cellID_i);
    double local[3], world[3];
    local[0] = xi_MC;
    local[1] = yi_MC;
    local[2] = zi_MC;
    startCell->LocalToWorld(local,world);
    xi_MC = world[0];
    yi_MC = world[1];
    zi_MC = world[2];

    const geo::CellGeo* stopCell = geom->Plane(planeID_f)->Cell(cellID_f);
    local[0] = xf_MC;
    local[1] = yf_MC;
    local[2] = zf_MC;
    stopCell->LocalToWorld(local,world);
    xf_MC = world[0];
    yf_MC = world[1];
    zf_MC = world[2];
    //1 monopole per event, so you can break out from the loop:
    break;
  }
  */
  //=====================================================================================
  //Now, start pseudo trigger:
  const int _xMax       = 383; 
  const int _yMax       = 383; 
  const int _zMax       = 895; 
  const int _xdelt      = 35;
  const int _ydelt      = 35;
  const int _zdelt      = 15;

  int PX_max, PY_max, PX_min, PY_min;
  double TX_max, TY_max, TX_min, TY_min;

  art::Handle< std::vector<rb::Cluster> > slices;
  evt.getByLabel(fClusterInput, slices);

  _sliced = slices->size(); 
  //If no slice reconstructed, just return the MC info
  if (!_sliced) RC_Tree->Fill();

  //Instead of splitting the hits into 2 views, just directly take the entire 3D slice:
  for (unsigned sliceIdx = 0; sliceIdx != slices->size(); ++sliceIdx) {
    const rb::Cluster& slice = (*slices)[sliceIdx];

    _nxhits_slice = 0;
    _nyhits_slice = 0;

    PX_max = 0;	
    PY_max = 0;
    PX_min = 999;
    PY_min = 999;
    TX_max = -9e7;	
    TY_max = -9e7;
    TX_min = 9e9;
    TY_min = 9e9;

    _penetrated = false;
    _nSatHits   = 0;
    _meanADC    = 0;
    noise_hits  = 0;

    bool Intrusion = false;
    int entry_id = -1;
    for (unsigned i = 0; i != slice.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = slice.Cell(i);
      //    if (chit->ADC() < 501) continue;
      if (chit->ADC() > 3100) ++_nSatHits;
      _meanADC += chit->ADC();
      float tns = chit->TNS();
      //   double logBeta = log10(beta_MC);
      /*
      if (!bt->IsNoise(chit) || (chit->ADC()>3000 && tns > ti_MC-1000 && tns < tf_MC+1000) ) {
	//Looking for the true hits:
	bool found = false;

	for (unsigned h = 0; h!=t_fls.size(); ++h) {
	  if (chit->Plane()==plane_fls[h] && chit->Cell()==cell_fls[h]) {
	    hitTdiff->Fill(beta_MC,tns-t_fls[h],1);
	    hitTdiff_log->Fill(logBeta,tns-t_fls[h],1);

	    DCS_beta->Fill(beta_MC, chit->ADC(), 1);
	    DCS_logbeta->Fill(logBeta,chit->ADC(), 1);
	    found = true;
	    break;
	  }
	}
	if (!found) ++noise_hits;
      }
    
      else ++noise_hits;
      */
      if (chit->View()==geo::kX) {       //X view
	++_nxhits_slice;
	if (PX_max < chit->Plane() ) PX_max = chit->Plane();
	if (PX_min > chit->Plane() ) PX_min = chit->Plane();
	if (TX_max < tns   ) TX_max = tns;
	if (TX_min > tns   ) TX_min = tns;

	if (!Intrusion) {
	  if (chit->Cell() < _xdelt){
	    entry_id  = 1;
	    Intrusion = true;
	  }
	  else if (chit->Cell() > _xMax - _xdelt){
	    entry_id  = 2;
	    Intrusion = true;
	  }
	}
	else if (!_penetrated) {
	  if      (chit->Cell() < _xdelt && entry_id!=1)       _penetrated = true;
	  else if (chit->Cell() > _xMax-_xdelt && entry_id!=2) _penetrated = true;
	}
      }

      else {       //Y view
	++_nyhits_slice;
	if (PY_max < chit->Plane() ) PY_max = chit->Plane();
	if (PY_min > chit->Plane() ) PY_min = chit->Plane();
	if (TY_max < chit->TNS()   ) TY_max = chit->TNS();
	if (TY_min > chit->TNS()   ) TY_min = chit->TNS();

	if (!Intrusion) {
	  if (chit->Cell() < _ydelt) {
	    entry_id  = 3;
	    Intrusion = true;
	  }
	  else if (chit->Cell() > _yMax - _ydelt) {
	    entry_id  = 4;
	    Intrusion = true;
	  }
	}

	else if (!_penetrated) {
	  if      (chit->Cell() < _ydelt && entry_id!=3)       _penetrated = true;
	  else if (chit->Cell() > _yMax-_ydelt && entry_id!=4) _penetrated = true;
	}
      }

      if (!Intrusion) {
	if (chit->Plane() < _zdelt) {
	  entry_id  = 5;
	  Intrusion = true;
	}
	else if (chit->Plane() > _zMax - _zdelt) {
	  entry_id  = 6;
	  Intrusion = true;
	}
      }
      else if (!_penetrated) {       //intrusion happened but not through yet
	if (chit->Plane() < _zdelt && entry_id!=5) _penetrated=true;
	else if (chit->Plane() > _zMax-_zdelt && entry_id!=6) _penetrated=true;
      }
    }
    //Record the info
    _meanADC     = _meanADC/(_nxhits_slice+_nyhits_slice+0.);
    _deltaP      = std::min(PX_max,PY_max) - std::max(PX_min,PY_min);
    _deltaTNS    = std::min(TX_max,TY_max) - std::max(TX_min,TY_min);    
    _tmin        = std::min(TX_min,TY_min);
    _tmax        = std::max(TX_max,TY_max);
    _pmin        = std::min(PX_min,PY_min);
    _pmax        = std::max(PX_max,PY_max);

    //   if (_nxhits_slice+_nyhits_slice < 10) _sliced = false;
    //   std::cout<<"xhits: "<<_nxhits_slice<<" yhits: "<<_nyhits_slice<<std::endl;

    RC_Tree->Fill();
  }

} //end analyzing 

void zcl::FmmTriggerAna::endJob(){

}

DEFINE_ART_MODULE(zcl::FmmTriggerAna)
