////////////////////////////////////////////////////////////////////////
// Class:       MMCheater
// Module Type: analyzer
// File:        MMCheater_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "MCCheater/BackTracker.h"
#include "Simulation/Particle.h"
#include "Simulation/ParticleNavigator.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"


#include "HoughPair.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"

#include "TH3.h"
#include "TVector3.h"
#include "TTree.h"

#include <algorithm> 
#include <utility>
#include <cmath>
#include <iostream>

namespace zcl {
  class MMCheater;
}

class zcl::MMCheater : public art::EDAnalyzer {
public:
  explicit MMCheater(fhicl::ParameterSet const & pset);
  virtual ~MMCheater();

  void beginJob();
  double distance(float c1, float c2, float p1, float p2);
  void analyze(const art::Event & evt);

private:
  std::string fHitInput;   
 
  TTree* Cheater_Tree;
  double beta, cosx, cosy, cosz;;
  double edep, pl, tof;
  int    NXhits, NYhits;
  int    D_TDC_max, D_Z_max;
  int    ADC_min;
  double ADC_mean;
  int    D_P_min, D_C_min;
  double Lx, D_lx_max, vpro_x, D_Tx_max_l, D_Tx_max_s;
  double Ly, D_ly_max, vpro_y, D_Ty_max_l, D_Ty_max_s;


};

zcl::MMCheater::MMCheater(fhicl::ParameterSet const & p) :
  EDAnalyzer(p),
  fHitInput          (p.get< std::string > ("CellHitInput")   )
{
}

zcl::MMCheater::~MMCheater()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::MMCheater::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;
  Cheater_Tree = tfs->make<TTree>("Cheater_Tree","Information from MC Cheater on event basis");
  Cheater_Tree -> Branch("beta",  &beta,  "beta/D");
  Cheater_Tree -> Branch("cosx",  &cosx,  "cosx/D");
  Cheater_Tree -> Branch("cosy",  &cosy,  "cosy/D");
  Cheater_Tree -> Branch("cosz",  &cosz,  "cosz/D");
  Cheater_Tree -> Branch("edep",  &edep,  "edep/D");
  Cheater_Tree -> Branch("PathLength",  &pl,  "PathLength/D");
  Cheater_Tree -> Branch("tof",    &tof,    "tof/D");
  Cheater_Tree -> Branch("nxhits",    &NXhits,    "nxhits/I");
  Cheater_Tree -> Branch("nyhits",    &NYhits,    "nyhits/I");
  Cheater_Tree -> Branch("D_TDC_max",    &D_TDC_max,    "D_TDC_max/I");
  Cheater_Tree -> Branch("D_Z_max",    &D_Z_max,    "D_Z_max/I");
  Cheater_Tree -> Branch("ADC_min",    &ADC_min,    "ADC_min/I");
  Cheater_Tree -> Branch("ADC_mean",    &ADC_mean,    "ADC_mean/D");
  Cheater_Tree -> Branch("D_P_min",    &D_P_min,    "D_P_min/I");
  Cheater_Tree -> Branch("D_C_min",    &D_C_min,    "D_C_min/I");
  Cheater_Tree -> Branch("Lx", &Lx,  "Lx/D");    //In DDT unit
  Cheater_Tree -> Branch("D_lx_max", &D_lx_max,  "D_lx_max/D");
  Cheater_Tree -> Branch("vpro_x", &vpro_x,  "vpro_x/D");
  Cheater_Tree -> Branch("D_Tx_max_l", &D_Tx_max_l,  "D_Tx_max_l/D");
  Cheater_Tree -> Branch("D_Tx_max_s", &D_Tx_max_s,  "D_Tx_max_s/D");
  Cheater_Tree -> Branch("Ly", &Ly,  "Ly/D");
  Cheater_Tree -> Branch("D_ly_max", &D_ly_max,  "D_ly_max/D");
  Cheater_Tree -> Branch("vpro_y", &vpro_y,  "vpro_y/D");
  Cheater_Tree -> Branch("D_Ty_max_l", &D_Ty_max_l,  "D_Ty_max_l/D");
  Cheater_Tree -> Branch("D_Ty_max_s", &D_Ty_max_s,  "D_Ty_max_s/D");
}

void zcl::MMCheater::analyze(const art::Event & evt)
{
  art::ServiceHandle<geo::Geometry> geom;
  art::ServiceHandle<cheat::BackTracker> bt;
  //Get MC info first:
  const sim::ParticleNavigator& pnav = bt->ParticleNavigator();
  for (sim::ParticleNavigator::const_iterator i = pnav.begin(); i != pnav.end(); ++i) {
    const sim::Particle *p = (*i).second;
    if (p->PdgCode()!=42 ) continue;
    const std::vector<sim::FLSHit>& flshits = bt->ParticleToFLSHit(p->TrackId());
 
    if (flshits.size()==0 ) continue;
    cosx = (p->Px())/(p->P());
    cosy = (p->Py())/(p->P());
    cosz = (p->Pz())/(p->P());
    double momentum2 = (p->P())*(p->P());
    double mass2     = (p->Mass())*(p->Mass());
    beta = sqrt(momentum2 / (momentum2+mass2) );

    double ti_MC = flshits[0].GetEntryT();
    double tf_MC = flshits[0].GetExitT();

    edep = flshits[0].GetEdep();
    pl   = flshits[0].GetTotalPathLength(); 
 
    for (unsigned h =1; h!= flshits.size(); ++h) {
      double ti = flshits[h].GetEntryT();
      double tf = flshits[h].GetExitT(); 
      if (ti < ti_MC) 
	ti_MC = ti;
      
      if (tf > tf_MC) 
	tf_MC = tf;
      
      pl   += flshits[h].GetTotalPathLength(); 
      edep += flshits[h].GetEdep();
    }
    tof = tf_MC-ti_MC;

    //1 monopole per event, so you can break out from the loop:
    break;
  }

  //  std::cout<<"particle info collected: FLS finish!"<<std::endl;

  std::vector<art::Ptr<rb::CellHit> > TrueHitsList;
  std::vector<art::Ptr<rb::CellHit> > Xhits;
  std::vector<art::Ptr<rb::CellHit> > Yhits;
  art::Handle<std::vector<rb::CellHit> > hitcol;
  evt.getByLabel(fHitInput, hitcol);

  for (unsigned int i = 0; i != hitcol->size(); ++i) {
    art::Ptr<rb::CellHit> hit_ptr(hitcol, i);
    if (!bt->IsNoise(hit_ptr)) 
      TrueHitsList.emplace_back(hit_ptr);
  }

  if (TrueHitsList.size()<6) return;
  rb::SortByTime(TrueHitsList);

  D_TDC_max = 0;
  D_Z_max   = 0;
  ADC_mean  = 0;
  ADC_min   = 4096;
  D_P_min   = 888;
  D_C_min   = 384;
  int  plane_tmp = TrueHitsList[0]->Plane();
  int  TDC_tmp = TrueHitsList[0]->TDC();

  int xti,xci,xpi,xtf,xcf,xpf; 
  int yti,yci,ypi,ytf,ycf,ypf; 
  xci = 384;
  yci = 384;
  xcf = -1;
  ycf = -1;
  xpi = 888;
  xpf = -1;
  ypi = 888;
  ypf = -1;
  xti = TDC_tmp;
  yti = TDC_tmp;
  xtf = TDC_tmp;
  ytf = TDC_tmp;

  for (unsigned i =0; i!= TrueHitsList.size(); ++i) {
    art::Ptr<rb::CellHit> hit_ptr = TrueHitsList[i];
    int ADC   = hit_ptr->ADC();
    int plane = hit_ptr->Plane();
    int cell  = hit_ptr->Cell(); 
    int TDC   = hit_ptr->TDC();

    if (hit_ptr->View()==geo::kX) {
      Xhits.emplace_back(hit_ptr);
      if (cell<xci) {
	xpi = plane;
	xci = cell;
	xti = TDC;
      }
      if (cell>xcf) {
	xpf = plane;
	xcf = cell;
	xtf = TDC;
      }
    }
    else {
      Yhits.emplace_back(hit_ptr);
      if (cell<yci) {
	ypi = plane;
	yci = cell;
	yti = TDC;
      }
      if (cell>ycf) {
	ypf = plane;
	ycf = cell;
	ytf = TDC;
      }
    }
    int delta_tdc = abs(TDC - TDC_tmp);
    TDC_tmp = TDC;
    if (delta_tdc > D_TDC_max) D_TDC_max = delta_tdc;
    
    int delta_z = abs(plane - plane_tmp);
    plane_tmp = plane;
    if (delta_z > D_Z_max) D_Z_max = delta_z;

    ADC_mean += ADC;
    if (ADC_min > ADC) ADC_min = ADC;
    
    int delta_p = std::min(887-plane, plane);
    int delta_c = std::min(383-cell, cell);
    if (D_P_min > delta_p) D_P_min = delta_p;
    if (D_C_min > delta_c) D_C_min = delta_c;
  }

  ADC_mean = ADC_mean/( TrueHitsList.size()+0.);

  NXhits = Xhits.size();
  NYhits = Yhits.size();

  //  std::cout<<"Cosmic Slice info collected!"<<std::endl;

  if (NXhits<3 || NYhits<3) return;

  //OK, now the slow MM rc and trigger tunes:
  //  rb::SortByPlaneAndCell(Xhits);
  Lx = distance(xci, xcf, xpi, xpf);
  //  std::cout<<"xci: "<<xci<<"  xcf: "<<xcf<<"  xpi: "<<xpi<<"  xpf: "<<xpf<<std::endl;
  vpro_x = (xtf-xti+0.)/Lx;
  D_lx_max = 0;
  D_Tx_max_l = 0;
  D_Tx_max_s = 0;
  for (unsigned i = 1; i!= Xhits.size()-1; ++i) {
    auto hit_ptr = Xhits[i];
    int pi = hit_ptr->Plane();
    int ci = hit_ptr->Cell();
    int ti = hit_ptr->TDC(); 
    double dl1 = distance(xci, ci, xpi, pi);
    double dl2 = distance(xcf, ci, xpf, pi);
    if (dl1+dl2-Lx > D_lx_max) D_lx_max=dl1+dl2-Lx;

    double dtl, dts;
    if (dl1>dl2) {
      dtl = fabs(ti-(xti+vpro_x*dl1));
      dts = fabs(ti-(xtf-vpro_x*dl2));
    }

    else {
      dts = fabs(ti-(xti+vpro_x*dl1));
      dtl = fabs(ti-(xtf-vpro_x*dl2));
    }

    if (dts>D_Tx_max_s) D_Tx_max_s = dts;
    if (dtl>D_Tx_max_l) D_Tx_max_l = dtl;
  }
  //  std::cout<<"X SMM finished!"<<std::endl;
  //Yview :
  //  rb::SortByPlaneAndCell(Yhits);
  Ly = distance(yci, ycf, ypi, ypf);
  vpro_y = (ytf-yti+0.)/Ly;
  D_ly_max = 0;
  D_Ty_max_l = 0;
  D_Ty_max_s = 0;
  for (unsigned i = 1; i!= Yhits.size()-1; ++i) {
    auto hit_ptr = Yhits[i];
    int pi = hit_ptr->Plane();
    int ci = hit_ptr->Cell();
    int ti = hit_ptr->TDC(); 
    double dl1 = distance(yci, ci, ypi, pi);
    double dl2 = distance(ycf, ci, ypf, pi);
    if (dl1+dl2-Ly > D_ly_max) D_ly_max=dl1+dl2-Ly;

    double dtl, dts;
    if (dl1>dl2) {
      dtl = fabs(ti-(yti+vpro_y*dl1));
      dts = fabs(ti-(ytf-vpro_y*dl2));
    }

    else {
      dts = fabs(ti-(yti+vpro_y*dl1));
      dtl = fabs(ti-(ytf-vpro_y*dl2));
    }

    if (dts>D_Ty_max_s) D_Ty_max_s = dts;
    if (dtl>D_Ty_max_l) D_Ty_max_l = dtl;
  }

  //  std::cout<<"Delta TDC: "<<D_TDC_max<<std::endl;
  Cheater_Tree->Fill();
  return;
} //end production function

double zcl::MMCheater::distance(float c1, float c2,
				float p1, float p2)	
{ 
  return sqrt((p1-p2)*(p1-p2)*2.82072+(c1-c2)*(c1-c2));	
}

DEFINE_ART_MODULE(zcl::MMCheater)
