////////////////////////////////////////////////////////////////////////
// Class:       SMMTriggerAna
// Module Type: producer
// File:        SMMTriggerAna_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "MCCheater/BackTracker.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"

#include "TH3.h"
#include "TVector3.h"
#include "TTree.h"

#include <algorithm> 
#include <utility>
#include <cmath>

namespace zcl {
  class SMMTriggerAna;
}

class zcl::SMMTriggerAna : public art::EDAnalyzer {
public:
  explicit SMMTriggerAna(fhicl::ParameterSet const & pset);
  virtual ~SMMTriggerAna();

  void analyze(const art::Event & evt);
  void beginJob();
  void endJob();

private:
  bool SurfAssign(art::Ptr<rb::CellHit> const & hptr);
  bool ContaineAssign(art::Ptr<rb::CellHit> const & hptr);

  //Return the indices of the earliest and latest matching elements in the other view:
  bool ViewCheck(art::Ptr<rb::CellHit> const &,
		 art::Ptr<rb::CellHit> const &);  

  float Dist(art::Ptr<rb::CellHit> const &,
	     art::Ptr<rb::CellHit> const &);  

  //This function return the index of the hit which earlier than and closest to the time you assigned
  unsigned IndexLocator(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
			float tns);



  // FHICL Parameters:
  std::string fCellHitInput;
  float _hf_min;     //The threshold of hits/distance
  //One should consider the speed of MM going to the next plane in the other view, 
  //so the timing cut should be looser in accomodating slower monopoles
  unsigned _sigmaT;    //Time resolution (ns) for single hit: same view parameter
  unsigned _deltaT;    //for checking mate hits on the other view
  unsigned _minHits;   //precut on hits number limit

  float _deltaP;    //This is used for surface determination and also for view matching determination
  float _deltaC;    //For surface determination
  float _sigmaL;    // Triangle cut for the max additional ratio allowed
  int   _ADCmin;
  int   _ADCmax;
  float _Xmax;
  float _Ymax;
  float _DistMin;   //Min distance requirement    
  float _vproMin;   //Max beta: 0.01
  float _vproMax;   //Min beta: 4e-5
  float _df;        //Maximum fraction of (plane gap)/(plane range) allowed

  //Tree record info:
  TTree  * RC_Tree;
  int    evtID, subRunID, RunID;
  double beta_MC;
  double Edep_MC;
  double xi_MC, xf_MC;
  double yi_MC, yf_MC;
  double zi_MC, zf_MC;
  double ti_MC, tf_MC;
  int plane_entry, plane_exit;
  int cell_entry,  cell_exit;
  
  int    nxhits_MC, nyhits_MC; 
  bool   trigger;
  double ti, tf;
  double dist;
  int    hitsAttach;
};

zcl::SMMTriggerAna::SMMTriggerAna(fhicl::ParameterSet const & p) :
  EDAnalyzer(p),
  fCellHitInput      (p.get< std::string  >("CellHitInput")      ),
  _hf_min            (p.get< float    >("hf_min")),     //Min hit number density in unit distance(in cell unit)
  _sigmaT     (1000),      //64TDC 
  _deltaT     (2000),      //128TDC 
  _minHits    (20),
  _deltaP     (2),
  _deltaC     (6),
  _sigmaL     (0.02),
  _ADCmin     (p.get<int>("ADCmin")),
  _ADCmax     (p.get<int>("ADCmax")),
  _Xmax       (p.get<float>("Xmax")), 
  _Ymax       (p.get<float>("Ymax")),            
  _DistMin    (p.get<float>("DistMin")),
  _vproMin    (p.get<float>("vproMin")),
  _vproMax    (p.get<float>("vproMax")),
  _df         (0.1)
{
}

zcl::SMMTriggerAna::~SMMTriggerAna()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::SMMTriggerAna::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;

  RC_Tree = tfs->make<TTree>("RC_Tree","Information from RC track info");
  RC_Tree -> Branch("event",  &evtID,      "event/I");
  RC_Tree -> Branch("subRun", &subRunID,   "subRun/I");     
  RC_Tree -> Branch("Run",    &RunID,      "Run/I");

  //Cheater info ntuple part
  RC_Tree -> Branch("betaMC",  &beta_MC,  "betaMC/D");
  RC_Tree -> Branch("edepMC",  &Edep_MC,  "edepMC/D");
  RC_Tree -> Branch("tiMC",    &ti_MC,    "tiMC/D");
  RC_Tree -> Branch("xiMC",    &xi_MC,    "xiMC/D");
  RC_Tree -> Branch("yiMC",    &yi_MC,    "yiMC/D");
  RC_Tree -> Branch("ziMC",    &zi_MC,    "ziMC/D");
  RC_Tree -> Branch("tfMC",    &tf_MC,    "tfMC/D");
  RC_Tree -> Branch("xfMC",    &xf_MC,    "xfMC/D");
  RC_Tree -> Branch("yfMC",    &yf_MC,    "yfMC/D");
  RC_Tree -> Branch("zfMC",    &zf_MC,    "zfMC/D");
  RC_Tree -> Branch("xhitsMC", &nxhits_MC,  "xhitsMC/I");
  RC_Tree -> Branch("yhitsMC", &nyhits_MC,  "yhitsMC/I");
  RC_Tree -> Branch("plane_entry", &plane_entry,  "plane_entry/I");
  RC_Tree -> Branch("cell_entry",  &cell_entry,   "cell_entry/I");
  RC_Tree -> Branch("plane_exit", &plane_exit,  "plane_exit/I");
  RC_Tree -> Branch("cell_exit",  &cell_exit,   "cell_exit/I");

  //Trigger info
  RC_Tree -> Branch("trigger",    &trigger,    "trigger/O");
  RC_Tree -> Branch("ti",         &ti,         "ti/D");
  RC_Tree -> Branch("tf",         &tf,         "tf/D");
  RC_Tree -> Branch("dist",       &dist,       "dist/D");
  RC_Tree -> Branch("hitsAttach", &hitsAttach, "hitsAttach/I");
}

void zcl::SMMTriggerAna::analyze(const art::Event & evt)
{
  evtID    = evt.id().event();
  subRunID = evt.subRun();
  RunID    = evt.run();

  trigger = false;
  ti  = -10000;
  tf  = -10000;
  dist = 0;
  hitsAttach = 0;

  //First record the MC truth Info:
  art::ServiceHandle<cheat::BackTracker> bt;
  art::ServiceHandle<geo::Geometry> geom;
  const sim::ParticleNavigator& pnav = bt->ParticleNavigator();
  for (sim::ParticleNavigator::const_iterator i = pnav.begin(); i != pnav.end(); ++i) {
    nxhits_MC = 0;
    nyhits_MC = 0;
    const sim::Particle *p = (*i).second;
    if (p->PdgCode()!=42 ) continue;
    const std::vector<sim::FLSHit>& flshits = bt->ParticleToFLSHit(p->TrackId());
    unsigned ntruehits_Tot = flshits.size();
 
    if ( ntruehits_Tot==0 ) continue;
    double momentum2 = (p->P())*(p->P());
    double mass2     = (p->Mass())*(p->Mass());
    beta_MC = sqrt(momentum2 / (momentum2+mass2) );

    ti_MC = flshits[0].GetEntryT();
    xi_MC = flshits[0].GetEntryX();
    yi_MC = flshits[0].GetEntryY();
    zi_MC = flshits[0].GetEntryZ();
    tf_MC = flshits[0].GetExitT();
    xf_MC = flshits[0].GetExitX();
    yf_MC = flshits[0].GetExitY();
    zf_MC = flshits[0].GetExitZ();
    Edep_MC = flshits[0].GetEdep();

    size_t planeID_i = flshits[0].GetPlaneID();
    size_t cellID_i  = flshits[0].GetCellID();
    size_t planeID_f = planeID_i;
    size_t cellID_f  = cellID_i;

    if (planeID_i%2) ++nxhits_MC;
    else ++nyhits_MC;


    for (size_t h =1; h!= flshits.size(); ++h) {
      double ti_tmp = flshits[h].GetEntryT();
      double tf_tmp = flshits[h].GetExitT(); 
 
      if (ti_tmp < ti_MC) {
	ti_MC = ti_tmp;
	xi_MC = flshits[h].GetEntryX();
	yi_MC = flshits[h].GetEntryY();
	zi_MC = flshits[h].GetEntryZ();
	planeID_i = flshits[h].GetPlaneID();
	cellID_i  = flshits[h].GetCellID();
      }
      if (tf_tmp > tf_MC) {
	tf_MC = tf_tmp;
	xf_MC = flshits[h].GetExitX();
	yf_MC = flshits[h].GetExitY();
	zf_MC = flshits[h].GetExitZ();
	planeID_f = flshits[h].GetPlaneID();
	cellID_f  = flshits[h].GetCellID();
      }

      if (flshits[h].GetPlaneID() % 2) ++nxhits_MC;
      else ++nyhits_MC;

      Edep_MC += flshits[h].GetEdep();
    }

    //By now, you have got the local coordinate, you have to translate it to world:
    const geo::CellGeo* startCell = geom->Plane(planeID_i)->Cell(cellID_i);
    double local[3], world[3];
    local[0] = xi_MC;
    local[1] = yi_MC;
    local[2] = zi_MC;
    startCell->LocalToWorld(local,world);
    xi_MC = world[0];
    yi_MC = world[1];
    zi_MC = world[2];
    plane_entry = planeID_i;
    cell_entry  = cellID_i;
    
    const geo::CellGeo* stopCell = geom->Plane(planeID_f)->Cell(cellID_f);
    local[0] = xf_MC;
    local[1] = yf_MC;
    local[2] = zf_MC;
    stopCell->LocalToWorld(local,world);
    xf_MC = world[0];
    yf_MC = world[1];
    zf_MC = world[2];
    plane_exit = planeID_f;
    cell_exit  = cellID_f;

    
    //1 monopole per event, so you can break out from the loop:
    break;
  }

  //  std::cout<<"Truth info obtained!"<<std::endl;
  
  //Now, pseudo trigger process:
  art::Handle< std::vector<rb::CellHit> > hitcol;
  evt.getByLabel(fCellHitInput, hitcol);
  // Load the cell hits into a vector for easy use
  std::vector<art::Ptr<rb::CellHit > > hitlist[2];
  
  //This process imitated the ADC filtering process
  for(unsigned int i = 0; i < hitcol->size(); ++i){
    art::Ptr<rb::CellHit> hit_ptr(hitcol, i);
    //    if (bt->IsNoise(hit_ptr) ) continue;
    
    if (hit_ptr->ADC()>_ADCmax || hit_ptr->ADC()<_ADCmin) continue;
    unsigned v = 0;
    if (hit_ptr->View()==geo::kY ) v = 1;
    hitlist[v].emplace_back(hit_ptr);
  }

  if (hitlist[0].empty() || hitlist[1].empty()) {
    RC_Tree -> Fill();
    return;
  }


  //Sort the hits by time:
  rb::SortByTime(hitlist[0]);
  rb::SortByTime(hitlist[1]);

  //X, Y, T  (plane, cell, tns)
  std::vector<art::Ptr<rb::CellHit> > c_PS;     //They will contain all Valid non surface hits on X view
  std::vector<art::Ptr<rb::CellHit> > s_PS;     //They will contain all Valid surface and mate hits on X view


  //View Check and doing surface hits collecion
  for (unsigned i = 0; i!= hitlist[0].size(); ++i ) {
    art::Ptr<rb::CellHit> const& hit = hitlist[0][i];
    float t = hit->TNS();
    unsigned start_id = IndexLocator(hitlist[1], t-_deltaT);
    unsigned stop_id  = IndexLocator(hitlist[1], t+_deltaT); 
    bool met = false;
    for (unsigned j = start_id; j<stop_id; ++j ) {
      art::Ptr<rb::CellHit> const& buddy = hitlist[1][j];
      if (ViewCheck(hit, buddy)) {
	if (SurfAssign(hit) || SurfAssign(buddy)) {
	  s_PS.emplace_back(hit);
	  break;
	}
	else if (!met ) c_PS.emplace_back(hit);
	met=true;
      }     
    }
  }
  //  std::cout<<"contained hits: "<<c_PS.size()<<"  surf hits: "<<s_PS.size()<<std::endl;
  
  if (c_PS.size()<_minHits) {
    RC_Tree->Fill();
    return;
  }
  if (s_PS.size()<2) {
    RC_Tree->Fill();
    return;
  }
   

  //Try to find entry hit on valid surface hits containers
  for (unsigned i=0; i<s_PS.size()-1; ++i) {
    //     bool found = false;
    art::Ptr<rb::CellHit> hit1 = s_PS[i];
    ti = hit1->TNS();
    int   pi = hit1->Plane();
    int   ci = hit1->Cell();
    //   std::cout<<"ci: "<<ci <<"  pi: "<<pi<<std::endl;
    
    unsigned ini = IndexLocator(c_PS, ti) + 1;
 
    if (ini+_minHits >= c_PS.size() ) break;
      
    for (unsigned j=s_PS.size()-1; j>i && ini+_minHits+1 < c_PS.size(); --j) {
      art::Ptr<rb::CellHit> hit2 = s_PS[j];
      tf = hit2->TNS();
      int   pf = hit2->Plane();
      int   cf = hit2->Cell();

      //    std::cout<<"cf: "<<cf<<"  pf: "<<pf<<std::endl;
      
      if (tf+ _deltaT  < c_PS[ini+_minHits]->TNS()) break;
      else if (tf > c_PS[c_PS.size()-1]->TNS() + _deltaT )  continue;
      

      int p_max = std::max(pi,pf);
      int p_min = std::min(pi,pf);
      int c_max = std::max(ci,cf);
      int c_min = std::min(ci,cf);

      if (p_max < p_min+_deltaP) continue;

      unsigned   fin = IndexLocator(c_PS, tf+_deltaT);
      dist = Dist(hit1, hit2);

      if (dist < _DistMin)	continue;
      if (fin<ini+dist*_hf_min) continue;

      float vpro = (tf-ti)/dist;
      //    std::cout<<"vpro: "<<vpro<<"  Dist: "<<dist<<std::endl;
      if (tf > ti+dist*_vproMax+_sigmaT) continue;
     
      else if (tf+_sigmaT < ti+dist*_vproMin) continue;

      //Here we implemente the CCB(continuity-check and break) mechanisim
      //First, calculate the time window for checking the previous hit on track:
      int  _dP = (p_max-p_min)*_df;
      float dt = (tf-ti)*_df;

      if (_dP<2) _dP = 2;
      else if (_dP>4) _dP = 4;

      if (dt>256) dt = 256;

      //Fish is coming!
      int hitsOnline = 0;
      double disCut = dist*_sigmaL;

      int h_cut = dist*_hf_min;

      int last_C = ci;
      int last_P = pi;
      float last_T = ti;
      for (unsigned k=ini; k!=fin; ++k) {
	art::Ptr<rb::CellHit> Mid = c_PS[k];
	int pk = Mid->Plane();
	int ck = Mid->Cell();
	float tk = Mid->TNS();

	if (pk > p_max || pk < p_min) continue;
	if (ck > c_max || ck < c_min) continue;

	//Now check if it is forward going:
	if (pf>pi) {
	  if (pk < last_P || pk> last_P+_dP ) {
	    if (tk>last_T+dt) break;
	    else continue;
	  }
	  if (cf > ci && ck < last_C) continue;
	  if (cf < ci && ck > last_C) continue;	  
	}

	else {
	  if ( pk+_dP< last_P || pk>last_P ) {
	    if (tk>last_T+dt) break;
	    else continue;
	  }
	  if (pk > last_P || pk+_dP < last_P) continue;
	  if (cf > ci && ck < last_C) continue;
	  if (cf < ci && ck > last_C) continue;
	}
	


	float t1 = tk-ti;
	float t2 = tf-tk;
	float d1 = Dist(hit1, Mid);
	float d2 = Dist(hit2, Mid);

	if (d1+d2 > dist+disCut) continue;

	if (d1>d2) {
	  if (t1 < d1*vpro + _sigmaT && t1 > d1*vpro - _sigmaT) {
	    ++hitsAttach;
	    last_C= ck;
	    last_P= pk;
	    last_T= tk;
	  }
	}
	else {
	  if (t2 > d2*vpro - _sigmaT && t2 < d2*vpro + _sigmaT) {
	    ++hitsOnline;
	    last_C= ck;
	    last_P= pk;
	    last_T= tk;
	  }
	}
      }

      if (hitsOnline > h_cut ) {
	trigger = true;
	if (hitsOnline > hitsAttach) 
	  hitsAttach = hitsOnline;
      }
    }
  } // X view finished
  

  RC_Tree->Fill();
  /*
  if (beta_MC < 0.005 && beta_MC > 0.0001 && nxhits_MC > 20 && !trigger) {
    std::cout<<"beta: "<<beta_MC<<"  xhitsMC: "<<nxhits_MC<<" yhitsMC: "<<nyhits_MC<<"\n"
	     <<"  Plane entry: "<<plane_entry<<"  Cell entry:   "<<cell_entry<<"\n" 
	     <<"  Plane exit: "<<plane_exit<<"   Cell exit: "<<cell_exit<<"\n"
      //<<"  tiMC: "<<ti_MC
	     <<"  xiMC: "<<xi_MC<<" yiMC: "<<yi_MC<<" ziMC: "<<zi_MC<<"\n" 
      //<<"  tf_MC: "<<tf_MC
	     <<"  xfMC: "<<xf_MC<<"  yfMC: "<<yf_MC<<"  zfMC: "<<zf_MC<<"\n"
	     <<"  edep: "<<Edep_MC
	     <<"  ti: "<<ti<<"  tf: "<<tf<<"  hitsAttach: "<<hitsAttach<<std::endl;
 
  } 
  */
} //end production function

void zcl::SMMTriggerAna::endJob(){

}

bool zcl::SMMTriggerAna::SurfAssign(art::Ptr<rb::CellHit> const & hptr) 
{
  if      ( hptr->Cell()  > _Ymax - _deltaC ) return true;
  else if ( hptr->Cell()  < _deltaC         ) return true;
  else if ( hptr->Plane() < _deltaP         ) return true;
  else if ( hptr->Plane() > _Xmax - _deltaP ) return true;

  return false;

}


//Note: Ts < Tm (Looking forwards). Time check is already done in the main loop.
bool zcl::SMMTriggerAna::ViewCheck(art::Ptr<rb::CellHit> const & hptr1,
				   art::Ptr<rb::CellHit> const & hptr2)
{
  if (hptr1->TNS() > hptr2->TNS()+_deltaT) return false;
  if (hptr2->TNS() > hptr1->TNS()+_deltaT) return false;
  if (hptr1->Plane() > hptr2->Plane() ) {
    if (hptr1->Plane() - hptr2->Plane() < _deltaP ) return true;
  }
  else if (hptr2->Plane() - hptr1->Plane() < _deltaP ) return true;
  return false; 
}

float zcl::SMMTriggerAna::Dist(art::Ptr<rb::CellHit> const & hptr1,
			       art::Ptr<rb::CellHit> const & hptr2)
{
  float dx = ((hptr1->Plane() )-  (hptr2->Plane()) )*1.6795;
  float dy = (hptr1->Cell() ) -  (hptr2->Cell());
  return sqrt(dx*dx+dy*dy);
}  

//It should return the index before T
unsigned zcl::SMMTriggerAna::IndexLocator(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
					  float tns)
{
  unsigned i = 0;
  unsigned f = hit_ptrs.size()-1;

  if (tns < (hit_ptrs[0])->TNS()) return i;
  if (tns > (hit_ptrs[f])->TNS()) return f;

  while (f>i+1) {
    unsigned m = (f+i)/2;
    if (tns >= (hit_ptrs[m])->TNS() ) i = m;
    else f = m; 
  }
  return i;
}

DEFINE_ART_MODULE(zcl::SMMTriggerAna)
