#include "g4nova/PrintTables.h"
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "Geant4/G4ParticleTable.hh"
#include "Geant4/G4Material.hh"
#include "Geant4/G4EmCalculator.hh"
using namespace g4n;

PrintTables::PrintTables()
{
  fMaterialTable = G4Material::GetMaterialTable();
  fParticleTable = G4ParticleTable::GetParticleTable();
}

//......................................................................

void PrintTables::Materials()
{
  std::ofstream f;
  f.open("g4nova-materials.txt");
  unsigned int i;
  f << "*= MATERIALS" << std::endl;
  f << "n\tName\tZeff\tDensity [g/cc]\tRadLen [cm]\t"
    << "Nucl. Int. Len [cm]\t" 
    << std::endl;
  for (i=0; i<fMaterialTable->size(); ++i) {
    const G4Material* m = (*fMaterialTable)[i];
    f << i << "\t" 
      << m->GetName() << "\t"
      << m->GetIonisation()->GetZeffective() << "\t"
      << m->GetDensity()/(gram/cm3) << "\t"
      << m->GetRadlen()/cm << "\t"
      << m->GetNuclearInterLength()/cm
      << std::endl;
  }
  f.close();
}

//......................................................................

void PrintTables::dEdx()
{
  static const int PDG_MUON = 13;
  unsigned int i, k;
  
  unsigned int nbin = 90;
  double ke[nbin];
  double kemin = 10*keV;
  double kemax = 10*TeV;
  double dp = log10(kemax/kemin)/(nbin-1);
  double dt = pow(10.0, dp);
  ke[0] = kemin;
  for (i=1; i<nbin; ++i) ke[i] = ke[i-1]*dt;
  
  G4ParticleDefinition* p = fParticleTable->FindParticle(PDG_MUON);

  G4EmCalculator g4EMCalc;
  for (i=0; i<fMaterialTable->size(); ++i) {
    const G4Material* m = (*fMaterialTable)[i];
    double rho = m->GetDensity();

    std::string fname = "g4nova-muon-dedx-";
    fname += m->GetName();
    fname += ".txt";

    std::ofstream f;
    f.open(fname);
    f << "# MUON ENERGY LOSS FOR " << m->GetName() << std::endl;
    f << "# T [GeV]\t dE/dx [MeV cm^2/g]" << std::endl;
    for (k=0; k<nbin; ++k) {
      f << ke[k]/GeV << "\t"
	<< g4EMCalc.ComputeTotalDEDX(ke[k], p, m)/rho/(MeV*cm2/g)
	<< std::endl;
    }
    f.close();
  }
}
////////////////////////////////////////////////////////////////////////
