////////////////////////////////////////////////////////////////////////
/// \brief Use Geant4's user "hooks" to kill particles in the rock
///
/// \author  rhatcher@fnal.gov
////////////////////////////////////////////////////////////////////////

/// This class implements the G4Base::UserAction interface in order to
/// decide whether to keep or kill particles

#ifndef G4NOVA_ROCKCUTTERACTION_H
#define G4NOVA_ROCKCUTTERACTION_H

#include "G4Base/UserAction.h"

// Forward declarations.
class G4Event;
class G4Track;
class G4Step;

///NOvA specific Geant4 interface
namespace g4n {

  // accumulate a list of particles modeled in G4
  class RockCutterAction : public g4b::UserAction {

  public:
    // Standard constructors and destructors;
    RockCutterAction();
    virtual ~RockCutterAction();

    void Config(fhicl::ParameterSet const& pset);
    void PrintConfig(std::string const& opt);

    /// UserActions method that we'll override, to obtain access to
    /// Geant4's particle tracks and trajectories.
    // void BeginOfEventAction(const G4Event*);
    // void EndOfEventAction(const G4Event*);
    // void PreTrackingAction(const G4Track*);
    // void PostTrackingAction(const G4Track*);
    // void SteppingAction(const G4Step*);

    /// Does this UserAction do stacking?  
    /// Override to return "true" if the following interfaces are implemented
    bool ProvidesStacking() { return true; } 
    /// G4UserStackingAction interfaces
    G4ClassificationOfNewTrack StackClassifyNewTrack(const G4Track*);
    // void StackNewStage();  // only needed if use fWaiting,fPostpone
    // void StackPrepareNewEvent();  // only needed if use fPostpone

  private:
    double Distance_Wall(double,double,double);

    int              fVerbose;    ///< verbosity
    double           fX0;
    double           fY0;
    double           fZ0;
    double           fX_max;
    double           fX_min;
    double           fY_max;
    double           fY_min;
    double           fZ_max;
    double           fZ_min;

   };

} // namespace g4n

#endif // G4NOVA_ROCKCUTTERACTION_h
